/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.collContext;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.processoai.XmlException;
import org.nsdl.searchtools.common.ListsMap;
import org.nsdl.searchtools.common.xml.XmlUtil;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * A cache for the Collection Context information. A client can ask for the
 * information for a particular collection (specified by OAI ID). If the
 * information is already in the cache, it is returned. If not, the OAI record
 * is fetched, parsed, cached, and returned.
 * 
 * The Collection of Collections is an exception: no information is inherited
 * from it.
 * 
 * If an error occurs in fetching or in parsing the information for a
 * Collection, the error is logged and an empty result is cached, so the error
 * won't be repeated.
 * 
 * @author jblake
 */
public class CollectionFieldsCache extends CollRecCache {
	private static final Logger LOG = Logger
			.getLogger(CollectionFieldsCache.class);

	public static CollectionFieldsCache getInstance() {
		return new CollectionFieldsCache();
	}

	private final Map<String, ListsMap<LuceneFieldSpec, String>> cache = new HashMap<String, ListsMap<LuceneFieldSpec, String>>();

	private final CollectionOaiParser parser = new CollectionOaiParser();

	/**
	 * Protected to enforce the factory pattern, but usable by unit tests.
	 */
	protected CollectionFieldsCache() {
		// Nothing to instantiate.
	}

	/**
	 * Get the information for a given collection.
	 */
	public ListsMap<LuceneFieldSpec, String> getCollContextFieldInfos(
			String oaiId) {
		if (oaiId == null) {
			return new ListsMap<LuceneFieldSpec, String>();
		}

		if (oaiId.equals(CollectionContextConstants.COLL_OF_COLL_OAI_ID)) {
			return new ListsMap<LuceneFieldSpec, String>();
		}

		if (!cache.containsKey(oaiId)) {
			Document oaiDoc = getCollectionOaiRecordAsDocument(oaiId);
			if (oaiDoc == null) {
				cache.put(oaiId, new ListsMap<LuceneFieldSpec, String>());
				LOG.warn("Found no OAI for Collection '" + oaiId + "'");
			} else {
				try {
					cache.put(oaiId, parser.parse(oaiId, oaiDoc));
					LOG.debug("Cached data for Collection '" + oaiId + "'");
				} catch (XmlException e) {
					cache.put(oaiId, new ListsMap<LuceneFieldSpec, String>());
					LOG.warn("Bad OAI for Collection '" + oaiId + "'", e);
				}
			}
		}

		return cache.get(oaiId);
	}

	/**
	 * Make this protected so we can override it for unit tests. If we fail to
	 * get the collection record for any reason, return null.
	 */
	protected Document getCollectionOaiRecordAsDocument(String collRecOaiId) {
		InputStream inputStream = null;
		try {
			String urlString = CollectionContextConstants.OAI_GETRECORD_REQUEST_PREFIX
					+ collRecOaiId;
			URL url = new URL(urlString);
			URLConnection conn = url.openConnection();
			inputStream = conn.getInputStream();

			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			factory.setAttribute(XmlUtil.NAMESPACES_FEATURE_ID, true);
			DocumentBuilder builder = factory.newDocumentBuilder();
			return builder.parse(inputStream);
		} catch (MalformedURLException e) {
			LOG.error("Failed to get collection OAI record", e);
		} catch (IOException e) {
			LOG.error("Failed to get collection OAI record", e);
		} catch (ParserConfigurationException e) {
			LOG.error("Failed to get collection OAI record", e);
		} catch (SAXException e) {
			LOG.error("Failed to get collection OAI record", e);
		} finally {
			if (inputStream != null) {
				try {
					inputStream.close();
				} catch (IOException e) {
					LOG.error("Failed to close input stream "
							+ "from collection OAI record", e);
				}
			}
		}
		return null;
	}

}
