/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.collContext;

import static org.nsdl.search.util.XmlNamespaceUtil.NS_NSDL_DC;
import static org.nsdl.search.util.XmlNamespaceUtil.NS_OAI;
import static org.nsdl.search.util.XmlNamespaceUtil.NS_W3C_XML_SCHEMA_INSTANCE;

import java.util.Set;

import javax.xml.xpath.XPathExpressionException;

import org.apache.log4j.Logger;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.processoai.MetadataElementSpec;
import org.nsdl.search.processoai.XmlException;
import org.nsdl.search.util.XmlNamespaceUtil;
import org.nsdl.searchtools.common.ListsMap;
import org.nsdl.searchtools.common.xml.ElementLister;
import org.nsdl.searchtools.common.xml.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Pull the inherited fields out of an OAI record for a Collection.
 */
public class CollectionOaiParser {
	private static final Logger LOG = Logger.getLogger(CollectionOaiParser.class);
	
	
	/**
	 * Parse the OAI document, extracting the information we need from the
	 * metadata.
	 */
	public ListsMap<LuceneFieldSpec, String> parse(String oaiId, Document oaiDoc)
			throws XmlException {
		if (oaiDoc == null) {
			throw new NullPointerException("'oaiDoc' may not be null.");
		}

		ListsMap<LuceneFieldSpec, String> fields = new ListsMap<LuceneFieldSpec, String>();

		Element topElement = oaiDoc.getDocumentElement();
		if (topElement == null) {
			throw new XmlException("XML document contains no elements.");
		}

		if ((!"OAI-PMH".equals(topElement.getNodeName()))
				|| (!NS_OAI.equals(topElement.getNamespaceURI()))) {
			throw new XmlException("File is not an OAI-PMH response file.");
		}

		Element getRecordElement = XmlUtil.getChildByNameNS(topElement, NS_OAI,
				"GetRecord");
		if (getRecordElement == null) {
			throw new XmlException("File contains no oai:GetRecord tag.");
		}

		Element recordElement = XmlUtil.getChildByNameNS(getRecordElement,
				NS_OAI, "record");
		if (recordElement == null) {
			throw new XmlException("File contains no oai:record tag.");
		}

		Element headerElement = XmlUtil.getChildByNameNS(recordElement, NS_OAI,
				"header");
		if (headerElement == null) {
			throw new XmlException("File contains no oai:header tag.");
		}

		// If the collection has been deleted, we will inherit no fields.
		if ("deleted".equals(headerElement.getAttribute("status"))) {
			LOG.warn("Collection '" + oaiId + "' has been deleted.");
			return fields;
		}

		Element metadataElement = XmlUtil.getChildByNameNS(recordElement,
				NS_OAI, "metadata");
		if (metadataElement == null) {
			throw new XmlException("File contains no oai:metadata tag.");
		}

		for (Element mdGroupElement : new ElementLister(metadataElement
				.getElementsByTagNameNS(NS_NSDL_DC, "nsdl_dc"))) {
			// ignore anything labeled "native_nsdl_dc"
			if (!mdGroupElement.getPrefix().startsWith("native_")) {
				parsePublicMetadataGroup(fields, mdGroupElement);
			}
		}

		return fields;
	}

	/**
	 * Pick out the metadata items that match the {@link MetadataElementSpec}s
	 * on our list. Add their values to the map of fields.
	 */
	private void parsePublicMetadataGroup(
			ListsMap<LuceneFieldSpec, String> fields, Element mdGroupElement)
			throws XmlException {
		Set<MetadataElementSpec> mdEls = CollectionContextMetadataElements
				.getAllSpecs();
		CollectionMetadataMapping mapper = CollectionMetadataMapping
				.getInstance();

		try {
			for (Element mdElement : XmlUtil.getChildElements(mdGroupElement)) {
				MetadataMatcher matcher = new MetadataMatcher(mdElement);
				for (MetadataElementSpec mdEl : mdEls) {
					if (matches(mdEl, matcher)) {
						String mdValue = XmlUtil.getElementValue(mdElement);
						LuceneFieldSpec luceneSpec = mapper.getLuceneSpec(mdEl);
						fields.addValue(luceneSpec, mdValue);
					}
				}
			}
		} catch (XPathExpressionException e) {
			throw new XmlException("Invalid format for OAI Harvest file", e);
		}
	}

	/**
	 * When does a metadata item match a {@link MetadataElementSpec}? The name,
	 * namespace and encoding type must all match, if supplied.
	 */
	private boolean matches(MetadataElementSpec mdEl, MetadataMatcher matcher)
			throws XmlException {
		return equiv(mdEl.getName(), matcher.getName())
				&& equiv(mdEl.getNamespace(), matcher.getNamespace())
				&& equiv(mdEl.getEncoding(), matcher.getEncoding())
				&& equiv(mdEl.getEncodingNamespace(), matcher
						.getEncodingNamespace());
	}

	private boolean equiv(String first, String second) {
		return (first == null) ? (second == null) : first.equals(second);
	}

	/**
	 * Get the fields that are used to match the metadata item to a spec. It's
	 * not as simple as you might think.
	 */
	private class MetadataMatcher {
		private final String name;
		private final String namespace;
		private final String encoding;
		private final String encodingNamespace;

		public MetadataMatcher(Element mdElement)
				throws XPathExpressionException {
			this.name = mdElement.getLocalName();
			this.namespace = mdElement.getNamespaceURI();

			String encodingType = XmlUtil.getAttributeValueNS(mdElement,
					NS_W3C_XML_SCHEMA_INSTANCE, "type");
			if (encodingType == null) {
				this.encoding = null;
				this.encodingNamespace = null;
			} else {
				int colonHere = encodingType.indexOf(':');
				if (colonHere == -1) {
					this.encoding = encodingType;
					this.encodingNamespace = null;
				} else {
					this.encoding = encodingType.substring(colonHere + 1);
					this.encodingNamespace = XmlNamespaceUtil
							.getNamespaceContext().getNamespaceURI(
									encodingType.substring(0, colonHere));
				}
			}
		}

		public String getEncoding() {
			return encoding;
		}

		public String getEncodingNamespace() {
			return encodingNamespace;
		}

		public String getName() {
			return name;
		}

		public String getNamespace() {
			return namespace;
		}

	}
}
