/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.config;

import static org.nsdl.search.config.OaiMetadataElements.*;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.*;

import org.nsdl.search.oai.AbstractMetadataMapping;

/**
 * <p>
 * Connect the metadata elements in the OAI with the fields in the Lucene index
 * that they feed directly into - for the purpose of harvesting OAI records.
 * </p>
 * <p>
 * Stemmed fields and compound fields are not accounted for in this mapping.
 * Those relationships are specified directly in the Lucene field specs.
 * </p>
 * <p>
 * Data that is inherited from collections is not specified here either. That
 * data requires a separate set of metadata mappings.
 * </p>
 * 
 * @author jblake
 */
public class MetadataMapping extends AbstractMetadataMapping {
	private static final MetadataMapping instance = new MetadataMapping();

	public static MetadataMapping getInstance() {
		return instance;
	}

	private MetadataMapping() {
		/* accessibility */
		addMapping(DCT_ACCESSIBILITY, MD_DCT_ACCESSIBILITY);

		/* accrualMethod */
		addMapping(DCT_ACCRUAL_METHOD, MD_DCT_ACCRUAL_METHOD);

		/* accrualPeriodicity */
		addMapping(DCT_ACCRUAL_PERIODICITY, MD_DCT_ACCRUAL_PERIODICITY);

		/* accrualPolicy */
		addMapping(DCT_ACCRUAL_POLICY, MD_DCT_ACCRUAL_POLICY);

		/* audience */
		addMapping(DCT_AUDIENCE, MD_DCT_AUDIENCE);
		addMapping(DCT_AUDIENCE_NSDL_AUDIENCE, MD_DCT_AUDIENCE_NSDL_AUDIENCE);
		addMapping(DCT_AUDIENCE_NSDL_EDLEVEL, MD_DCT_AUDIENCE_NSDL_EDLEVEL);
		addMapping(DCT_EDLEVEL, MD_DCT_EDLEVEL);
		addMapping(DCT_EDLEVEL_NSDL_EDLEVEL, MD_DCT_EDLEVEL_NSDL_EDLEVEL);
		addMapping(DCT_MEDIATOR, MD_DCT_MEDIATOR);

		/* contributor */
		addMapping(DC_CONTRIBUTOR, MD_DC_CONTRIBUTOR);

		/* coverage */
		addMapping(DC_COVERAGE, MD_DC_COVERAGE);
		addMapping(DC_COVERAGE_BOX, MD_DC_COVERAGE_BOX);
		addMapping(DC_COVERAGE_ISO3166, MD_DC_COVERAGE_ISO3166);
		addMapping(DC_COVERAGE_POINT, MD_DC_COVERAGE_POINT);
		addMapping(DC_COVERAGE_TGN, MD_DC_COVERAGE_TGN);
		addMapping(DC_COVERAGE_ISO8601, MD_DC_COVERAGE_ISO8601);
		addMapping(DC_COVERAGE_PERIOD, MD_DC_COVERAGE_PERIOD);
		addMapping(DC_COVERAGE_W3CDTF, MD_DC_COVERAGE_W3CDTF);
		addMapping(DCT_SPATIAL, MD_DCT_SPATIAL);
		addMapping(DCT_SPATIAL_BOX, MD_DCT_SPATIAL_BOX);
		addMapping(DCT_SPATIAL_ISO3166, MD_DCT_SPATIAL_ISO3166);
		addMapping(DCT_SPATIAL_POINT, MD_DCT_SPATIAL_POINT);
		addMapping(DCT_SPATIAL_TGN, MD_DCT_SPATIAL_TGN);
		addMapping(DCT_TEMPORAL, MD_DCT_TEMPORAL);
		addMapping(DCT_TEMPORAL_ISO8601, MD_DCT_TEMPORAL_ISO8601);
		addMapping(DCT_TEMPORAL_PERIOD, MD_DCT_TEMPORAL_PERIOD);
		addMapping(DCT_TEMPORAL_W3CDTF, MD_DCT_TEMPORAL_W3CDTF);

		/* creator */
		addMapping(DC_CREATOR, MD_DC_CREATOR);

		/* date */
		addMapping(DC_DATE, MD_DC_DATE);
		addMapping(DC_DATE_ISO8601, MD_DC_DATE_ISO8601);
		addMapping(DC_DATE_PERIOD, MD_DC_DATE_PERIOD);
		addMapping(DC_DATE_W3CDTF, MD_DC_DATE_W3CDTF);
		addMapping(DCT_AVAILABLE, MD_DCT_AVAILABLE);
		addMapping(DCT_AVAILABLE_ISO8601, MD_DCT_AVAILABLE_ISO8601);
		addMapping(DCT_AVAILABLE_PERIOD, MD_DCT_AVAILABLE_PERIOD);
		addMapping(DCT_AVAILABLE_W3CDTF, MD_DCT_AVAILABLE_W3CDTF);
		addMapping(DCT_CREATED, MD_DCT_CREATED);
		addMapping(DCT_CREATED_ISO8601, MD_DCT_CREATED_ISO8601);
		addMapping(DCT_CREATED_PERIOD, MD_DCT_CREATED_PERIOD);
		addMapping(DCT_CREATED_W3CDTF, MD_DCT_CREATED_W3CDTF);
		addMapping(DCT_DATE_ACCEPTED, MD_DCT_DATE_ACCEPTED);
		addMapping(DCT_DATE_ACCEPTED_ISO8601, MD_DCT_DATE_ACCEPTED_ISO8601);
		addMapping(DCT_DATE_ACCEPTED_PERIOD, MD_DCT_DATE_ACCEPTED_PERIOD);
		addMapping(DCT_DATE_ACCEPTED_W3CDTF, MD_DCT_DATE_ACCEPTED_W3CDTF);
		addMapping(DCT_DATE_COPYRIGHTED, MD_DCT_DATE_COPYRIGHTED);
		addMapping(DCT_DATE_COPYRIGHTED_ISO8601,
				MD_DCT_DATE_COPYRIGHTED_ISO8601);
		addMapping(DCT_DATE_COPYRIGHTED_PERIOD, MD_DCT_DATE_COPYRIGHTED_PERIOD);
		addMapping(DCT_DATE_COPYRIGHTED_W3CDTF, MD_DCT_DATE_COPYRIGHTED_W3CDTF);
		addMapping(DCT_DATE_SUBMITTED, MD_DCT_DATE_SUBMITTED);
		addMapping(DCT_DATE_SUBMITTED_ISO8601, MD_DCT_DATE_SUBMITTED_ISO8601);
		addMapping(DCT_DATE_SUBMITTED_PERIOD, MD_DCT_DATE_SUBMITTED_PERIOD);
		addMapping(DCT_DATE_SUBMITTED_W3CDTF, MD_DCT_DATE_SUBMITTED_W3CDTF);
		addMapping(DCT_ISSUED, MD_DCT_ISSUED);
		addMapping(DCT_ISSUED_ISO8601, MD_DCT_ISSUED_ISO8601);
		addMapping(DCT_ISSUED_PERIOD, MD_DCT_ISSUED_PERIOD);
		addMapping(DCT_ISSUED_W3CDTF, MD_DCT_ISSUED_W3CDTF);
		addMapping(DCT_MODIFIED, MD_DCT_MODIFIED);
		addMapping(DCT_MODIFIED_ISO8601, MD_DCT_MODIFIED_ISO8601);
		addMapping(DCT_MODIFIED_PERIOD, MD_DCT_MODIFIED_PERIOD);
		addMapping(DCT_MODIFIED_W3CDTF, MD_DCT_MODIFIED_W3CDTF);
		addMapping(DCT_VALID, MD_DCT_VALID);
		addMapping(DCT_VALID_ISO8601, MD_DCT_VALID_ISO8601);
		addMapping(DCT_VALID_PERIOD, MD_DCT_VALID_PERIOD);
		addMapping(DCT_VALID_W3CDTF, MD_DCT_VALID_W3CDTF);

		/* description */
		addMapping(DC_DESCRIPTION, MD_DC_DESCRIPTION);
		addMapping(DCT_ABSTRACT, MD_DCT_ABSTRACT);
		addMapping(DCT_TABLE_OF_CONTENTS, MD_DCT_TABLE_OF_CONTENTS);

		/* format */
		addMapping(DC_FORMAT, MD_DC_FORMAT);
		addMapping(DC_FORMAT_IMT, MD_DC_FORMAT_IMT);
		addMapping(DCT_EXTENT, MD_DCT_EXTENT);
		addMapping(DCT_MEDIUM, MD_DCT_MEDIUM);
		addMapping(DCT_MEDIUM_IMT, MD_DCT_MEDIUM_IMT);

		/* identifier */
		addMapping(DC_IDENTIFIER, MD_DC_IDENTIFIER);
		addMapping(DC_IDENTIFIER_URI, MD_DC_IDENTIFIER_URI);
		addMapping(DC_IDENTIFIER_ISBN, MD_DC_IDENTIFIER_ISBN);
		addMapping(DC_IDENTIFIER_ISSN, MD_DC_IDENTIFIER_ISSN);
		addMapping(DCT_BIBLIOGRAPHIC_CITATION, MD_DCT_BIBLIOGRAPHIC_CITATION);

		/* instructionalMethod */
		addMapping(DCT_INSTRUCTIONAL_METHOD, MD_DCT_INSTRUCTIONAL_METHOD);

		/* interactivityLevel */
		addMapping(IEEE_INTERACTIVITY_LEVEL, MD_IEEE_INTERACTIVITY_LEVEL);

		/* interactivityType */
		addMapping(IEEE_INTERACTIVITY_TYPE, MD_IEEE_INTERACTIVITY_TYPE);

		/* language */
		addMapping(DC_LANGUAGE, MD_DC_LANGUAGE);
		addMapping(DC_LANGUAGE_ISO639_2, MD_DC_LANGUAGE_ISO639_2);
		addMapping(DC_LANGUAGE_RFC3066, MD_DC_LANGUAGE_RFC3066);

		/* provenance */
		addMapping(DCT_PROVENANCE, MD_DCT_PROVENANCE);

		/* publisher */
		addMapping(DC_PUBLISHER, MD_DC_PUBLISHER);

		/* relation */
		addMapping(DC_RELATION, MD_DC_RELATION);
		addMapping(DC_RELATION_URI, MD_DC_RELATION_URI);
		addMapping(DCT_CONFORMS_TO, MD_DCT_CONFORMS_TO);
		addMapping(DCT_CONFORMS_TO_URI, MD_DCT_CONFORMS_TO_URI);
		addMapping(DCT_HAS_FORMAT, MD_DCT_HAS_FORMAT);
		addMapping(DCT_HAS_FORMAT_URI, MD_DCT_HAS_FORMAT_URI);
		addMapping(DCT_HAS_PART, MD_DCT_HAS_PART);
		addMapping(DCT_HAS_PART_URI, MD_DCT_HAS_PART_URI);
		addMapping(DCT_HAS_VERSION, MD_DCT_HAS_VERSION);
		addMapping(DCT_HAS_VERSION_URI, MD_DCT_HAS_VERSION_URI);
		addMapping(DCT_IS_FORMAT_OF, MD_DCT_IS_FORMAT_OF);
		addMapping(DCT_IS_FORMAT_OF_URI, MD_DCT_IS_FORMAT_OF_URI);
		addMapping(DCT_IS_PART_OF, MD_DCT_IS_PART_OF);
		addMapping(DCT_IS_PART_OF_URI, MD_DCT_IS_PART_OF_URI);
		addMapping(DCT_IS_REFERENCED_BY, MD_DCT_IS_REFERENCED_BY);
		addMapping(DCT_IS_REFERENCED_BY_URI, MD_DCT_IS_REFERENCED_BY_URI);
		addMapping(DCT_IS_REPLACED_BY, MD_DCT_IS_REPLACED_BY);
		addMapping(DCT_IS_REPLACED_BY_URI, MD_DCT_IS_REPLACED_BY_URI);
		addMapping(DCT_IS_REQUIRED_BY, MD_DCT_IS_REQUIRED_BY);
		addMapping(DCT_IS_REQUIRED_BY_URI, MD_DCT_IS_REQUIRED_BY_URI);
		addMapping(DCT_IS_VERSION_OF, MD_DCT_IS_VERSION_OF);
		addMapping(DCT_IS_VERSION_OF_URI, MD_DCT_IS_VERSION_OF_URI);
		addMapping(DCT_REFERENCES, MD_DCT_REFERENCES);
		addMapping(DCT_REFERENCES_URI, MD_DCT_REFERENCES_URI);
		addMapping(DCT_REPLACES, MD_DCT_REPLACES);
		addMapping(DCT_REPLACES_URI, MD_DCT_REPLACES_URI);
		addMapping(DCT_REQUIRES, MD_DCT_REQUIRES);
		addMapping(DCT_REQUIRES_URI, MD_DCT_REQUIRES_URI);

		/* rights */
		addMapping(DC_RIGHTS, MD_DC_RIGHTS);
		addMapping(DC_RIGHTS_URI, MD_DC_RIGHTS_URI);
		addMapping(DCT_ACCESS_RIGHTS, MD_DCT_ACCESS_RIGHTS);
		addMapping(DCT_ACCESS_RIGHTS_NSDL_ACCESS,
				MD_DCT_ACCESS_RIGHTS_NSDL_ACCESS);
		addMapping(DCT_ACCESS_RIGHTS_URI, MD_DCT_ACCESS_RIGHTS_URI);
		addMapping(DCT_LICENSE, MD_DCT_LICENSE);
		addMapping(DCT_LICENSE_URI, MD_DCT_LICENSE_URI);

		/* rightsHolder */
		addMapping(DCT_RIGHTS_HOLDER, MD_DCT_RIGHTS_HOLDER);

		/* source */
		addMapping(DC_SOURCE, MD_DC_SOURCE);
		addMapping(DC_SOURCE_URI, MD_DC_SOURCE_URI);

		/*
		 * subject (tokenize-able fields also mapped to "subject" field for
		 * nsdl.org searches
		 */
		addMapping(DC_SUBJECT, MD_DC_SUBJECT);
		addMapping(DC_SUBJECT_DDC, MD_DC_SUBJECT_DDC);
		addMapping(DC_SUBJECT_GEM, MD_DC_SUBJECT_GEM);
		addMapping(DC_SUBJECT_LCC, MD_DC_SUBJECT_LCC);
		addMapping(DC_SUBJECT_LCSH, MD_DC_SUBJECT_LCSH);
		addMapping(DC_SUBJECT_MESH, MD_DC_SUBJECT_MESH);
		addMapping(DC_SUBJECT_UDC, MD_DC_SUBJECT_UDC);

		/* title */
		addMapping(DC_TITLE, MD_DC_TITLE);
		addMapping(DCT_ALTERNATIVE, MD_DCT_ALTERNATIVE);

		/* type */
		addMapping(DC_TYPE, MD_DC_TYPE);
		addMapping(DC_TYPE_DCMITYPE, MD_DC_TYPE_DCMITYPE);
		addMapping(DC_TYPE_NSDLTYPE, MD_DC_TYPE_NSDLTYPE);

		/* typicalLearningTime */
		addMapping(IEEE_TYPICAL_LEARNING_TIME, MD_IEEE_TYPICAL_LEARNING_TIME);
	}
}
