/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.content;

import java.io.Reader;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import net.jcip.annotations.ThreadSafe;

import org.apache.log4j.Logger;
import org.nsdl.search.generated.contentcache.ContentStatusType;
import org.nsdl.search.generated.contentcache.GetPageResponseType;
import org.nsdl.search.generated.contentcache.NSDLContentCacheType;
import org.nsdl.search.generated.contentcache.ResultCodeType;

/**
 * <p>
 * Handle the response from the Content Cache - break it into bite-sized chunks.
 * </p>
 * <p>
 * This contains some heavy-weight objects, so don't create a new instance for
 * each call, only for each thread.
 * </p>
 * 
 * @author jblake
 */
@ThreadSafe
public class ContentCacheResponseParser {
	private static final Logger LOG = Logger
			.getLogger(ContentCacheResponseParser.class);

	/**
	 * Keep one instance for each thread.
	 */
	private static final ThreadLocal<ContentCacheResponseParser> instances = new ThreadLocal<ContentCacheResponseParser>() {
		@Override
		protected ContentCacheResponseParser initialValue() {
			return new ContentCacheResponseParser();
		}
	};

	/**
	 * Get the instance for this thread.
	 */
	public static ContentCacheResponseParser getInstance() {
		return instances.get();
	}

	private final JAXBContext context;

	/**
	 * Only available through {@link #getInstance()}, to keep it thread-safe.
	 * 
	 * @throws JAXBException
	 */
	private ContentCacheResponseParser() {
		try {
			context = JAXBContext
					.newInstance("org.nsdl.search.generated.contentcache");
		} catch (JAXBException e) {
			LOG.error("Failed to create instance of JAXBContext", e);
			throw new IllegalStateException(e);
		}
	}

	/**
	 * Pull the response apart into its components and create a content object.
	 */
	public PageContent parse(Reader contentCacheResponse) throws JAXBException {
		NSDLContentCacheType cc = unmarshallResponse(contentCacheResponse);

		// Problem with the call? No content.
		ResultCodeType resultCode = cc.getResultCode().getCode();
		if (ResultCodeType.SUCCESS != resultCode) {
			LOG.debug("No content: result code was '" + resultCode + "'");
			return PageContent.EMPTY_CONTENT;
		}

		// Not fetched successfully? No content.
		GetPageResponseType gprt = cc.getGetPage();
		ContentStatusType status = gprt.getBestFetch().getStatus();
		if (ContentStatusType.FETCHED != status) {
			LOG.debug("No content: status was '" + status+ "'");
			return PageContent.EMPTY_CONTENT;
		}

		// Successful fetch? Get the pieces.
		return new PageContent(gprt.getContentType(), gprt.getBestFetch()
				.getDate(), gprt.getLatestFetch().getDate(), gprt
				.getModifyDate(), gprt.getParsedText(), gprt.getTitle(), true);
	}

	/**
	 * Create an {@link Unmarshaller} and unmarshall the response.
	 */
	@SuppressWarnings("unchecked")
	private NSDLContentCacheType unmarshallResponse(Reader contentCacheResponse)
			throws JAXBException {
		Unmarshaller unmarshaller = context.createUnmarshaller();
		Object ccObject = unmarshaller.unmarshal(contentCacheResponse);
		return ((JAXBElement<NSDLContentCacheType>) ccObject).getValue();
	}

}
