/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.content;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.Charset;

import javax.xml.bind.JAXBException;

import net.jcip.annotations.ThreadSafe;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.log4j.Logger;
import org.nsdl.search.util.Config;
import org.nsdl.searchtools.common.DateUtil;

/**
 * <p>
 * A Singleton wrapper that does an HTTP request to the ContentCache and parses
 * the reply.
 * </p>
 * <p>
 * A timing mechanism has been added, so if we get an {@link IOException} when
 * trying to contact the ContentCache, we will not try to contact it again until
 * a decent interval of time has passed. There are two reasons for this:
 * <ul>
 * <li>We don't want to flood the log with messages if the cache is offline (not
 * too important).</li>
 * <li>On a search request that yields 50 results, we don't want to wait for 50
 * timeouts from the ContentCache (much more important).</li>
 * </ul>
 * So, if we get an {@link IOException}, we record the time. Then we won't even
 * attempt any connections within the next X milliseconds, instead simply
 * returning an {@link PageContent#EMPTY_CONTENT "empty content"} result.
 * </p>
 * 
 * @author jblake
 */
@ThreadSafe
public class ContentCacheWrapperImpl extends ContentCacheWrapper {
	private static final Logger LOG = Logger
			.getLogger(ContentCacheWrapperImpl.class);

	private static ContentCacheWrapperImpl instance;

	public static synchronized ContentCacheWrapperImpl getInstance() {
		if (instance == null) {
			instance = new ContentCacheWrapperImpl();
		}
		return instance;
	}

	/**
	 * The base URL for the ContentCache service, or an empty string if no URL
	 * was supplied.
	 */
	private final String ccBaseUrl;

	/**
	 * The time of our most recent failure to contact the ContentCache.
	 */
	private long ioErrorTimestamp;

	/**
	 * If we fail to contact the ContentCache, how long before we even attempt
	 * it again? Milliseconds.
	 */
	private static final long MINIMUM_MILLIS_BETWEEN_IO_ERRORS = 10000;

	/**
	 * Private to enforce the factory pattern.
	 */
	private ContentCacheWrapperImpl() {
		ContentCacheConfig config = Config.getInstance()
				.getContentCacheConfig();
		String baseUrl = config.getContentCacheBaseUrl();
		if (baseUrl == null) {
			LOG.warn("ContentCacheBaseUrl is NULL");
			ccBaseUrl = "";
		} else {
			String trimmed = baseUrl.trim();
			if (trimmed.length() == 0) {
				LOG.warn("ContentCacheBaseUrl is empty");
			}
			ccBaseUrl = trimmed;
		}
	}

	/**
	 * Go get any content that's available for this URL. If any errors occur,
	 * log them and return {@link PageContent#EMPTY_CONTENT}. If the baseUrl has
	 * not been specified, return {@link PageContent#EMPTY_CONTENT} also.
	 */
	public PageContent getPageContent(String pageUrl) {
		LOG.debug("getPageContent for '" + pageUrl + "'");

		if (ccBaseUrl.length() == 0) {
			LOG.debug("no base url");
			return PageContent.EMPTY_CONTENT;
		}

		if (recentIoError()) {
			LOG.debug("recent timeout - not even trying");
			return PageContent.EMPTY_CONTENT;
		}

		String requestUrl = ccBaseUrl + "/getPage?url="
				+ encodePageUrl(pageUrl);

		HttpClient httpClient = new HttpClient();
		HttpMethod method = new GetMethod(requestUrl);
		try {
			httpClient.executeMethod(method);
			Reader reader = new InputStreamReader(method
					.getResponseBodyAsStream(), Charset.forName("UTF-8"));
			return ContentCacheResponseParser.getInstance().parse(reader);
		} catch (IOException e) {
			// This might happen, and if it does it will likely flood the log.
			LOG.warn("Can't contact Content Cache at '" + requestUrl + "': "
					+ e);
			ioErrorTimestamp = DateUtil.getInstance().now().getTime();
		} catch (JAXBException e) {
			// Non-schema-valid response from Content Cache. That's a problem.
			LOG.error("invalid response for page '" + pageUrl + "'", e);
		} finally {
			method.releaseConnection();
		}
		return PageContent.EMPTY_CONTENT;
	}

	/**
	 * Encode any special characters into "%" notation.
	 */
	private String encodePageUrl(String pageUrl) {
		try {
			return URLEncoder.encode(pageUrl, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			// This should never happen - the encoding name is hardcoded!
			LOG.error("OMG", e);
			return pageUrl;
		}
	}

	/**
	 * Have we recently encountered an IO error?
	 */
	private boolean recentIoError() {
		long now = DateUtil.getInstance().now().getTime();
		return (now - ioErrorTimestamp) < MINIMUM_MILLIS_BETWEEN_IO_ERRORS;
	}
}
