/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.content;

import java.util.Date;

import net.jcip.annotations.Immutable;

import org.nsdl.search.lucene.searchdoc.SearchDocumentContent;
import org.nsdl.searchtools.common.DateUtil;

/**
 * An immutable data object that holds the response from the ContentCache.
 * 
 * @author jblake
 */
@Immutable
public class PageContent implements SearchDocumentContent {
	/**
	 * A page with no valid content. Use non-null values for the fields, just in
	 * case the user violates the contract and queries them.
	 */
	public static final PageContent EMPTY_CONTENT = new PageContent("",
			new Date(0), new Date(0), new Date(0), "", "", false);

	private final String contentType;
	private final Date fetchDate;
	private final Date modifiedDate;
	private final int livenessLapseDays;
	private final String text;
	private final String title;
	private final boolean valid;

	public PageContent(String contentType, Date bestFetchDate,
			Date latestFetchDate, Date modifiedDate, String text, String title,
			boolean valid) {
		this.contentType = contentType;
		this.fetchDate = bestFetchDate;
		this.modifiedDate = modifiedDate;
		this.livenessLapseDays = figureLivenessLapse(latestFetchDate,
				bestFetchDate);
		this.text = text;
		this.title = title;
		this.valid = valid;
	}

	/**
	 * How many days between the latest and best fetches?
	 */
	private int figureLivenessLapse(Date latestFetchDate, Date bestFetchDate) {
		if ((latestFetchDate == null) || (bestFetchDate == null)
				|| latestFetchDate.before(bestFetchDate)) {
			return 0;
		}
		long interval = latestFetchDate.getTime() - bestFetchDate.getTime();
		return (int) (interval / DateUtil.MILLIS_IN_A_DAY);
	}

	public String getContentType() {
		return contentType;
	}

	public Date getFetchDate() {
		return fetchDate;
	}

	/**
	 * The number of elapsed days between the best fetch and the latest fetch.
	 * For a live link, this should be 0.
	 */
	public int getLivenessLapseDays() {
		return livenessLapseDays;
	}

	public Date getModifiedDate() {
		return modifiedDate;
	}

	public String getText() {
		return text;
	}

	public String getTitle() {
		return title;
	}

	public boolean isValid() {
		return valid;
	}

}
