/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene;

import net.jcip.annotations.Immutable;

import org.apache.lucene.document.Field;

/**
 * Immutable description of a field in the Lucene index, and how that field
 * should be handled.
 * 
 * @author jblake
 */
@Immutable
public final class LuceneFieldSpec implements Comparable<LuceneFieldSpec> {
	public static final String STEMMING_SUFFIX = ".Stems";
	public static final float DEFAULT_BOOST_VALUE = 1.0F;

	/** The name of the Lucene Field. Not null. */
	private final String name;

	/** Is the value of the field stored in the index? How? */
	private final Field.Store store;

	/** Is the field indexed? Tokenized? */
	private final Field.Index index;

	/** Converted to lower case before being indexed? */
	private final boolean caseFold;

	/** Will the field have term vectors? What sort? */
	private final Field.TermVector termVector;

	/** Boost factor for the field. Default is 1.0 */
	private final float boost;

	/** A compound field that this field feeds to. May be null. */
	private final LuceneFieldSpec compoundField;

	/** Should the lucene index also have a stemmed version of this field? */
	private final boolean stemmable;

	/** Is this field tokenized? If not, will require special query parsing! */
	private final boolean tokenized;

	/** Are the contents of this field derived from other fields? */
	private final boolean derived;

	/** Are the contents of this field added to the default search field? */
	private final boolean inDefaultField;

	/** Value displayed by toString(). */
	private final String stringRepresentation;

	/** Value returned by hashCode(). */
	private final int hashCode;

	/** Value returned by getStemmedSpec(). */
	private final LuceneFieldSpec stemmedSpec;

	/**
	 * Build from individual data elements. The compound field may be null, but
	 * all other data are required. If present, the compound field must be a
	 * derived field.
	 */
	public LuceneFieldSpec(String name, Field.Store store, Field.Index index,
			boolean caseFold, float boost, LuceneFieldSpec compoundField,
			boolean stemmable, boolean tokenized, boolean derived,
			boolean inDefaultField) {
		if (name == null) {
			throw new IllegalArgumentException("name may not be null.");
		}
		if (store == null) {
			throw new IllegalArgumentException("store may not be null.");
		}
		if (index == null) {
			throw new IllegalArgumentException("index may not be null.");
		}
		if ((compoundField != null) && (!compoundField.isDerived())) {
			throw new IllegalArgumentException(
					"compoundField must be a derived field.");
		}
		this.name = name;
		this.store = store;
		this.index = index;
		this.caseFold = caseFold;
		this.termVector = Field.TermVector.NO;
		this.boost = boost;
		this.compoundField = compoundField;
		this.stemmable = stemmable;
		this.tokenized = tokenized;
		this.derived = derived;
		this.inDefaultField = inDefaultField;
		this.stringRepresentation = figureStringRepresentation();
		this.hashCode = figureHashCode();
		this.stemmedSpec = figureStemmedSpec();
	}

	/** Since the object is immutable, we only need to figure this once. */
	private String figureStringRepresentation() {
		return "LuceneFieldSpec:[name='" + name + "', store=" + store
				+ ", index=" + index + ", caseFold=" + caseFold
				+ ", termVector=" + termVector + ", boost=" + boost
				+ "F, compoundFieldName='"
				+ ((compoundField == null) ? null : compoundField.getName())
				+ "', stemmable=" + stemmable + ", tokenized=" + tokenized
				+ ", derived=" + derived + ", inDefaultField=" + inDefaultField
				+ "]";
	}

	/** Since the object is immutable, we only need to figure this once. */
	private int figureHashCode() {
		return name.hashCode() ^ store.hashCode() ^ index.hashCode()
				^ termVector.hashCode() ^ Boolean.valueOf(caseFold).hashCode()
				^ Boolean.valueOf(stemmable).hashCode()
				^ Boolean.valueOf(tokenized).hashCode()
				^ Boolean.valueOf(derived).hashCode()
				^ Boolean.valueOf(inDefaultField).hashCode()
				^ Float.valueOf(boost).hashCode()
				^ ((compoundField == null) ? 0 : compoundField.hashCode());
	}

	/**
	 * The stemmed version of this has the stemmed name, is not stored, and is
	 * not stemmable. Since the object is immutable, we only need to figure this
	 * once.
	 */
	private LuceneFieldSpec figureStemmedSpec() {
		if (stemmable) {
			return new LuceneFieldSpec(name + STEMMING_SUFFIX, Field.Store.NO,
					index, caseFold, boost, compoundField, false, tokenized,
					derived, inDefaultField);
		} else {
			return null;
		}
	}

	public float getBoost() {
		return boost;
	}

	public boolean isCaseFold() {
		return caseFold;
	}

	public LuceneFieldSpec getCompoundField() {
		return compoundField;
	}

	public String getCompoundFieldName() {
		return (compoundField == null) ? null : compoundField.getName();
	}

	public Field.Index getIndex() {
		return index;
	}

	public boolean isIndexed() {
		return (index == Field.Index.TOKENIZED);
	}

	public Field.Store getStore() {
		return store;
	}

	public Field.TermVector getTermVector() {
		return termVector;
	}

	public boolean isStemmable() {
		return stemmable;
	}

	public String getName() {
		return name;
	}

	public boolean isSpecialParsing() {
		return !tokenized;
	}

	public boolean isTokenized() {
		return tokenized;
	}

	public boolean isDerived() {
		return derived;
	}

	public boolean isInDefaultField() {
		return inDefaultField;
	}

	public LuceneFieldSpec getStemmedSpec() {
		if (stemmable) {
			return stemmedSpec;
		} else {
			throw new UnsupportedOperationException("LuceneFieldSpec '" + name
					+ "' is not stemmable.");
		}
	}

	@Override
	public boolean equals(Object other) {
		if (other == null) {
			return false;
		}
		if (other == this) {
			return true;
		}
		if (!LuceneFieldSpec.class.equals(other.getClass())) {
			return false;
		}
		LuceneFieldSpec that = (LuceneFieldSpec) other;
		return (this.stringRepresentation.equals(that.stringRepresentation));
	}

	@Override
	public int hashCode() {
		return hashCode;
	}

	@Override
	public String toString() {
		return stringRepresentation;
	}

	/**
	 * Sort by their string representations.
	 */
	public int compareTo(LuceneFieldSpec that) {
		return this.stringRepresentation.compareTo(that.stringRepresentation);
	}

}
