/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene;

/**
 * Some helpful methods for dealing with the Lucene index.
 * 
 * @author jblake
 */
public class LuceneUtil {
	/**
	 * According to http://lucene.apache.org/java/docs/queryparsersyntax.html on
	 * 2005-03-31, the following chars need escaping for the Lucene QueryParser.
	 */
	public static final char[] SPECIAL_CHARACTERS = new char[]{'+', '-', '&', '|',
			'!', '{', '}', '(', ')', '[', ']', '^', '"', '~', '*', '?', ':',
			'\\'};

	private static LuceneUtil instance = new LuceneUtil();

	public static LuceneUtil getInstance() {
		return instance;
	}

	private LuceneUtil() {
		// keep it private to enforce the Singleton pattern.
	}

	/**
	 * <p>
	 * Escape special characters in a search term or field name, so they will
	 * not be interpreted as special by the Lucene query parser.
	 * </p>
	 * <p>
	 * The documentation for QueryParser refers to a page on query parser
	 * syntax. This has a section on escaping special character, that reads as
	 * follows:
	 * </p>
	 * <p>
	 * Lucene supports escaping special characters that are part of the query
	 * syntax. The current list special characters are
	 * 
	 * <pre>
	 * + - &amp;&amp; || ! ( ) { } [ ] &circ; &quot; &tilde; * ? : \
	 * </pre>
	 * 
	 * To escape these character use a back-slash before the character.
	 * </p>
	 * <p>
	 * It's not clear from this whether the "or" operator ("||") needs a single
	 * back-slash before it, or two. Experimenting with Luke indicates that
	 * either will work. We go with back-slashes before each stroke (and before
	 * each ampersand in the "and" operator), because it's simpler.
	 * </p>
	 * 
	 * @param raw
	 *            the original string.
	 * @return the string with special characters escaped.
	 */
	public String encodeLuceneChars(String raw) {
		if (raw == null) {
			return null;
		}

		StringBuilder result = new StringBuilder();
		for (int i = 0; i < raw.length(); i++) {
			char thisChar = raw.charAt(i);
			for (char special : SPECIAL_CHARACTERS) {
				if (thisChar == special) {
					result.append('\\');
					break;
				}
			}
			result.append(thisChar);
		}
		return result.toString();
	}
}
