/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.lucene.document.Field;

/**
 * <p>
 * Functional descriptions of the fields in the Lucene index.
 * </p>
 * <p>
 * This describes all of the fields in the index. A "shorthand" is used to
 * derive the stemmed fields from their "parent" fields, but once included they
 * appear as first-class members.
 * </p>
 * 
 * <p>
 * Fields inherited from Collections are listed here also. We want a collection
 * of all fields in the index, regardless of their sources.
 * </p>
 * 
 * @author jblake
 */
public class SearchIndexFieldSpecs {
	private static final Map<String, LuceneFieldSpec> specsByName = new HashMap<String, LuceneFieldSpec>();

	/* --------------------------------------------------------------------- */
	/* Resource-based fields. */
	/* --------------------------------------------------------------------- */

	public static final LuceneFieldSpec RESOURCE_IDENTIFIER = keywordField(
			"resourceIdentifier", null, false, false);
	public static final LuceneFieldSpec DOCUMENT_CREATED = keywordField(
			"documentCreated", null, true, false);
	public static final LuceneFieldSpec DOCUMENT_LAST_UPDATED = keywordField(
			"documentLastUpdated", null, true, false);
	public static final LuceneFieldSpec METADATA_LAST_MODIFIED = keywordField(
			"metadataLastModified", null, true, false);
	public static final LuceneFieldSpec CONTENT = notStored("content");
	public static final LuceneFieldSpec CONTENT_LAST_MODIFIED = keywordField(
			"contentLastModified", null, true, false);
	public static final LuceneFieldSpec CONTENT_LAST_FETCHED = keywordField(
			"contentLastFetched", null, true, false);

	public static final LuceneFieldSpec ALL_FIELDS = defaultSearchField("allFields");

	/* --------------------------------------------------------------------- */
	/* Metadata-related fields. */
	/* --------------------------------------------------------------------- */

	public static final LuceneFieldSpec METADATA_SOURCE = keywordField(
			"metadataSource", null, false, false);
	public static final LuceneFieldSpec IN_COLLECTION = keywordField(
			"inCollection", null, false, false);
	public static final LuceneFieldSpec COLL_IMAGE_URL = notIndexed("collImageUrl");
	public static final LuceneFieldSpec COLL_IMAGE_WIDTH = notIndexed("collImageWidth");
	public static final LuceneFieldSpec COLL_IMAGE_HEIGHT = notIndexed("collImageHeight");
	public static final LuceneFieldSpec COLL_IMAGE_ALT_TEXT = notIndexed("collImageAltText");

	public static final LuceneFieldSpec COMPOUND_AGENT = compoundField("compoundAgent");
	public static final LuceneFieldSpec COMPOUND_AUDIENCE = compoundField("compoundAudience");
	public static final LuceneFieldSpec COMPOUND_DESCRIPTION = compoundField("compoundDescription");
	public static final LuceneFieldSpec COMPOUND_GENRE = compoundField("compoundGenre");
	public static final LuceneFieldSpec COMPOUND_SUBJECT = compoundField("compoundSubject");
	public static final LuceneFieldSpec COMPOUND_TITLE = compoundField("compoundTitle");

	/* --------------------------------------------------------------------- */
	/* Metadata: Dublin core and extensions. */
	/* --------------------------------------------------------------------- */

	/* accessibility */
	public static final LuceneFieldSpec DCT_ACCESSIBILITY = standardField(
			"dct:accessibility", null, true, 1.0F);

	/* accrualMethod */
	public static final LuceneFieldSpec DCT_ACCRUAL_METHOD = standardField(
			"dct:accrualMethod", null, true, 1.0F);

	/* accrualPeriodicity */
	public static final LuceneFieldSpec DCT_ACCRUAL_PERIODICITY = standardField(
			"dct:accrualPeriodicity", null, true, 1.0F);

	/* accrualPolicy */
	public static final LuceneFieldSpec DCT_ACCRUAL_POLICY = standardField(
			"dct:accrualPolicy", null, true, 1.0F);

	/* audience */
	public static final LuceneFieldSpec DCT_AUDIENCE = standardField(
			"dct:audience", COMPOUND_AUDIENCE, true, 1.0F);
	public static final LuceneFieldSpec DCT_AUDIENCE_NSDL_AUDIENCE = keywordField(
			"dct:audience.nsdl_dc:NSDLAudience", COMPOUND_AUDIENCE, true, true);
	public static final LuceneFieldSpec DCT_AUDIENCE_NSDL_EDLEVEL = keywordField(
			"dct:audience.nsdl_dc:NSDLEdLevel", COMPOUND_AUDIENCE, true, true);
	public static final LuceneFieldSpec DCT_EDLEVEL = standardField(
			"dct:educationLevel", COMPOUND_AUDIENCE, true, 1.0F);
	public static final LuceneFieldSpec DCT_EDLEVEL_NSDL_EDLEVEL = keywordField(
			"dct:educationLevel.nsdl_dc:NSDLEdLevel", COMPOUND_AUDIENCE, true,
			true);
	public static final LuceneFieldSpec DCT_MEDIATOR = standardField(
			"dct:mediator", COMPOUND_AUDIENCE, true, 1.0F);

	/* contributor */
	public static final LuceneFieldSpec DC_CONTRIBUTOR = standardField(
			"dc:contributor", COMPOUND_AGENT, true, 1.0F);

	/* coverage */
	public static final LuceneFieldSpec DC_COVERAGE = standardField(
			"dc:coverage", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DC_COVERAGE_BOX = standardField(
			"dc:coverage.dct:Box", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DC_COVERAGE_ISO3166 = standardField(
			"dc:coverage.dct:ISO3166", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DC_COVERAGE_POINT = standardField(
			"dc:coverage.dct:Point", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DC_COVERAGE_TGN = standardField(
			"dc:coverage.dct:TGN", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DC_COVERAGE_ISO8601 = keywordField(
			"dc:coverage.dct:ISO8601", COMPOUND_SUBJECT, true, true);
	public static final LuceneFieldSpec DC_COVERAGE_PERIOD = standardField(
			"dc:coverage.dct:Period", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DC_COVERAGE_W3CDTF = keywordField(
			"dc:coverage.dct:W3CDTF", COMPOUND_SUBJECT, true, true);
	public static final LuceneFieldSpec DCT_SPATIAL = standardField(
			"dct:spatial", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DCT_SPATIAL_BOX = standardField(
			"dct:spatial.dct:Box", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DCT_SPATIAL_ISO3166 = standardField(
			"dct:spatial.dct:ISO3166", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DCT_SPATIAL_POINT = standardField(
			"dct:spatial.dct:Point", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DCT_SPATIAL_TGN = standardField(
			"dct:spatial.dct:TGN", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DCT_TEMPORAL = standardField(
			"dct:temporal", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DCT_TEMPORAL_ISO8601 = keywordField(
			"dct:temporal.dct:ISO8601", COMPOUND_SUBJECT, true, true);
	public static final LuceneFieldSpec DCT_TEMPORAL_PERIOD = standardField(
			"dct:temporal.dct:Period", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DCT_TEMPORAL_W3CDTF = keywordField(
			"dct:temporal.dct:W3CDTF", COMPOUND_SUBJECT, true, true);

	/* creator */
	public static final LuceneFieldSpec DC_CREATOR = standardField(
			"dc:creator", COMPOUND_AGENT, true, 1.0F);

	/* date */
	public static final LuceneFieldSpec DC_DATE = standardField("dc:date",
			null, true, 1.0F);
	public static final LuceneFieldSpec DC_DATE_ISO8601 = keywordField(
			"dc:date.dct:ISO8601", null, true, true);
	public static final LuceneFieldSpec DC_DATE_PERIOD = standardField(
			"dc:date.dct:Period", null, true, 1.0F);
	public static final LuceneFieldSpec DC_DATE_W3CDTF = keywordField(
			"dc:date.dct:W3CDTF", null, true, true);
	public static final LuceneFieldSpec DCT_AVAILABLE = standardField(
			"dct:available", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_AVAILABLE_ISO8601 = keywordField(
			"dct:available.dct:ISO8601", null, true, true);
	public static final LuceneFieldSpec DCT_AVAILABLE_PERIOD = standardField(
			"dct:available.dct:Period", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_AVAILABLE_W3CDTF = keywordField(
			"dct:available.dct:W3CDTF", null, true, true);
	public static final LuceneFieldSpec DCT_CREATED = standardField(
			"dct:created", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_CREATED_ISO8601 = keywordField(
			"dct:created.dct:ISO8601", null, true, true);
	public static final LuceneFieldSpec DCT_CREATED_PERIOD = standardField(
			"dct:created.dct:Period", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_CREATED_W3CDTF = keywordField(
			"dct:created.dct:W3CDTF", null, true, true);
	public static final LuceneFieldSpec DCT_DATE_ACCEPTED = standardField(
			"dct:dateAccepted", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_DATE_ACCEPTED_ISO8601 = keywordField(
			"dct:dateAccepted.dct:ISO8601", null, true, true);
	public static final LuceneFieldSpec DCT_DATE_ACCEPTED_PERIOD = standardField(
			"dct:dateAccepted.dct:Period", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_DATE_ACCEPTED_W3CDTF = keywordField(
			"dct:dateAccepted.dct:W3CDTF", null, true, true);
	public static final LuceneFieldSpec DCT_DATE_COPYRIGHTED = standardField(
			"dct:dateCopyrighted", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_DATE_COPYRIGHTED_ISO8601 = keywordField(
			"dct:dateCopyrighted.dct:ISO8601", null, true, true);
	public static final LuceneFieldSpec DCT_DATE_COPYRIGHTED_PERIOD = standardField(
			"dct:dateCopyrighted.dct:Period", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_DATE_COPYRIGHTED_W3CDTF = keywordField(
			"dct:dateCopyrighted.dct:W3CDTF", null, true, true);
	public static final LuceneFieldSpec DCT_DATE_SUBMITTED = standardField(
			"dct:dateSubmitted", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_DATE_SUBMITTED_ISO8601 = keywordField(
			"dct:dateSubmitted.dct:ISO8601", null, true, true);
	public static final LuceneFieldSpec DCT_DATE_SUBMITTED_PERIOD = standardField(
			"dct:dateSubmitted.dct:Period", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_DATE_SUBMITTED_W3CDTF = keywordField(
			"dct:dateSubmitted.dct:W3CDTF", null, true, true);
	public static final LuceneFieldSpec DCT_ISSUED = standardField(
			"dct:issued", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_ISSUED_ISO8601 = keywordField(
			"dct:issued.dct:ISO8601", null, true, true);
	public static final LuceneFieldSpec DCT_ISSUED_PERIOD = standardField(
			"dct:issued.dct:Period", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_ISSUED_W3CDTF = keywordField(
			"dct:issued.dct:W3CDTF", null, true, true);
	public static final LuceneFieldSpec DCT_MODIFIED = standardField(
			"dct:modified", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_MODIFIED_ISO8601 = keywordField(
			"dct:modified.dct:ISO8601", null, true, true);
	public static final LuceneFieldSpec DCT_MODIFIED_PERIOD = standardField(
			"dct:modified.dct:Period", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_MODIFIED_W3CDTF = keywordField(
			"dct:modified.dct:W3CDTF", null, true, true);
	public static final LuceneFieldSpec DCT_VALID = standardField("dct:valid",
			null, true, 1.0F);
	public static final LuceneFieldSpec DCT_VALID_ISO8601 = keywordField(
			"dct:valid.dct:ISO8601", null, true, true);
	public static final LuceneFieldSpec DCT_VALID_PERIOD = standardField(
			"dct:valid.dct:Period", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_VALID_W3CDTF = keywordField(
			"dct:valid.dct:W3CDTF", null, true, true);

	/* description */
	public static final LuceneFieldSpec DC_DESCRIPTION = standardField(
			"dc:description", COMPOUND_DESCRIPTION, true, 1.0F);
	public static final LuceneFieldSpec DCT_ABSTRACT = standardField(
			"dct:abstract", COMPOUND_DESCRIPTION, true, 1.0F);
	public static final LuceneFieldSpec DCT_TABLE_OF_CONTENTS = standardField(
			"dct:tableOfContents", COMPOUND_DESCRIPTION, true, 1.0F);

	/* format */
	public static final LuceneFieldSpec DC_FORMAT = standardField("dc:format",
			COMPOUND_GENRE, true, 1.0F);
	public static final LuceneFieldSpec DC_FORMAT_IMT = keywordField(
			"dc:format.dct:IMT", COMPOUND_GENRE, true, true);
	public static final LuceneFieldSpec DCT_EXTENT = standardField(
			"dct:extent", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_MEDIUM = standardField(
			"dct:medium", COMPOUND_GENRE, true, 1.0F);
	// NOTE: medium IMT is an error according to DC, but we get it anyway.
	public static final LuceneFieldSpec DCT_MEDIUM_IMT = keywordField(
			"dct:medium.dct:IMT", COMPOUND_GENRE, true, true);

	/* identifier */
	public static final LuceneFieldSpec DC_IDENTIFIER = standardField(
			"dc:identifier", null, true, 1.0F);
	public static final LuceneFieldSpec DC_IDENTIFIER_URI = keywordField(
			"dc:identifier.dct:URI", null, false, false);
	public static final LuceneFieldSpec DC_IDENTIFIER_ISBN = keywordField(
			"dc:identifier.dct:ISBN", null, true, false);
	public static final LuceneFieldSpec DC_IDENTIFIER_ISSN = keywordField(
			"dc:identifier.dct:ISSN", null, true, false);
	public static final LuceneFieldSpec DCT_BIBLIOGRAPHIC_CITATION = standardField(
			"dct:bibliographicCitation", null, true, 1.0F);

	/* instructionalMethod */
	public static final LuceneFieldSpec DCT_INSTRUCTIONAL_METHOD = standardField(
			"dct:instructionalMethod", null, true, 1.0F);

	/* interactivityLevel */
	public static final LuceneFieldSpec IEEE_INTERACTIVITY_LEVEL = standardField(
			"ieee:interactivityLevel", null, true, 1.0F);

	/* interactivityType */
	public static final LuceneFieldSpec IEEE_INTERACTIVITY_TYPE = standardField(
			"ieee:interactivityType", null, true, 1.0F);

	/* language */
	public static final LuceneFieldSpec DC_LANGUAGE = standardField(
			"dc:language", null, true, 1.0F);
	public static final LuceneFieldSpec DC_LANGUAGE_ISO639_2 = keywordField(
			"dc:language.dct:ISO639-2", null, true, false);
	public static final LuceneFieldSpec DC_LANGUAGE_RFC3066 = keywordField(
			"dc:language.dct:RFC3066", null, true, false);

	/* provenance */
	public static final LuceneFieldSpec DCT_PROVENANCE = standardField(
			"dct:provenance", null, true, 1.0F);

	/* publisher */
	public static final LuceneFieldSpec DC_PUBLISHER = standardField(
			"dc:publisher", COMPOUND_AGENT, true, 1.0F);

	/* relation */
	public static final LuceneFieldSpec DC_RELATION = standardField(
			"dc:relation", null, true, 1.0F);
	public static final LuceneFieldSpec DC_RELATION_URI = keywordField(
			"dc:relation.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_CONFORMS_TO = standardField(
			"dct:conformsTo", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_CONFORMS_TO_URI = keywordField(
			"dct:conformsTo.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_HAS_FORMAT = standardField(
			"dct:hasFormat", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_HAS_FORMAT_URI = keywordField(
			"dct:hasFormat.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_HAS_PART = standardField(
			"dct:hasPart", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_HAS_PART_URI = keywordField(
			"dct:hasPart.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_HAS_VERSION = standardField(
			"dct:hasVersion", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_HAS_VERSION_URI = keywordField(
			"dct:hasVersion.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_IS_FORMAT_OF = standardField(
			"dct:isFormatOf", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_IS_FORMAT_OF_URI = keywordField(
			"dct:isFormatOf.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_IS_PART_OF = standardField(
			"dct:isPartOf", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_IS_PART_OF_URI = keywordField(
			"dct:isPartOf.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_IS_REFERENCED_BY = standardField(
			"dct:isReferencedBy", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_IS_REFERENCED_BY_URI = keywordField(
			"dct:isReferencedBy.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_IS_REPLACED_BY = standardField(
			"dct:isReplacedBy", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_IS_REPLACED_BY_URI = keywordField(
			"dct:isReplacedBy.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_IS_REQUIRED_BY = standardField(
			"dct:isRequiredBy", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_IS_REQUIRED_BY_URI = keywordField(
			"dct:isRequiredBy.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_IS_VERSION_OF = standardField(
			"dct:isVersionOf", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_IS_VERSION_OF_URI = keywordField(
			"dct:isVersionOf.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_REFERENCES = standardField(
			"dct:references", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_REFERENCES_URI = keywordField(
			"dct:references.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_REPLACES = standardField(
			"dct:replaces", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_REPLACES_URI = keywordField(
			"dct:replaces.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_REQUIRES = standardField(
			"dct:requires", null, true, 1.0F);
	public static final LuceneFieldSpec DCT_REQUIRES_URI = keywordField(
			"dct:requires.dct:URI", null, false, false);

	/* rights */
	public static final LuceneFieldSpec DC_RIGHTS = standardField("dc:rights",
			null, false, 1.0F);
	public static final LuceneFieldSpec DC_RIGHTS_URI = keywordField(
			"dc:rights.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_ACCESS_RIGHTS = standardField(
			"dct:accessRights", null, false, 1.0F);
	public static final LuceneFieldSpec DCT_ACCESS_RIGHTS_NSDL_ACCESS = keywordField(
			"dct:accessRights.nsdl_dc:NSDLAccess", null, true, true);
	public static final LuceneFieldSpec DCT_ACCESS_RIGHTS_URI = keywordField(
			"dct:accessRights.dct:URI", null, false, false);
	public static final LuceneFieldSpec DCT_LICENSE = standardField(
			"dct:license", null, false, 1.0F);
	public static final LuceneFieldSpec DCT_LICENSE_URI = keywordField(
			"dct:license.dct:URI", null, false, false);

	/* rightsHolder */
	public static final LuceneFieldSpec DCT_RIGHTS_HOLDER = standardField(
			"dct:rightsHolder", COMPOUND_AGENT, true, 1.0F);

	/* source */
	public static final LuceneFieldSpec DC_SOURCE = standardField("dc:source",
			null, true, 1.0F);
	public static final LuceneFieldSpec DC_SOURCE_URI = keywordField(
			"dc:source.dct:URI", null, false, false);

	/*
	 * subject (tokenize-able fields also mapped to "subject" field for nsdl.org
	 * searches
	 */
	public static final LuceneFieldSpec DC_SUBJECT = standardField(
			"dc:subject", COMPOUND_SUBJECT, true, 1.0F);
	public static final LuceneFieldSpec DC_SUBJECT_DDC = keywordField(
			"dc:subject.dct:DDC", null, true, false);
	public static final LuceneFieldSpec DC_SUBJECT_GEM = keywordField(
			"dc:subject.nsdl_dc:GEM", COMPOUND_SUBJECT, true, true);
	public static final LuceneFieldSpec DC_SUBJECT_LCC = keywordField(
			"dc:subject.dct:LCC", null, true, false);
	public static final LuceneFieldSpec DC_SUBJECT_LCSH = keywordField(
			"dc:subject.dct:LCSH", COMPOUND_SUBJECT, true, true);
	public static final LuceneFieldSpec DC_SUBJECT_MESH = keywordField(
			"dc:subject.dct:MESH", COMPOUND_SUBJECT, true, true);
	public static final LuceneFieldSpec DC_SUBJECT_UDC = keywordField(
			"dc:subject.dct:UDC", null, true, false);

	/* title */
	public static final LuceneFieldSpec DC_TITLE = standardField("dc:title",
			COMPOUND_TITLE, true, 1.0F);
	public static final LuceneFieldSpec DCT_ALTERNATIVE = standardField(
			"dct:alternative", COMPOUND_TITLE, true, 1.0F);

	/* type */
	public static final LuceneFieldSpec DC_TYPE = standardField("dc:type",
			COMPOUND_GENRE, true, 1.0F);
	public static final LuceneFieldSpec DC_TYPE_DCMITYPE = keywordField(
			"dc:type.dct:DCMIType", COMPOUND_GENRE, true, true);
	public static final LuceneFieldSpec DC_TYPE_NSDLTYPE = keywordField(
			"dc:type.nsdl_dc:NSDLType", COMPOUND_GENRE, true, true);

	/* typicalLearningTime */
	public static final LuceneFieldSpec IEEE_TYPICAL_LEARNING_TIME = standardField(
			"ieee:typicalLearningTime", null, true, 1.0F);

	/* --------------------------------------------------------------------- */
	/* Collection context metadata: inherited from Collection records. */
	/* --------------------------------------------------------------------- */

	public static final LuceneFieldSpec COLL_DCT_ACCESS_RIGHTS = standardField(
			"coll.dct:accessRights", null, false, 0.3F);
	public static final LuceneFieldSpec COLL_DCT_ALTERNATIVE = standardField(
			"coll.dct:alternative", COMPOUND_TITLE, true, 0.8F);
	public static final LuceneFieldSpec COLL_DCT_AUDIENCE = standardField(
			"coll.dct:audience", COMPOUND_AUDIENCE, true, 1.0F);
	public static final LuceneFieldSpec COLL_DCT_AUDIENCE_NSDL_EDLEVEL = keywordField(
			"coll.dct:audience.nsdl_dc:NSDLEdLevel", COMPOUND_AUDIENCE, true,
			true);
	public static final LuceneFieldSpec COLL_DC_DESCRIPTION = standardField(
			"coll.dc:description", COMPOUND_DESCRIPTION, true, 0.8F);
	public static final LuceneFieldSpec COLL_DCT_EDLEVEL = standardField(
			"coll.dct:educationLevel", COMPOUND_AUDIENCE, true, 1.0F);
	public static final LuceneFieldSpec COLL_DCT_EDLEVEL_NSDL_EDLEVEL = keywordField(
			"coll.dct:educationLevel.nsdl_dc:NSDLEdLevel", COMPOUND_AUDIENCE,
			true, true);
	public static final LuceneFieldSpec COLL_DCT_LICENSE = standardField(
			"coll.dct:license", null, false, 0.3F);
	public static final LuceneFieldSpec COLL_DCT_MEDIATOR = standardField(
			"coll.dct:mediator", COMPOUND_AUDIENCE, true, 1.0F);
	public static final LuceneFieldSpec COLL_DC_RIGHTS = standardField(
			"coll.dc:rights", null, false, 0.3F);
	public static final LuceneFieldSpec COLL_DC_TITLE = standardField(
			"coll.dc:title", COMPOUND_TITLE, true, 0.8F);

	/**
	 * <p>
	 * Create a standard field.
	 * </p>
	 * <p>
	 * Standard fields are:
	 * <ul>
	 * <li>case-folded</li>
	 * <li>tokenized</li>
	 * <li>indexed</li>
	 * <li>stored</li>
	 * <li>stemmed</li>
	 * <li>not derived</li>
	 * </ul>
	 * 
	 * @param compound
	 *            Compound field that this field contributes to, or null.
	 * @param inDefaultField
	 *            Does this field contribute to the default search field?
	 * @param boost
	 *            Do we want to rank this field higher or lower than the others?
	 */
	private static LuceneFieldSpec standardField(String name,
			LuceneFieldSpec compound, boolean inDefaultField, float boost) {
		return addSpec(name, Field.Store.YES, Field.Index.TOKENIZED, true,
				boost, compound, true, true, false, inDefaultField);
	}

	/**
	 * <p>
	 * Create a keyword field.
	 * </p>
	 * <p>
	 * Keywords fields are:
	 * <ul>
	 * <li>not tokenized</li>
	 * <li>indexed</li>
	 * <li>stored</li>
	 * <li>not stemmed</li>
	 * <li>not derived</li>
	 * </ul>
	 * 
	 * @param compound
	 *            Compound field that this field contributes to, or null.
	 * @param caseFolded
	 *            Is the text converted to lower-case?
	 * @param inDefaultField
	 *            Does this field contribute to the default search field?
	 */
	private static LuceneFieldSpec keywordField(String name,
			LuceneFieldSpec compound, boolean caseFolded, boolean inDefaultField) {
		return addSpec(name, Field.Store.YES, Field.Index.TOKENIZED,
				caseFolded, 1.0F, compound, false, false, false, inDefaultField);
	}

	/**
	 * Create a field that is searchable and contributes to the default field,
	 * but is not stored in the index.
	 */
	private static LuceneFieldSpec notStored(String name) {
		return addSpec(name, Field.Store.NO, Field.Index.TOKENIZED, true, 1.0F,
				null, true, true, false, true);
	}

	/**
	 * Create a field that is searchable but not stored in the index.
	 */
	private static LuceneFieldSpec defaultSearchField(String name) {
		return addSpec(name, Field.Store.NO, Field.Index.TOKENIZED, true, 1.0F,
				null, true, true, true, false);
	}

	/**
	 * Create a field that is stored in the index but is not searchable.
	 */
	private static LuceneFieldSpec notIndexed(String name) {
		return addSpec(name, Field.Store.YES, Field.Index.NO, true, 1.0F, null,
				false, true, false, false);
	}

	/**
	 * Create a compound field.
	 */
	private static LuceneFieldSpec compoundField(String name) {
		return addSpec(name, Field.Store.YES, Field.Index.TOKENIZED, true,
				1.0F, null, true, true, true, false);
	}

	/**
	 * Create a field specification from all data items, and add it to the class
	 * map. Also add its stemmed version, if there is one.
	 */
	private static LuceneFieldSpec addSpec(String name, Field.Store store,
			Field.Index index, boolean caseFold, float boost,
			LuceneFieldSpec compoundField, boolean stemmable,
			boolean tokenized, boolean derived, boolean inDefaultField) {
		LuceneFieldSpec spec = new LuceneFieldSpec(name, store, index,
				caseFold, boost, compoundField, stemmable, tokenized, derived,
				inDefaultField);

		specsByName.put(name, spec);

		if (stemmable) {
			LuceneFieldSpec stemmed = spec.getStemmedSpec();
			specsByName.put(stemmed.getName(), stemmed);
		}

		return spec;
	}

	/**
	 * Get the specification that corresponds to the given name, if any.
	 * 
	 * @return either the desired spec, or null.
	 */
	public static LuceneFieldSpec getSpec(String fieldName) {
		return specsByName.get(fieldName);
	}

	/**
	 * Get all of the field specs.
	 */
	public static Set<LuceneFieldSpec> getSpecs() {
		return new HashSet<LuceneFieldSpec>(specsByName.values());
	}

	/**
	 * Don't create any instances: all methods and constants are static.
	 */
	private SearchIndexFieldSpecs() {
		// Nothing to instantiate.
	}
}
