/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.nsdl;

import java.io.Reader;

import org.apache.log4j.Logger;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.KeywordTokenizer;
import org.apache.lucene.analysis.LowerCaseFilter;
import org.apache.lucene.analysis.PerFieldAnalyzerWrapper;
import org.apache.lucene.analysis.StopFilter;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.standard.StandardFilter;
import org.apache.lucene.analysis.standard.StandardTokenizer;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;

/**
 * A Lucene Analyzer that handles case, whitespace, stopwords, stemming, special
 * chars, etc. as desired for the NSDL Search Service. Note this is a
 * PerFieldAnalyzerWrapper -- we use different analyzers on different fields.
 * 
 * @author jblake
 */
public class NsdlAnalyzer extends PerFieldAnalyzerWrapper {
	private static final Logger LOG = Logger.getLogger(NsdlAnalyzer.class);

	private static final Analyzer keywordAnalyzer = new KeywordAnalyzer();
	private static final Analyzer foldedKeywordAnalyzer = new FoldedKeywordAnalyzer();
	private static final Analyzer textAnalyzer = new TextAnalyzer();
	private static final Analyzer stemmedTextAnalyzer = new StemmedTextAnalyzer();

	private static final NsdlAnalyzer instance = new NsdlAnalyzer();

	public static NsdlAnalyzer getInstance() {
		return instance;
	}

	/**
	 * Builds an NSDL analyzer using the usual analyzers and stopwords. Keep it
	 * package-access to enforce the Factory pattern.
	 */
	NsdlAnalyzer() {
		// set the default analyzer for fields without a mapping.
		super(textAnalyzer);

		// populate the per-field analyzers.
		for (LuceneFieldSpec spec : SearchIndexFieldSpecs.getSpecs()) {
			if (!spec.getName().endsWith(".Stems")) {
				addAnalyzer(spec);
			}
		}
		
		LOG.debug(this);
	}

	/**
	 * Add a field-specific analyzer for the given field spec. If the field is
	 * stemmable, add an analyzer for the stemmed field, too.
	 */
	public void addAnalyzer(LuceneFieldSpec spec) {
		if (spec.isIndexed()) {
			if (spec.isTokenized()) {
				addAnalyzer(spec, textAnalyzer);
			} else {
				if (spec.isCaseFold()) {
					addAnalyzer(spec, foldedKeywordAnalyzer);
				} else {
					addAnalyzer(spec, keywordAnalyzer);
				}
			}

			if (spec.isStemmable()) {
				addAnalyzer(spec.getStemmedSpec(), stemmedTextAnalyzer);
			}
		}
	}

	/**
	 * Really add it, and log it if debug is enabled.
	 */
	private void addAnalyzer(LuceneFieldSpec spec, Analyzer analyzer) {
		String fieldName = spec.getName();
		addAnalyzer(fieldName, analyzer);
		LOG.debug(fieldName + " --> " + analyzer.getClass().getSimpleName());
	}

	/**
	 * An Analyzer that doesn't tokenize, doesn't fold case, doesn't remove
	 * stopwords, doesn't stem words.
	 */
	static class KeywordAnalyzer extends Analyzer {
		public TokenStream tokenStream(String fieldName, final Reader reader) {
			return new KeywordTokenizer(reader);
		}
	}

	/**
	 * An Analyzer that doesn't tokenize, folds case, doesn't remove stopwords,
	 * doesn't stem words.
	 */
	static class FoldedKeywordAnalyzer extends Analyzer {
		public TokenStream tokenStream(String fieldName, final Reader reader) {
			TokenStream result = new KeywordTokenizer(reader);
			result = new LowerCaseFilter(result);
			return result;
		}
	}

	/**
	 * An Analyzer that tokenizes, folds case, removes stopwords, doesn't stem
	 * words.
	 */
	static class TextAnalyzer extends Analyzer {
		public TokenStream tokenStream(String fieldName, final Reader reader) {
			TokenStream result = new StandardTokenizer(reader);
			result = new StandardFilter(result);
			result = new LowerCaseFilter(result);
			result = new StopFilter(result, StopWords.NSDL_ENGLISH_STOPWORDS);
			return result;
		}
	}

	/**
	 * An Analyzer that tokenizes, folds case, removes stopwords, stems words.
	 */
	static class StemmedTextAnalyzer extends Analyzer {
		public TokenStream tokenStream(String fieldName, final Reader reader) {
			TokenStream result = new StandardTokenizer(reader);
			result = new StandardFilter(result);
			result = new LowerCaseFilter(result);
			result = new StopFilter(result, StopWords.NSDL_ENGLISH_STOPWORDS);
			result = new org.apache.lucene.analysis.PorterStemFilter(result);
			return result;
		}
	}

}
