/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.nsdl;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.PrefixQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;
import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.searchtools.common.net.UrlUtil;

/**
 * <p>
 * NSDL implementation of the Lucene QueryParser. Provides special parsing to
 * accomodate the way we store data in the index.
 * </p>
 * <p>
 * Handle fielded queries on metadata-provided keyword fields. In the index,
 * these field values are followed by a delimiter and the metadata-source-refID,
 * and we want to search on these fields as if the refID didn't exist.
 * </p>
 * <p>
 * Handle fielded queries fields that contain URLs. The query text must be
 * normalized to match our URL normalization.
 * </p>
 * <p>
 * What about non-metadata-provided keyword fields, like dates?
 * </p>
 * 
 * @author jblake
 */
public class NsdlQueryParser extends QueryParser {
	private static final NsdlQueryParser instance = new NsdlQueryParser();

	public static NsdlQueryParser getInstance() {
		return instance;
	}

	/**
	 * Package-access to enforce the Factory pattern, but allow unit tests.
	 */
	NsdlQueryParser() {
		// default field is allFields.Stems
		super(SearchIndexFieldSpecs.ALL_FIELDS.getStemmedSpec().getName(),
				NsdlAnalyzer.getInstance());
		this.setDefaultOperator(QueryParser.AND_OPERATOR);
	}

	/**
	 * @deprecated in lucene 1.9.1
	 */
	@SuppressWarnings("unused")
	protected Query getFieldQuery(String fieldname, Analyzer analyzer,
			String rawQueryText) throws ParseException {
		return getFieldQuery(fieldname, rawQueryText);
	}

	/**
	 * Get a field query with no slop.
	 */
	protected Query getFieldQuery(String fieldname, String rawQueryText)
			throws ParseException {
		Query q = prepareSpecialFieldedQuery(fieldname, rawQueryText);
		if (q == null) {
			// use vanilla QueryParser
			return super.getFieldQuery(fieldname, rawQueryText);
		} else {
			return q;
		}
	}

	/**
	 * @deprecated in lucene 1.9.1
	 */
	@SuppressWarnings("unused")
	protected Query getFieldQuery(String fieldname, Analyzer analyzer,
			String rawQueryText, int slop) throws ParseException {
		return getFieldQuery(fieldname, rawQueryText, slop);
	}

	/**
	 * Get a field query with slop.
	 */
	protected Query getFieldQuery(String fieldname, String rawQueryText,
			int slop) throws ParseException {
		Query q = prepareSpecialFieldedQuery(fieldname, rawQueryText);
		if (q == null) {
			// use vanilla QueryParser
			return super.getFieldQuery(fieldname, rawQueryText, slop);
		} else {
			return q;
		}
	}

	/**
	 * If the fielded query fits one of our special cases, return an appropriate
	 * Query; otherwise return null.
	 */
	private Query prepareSpecialFieldedQuery(String fieldName,
			String rawQueryText) {
		LuceneFieldSpec spec = SearchIndexFieldSpecs.getSpec(fieldName);

		// If we don't recognize the field, or if it's not a keyword field, let
		// the superclass handle it.
		if ((spec == null) || (spec.isTokenized())) {
			return null;
		}

		/*
		 * TODO: This is weird. Why were these dates handled as if they were
		 * tokenized in the previous implementation? Do I still need to handle
		 * them that way?
		 */
		if ((spec == SearchIndexFieldSpecs.DOCUMENT_CREATED)
				|| (spec == SearchIndexFieldSpecs.DOCUMENT_LAST_UPDATED)
				|| (spec == SearchIndexFieldSpecs.CONTENT_LAST_FETCHED)
				|| (spec == SearchIndexFieldSpecs.CONTENT_LAST_MODIFIED)
				|| (spec == SearchIndexFieldSpecs.METADATA_LAST_MODIFIED)) {
			return null;
		}

		/*
		 * URL-based fields are trimmed and normalized as they are stored, so we
		 * need to handle the query text the same way.
		 */
		if (spec == SearchIndexFieldSpecs.RESOURCE_IDENTIFIER) {
			String rawUrl = rawQueryText.trim();
			String normalized = UrlUtil.getInstance().normalizeOrNot(rawUrl);
			return new TermQuery(new Term(fieldName, normalized));
		}

		/**
		 * Use a PrefixQuery for other keyword fields, folded or not, as
		 * appropriate.
		 */
		String queryText = (spec.isCaseFold())
				? rawQueryText.toLowerCase()
				: rawQueryText;
		return new PrefixQuery(new Term(fieldName, queryText
				+ SearchDocument.SEPARATOR));
	}

}
