/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.nsdl;

/**
 * This class simply loads an arraylist of stop words. This way a single lookup
 * can be used instead of iterating over the array every time.
 * 
 * @author Naomi Dushay
 */
public class StopWords {
	// NOTE: starting with smallest list (words on all lists) - vanilla lucene
	// except "an" (not in nutch or umass), including "s", "t" (not in long
	// umass - prob due to their parsing of contractions) 2005-03-15
	/**
	 * Currently used NSDL chosen stop words - words so common in the index that
	 * they are not useful for selecting documents.
	 */
	public static final String[] NSDL_ENGLISH_STOPWORDS = {"a", "and", "are",
			"as", "at", "be", "but", "by", "for", "if", "in", "into", "is",
			"it", "no", "not", "of", "on", "or", "s", "such", "t", "that",
			"the", "their", "then", "there", "these", "they", "this", "to",
			"was", "will", "with"};

	/** Lucene supplied stop words - here for reference. */
	@SuppressWarnings("unused")
	private static final String[] LUCENE_ENGLISH_STOPWORDS = {"a", "an", "and",
			"are", "as", "at", "be", "but", "by", "for", "if", "in", "into",
			"is", "it", "no", "not", "of", "on", "or", "s", "such", "t",
			"that", "the", "their", "then", "there", "these", "they", "this",
			"to", "was", "will", "with"};

	/**
	 * Nutch supplied stop words - here for reference. From
	 * net.nutch.analysis.NutchAnalysis.jj, which javaCC compiles into
	 * NutchAnalysis.java ... so it has the same stopwords.
	 */
	@SuppressWarnings("unused")
	private static final String[] NUTCH_ENGLISH_STOP_WORDS = {"a", "and",
			"are", "as", "at", "be", "but", "by", "for", "if", "in", "into",
			"is", "it", "no", "not", "of", "on", "or", "s", "such", "t",
			"that", "the", "their", "then", "there", "these", "they", "this",
			"to", "was", "will", "with"};

	/**
	 * UMass supplied stopwords - here for reference. From their NSDL search
	 * service implementation - org.apache.lucene.analysis.StopAnalyzer).
	 */
	@SuppressWarnings("unused")
	private static final String[] UMASS_STOPWORDS_FROM_STOP_ANALYZER = {"a",
			"and", "are", "as", "at", "be", "but", "by", "for", "if", "in",
			"into", "is", "it", "no", "not", "of", "on", "or", "s", "such",
			"t", "that", "the", "their", "then", "there", "these", "they",
			"this", "to", "was", "will", "with"};

	/**
	 * UMass supplied stopwords - here for reference. From their NSDL search
	 * service implementation (org.apache.lucene.analysis.standard.NSDLAnalyzer)
	 */
	@SuppressWarnings("unused")
	private static final String[] UMASS_STOPWORDS_FROM_NSDL_ANALYZER = {"a",
			"about", "above", "according", "across", "after", "afterwards",
			"again", "against", "albeit", "all", "almost", "alone", "along",
			"already", "also", "although", "always", "among", "amongst", "am",
			"an", "and", "another", "any", "anybody", "anyhow", "anyone",
			"anything", "anyway", "anywhere", "apart", "are", "around", "as",
			"at", "av", "be", "became", "because", "become", "becomes",
			"becoming", "been", "before", "beforehand", "behind", "being",
			"below", "beside", "besides", "between", "beyond", "both", "but",
			"by", "can", "cannot", "canst", "certain", "cf", "choose",
			"contrariwise", "cos", "could", "cu", "day", "do", "does", "doesn",
			"doing", "dost", "doth", "double", "down", "dual", "during",
			"each", "either", "else", "elsewhere", "enough", "et", "etc",
			"even", "ever", "every", "everybody", "everyone", "everything",
			"everywhere", "except", "excepted", "excepting", "exception",
			"exclude", "excluding", "exclusive", "far", "farther", "farthest",
			"few", "ff", "first", "for", "formerly", "forth", "forward",
			"from", "front", "further", "furthermore", "furthest", "get", "go",
			"had", "halves", "hardly", "has", "hast", "hath", "have", "he",
			"hence", "henceforth", "her", "here", "hereabouts", "hereafter",
			"hereby", "herein", "hereto", "hereupon", "hers", "herself", "him",
			"himself", "hindmost", "his", "hither", "hitherto", "how",
			"however", "howsoever", "i", "ie", "if", "in", "inasmuch", "inc",
			"include", "included", "including", "indeed", "indoors", "inside",
			"insomuch", "instead", "into", "inward", "inwards", "is", "it",
			"its", "itself", "just", "kind", "kg", "km", "last", "latter",
			"latterly", "less", "lest", "let", "like", "little", "ltd", "many",
			"may", "maybe", "me", "meantime", "meanwhile", "might", "moreover",
			"most", "mostly", "more", "mr", "mrs", "ms", "much", "must", "my",
			"myself", "namely", "need", "neither", "never", "nevertheless",
			"next", "no", "nobody", "none", "nonetheless", "noone", "nope",
			"nor", "not", "nothing", "notwithstanding", "now", "nowadays",
			"nowhere", "of", "off", "often", "ok", "on", "once", "one", "only",
			"onto", "or", "other", "others", "otherwise", "ought", "our",
			"ours", "ourselves", "out", "outside", "over", "own", "per",
			"perhaps", "plenty", "provide", "quite", "rather", "really",
			"round", "said", "sake", "same", "sang", "save", "saw", "see",
			"seeing", "seem", "seemed", "seeming", "seems", "seen", "seldom",
			"selves", "sent", "several", "shalt", "she", "should", "shown",
			"sideways", "since", "slept", "slew", "slung", "slunk", "smote",
			"so", "some", "somebody", "somehow", "someone", "something",
			"sometime", "sometimes", "somewhat", "somewhere", "spake", "spat",
			"spoke", "spoken", "sprang", "sprung", "stave", "staves", "still",
			"such", "supposing", "than", "that", "the", "thee", "their",
			"them", "themselves", "then", "thence", "thenceforth", "there",
			"thereabout", "thereabouts", "thereafter", "thereby", "therefore",
			"therein", "thereof", "thereon", "thereto", "thereupon", "these",
			"they", "this", "those", "thou", "though", "thrice", "through",
			"throughout", "thru", "thus", "thy", "thyself", "till", "to",
			"together", "too", "toward", "towards", "ugh", "unable", "under",
			"underneath", "unless", "unlike", "until", "up", "upon", "upward",
			"upwards", "us", "use", "used", "using", "very", "via", "vs",
			"want", "was", "we", "week", "well", "were", "what", "whatever",
			"whatsoever", "when", "whence", "whenever", "whensoever", "where",
			"whereabouts", "whereafter", "whereas", "whereat", "whereby",
			"wherefore", "wherefrom", "wherein", "whereinto", "whereof",
			"whereon", "wheresoever", "whereto", "whereunto", "whereupon",
			"wherever", "wherewith", "whether", "whew", "which", "whichever",
			"whichsoever", "while", "whilst", "whither", "who", "whoa",
			"whoever", "whole", "whom", "whomever", "whomsoever", "whose",
			"whosoever", "why", "will", "wilt", "with", "within", "without",
			"worse", "worst", "would", "wow", "ye", "yet", "year", "yippee",
			"you", "your", "yours", "yourself", "yourselves"};

	/**
	 * From
	 * http://www.onjava.com/onjava/2003/01/15/examples/EnglishStopWords.txt -
	 * here for reference.
	 */
	@SuppressWarnings("unused")
	private static final String[] ON_JAVA_SMART_STOP_WORDS = {"a", "able",
			"about", "above", "according", "accordingly", "across", "actually",
			"after", "afterwards", "again", "against", "all", "allow",
			"allows", "almost", "alone", "along", "already", "also",
			"although", "always", "am", "among", "amongst", "an", "and",
			"another", "any", "anybody", "anyhow", "anyone", "anything",
			"anyway", "anyways", "anywhere", "apart", "appear", "appreciate",
			"appropriate", "are", "around", "as", "aside", "ask", "asking",
			"associated", "at", "available", "away", "awfully", "b", "be",
			"became", "because", "become", "becomes", "becoming", "been",
			"before", "beforehand", "behind", "being", "believe", "below",
			"beside", "besides", "best", "better", "between", "beyond", "both",
			"brief", "but", "by", "c", "came", "can", "cannot", "cant",
			"cause", "causes", "certain", "certainly", "changes", "clearly",
			"co", "com", "come", "comes", "concerning", "consequently",
			"consider", "considering", "contain", "containing", "contains",
			"corresponding", "could", "course", "currently", "d", "definitely",
			"described", "despite", "did", "different", "do", "does", "doing",
			"done", "down", "downwards", "during", "e", "each", "edu", "eg",
			"eight", "either", "else", "elsewhere", "enough", "entirely",
			"especially", "et", "etc", "even", "ever", "every", "everybody",
			"everyone", "everything", "everywhere", "ex", "exactly", "example",
			"except", "f", "far", "few", "fifth", "first", "five", "followed",
			"following", "follows", "for", "former", "formerly", "forth",
			"four", "from", "further", "furthermore", "g", "get", "gets",
			"getting", "given", "gives", "go", "goes", "going", "gone", "got",
			"gotten", "greetings", "h", "had", "happens", "hardly", "has",
			"have", "having", "he", "hello", "help", "hence", "her", "here",
			"hereafter", "hereby", "herein", "hereupon", "hers", "herself",
			"hi", "him", "himself", "his", "hither", "hopefully", "how",
			"howbeit", "however", "i", "ie", "if", "ignored", "immediate",
			"in", "inasmuch", "inc", "indeed", "indicate", "indicated",
			"indicates", "inner", "insofar", "instead", "into", "inward", "is",
			"it", "its", "itself", "j", "just", "k", "keep", "keeps", "kept",
			"know", "knows", "known", "l", "last", "lately", "later", "latter",
			"latterly", "least", "less", "lest", "let", "like", "liked",
			"likely", "little", "look", "looking", "looks", "ltd", "m",
			"mainly", "many", "may", "maybe", "me", "mean", "meanwhile",
			"merely", "might", "more", "moreover", "most", "mostly", "much",
			"must", "my", "myself", "n", "name", "namely", "nd", "near",
			"nearly", "necessary", "need", "needs", "neither", "never",
			"nevertheless", "new", "next", "nine", "no", "nobody", "non",
			"none", "noone", "nor", "normally", "not", "nothing", "novel",
			"now", "nowhere", "o", "obviously", "of", "off", "often", "oh",
			"ok", "okay", "old", "on", "once", "one", "ones", "only", "onto",
			"or", "other", "others", "otherwise", "ought", "our", "ours",
			"ourselves", "out", "outside", "over", "overall", "own", "p",
			"particular", "particularly", "per", "perhaps", "placed", "please",
			"plus", "possible", "presumably", "probably", "provides", "q",
			"que", "quite", "qv", "r", "rather", "rd", "re", "really",
			"reasonably", "regarding", "regardless", "regards", "relatively",
			"respectively", "right", "s", "said", "same", "saw", "say",
			"saying", "says", "second", "secondly", "see", "seeing", "seem",
			"seemed", "seeming", "seems", "seen", "self", "selves", "sensible",
			"sent", "serious", "seriously", "seven", "several", "shall", "she",
			"should", "since", "six", "so", "some", "somebody", "somehow",
			"someone", "something", "sometime", "sometimes", "somewhat",
			"somewhere", "soon", "sorry", "specified", "specify", "specifying",
			"still", "sub", "such", "sup", "sure", "t", "take", "taken",
			"tell", "tends", "th", "than", "thank", "thanks", "thanx", "that",
			"thats", "the", "their", "theirs", "them", "themselves", "then",
			"thence", "there", "thereafter", "thereby", "therefore", "therein",
			"theres", "thereupon", "these", "they", "think", "third", "this",
			"thorough", "thoroughly", "those", "though", "three", "through",
			"throughout", "thru", "thus", "to", "together", "too", "took",
			"toward", "towards", "tried", "tries", "truly", "try", "trying",
			"twice", "two", "u", "un", "under", "unfortunately", "unless",
			"unlikely", "until", "unto", "up", "upon", "us", "use", "used",
			"useful", "uses", "using", "usually", "uucp", "v", "value",
			"various", "very", "via", "viz", "vs", "w", "want", "wants", "was",
			"way", "we", "welcome", "well", "went", "were", "what", "whatever",
			"when", "whence", "whenever", "where", "whereafter", "whereas",
			"whereby", "wherein", "whereupon", "wherever", "whether", "which",
			"while", "whither", "who", "whoever", "whole", "whom", "whose",
			"why", "will", "willing", "wish", "with", "within", "without",
			"wonder", "would", "would", "x", "y", "yes", "yet", "you", "your",
			"yours", "yourself", "yourselves", "z", "zero"};

	private StopWords() {
		// Nothing to instantiate here.
	}

}
