/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.searchdoc;

import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.apache.lucene.document.Document;
import org.nsdl.search.content.ContentCacheWrapper;
import org.nsdl.searchtools.common.DateUtil;

/**
 * Hold all of the information needed for a record in the search index.
 */
public class SearchDocument {
	public static final String SEPARATOR = " ~^ ";
	public static final String SEPARATOR_PATTERN = " ~\\^ ";
	public static final String STEM_SUFFIX = ".Stems";
	public static final String DATE_FORMAT = DateUtil.ISO8601_DATE_TIME_Z;

	/** Has this document changed since we read it from the search index? */
	private boolean changed;

	private final String identifier;
	private final Date documentCreated;

	/** Set only when writing to the index, and only if the document is changed. */
	private Date documentLastUpdated;

	private Date metadataLastModified;

	/** The metadata sources that apply to this document, keyed by OAI ID. */
	private final Map<String, SearchDocumentMetadata> metadataSources;

	private SearchDocumentContent content;

	public SearchDocument(String identifier) {
		// This is not from the index, so it's clearly changed.
		this.changed = true;

		this.identifier = identifier;
		this.documentCreated = DateUtil.getInstance().now();
		this.documentLastUpdated = new Date(0L);
		this.metadataLastModified = new Date(0L);
		this.metadataSources = new HashMap<String, SearchDocumentMetadata>();
		loadContent();
	}

	public SearchDocument(Document luceneDoc) {
		this.changed = false;
		SearchDocumentLoader loader = new SearchDocumentLoader();
		loader.load(luceneDoc);
		this.identifier = loader.getIdentifier();
		this.documentCreated = loader.getDocumentCreated();
		this.documentLastUpdated = loader.getDocumentLastUpdated();
		this.metadataLastModified = loader.getMetadataLastModified();
		this.metadataSources = loader.getMetadataSources();
		loadContent();
	}

	/**
	 * Get the content from the cache. If it's newer than this record, then the
	 * record has changed.
	 */
	private void loadContent() {
		this.content = ContentCacheWrapper.getInstance().getPageContent(
				this.identifier);
		if (this.content.isValid()
				&& (this.content.getModifiedDate()
						.after(this.documentLastUpdated))) {
			this.changed = true;
		}
	}

	public Document toDocument() {
		SearchDocumentUnloader unloader = new SearchDocumentUnloader();
		return unloader.unload(this);
	}

	public boolean isChanged() {
		return changed;
	}

	public String getIdentifier() {
		return this.identifier;
	}

	public Date getDocumentCreated() {
		return this.documentCreated;
	}

	public Date getDocumentLastUpdated() {
		return this.documentLastUpdated;
	}

	public Date getMetadataLastModified() {
		return this.metadataLastModified;
	}

	public Collection<SearchDocumentMetadata> getMetadataSources() {
		return Collections
				.unmodifiableCollection(this.metadataSources.values());
	}

	public Set<String> getMetadataSourceIds() {
		return Collections.unmodifiableSet(this.metadataSources.keySet());
	}

	public SearchDocumentMetadata getMetadata(String oaiId) {
		return this.metadataSources.get(oaiId);
	}

	/**
	 * Record the metadata from a new OAI record. If there already is metadata
	 * from this OAI record, replace it.
	 * 
	 * This might affect the value of {@link #metadataLastModified}
	 */
	public void addMetadataSource(SearchDocumentMetadata metadata) {
		this.metadataSources.put(metadata.getOaiId(), metadata);

		if (metadata.getOaiDateStamp().after(this.metadataLastModified)) {
			this.metadataLastModified = metadata.getOaiDateStamp();
		}

		this.changed = true;
	}

	/**
	 * Remove any metadata associated with this OAI id. If there is none, the
	 * document remains unchanged.
	 */
	public void removeMetadataSource(String oaiId) {
		SearchDocumentMetadata oldMd = this.metadataSources.remove(oaiId);
		if (oldMd != null) {
			this.changed = true;
		}
	}

	/**
	 * Are there any metadata sources for this record?
	 */
	public boolean hasMetadata() {
		return !this.metadataSources.isEmpty();
	}

	public boolean hasContent() {
		return (content != null) && content.isValid();
	}

	public SearchDocumentContent getContent() {
		if (hasContent()) {
			return content;
		} else {
			throw new IllegalStateException(
					"Test for content with hasContent() first.");
		}
	}

	public void setContent(SearchDocumentContent content) {
		this.content = content;
		this.changed = true;
	}

	/**
	 * Test all fields for equality, except {link {@link #changed}. We don't
	 * care where the doc came from when testing for equality.
	 */
	@Override
	public boolean equals(Object other) {
		if (this == other) {
			return true;
		}
		if (other == null) {
			return false;
		}
		if (this.getClass() != other.getClass()) {
			return false;
		}

		SearchDocument that = (SearchDocument) other;

		return this.identifier.equals(that.identifier)
				&& this.documentCreated.equals(that.documentCreated)
				&& this.documentLastUpdated.equals(that.documentLastUpdated)
				&& this.metadataLastModified.equals(that.metadataLastModified)
				&& this.metadataSources.equals(that.metadataSources)
				&& this.content.equals(that.content);
	}

	/**
	 * It should be extremely rare (or an error) to have two documents with the
	 * same identifier, so this is adequate.
	 */
	@Override
	public int hashCode() {
		return identifier.hashCode();
	}

	/**
	 * Convert to string for debugging.
	 */
	@Override
	public String toString() {
		String result = "SearchDocument[";
		result += "identifier='" + identifier + "', created=" + documentCreated
				+ ", updated=" + documentLastUpdated + ", metadataModified="
				+ metadataLastModified + ", changed=" + changed
				+ ", metadataSources=[";
		for (SearchDocumentMetadata md : metadataSources.values()) {
			result += md.toString() + ", ";
		}
		result += ", content=" + content + "'";
		result += "]";
		return result;
	}

}
