/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.searchdoc;

import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COLL_IMAGE_ALT_TEXT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COLL_IMAGE_HEIGHT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COLL_IMAGE_URL;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COLL_IMAGE_WIDTH;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.CONTENT_LAST_FETCHED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.CONTENT_LAST_MODIFIED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DOCUMENT_CREATED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DOCUMENT_LAST_UPDATED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.IN_COLLECTION;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.METADATA_LAST_MODIFIED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.METADATA_SOURCE;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.RESOURCE_IDENTIFIER;
import static org.nsdl.search.lucene.searchdoc.SearchDocument.DATE_FORMAT;
import static org.nsdl.search.lucene.searchdoc.SearchDocument.SEPARATOR_PATTERN;
import static org.nsdl.search.lucene.searchdoc.SearchDocument.STEM_SUFFIX;

import java.text.ParseException;
import java.util.Arrays;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.nsdl.search.lucene.FatalSearchIndexException;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;
import org.nsdl.search.processoai.OaiCollectionInfo;
import org.nsdl.searchtools.common.DateUtil;
import org.nsdl.searchtools.common.ListsMap;

/**
 * Parses a {@link Document Lucene Document} and loads the contents into a
 * {@link SearchDocument}.
 * 
 * @author jblake
 */
public class SearchDocumentLoader {
	private static final Logger LOG = Logger
			.getLogger(SearchDocumentLoader.class);

	private final DateUtil dateUtil = DateUtil.getInstance();

	private Map<Integer, SearchDocumentMetadata> metadataIndexes;
	private Map<String, SearchDocumentMetadata> metadataSources;

	private String identifier;
	private Date documentCreated;
	private Date documentLastUpdated;
	private Date metadataLastModified;

	SearchDocumentLoader() {
		this.metadataIndexes = new HashMap<Integer, SearchDocumentMetadata>();
	}

	void load(Document luceneDoc) {
		ListsMap<String, String> fields = buildFieldValuesMap(luceneDoc);
		LOG.debug("fields=" + fields);

		this.identifier = getOneFieldValue(fields, RESOURCE_IDENTIFIER);
		this.documentCreated = getDateFieldValue(fields, DOCUMENT_CREATED);
		this.documentLastUpdated = getOptionalDateFieldValue(fields,
				DOCUMENT_LAST_UPDATED);
		this.metadataLastModified = getDateFieldValue(fields,
				METADATA_LAST_MODIFIED);

		// Ignore content-related dates, we'll get them when we get the content.
		fields.remove(CONTENT_LAST_FETCHED.getName());
		fields.remove(CONTENT_LAST_MODIFIED.getName());

		// Determine the metadata sources, mapped by OAI ID and by index.
		this.metadataSources = new HashMap<String, SearchDocumentMetadata>();
		this.metadataIndexes = new HashMap<Integer, SearchDocumentMetadata>();
		buildMetadataMaps(fields);

		// Go through the fields, assigning each to the appropriate metadata.
		for (String fieldName : fields.keySet()) {
			loadMetadataField(fieldName, fields.get(fieldName));
		}

		// Go through the metadata source, loading fields into Collection Infos.
		for (SearchDocumentMetadata metadata : this.metadataSources.values()) {
			loadOaiCollectionInfo(metadata);
		}
	}

	/**
	 * Take the fields from the Lucene document and arrange them in lists,
	 * mapped by field name.
	 */
	private ListsMap<String, String> buildFieldValuesMap(Document luceneDoc) {
		ListsMap<String, String> map = new ListsMap<String, String>();
		Enumeration<Field> fields = getFieldsFromDocument(luceneDoc);
		while (fields.hasMoreElements()) {
			Field field = fields.nextElement();
			map.addValue(field.name(), field.stringValue());
		}
		return map;
	}

	/**
	 * Isolate the unchecked cast.
	 */
	@SuppressWarnings("unchecked")
	private Enumeration<Field> getFieldsFromDocument(Document doc) {
		return doc.fields();
	}

	/**
	 * We need exactly one field value. Get it and remove it from the map.
	 */
	private String getOneFieldValue(ListsMap<String, String> fieldMap,
			LuceneFieldSpec spec) {
		List<String> identifierList = fieldMap.get(spec.getName());
		fieldMap.remove(spec.getName());

		if (identifierList.size() == 0) {
			throw new FatalSearchIndexException("Document has no "
					+ spec.getName() + " field.");
		}
		if (identifierList.size() > 1) {
			LOG.warn("Document for '" + this.identifier
					+ "' has more than one " + spec.getName() + " field: "
					+ identifierList);
		}

		return identifierList.get(0);
	}

	/**
	 * Get exactly one date field value and parse it according to the supplied
	 * format.
	 */
	private Date getDateFieldValue(ListsMap<String, String> fieldMap,
			LuceneFieldSpec spec) {
		String dateString = getOneFieldValue(fieldMap, spec);
		try {
			return dateUtil.parseDate(dateString, DATE_FORMAT);
		} catch (ParseException e) {
			throw new FatalSearchIndexException(spec.getName()
					+ " for document " + this.identifier
					+ " is in an invalid format: '" + dateString + "'");
		}
	}

	/**
	 * Get a date field value and parse it according to the supplied format. If
	 * the value is not present, go with the minimum date.
	 */
	private Date getOptionalDateFieldValue(ListsMap<String, String> fieldMap,
			LuceneFieldSpec spec) {
		List<String> identifierList = fieldMap.get(spec.getName());
		fieldMap.remove(spec.getName());

		if (identifierList.size() == 0) {
			return new Date(0L);
		} else if (identifierList.size() > 1) {
			throw new FatalSearchIndexException("Document has more than one "
					+ spec.getName() + " field: " + identifierList);
		}

		String dateString = identifierList.get(0);
		if (dateString.length() == 0) {
			return new Date(0L);
		} else {
			try {
				return dateUtil.parseDate(dateString, DATE_FORMAT);
			} catch (ParseException e) {
				throw new FatalSearchIndexException(spec.getName()
						+ " for document " + this.identifier
						+ " is in an invalid format: '" + dateString + "'");
			}
		}
	}

	private void buildMetadataMaps(ListsMap<String, String> fields) {
		for (String msLink : fields.get(METADATA_SOURCE.getName())) {
			try {
				String[] parts = msLink.split(SEPARATOR_PATTERN);
				if (parts.length != 3) {
					throw new FatalSearchIndexException("Can't parse "
							+ METADATA_SOURCE.getName() + " field: '" + msLink
							+ "'");
				}
				String oaiId = parts[0];
				Integer index = Integer.valueOf(parts[1]);
				String rawUrl = parts[2];
				SearchDocumentMetadata metadata = new SearchDocumentMetadata(
						oaiId, rawUrl);
				this.metadataSources.put(oaiId, metadata);
				this.metadataIndexes.put(index, metadata);
			} catch (NumberFormatException e) {
				throw new FatalSearchIndexException("Can't parse "
						+ METADATA_SOURCE.getName() + " field: '" + msLink
						+ "'");
			}
		}
		fields.remove(METADATA_SOURCE.getName());
	}

	/**
	 * Any remaining fields must be parcelled out to the appropriate metadata
	 * souce. But ignore any fields that are derived from others, or are stemmed
	 * versions of other fields.
	 */
	private void loadMetadataField(String fieldName, List<String> values) {
		if (fieldName.endsWith(STEM_SUFFIX)) {
			return;
		}

		// If we don't recognize the field, log it and go on.
		LuceneFieldSpec spec = SearchIndexFieldSpecs.getSpec(fieldName);
		if (spec == null) {
			// KLUGE: some unknown fields are expected!
			if ("publicNativeMetadata".equals(fieldName)) {
				LOG.debug("Loading document '" + this.identifier +
						"' -- No field spec for '" + fieldName + "'");
			} else {
				LOG.warn("Loading document '" + this.identifier +
						"' -- No field spec for '" + fieldName + "'");
			}
			return;
		}

		if (spec.isDerived()) {
			return;
		}

		for (String value : values) {
			String[] parts = value.split(SEPARATOR_PATTERN);
			if (parts.length == 1) {
				LOG.info("Loading document '" + this.identifier +
						"' -- No metadata index for " + fieldName + ":'"
						+ value + "'");
				return;
			}
			List<String> partsList = Arrays.asList(parts);
			if (spec.isTokenized()) {
				loadFieldValues(spec, parts[0], partsList.subList(1,
						parts.length));
			} else {
				loadFieldValues(spec, parts[parts.length - 1], partsList
						.subList(0, parts.length - 1));
			}
		}
	}

	/**
	 * Find the metadata object that corresponds to this indexString, and store
	 * these field values into it.
	 */
	private void loadFieldValues(LuceneFieldSpec spec, String indexString,
			List<String> values) {
		try {
			Integer metadataIndex = Integer.valueOf(indexString);
			SearchDocumentMetadata metadata = this.metadataIndexes
					.get(metadataIndex);
			for (String value : values) {
				metadata.addFieldValue(spec, value);
			}
		} catch (NumberFormatException e) {
			try {
				Integer metadataIndex = Integer.valueOf(values.get(0));
				SearchDocumentMetadata metadata = this.metadataIndexes
						.get(metadataIndex);
				metadata.addFieldValue(spec, indexString);
				for (int i = 1; i < values.size(); i++) {
					metadata.addFieldValue(spec, values.get(i));
				}
				LOG.info("Incorrect storage format for " + spec.getName()
						+ ", fixing...");
			} catch (NumberFormatException e1) {
				LOG.info("Invalid metadataIndex: " + spec.getName() + ": '"
						+ indexString + "', " + values);
			}
		} catch (NullPointerException e) {
			LOG.info("Can't find metadata for index: " + spec.getName() + ": '"
					+ indexString + "', " + values);
		}
	}

	/**
	 * Find the fields that compose the OaiCollectionInfo for this metadata.
	 */
	private void loadOaiCollectionInfo(SearchDocumentMetadata metadata) {
		List<String> values;

		values = metadata.getFieldValues(IN_COLLECTION);
		String inCollection = values.isEmpty() ? null : values.get(0);
		metadata.clearField(IN_COLLECTION);

		values = metadata.getFieldValues(COLL_IMAGE_URL);
		String imageUrl = values.isEmpty() ? "" : values.get(0);
		metadata.clearField(COLL_IMAGE_URL);

		values = metadata.getFieldValues(COLL_IMAGE_ALT_TEXT);
		String altText = values.isEmpty() ? "" : values.get(0);
		metadata.clearField(COLL_IMAGE_ALT_TEXT);

		values = metadata.getFieldValues(COLL_IMAGE_HEIGHT);
		String imageHeight = values.isEmpty() ? "30" : values.get(0);
		metadata.clearField(COLL_IMAGE_HEIGHT);

		values = metadata.getFieldValues(COLL_IMAGE_WIDTH);
		String imageWidth = values.isEmpty() ? "100" : values.get(0);
		metadata.clearField(COLL_IMAGE_WIDTH);

		metadata.setCollInfo(new OaiCollectionInfo(inCollection, imageUrl,
				imageWidth, imageHeight, altText));
	}

	public String getIdentifier() {
		return this.identifier;
	}

	public Date getDocumentCreated() {
		return this.documentCreated;
	}

	public Date getDocumentLastUpdated() {
		return this.documentLastUpdated;
	}

	public Date getMetadataLastModified() {
		return this.metadataLastModified;
	}

	public Map<String, SearchDocumentMetadata> getMetadataSources() {
		return this.metadataSources;
	}

}
