/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.searchdoc;

import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.nsdl.search.collContext.CollRecCache;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.processoai.OaiCollectionInfo;
import org.nsdl.search.processoai.OaiHarvestRecord;
import org.nsdl.searchtools.common.ListsMap;

/**
 * Of the information in a {@link SearchDocument} this part is attributable to a
 * single source of Metadata.
 * 
 * @author jblake
 */
public class SearchDocumentMetadata {
	private final String oaiId;
	private final String rawUrl;
	private Date oaiDateStamp;
	private OaiCollectionInfo collInfo;
	private final ListsMap<LuceneFieldSpec, String> fields;

	public SearchDocumentMetadata(String oaiId, String rawUrl) {
		this.oaiId = oaiId;
		this.rawUrl = rawUrl;
		this.oaiDateStamp = new Date(0L);
		this.collInfo = new OaiCollectionInfo(null, null, null, null, null);
		this.fields = new ListsMap<LuceneFieldSpec, String>();
	}

	public SearchDocumentMetadata(String rawUrl, OaiHarvestRecord oai) {
		SearchDocumentMetadataLoader loader = new SearchDocumentMetadataLoader();
		loader.load(oai);
		this.oaiId = oai.getOaiRecId();
		this.rawUrl = rawUrl;
		this.oaiDateStamp = loader.getOaiDateStamp();
		this.collInfo = loader.getCollInfo();
		this.fields = loader.getFields();

		// Load the inherited collection context info.
		ListsMap<LuceneFieldSpec, String> collContextFields = CollRecCache
				.getInstance().getCollContextFieldInfos(
						oai.getCollectionInfo().getCollectionOaiId());
		for (LuceneFieldSpec spec : collContextFields.keySet()) {
			fields.put(spec, collContextFields.get(spec));
		}

	}

	public String getOaiId() {
		return oaiId;
	}

	public String getRawUrl() {
		return rawUrl;
	}

	public void addFieldValue(LuceneFieldSpec spec, String value) {
		fields.addValue(spec, value);
	}

	public void clearField(LuceneFieldSpec spec) {
		fields.remove(spec);
	}

	public Set<LuceneFieldSpec> getFields() {
		return Collections.unmodifiableSet(fields.keySet());
	}

	public List<String> getFieldValues(LuceneFieldSpec spec) {
		return Collections.unmodifiableList(fields.get(spec));
	}

	public Date getOaiDateStamp() {
		return oaiDateStamp;
	}

	public void setOaiDateStamp(Date oaiDateStamp) {
		this.oaiDateStamp = oaiDateStamp;
	}

	public OaiCollectionInfo getCollInfo() {
		return collInfo;
	}

	public void setCollInfo(OaiCollectionInfo collInfo) {
		this.collInfo = collInfo;
	}

	@Override
	public boolean equals(Object other) {
		if (this == other) {
			return true;
		}
		if (other == null) {
			return false;
		}
		if (!this.getClass().equals(other.getClass())) {
			return false;
		}

		SearchDocumentMetadata that = (SearchDocumentMetadata) other;

		return this.oaiId.equals(that.oaiId) && this.rawUrl.equals(that.rawUrl)
				&& this.oaiDateStamp.equals(that.oaiDateStamp)
				&& this.collInfo.equals(that.collInfo)
				&& this.fields.equals(that.fields);
	}

	/**
	 * We should very rarely see two metadata objects with the same ID, so this
	 * should be adequate.
	 */
	@Override
	public int hashCode() {
		return oaiId.hashCode();
	}

	/**
	 * Convert to string for debugging.
	 */
	@Override
	public String toString() {
		String result = "SearchDocumentMetadata[oaiId=" + oaiId + ", rawUrl="
				+ rawUrl + ", oaiDate=" + oaiDateStamp + ", collectionInfo="
				+ collInfo + ", fields=[";
		for (Iterator<LuceneFieldSpec> specs = fields.keySet().iterator(); specs
				.hasNext();) {
			LuceneFieldSpec spec = specs.next();
			if (spec == null) {
				result += "NULL SPEC";
			} else {
				result += spec.getName() + ":" + fields.get(spec);
			}
			if (specs.hasNext()) {
				result += ", ";
			}
		}
		result += "]";
		result += "]";
		return result;
	}

}
