/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.searchdoc;

import static org.nsdl.search.lucene.SearchIndexFieldSpecs.ALL_FIELDS;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COLL_IMAGE_ALT_TEXT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COLL_IMAGE_HEIGHT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COLL_IMAGE_URL;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COLL_IMAGE_WIDTH;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COMPOUND_GENRE;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COMPOUND_TITLE;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.CONTENT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.CONTENT_LAST_FETCHED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.CONTENT_LAST_MODIFIED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DOCUMENT_CREATED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DOCUMENT_LAST_UPDATED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.IN_COLLECTION;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.METADATA_LAST_MODIFIED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.METADATA_SOURCE;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.RESOURCE_IDENTIFIER;
import static org.nsdl.search.lucene.searchdoc.SearchDocument.DATE_FORMAT;
import static org.nsdl.search.lucene.searchdoc.SearchDocument.SEPARATOR;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.processoai.OaiCollectionInfo;
import org.nsdl.searchtools.common.DateUtil;
import org.nsdl.searchtools.common.ListsMap;

/**
 * Creates a {@link Document Lucene Document} from the contents of a
 * {@link SearchDocument}.
 * 
 * @author jblake
 */
public class SearchDocumentUnloader {
	private final DateUtil dateUtil = DateUtil.getInstance();

	public Document unload(SearchDocument searchDoc) {
		Document luceneDoc = new Document();

		// If the doc has changed, the "last updated" date is right now!
		Date updateDate = (searchDoc.isChanged()
				? DateUtil.getInstance().now()
				: searchDoc.getDocumentLastUpdated());

		addField(luceneDoc, RESOURCE_IDENTIFIER, searchDoc.getIdentifier());
		addDateField(luceneDoc, DOCUMENT_CREATED, searchDoc
				.getDocumentCreated());
		addDateField(luceneDoc, DOCUMENT_LAST_UPDATED, updateDate);
		addDateField(luceneDoc, METADATA_LAST_MODIFIED, searchDoc
				.getMetadataLastModified());

		if (searchDoc.hasContent()) {
			addContentFields(luceneDoc, searchDoc.getContent());
		}

		List<SearchDocumentMetadata> metadataList = new ArrayList<SearchDocumentMetadata>(
				searchDoc.getMetadataSources());
		for (int i = 0; i < metadataList.size(); i++) {
			SearchDocumentMetadata metadata = metadataList.get(i);
			addMetadataFields(luceneDoc, metadata, i);
			addMetadataCollectionInfo(luceneDoc, metadata, i);
		}

		addDefaultField(luceneDoc, searchDoc.getMetadataSources());

		return luceneDoc;
	}

	/**
	 * We have content in the document, so dump it.
	 */
	private void addContentFields(Document luceneDoc,
			SearchDocumentContent content) {
		addDateField(luceneDoc, CONTENT_LAST_MODIFIED, content
				.getModifiedDate());
		addDateField(luceneDoc, CONTENT_LAST_FETCHED, content.getFetchDate());

		List<String> contentValues = new ArrayList<String>();

		String contentType = content.getContentType();
		if (contentType != null) {
			addField(luceneDoc, COMPOUND_GENRE, "X ~^ " + contentType);
			contentValues.add(contentType);
		}

		String title = content.getTitle();
		if (title != null) {
			addField(luceneDoc, COMPOUND_TITLE, "X ~^ " + title);
			contentValues.add(title);
		}

		String text = content.getText();
		if (text != null) {
			addField(luceneDoc, CONTENT, text);
			contentValues.add(text);
		}

		// everything but the dates goes into the default search.
		StringBuffer buffer = new StringBuffer("X ~^ ");
		appendFieldValues(buffer, contentValues);
		addField(luceneDoc, ALL_FIELDS, buffer.toString());
	}

	private void addDateField(Document doc, LuceneFieldSpec spec, Date date) {
		addField(doc, spec, dateUtil.formatDate(date, DATE_FORMAT));
	}

	private void addField(Document doc, LuceneFieldSpec spec, String value) {
		doc.add(buildField(spec, value));

		if (spec.isStemmable()) {
			doc.add(buildField(spec.getStemmedSpec(), value));
		}
	}

	private Field buildField(LuceneFieldSpec spec, String value) {
		Field field = new Field(spec.getName(), value, spec.getStore(), spec
				.getIndex(), spec.getTermVector());
		field.setBoost(spec.getBoost());
		return field;
	}

	private void addMetadataFields(Document doc,
			SearchDocumentMetadata metadata, int index) {
		String value = metadata.getOaiId() + SEPARATOR + index + SEPARATOR
				+ metadata.getRawUrl();
		addField(doc, METADATA_SOURCE, value);

		addRawMetadataFields(doc, metadata, index);

		addCompoundMetadataFields(doc, metadata, index);
	}

	private void addRawMetadataFields(Document doc,
			SearchDocumentMetadata metadata, int index) {
		for (LuceneFieldSpec field : metadata.getFields()) {
			addMetadataField(doc, field, metadata.getFieldValues(field), index);
		}
	}

	private void addCompoundMetadataFields(Document doc,
			SearchDocumentMetadata metadata, int index) {
		ListsMap<LuceneFieldSpec, String> compoundFieldValues = new ListsMap<LuceneFieldSpec, String>();

		for (LuceneFieldSpec field : metadata.getFields()) {
			if (field.getCompoundField() != null) {
				compoundFieldValues.put(field.getCompoundField(), metadata
						.getFieldValues(field));
			}
		}
		for (LuceneFieldSpec compoundField : compoundFieldValues.keySet()) {
			addMetadataField(doc, compoundField, compoundFieldValues
					.get(compoundField), index);
		}
	}

	private void addMetadataField(Document doc, LuceneFieldSpec spec,
			List<String> values, int index) {
		if (!values.isEmpty()) {
			if (spec.isTokenized()) {
				StringBuffer buffer = new StringBuffer();
				buffer.append(index).append(SEPARATOR);
				appendFieldValues(buffer, values);
				addField(doc, spec, buffer.toString());
			} else {
				for (String value : values) {
					StringBuffer buffer = new StringBuffer(value);
					buffer.append(SEPARATOR).append(index);
					addField(doc, spec, buffer.toString());
				}
			}
		}
	}

	private void appendFieldValues(StringBuffer buffer, List<String> values) {
		for (int i = 0; i < values.size(); i++) {
			buffer.append(values.get(i));
			if (i < values.size() - 1) {
				buffer.append(SEPARATOR);
			}
		}
	}

	private void addMetadataCollectionInfo(Document luceneDoc,
			SearchDocumentMetadata metadata, int index) {
		OaiCollectionInfo collInfo = metadata.getCollInfo();
		addCollInfoField(luceneDoc, IN_COLLECTION, collInfo
				.getCollectionOaiId(), index);
		addCollInfoField(luceneDoc, COLL_IMAGE_ALT_TEXT, collInfo
				.getImageAltText(), index);
		addCollInfoField(luceneDoc, COLL_IMAGE_HEIGHT, collInfo
				.getImageHeight(), index);
		addCollInfoField(luceneDoc, COLL_IMAGE_WIDTH, collInfo.getImageWidth(),
				index);
		addCollInfoField(luceneDoc, COLL_IMAGE_URL, collInfo.getImageUrl(),
				index);
	}

	private void addCollInfoField(Document luceneDoc, LuceneFieldSpec spec,
			String value, int index) {
		if (value != null) {
			addMetadataField(luceneDoc, spec, Collections.singletonList(value),
					index);
		}
	}

	/**
	 * Create the default field value from all of the relevant metadata fields.
	 * Add it to the document.
	 * 
	 * @param luceneDoc
	 * @param metadataList
	 * @param contentText
	 */
	private void addDefaultField(Document luceneDoc,
			Collection<SearchDocumentMetadata> metadataList) {
		List<String> values = new ArrayList<String>();
		for (SearchDocumentMetadata metadata : metadataList) {
			for (LuceneFieldSpec field : metadata.getFields()) {
				if (field.isInDefaultField()) {
					values.addAll(metadata.getFieldValues(field));
				}
			}
		}

		StringBuffer buffer = new StringBuffer();
		appendFieldValues(buffer, values);
		addField(luceneDoc, ALL_FIELDS, buffer.toString());
	}
}
