/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.wrapper;

import java.io.IOException;
import java.io.StringReader;

import org.apache.log4j.Logger;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.search.highlight.Highlighter;
import org.apache.lucene.search.highlight.QueryScorer;
import org.apache.lucene.search.highlight.SimpleHTMLFormatter;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;
import org.nsdl.search.lucene.nsdl.NsdlAnalyzer;

/**
 * <p>
 * An implementation of {@link BestPassageGenerator}. There are no instance
 * variables, so the instance methods are thread-safe.
 * </p>
 * 
 * @author jblake
 */
public class BestPassageGeneratorImpl extends BestPassageGenerator {
	private static final Logger LOG = Logger
			.getLogger(BestPassageGeneratorImpl.class);

	/**
	 * If several fragments are found by the highlighter, use only this many.
	 */
	public static final int MAX_BEST_PASSAGE_FRAGMENTS = 3;

	/**
	 * If we don't find any fragments, use the beginning of the content, up to
	 * this many characters.
	 */
	public static final int MAX_BEST_PASSAGE_LENGTH = 300;

	/**
	 * The name of the content field (stemmed).
	 */
	public static final String CONTENT_FIELD_NAME = SearchIndexFieldSpecs.CONTENT
			.getStemmedSpec().getName();

	/**
	 * The instances are light-weight, so just get a fresh one.
	 */
	public static BestPassageGeneratorImpl getInstance() {
		return new BestPassageGeneratorImpl();
	}

	/**
	 * Private, to enforce the factory pattern.
	 */
	private BestPassageGeneratorImpl() {
	}

	/**
	 * <p>
	 * Scan this content for the best passage, relative to the given query, and
	 * highlight the query terms wherever they are found inside the passage.
	 * </p>
	 * <p>
	 * If the content is empty, or if there are any problems, don't throw an
	 * exception. Just return an an empty string.
	 * </p>
	 */
	@Override
	String getHighlightedBestPassage(IndexWrapperQuery query, String content) {
		LuceneIndexQuery liQuery = LuceneIndexQuery.confirmQueryClass(query,
				this.getClass());

		if ((content == null) || (content.length() == 0)) {
			return "";
		}

		try {
			// Create a scorer from the query: it must be a "primitive" query.
			QueryScorer scorer = new QueryScorer(liQuery.getPrimitiveQuery());

			// Create a highlighter from the scorer.
			SimpleHTMLFormatter formatter = new SimpleHTMLFormatter();
			Highlighter highlighter = new Highlighter(formatter, scorer);

			// Tokenize the content
			Analyzer analyzer = NsdlAnalyzer.getInstance();
			TokenStream tokenStream = analyzer.tokenStream(CONTENT_FIELD_NAME,
					new StringReader(content));

			// And go through highlighting the favored tokens.
			String result = highlighter.getBestFragments(tokenStream, content,
					MAX_BEST_PASSAGE_FRAGMENTS, "... ");

			// If it's empty for some reason, use the beginning of the content.
			if (result.length() > 0) {
				return result;
			} else {
				return getBeginningOfContent(content);
			}
		} catch (IOException e) {
			LOG.error("problem with best passage", e);
			return "";
		}
	}

	/**
	 * Grab the beginning of the content, up to a pre-set maximum length.
	 */
	private String getBeginningOfContent(String content) {
		return (content.length() <= MAX_BEST_PASSAGE_LENGTH)
				? content
				: content.substring(0, MAX_BEST_PASSAGE_LENGTH + 1);
	}
}
