/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.wrapper;

import java.io.IOException;

import org.apache.lucene.queryParser.ParseException;
import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.searchtools.common.InstanceFactory;

/**
 * A shell around the search index. This the abstract interface/factory.
 * 
 * @author jblake
 */
public abstract class IndexWrapper {

	// ----------------------------------------------------------------------
	// Factory implementation
	// ----------------------------------------------------------------------

	private static final Class<?>[] PARAMETER_TYPES = new Class[0];
	private static final Object[] PARAMETER_VALUES = new Object[0];

	/**
	 * Get an object that instantiates IndexWrapper.
	 */
	public static IndexWrapper getInstance() {
		return InstanceFactory.getInstance(IndexWrapper.class,
				IndexWrapperImpl.class, PARAMETER_TYPES, PARAMETER_VALUES);
	}

	// ----------------------------------------------------------------------
	// Abstract interface
	// ----------------------------------------------------------------------

	/**
	 * Create a fresh index in the configured directory. CAUTION: do you really
	 * want to do this?
	 */
	public abstract void createIndex() throws IOException;

	/**
	 * Create a query object from a text string.
	 * 
	 * @param queryString
	 *            The text of the query, to be parsed and processed.
	 * @return the query object, suitable for use in searches.
	 */
	public abstract IndexWrapperQuery parseQuery(String queryString)
			throws ParseException, IOException;

	/**
	 * Create a query object that will search a given field for a value that
	 * begins with a given string.
	 * 
	 * @param fieldName
	 *            the field to search.
	 * @param valuePrefix
	 *            the prefix to match against field values.
	 * @return the query object, suitable for use in searches.
	 */
	public abstract IndexWrapperQuery createPrefixQuery(String fieldName,
			String valuePrefix) throws IOException;

	/**
	 * Search the index and return the results.
	 * 
	 * @param query
	 *            A query object to be executed.
	 * @param skipCount
	 *            In the results, skip over (ignore) this many documents that
	 *            match the query.
	 * @param maxReturnCount
	 *            If, after skipping, we have more matching documents than this,
	 *            ignore the remainder.
	 */
	public abstract SearchResults search(IndexWrapperQuery query,
			int skipCount, int maxReturnCount) throws IOException;

	/**
	 * Search the index for a document with this ID.
	 * 
	 * @param docId
	 *            the ID of the document we are seeking
	 * @return A {@link SearchResults} object with either 0 or 1 result. Does
	 *         not return null.
	 */
	public abstract SearchResults getDocument(String docId) throws IOException;

	/**
	 * Given a query and some content, identify the best passages based on that
	 * query, and highlight the query terms in the passages.
	 * 
	 * @return text with embedded HTML tags to highlight the terms.
	 */
	public abstract String getBestPassage(IndexWrapperQuery query,
			String content);

	/**
	 * Write a document to the index, replacing any previous version. This
	 * request will be cached until the index wrapper is synchronized to the
	 * index.
	 * 
	 * @param doc
	 *            the {@link SearchDocument} to be written.
	 */
	public abstract void writeDocument(SearchDocument doc) throws IOException;

	/**
	 * Remove this document from the index (or any document with this ID). This
	 * request will be cached until the index wrapper is synchronized to the
	 * index.
	 * 
	 * @param doc
	 *            the {@link SearchDocument} to be deleted.
	 */
	public abstract void deleteDocument(SearchDocument doc) throws IOException;

	/**
	 * Synchronize the index wrapper to the index. This insures that all pending
	 * deletes and additions are performed, and that the index searcher is
	 * current.
	 * 
	 * Also, all readers, writers and searchers are closed. This doesn't mean
	 * that the wrapper is unusable, it just means that subsequent access will
	 * create new readers, writers, and searchers as needed.
	 */
	public abstract void synchronize() throws IOException;

	/**
	 * Apply a listener to record the writes and deletes that are handled
	 * internally by the {@link IndexWrapper}.
	 */
	public abstract void setListener(IndexWrapperListener listener);

	/**
	 * Get some summary information about the index.
	 */
	public abstract IndexSummaryInfo getIndexInfo() throws IOException;
}
