/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.wrapper;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.apache.log4j.Logger;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.IndexReader.FieldOption;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.Hits;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;
import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.search.lucene.wrapper.IndexWrapperListener.EventType;
import org.nsdl.search.util.Config;

/**
 * <p>
 * The standard implementation of {@link IndexWrapper}.
 * </p>
 * <p>
 * This wrapper caches change requests (writes and deletes), for the sake of
 * efficiency. Until a sufficient number of requests have come in, or until
 * enough idle time passes, the requests are stored here in the wrapper.
 * Eventually a threshold is reached, and the cache is flushed to the Lucene
 * index.
 * </p>
 * <p>
 * When a search is done, the cached requests are applied to the search results,
 * as well as possible. If a result document is deleted in the cache, it will be
 * removed from the search results. If a result document is updated in the
 * cache, the updated form is put into the results.
 * </p>
 * <p>
 * This leads to some data inconsistencies. Perhaps the updated document would
 * not have been found by the query, although the original document was. Perhaps
 * one of the new documents in the cache would have been found in the query if
 * the cache had been flushed. If a search document is deleted in the cache,
 * then we might return fewer results than were requested.
 * </p>
 * <p>
 * This is a Singleton, so we need to be mindful of multi-threading issues. For
 * now, this is done simply by synchronizing all public methods.
 * </p>
 * 
 * @author jblake
 */
public class IndexWrapperImpl extends IndexWrapper {
	private static final Logger LOG = Logger.getLogger(IndexWrapperImpl.class);

	private static IndexWrapperImpl instance;

	public static synchronized IndexWrapperImpl getInstance() {
		if (instance == null) {
			instance = new IndexWrapperImpl();
		}
		return instance;
	}

	/**
	 * Generate a new instance, the next time one is requested.
	 */
	public static synchronized void reset() {
		instance = null;
	}

	/**
	 * An even lower layer to control freshness of readers, searchers, etc.
	 */
	private final LuceneAccessHandler access = new LuceneAccessHandler();

	/**
	 * This holds any parameters we need to find or work with the index.
	 */
	private final IndexWrapperConfig config = Config.getInstance()
			.getIndexWrapperConfig();

	/**
	 * The IDs of the docs that we have been told to delete, but haven't gotten
	 * around to it yet. (Does not include the IDs of docs that we are writing).
	 */
	private final Set<String> docsToDelete = new HashSet<String>();

	/**
	 * The docs that we have been asked to write, but haven't gotten around to
	 * it yet, keyed by their IDs.
	 */
	private final Map<String, SearchDocument> docsToWrite = new HashMap<String, SearchDocument>();

	/**
	 * A listener to record index events.
	 */
	private IndexWrapperListener listener = IndexWrapperListener.NULL_LISTENER;

	/**
	 * Keep it private to enforce the Factory pattern.
	 */
	private IndexWrapperImpl() {
	}

	/**
	 * Set a new listener to record index events.
	 */
	@Override
	public void setListener(IndexWrapperListener listener) {
		if (listener == null) {
			this.listener = IndexWrapperListener.NULL_LISTENER;
		} else {
			this.listener = listener;
		}
	}

	/**
	 * Create a fresh index in the configured directory. CAUTION: do you really
	 * want to do this?
	 */
	@Override
	public void createIndex() throws IOException {
		synchronized (access) {
			access.createIndex();
		}
	}

	/**
	 * Create a query object from a text string.
	 */
	@Override
	public IndexWrapperQuery parseQuery(String queryString)
			throws ParseException, IOException {
		synchronized (access) {
			try {
				IndexReader reader = access.getReader();
				return LuceneIndexQuery.createGeneralQuery(queryString, reader);
			} finally {
				access.releaseReader();
			}
		}
	}

	/**
	 * Create a query that will search a given field for a value that begins
	 * with a given string.
	 */
	@Override
	public IndexWrapperQuery createPrefixQuery(String fieldName,
			String valuePrefix) throws IOException {
		synchronized (access) {
			try {
				IndexReader reader = access.getReader();
				return LuceneIndexQuery.createPrefixQuery(fieldName,
						valuePrefix, reader);
			} finally {
				access.releaseReader();
			}
		}
	}

	/**
	 * Search the index, and return the results.
	 */
	@Override
	public synchronized SearchResults search(IndexWrapperQuery query,
			int skipCount, int maxReturnCount) throws IOException {
		LuceneIndexQuery liQuery = LuceneIndexQuery.confirmQueryClass(query,
				this.getClass());

		synchronized (access) {
			try {
				LOG.debug("search on this query: " + liQuery.getBaseQuery());
				Hits hits = access.getSearcher().search(liQuery.getBaseQuery());
				return new SearchResultsImpl(liQuery, hits, maxReturnCount,
						skipCount, docsToDelete, docsToWrite);
			} finally {
				access.releaseSearcher();
			}
		}
	}

	/**
	 * Search the index for a document with this ID. Return a
	 * {@link SearchResults} object with either 0 or 1 result.
	 */
	@Override
	public SearchResults getDocument(String docId) throws IOException {
		LuceneIndexQuery liQuery = LuceneIndexQuery.createDocumentQuery(docId);
		return search(liQuery, 0, 1);
	}

	/**
	 * Get the best passage and highlight the query terms with HTML tags. If
	 * there are problems, return an empty string.
	 */
	@Override
	public String getBestPassage(IndexWrapperQuery query, String content) {
		return BestPassageGenerator.getInstance().getHighlightedBestPassage(
				query, content);
	}

	/**
	 * Write a document to the index, replacing any previous version. This
	 * request will be cached until the index wrapper is synchronized to the
	 * index.
	 */
	public synchronized void writeDocument(SearchDocument doc)
			throws IOException {
		LOG.debug("writing document: " + doc.getIdentifier());

		// if we have already been asked to delete this document, forget that.
		docsToDelete.remove(doc.getIdentifier());

		// if we have already been asked to write this document, write over it.
		docsToWrite.put(doc.getIdentifier(), doc);

		synchronizeIfReady(false);
	}

	/**
	 * Remove this document from the index (or any document with this ID). This
	 * request will be cached until the index wrapper is synchronized to the
	 * index.
	 */
	@Override
	public synchronized void deleteDocument(SearchDocument doc)
			throws IOException {
		// if we have previously been asked to write this document, forget that.
		docsToWrite.keySet().remove(doc.getIdentifier());

		docsToDelete.add(doc.getIdentifier());

		synchronizeIfReady(false);
	}

	/**
	 * The client code requests a synchronize.
	 */
	@Override
	public synchronized void synchronize() throws IOException {
		synchronizeIfReady(true);
	}

	/**
	 * <p>
	 * Check to see if we are ready to synchronize to the index.
	 * </p>
	 * <p>
	 * If the user has requested a synchronize, or if the number of cached
	 * writes and deletes has reached the cache limit, then we're ready.
	 * </p>
	 */
	private void synchronizeIfReady(boolean force) throws IOException {
		if (force) {
			doSynchronize(EventType.USER_SYNCHRONIZE);
			return;
		}

		int waitingInCache = docsToDelete.size() + docsToWrite.size();
		int allowedInCache = config.getMaximumCachedWrites();
		if (waitingInCache >= allowedInCache) {
			doSynchronize(EventType.CACHE_FULL);
			return;
		}
	}

	/**
	 * Synchronize the index wrapper to the index. This insures that all pending
	 * deletes and additions are performed, and that the index searcher is
	 * current.
	 */
	private void doSynchronize(EventType eventType) throws IOException {
		// Record it, if anyone is listening.
		listener
				.recordEvent(eventType, docsToDelete.size(), docsToWrite.size());

		synchronized (access) {
			if (!docsToDelete.isEmpty()) {
				try {
					// go through the pending deletes, deleting them.
					IndexReader reader = access.getReader();
					for (String docId : docsToDelete) {
						reader.deleteDocuments(new Term(config
								.getDocIdFieldName(), docId));
					}
				} finally {
					access.releaseReader();
				}
			}
			docsToDelete.clear();

			if (!docsToWrite.isEmpty()) {
				try {
					// go through the pending writes, deleting any old versions.
					IndexReader reader = access.getReader();
					for (String docId : docsToWrite.keySet()) {
						reader.deleteDocuments(new Term(config
								.getDocIdFieldName(), docId));
					}
				} finally {
					access.releaseReader();
				}

				try {
					// go through the writes again, adding them.
					IndexWriter writer = access.getWriter();
					for (SearchDocument doc : docsToWrite.values()) {
						writer.addDocument(doc.toDocument());
					}
				} finally {
					access.releaseWriter();
				}
			}
			docsToWrite.clear();

			// The next search will see the changes.
			access.quiesce();
		}
	}

	/**
	 * The cache should be flushed before the system shuts down. If not, try to
	 * do it now.
	 */
	@Override
	protected void finalize() throws Throwable {
		if (!docsToDelete.isEmpty() || !docsToWrite.isEmpty()) {
			LOG.error("Lucene index was not closed properly. Flushing cache!");
			doSynchronize(EventType.FINALIZE);
		}

		super.finalize();
	}

	/**
	 * Create an IndexInfo object.
	 * 
	 * @see org.nsdl.search.lucene.wrapper.IndexWrapper#getIndexInfo()
	 * @throws IOException
	 */
	@Override
	public IndexSummaryInfo getIndexInfo() throws IOException {
		IndexSummaryInfoImpl info = new IndexSummaryInfoImpl();

		// These items can be obtained without a reader instance.
		info.setIndexDirectory(new File(config.getLuceneIndexDirectory()));
		info.setLastModified(new Date(IndexReader.lastModified(config
				.getLuceneIndexDirectory())));

		// Get info through the current reader, and release it.
		synchronized (access) {
			try {
				IndexReader reader = access.getReader();
				info.setNumDocs(reader.numDocs());
				info.setVersion(reader.getVersion());
				info.setFieldNames(new TreeSet<String>(getFieldNames(reader,
						FieldOption.ALL)));
				info.setSearchableFieldNames(new TreeSet<String>(getFieldNames(
						reader, FieldOption.INDEXED)));
			} finally {
				access.releaseReader();
			}
		}

		// The list of stored fields is derived from the list of all fields.
		Set<String> storedFieldNames = new TreeSet<String>();
		for (String fieldName : info.getFieldNames()) {
			LuceneFieldSpec spec = SearchIndexFieldSpecs.getSpec(fieldName);
			if ((spec != null) && (spec.getStore().equals(Field.Store.YES))) {
				storedFieldNames.add(fieldName);
			}
		}
		info.setStoredFieldNames(storedFieldNames);

		return info;
	}

	/**
	 * Get field names in a separate method just so we can limit the scope of
	 * the "unchecked" suppression.
	 */
	@SuppressWarnings("unchecked")
	private Collection<String> getFieldNames(IndexReader reader,
			FieldOption fieldOption) {
		return reader.getFieldNames(fieldOption);
	}

}
