/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.wrapper;

import java.io.IOException;

import org.apache.log4j.Logger;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.search.IndexSearcher;
import org.nsdl.search.lucene.nsdl.NsdlAnalyzer;
import org.nsdl.search.util.Config;

/**
 * Acts as the "traffic cop", allowing access to the search index via
 * {@link IndexSearcher}s, {@link IndexReader}s, and {@link IndexWriter}s.
 * Used only by the {@link IndexWrapperImpl}.
 * 
 * TODO: This class has gone halfway toward proper use. Right now, the searcher
 * is only closed on {@link #quiesce()} and the reader and writer are closed
 * either on {@link #quiesce()} or when the other is opened.
 * 
 * TODO: But to make this work properly, access is synchronized by the
 * {@link IndexWrapperImpl}, and that's not appropriate or efficient.
 * 
 * TODO: Need to do more work with this, probably using the new Threading
 * package.
 * 
 * @author jblake
 */
public class LuceneAccessHandler {
	private static final Logger LOG = Logger
			.getLogger(LuceneAccessHandler.class);

	private final String luceneIndexDirectory;

	private IndexSearcher currentSearcher;
	private boolean searcherInUse;

	private IndexWriter currentWriter;
	private boolean writerInUse;

	private IndexReader currentReader;
	private boolean readerInUse;

	LuceneAccessHandler() {
		this.luceneIndexDirectory = Config.getInstance()
				.getIndexWrapperConfig().getLuceneIndexDirectory();
	}

	/**
	 * Create a new index, in the configured directory.
	 * 
	 * @throws IOException
	 * @throws IllegalStateException
	 *             if the current index is in use.
	 */
	void createIndex() throws IOException {
		// Can't create a new index if an old one is already in use.
		quiesce();

		IndexWriter writer = new IndexWriter(luceneIndexDirectory, NsdlAnalyzer
				.getInstance(), true);
		writer.close();
	}

	/**
	 * Get an {@link IndexSearcher} for use on the index. This searcher might
	 * not be current unless this call follows a call to {@link #quiesce()}.
	 * This call must be followed by a call to {@link #releaseSearcher()}.
	 * 
	 * @throws IOException
	 * @throws IllegalStateException
	 *             if a previous call was not released.
	 */
	IndexSearcher getSearcher() throws IOException {
		if (searcherInUse) {
			throw new IllegalStateException("Searcher is already in use.");
		}
		openSearcher();
		searcherInUse = true;
		return currentSearcher;
	}

	/**
	 * <p>
	 * Indicate that the current {@link IndexSearcher} is no longer in use. This
	 * follows a call to {@link #getSearcher()}.
	 * </p>
	 * <p>
	 * This does not throw a IOException on error, so it can be safely used in a
	 * "finally" block.
	 * </p>
	 */
	void releaseSearcher() {
		searcherInUse = false;
	}

	/**
	 * Get an {@link IndexReader} for use on the index. This reader might not be
	 * current unless this call follows a call to {@link #quiesce()}. This call
	 * must be followed by a call to {@link #releaseReader()}.
	 * 
	 * NOTE: we can't have both a reader and a writer open at the same time, so
	 * if a writer is open, we need to close it.
	 * 
	 * @throws IOException
	 * @throws IllegalStateException
	 *             if a previous call was not released.
	 */
	IndexReader getReader() throws IOException {
		if (readerInUse) {
			throw new IllegalStateException("Reader is already in use.");
		}
		if (writerInUse) {
			throw new IllegalStateException("Writer is still in use.");
		}

		closeWriter();

		openReader();
		readerInUse = true;
		return currentReader;
	}

	/**
	 * <p>
	 * Indicate that the current {@link IndexReader} is no longer in use. This
	 * follows a call to {@link #getReader()}.
	 * </p>
	 * <p>
	 * This does not throw a IOException on error, so it can be safely used in a
	 * "finally" block.
	 * </p>
	 */
	void releaseReader() {
		readerInUse = false;
	}

	/**
	 * Get an {@link IndexWriter} for use on the index. This call must be
	 * followed by a call to {@link #releaseWriter()}.
	 * 
	 * NOTE: we can't have both a reader and a writer open at the same time, so
	 * if a reader is open, close it.
	 * 
	 * @throws IOException
	 * @throws IllegalStateException
	 *             if a previous call was not released.
	 */
	IndexWriter getWriter() throws IOException {
		if (writerInUse) {
			throw new IllegalStateException("Writer is already in use.");
		}
		if (readerInUse) {
			throw new IllegalStateException("Reader is still in use.");
		}

		closeReader();
		openWriter();
		writerInUse = true;
		return currentWriter;
	}

	/**
	 * <p>
	 * Indicate that the current {@link IndexWriter} is no longer in use. This
	 * follows a call to {@link #getWriter()}.
	 * </p>
	 * <p>
	 * This does not throw a IOException on error, so it can be safely used in a
	 * "finally" block.
	 * </p>
	 */
	void releaseWriter() {
		writerInUse = false;
	}

	/**
	 * Close current reader, writer, and searcher. The index can then be removed
	 * or replaced. The next calls to {@link #getSearcher()} of
	 * {@link #getReader()} will return current objects.
	 * 
	 * @throws IllegalStateException
	 *             if any of the reader, writer or searcher is currently in use.
	 */
	void quiesce() {
		if (searcherInUse) {
			throw new IllegalStateException(
					"Can't quiesce: Searcher is in use.");
		}
		closeSearcher();

		if (readerInUse) {
			throw new IllegalStateException("Can't quiesce: Reader is in use.");
		}
		closeReader();

		if (writerInUse) {
			throw new IllegalStateException("Can't quiesce: Writer is in use.");
		}
		closeWriter();

	}

	private void openWriter() throws IOException {
		if (currentWriter == null) {
			currentWriter = new IndexWriter(luceneIndexDirectory, NsdlAnalyzer
					.getInstance(), false);
		}
	}

	private void closeWriter() {
		if (currentWriter != null) {
			try {
				currentWriter.close();
			} catch (IOException e) {
				LOG.error("failed to close IndexWriter", e);
			}
			currentWriter = null;
		}
	}

	private void openReader() throws IOException {
		if (currentReader == null) {
			currentReader = IndexReader.open(luceneIndexDirectory);
		}
	}

	private void closeReader() {
		if (currentReader != null) {
			try {
				currentReader.close();
			} catch (IOException e) {
				LOG.error("failed to close IndexReader", e);
			}
			currentReader = null;
		}
	}

	private void openSearcher() throws IOException {
		if (currentSearcher == null) {
			currentSearcher = new IndexSearcher(luceneIndexDirectory);
		}
	}

	private void closeSearcher() {
		if (currentSearcher != null) {
			try {
				currentSearcher.close();
			} catch (IOException e) {
				LOG.error("failed to close IndexSearcher", e);
			}
			currentSearcher = null;
		}
	}

}
