/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.wrapper;

import java.io.IOException;

import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.PrefixQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;
import org.nsdl.search.lucene.nsdl.NsdlQueryParser;

/**
 * A Lucene-based implementation of {@link IndexWrapperQuery}.
 * 
 * @author jblake
 */
public class LuceneIndexQuery implements IndexWrapperQuery {

	/**
	 * Create a query object from a query string. Need access to an
	 * {@link IndexReader} in order to create the primitive version of the base
	 * query.
	 */
	public static LuceneIndexQuery createGeneralQuery(String queryString,
			IndexReader reader) throws ParseException, IOException {
		Query baseQuery = NsdlQueryParser.getInstance().parse(queryString);
		Query primitiveQuery = baseQuery.rewrite(reader);
		return new LuceneIndexQuery(baseQuery, primitiveQuery);
	}

	/**
	 * Create a query object from a document ID.
	 */
	public static LuceneIndexQuery createDocumentQuery(String docId) {
		Query baseQuery = new TermQuery(new Term(
				SearchIndexFieldSpecs.RESOURCE_IDENTIFIER.getName(), docId));
		// This single TermQuery is already primitive.
		return new LuceneIndexQuery(baseQuery, baseQuery);
	}

	/**
	 * Create a prefix query object from a field name and a prefix string. Need
	 * access to an {@link IndexReader} in order to create the primitive version
	 * of the base query.
	 */
	public static LuceneIndexQuery createPrefixQuery(String fieldName,
			String valuePrefix, IndexReader reader) throws IOException {
		Query baseQuery = new PrefixQuery(new Term(fieldName, valuePrefix));
		Query primitiveQuery = baseQuery.rewrite(reader);
		return new LuceneIndexQuery(baseQuery, primitiveQuery);
	}

	/**
	 * Do a safe cast from {@link IndexWrapperQuery} to {@link LuceneIndexQuery}.
	 */
	public static LuceneIndexQuery confirmQueryClass(IndexWrapperQuery query,
			Class<?> usingClass) {
		if (query instanceof LuceneIndexQuery) {
			return (LuceneIndexQuery) query;
		} else if (query == null) {
			throw new NullPointerException("query may not be null.");
		} else {
			throw new IllegalArgumentException(usingClass.getSimpleName()
					+ " can only accept queries that are instances of "
					+ LuceneIndexQuery.class.getSimpleName());
		}
	}

	/**
	 * For searching.
	 */
	private final Query baseQuery;

	/**
	 * For figuring best passage. This must be a primitive query.
	 */
	private final Query primitiveQuery;

	/**
	 * Create a query object from a query string.
	 */
	private LuceneIndexQuery(Query baseQuery, Query primitiveQuery) {
		this.baseQuery = baseQuery;
		this.primitiveQuery = primitiveQuery;
	}

	public Query getBaseQuery() {
		return baseQuery;
	}

	public Query getPrimitiveQuery() {
		return primitiveQuery;
	}

	@Override
	public String toString() {
		return "LuceneIndexQuery[baseQuery=" + baseQuery + ", primitiveQuery="
				+ primitiveQuery + "]";
	}

}
