/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.wrapper;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.lucene.search.Hits;
import org.nsdl.search.lucene.searchdoc.SearchDocument;

/**
 * <p>
 * The basic implementation of {@link SearchResults}.
 * </p>
 * <p>
 * This object contains only "dead" data. That is to say, it is not dependent on
 * the Lucene index except during the constructor.
 * </p>
 * 
 * @author jblake
 */
public class SearchResultsImpl implements SearchResults {
	private final int maxReturnCount;
	private final int skipCount;
	private final int totalHitCount;
	private final LuceneIndexQuery query;
	private final List<SearchResult> results;

	/**
	 * Write down all of the interesting information. Filter the Hits against
	 * the cached deletions and updates, so the view for the client is as
	 * current as we can make it.
	 */
	SearchResultsImpl(LuceneIndexQuery query, Hits hits, int maxReturnCount,
			int skipCount, Set<String> docsToDelete,
			Map<String, SearchDocument> docsToWrite) throws IOException {
		if (query == null) {
			throw new NullPointerException("'query' cannot be null.");
		}
		if (hits == null) {
			throw new NullPointerException("'hits' cannot be null.");
		}
		if (maxReturnCount < 0) {
			throw new IllegalArgumentException(
					"'maxReturnCount' cannot be less than 0.");
		}
		if (skipCount < 0) {
			throw new IllegalArgumentException(
					"'skipCount' cannot be less than 0.");
		}
		if (docsToDelete == null) {
			throw new NullPointerException("'docsToDelete' cannot be null.");
		}
		if (docsToWrite == null) {
			throw new NullPointerException("'docsToWrite' cannot be null.");
		}

		this.query = query;
		this.maxReturnCount = maxReturnCount;
		this.skipCount = skipCount;
		this.totalHitCount = hits.length();

		List<SearchResult> results = assembleResults(query, hits, docsToDelete,
				docsToWrite);
		this.results = Collections.unmodifiableList(results);
	}

	/**
	 * Go through the Hits, fetching Lucene documents and building
	 * {@link SearchDocument}s from them. These {@link SearchDocument}s are
	 * then assembled into {@link SearchResult}s for distribution.
	 * 
	 * If any hit is found in the docs to delete, skip it.
	 * 
	 * If any hit is found in the docs to write, use that version instead.
	 */
	private List<SearchResult> assembleResults(IndexWrapperQuery query, Hits hits,
			Set<String> docsToDelete, Map<String, SearchDocument> docsToWrite)
			throws IOException {
		List<SearchResult> results = new ArrayList<SearchResult>();
		for (int position = this.skipCount; position < hits.length(); position++) {
			SearchDocument doc = new SearchDocument(hits.doc(position));
			String docId = doc.getIdentifier();
			if (docsToDelete.contains(docId)) {
				continue;
			}
			if (docsToWrite.containsKey(docId)) {
				doc = docsToWrite.get(docId);
			}

			String bestPassage = "";
			if (doc.hasContent()) {
				String contentText = doc.getContent().getText();
				bestPassage = IndexWrapper.getInstance().getBestPassage(query,
						contentText);
			}

			results.add(new SearchResultImpl(position, hits.score(position),
					doc, bestPassage));

			if (results.size() >= this.maxReturnCount) {
				break;
			}
		}
		return results;
	}

	/**
	 * We return no more than you asked for, and maybe less.
	 */
	public int getReturnCount() {
		return results.size();
	}

	/**
	 * How many result documents did you ask to skip?
	 */
	public int getSkipCount() {
		return this.skipCount;
	}

	/**
	 * How many documents in the index matched your query?
	 */
	public int getTotalCount() {
		return this.totalHitCount;
	}

	/**
	 * Get a specific result, or null if index is out of range.
	 */
	public SearchResult getResult(int index) {
		if ((index >= 0) && (index < results.size())) {
			return results.get(index);
		} else {
			return null;
		}
	}

	/**
	 * Get an iterator against this object.
	 */
	public Iterator<SearchResult> iterator() {
		return results.iterator();
	}

	@Override
	public String toString() {
		return "SearchResultsImpl[totalHits=" + this.totalHitCount
				+ ", skipCount=" + this.skipCount + ", maxCount="
				+ this.maxReturnCount + ", query=" + this.query + "]";
	}

}
