/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.oai.harvest;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.zip.GZIPInputStream;

import org.apache.log4j.Logger;
import org.nsdl.searchtools.common.history.HistoryFile;

/**
 * An OAI harvest process creates a directory of files. This performs operations
 * on those files.
 * 
 * @author jblake
 */
public class HarvestDirectoryWrapper {
	private static final Logger LOG = Logger
			.getLogger(HarvestDirectoryWrapper.class);

	private static final String CHAR_ENCODING = "UTF-8";

	private final HistoryFile historyFile;
	private final File directory;

	/**
	 * Package access -- for use only by {@link HarvestWrapper}.
	 */
	HarvestDirectoryWrapper(File directory, HistoryFile historyFile) {
		this.directory = directory;
		this.historyFile = historyFile;
	}

	/**
	 * Where is this directory?
	 */
	public String getPath() {
		return this.directory.getPath();
	}

	/**
	 * Has this harvest already been applied to the current search index?
	 */
	public boolean isIndexed() {
		return historyFile.isEventPresent(directory.getAbsolutePath());
	}

	/**
	 * Finished applying this harvest to the current search index. Write it
	 * down.
	 */
	public void setIndexed() {
		historyFile.recordEvent(directory.getAbsolutePath());
	}

	/**
	 * Get a list of the harvest files (whether gzipped or not), sorted by
	 * filename, sort of: names are "ListRecords.31.xml.gz", or
	 * "ListRecords.31.xml", so find the integer portion and compare based on
	 * that.
	 */
	public List<File> getFiles() {
		File[] fileArray = directory.listFiles(new FileFilter() {
			public boolean accept(File file) {
				return file.getName().matches(
						"ListRecords\\.[0-9]+\\.xml(\\.gz)?");
			}
		});

		List<File> fileList = new ArrayList<File>(Arrays.asList(fileArray));
		Collections.sort(fileList, new Comparator<File>() {
			public int compare(File first, File second) {
				return getFileInteger(first) - getFileInteger(second);
			}
			private int getFileInteger(File file) {
				String iString = file.getName().split("\\.")[1];
				return Integer.parseInt(iString);
			}
		});

		return fileList;
	}

	/**
	 * Get a Reader on this file, regardless of whether the file is GZipped or
	 * not.
	 */
	public Reader openHarvestFile(File file) throws IOException {
		// This is a crude check for a gzipped file
		InputStream stream;
		if (file.getName().endsWith(".gz")) {
			stream = new GZIPInputStream(new FileInputStream(file));
		} else {
			stream = new FileInputStream(file);
		}
		return new BufferedReader(new InputStreamReader(stream, CHAR_ENCODING));
	}

	/**
	 * Check the harvest log to find out how many OAI records were harvested
	 * into this directory.
	 */
	public int getNumberOfHarvestedRecords() {
		try {
			OaiHarvestLogSummary harvestLogAnalyzer = OaiHarvestLogSummary
					.getSummary(directory.getAbsolutePath());
			return (int) harvestLogAnalyzer.getNumRecsHarvested();
		} catch (IOException e) {
			LOG.error("problem with OAI harvest log for "
					+ directory.getAbsolutePath(), e);
			return -1;
		} catch (OaiLogFormatException e) {
			LOG.error("problem with OAI harvest log for "
					+ directory.getAbsolutePath(), e);
			return -1;
		}
	}

}
