/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.oai.harvest;

import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.nsdl.searchtools.common.history.HistoryFile;

/**
 * A wrapper around the parent directory that contains all harvest directories.
 * 
 * @author jblake
 */
public class HarvestWrapperImpl extends HarvestWrapper {
	private static final Map<String, HarvestWrapperImpl> instances = new HashMap<String, HarvestWrapperImpl>();

	/**
	 * Get an instance based around this directory and this history file. If we
	 * already have such an instance, use it.
	 */
	public static HarvestWrapperImpl getInstance(File harvestParentDirectory,
			HistoryFile historyFile) {
		String key = makeKey(harvestParentDirectory, historyFile);

		if (!instances.containsKey(key)) {
			instances.put(key, new HarvestWrapperImpl(harvestParentDirectory,
					historyFile));
		}

		return instances.get(key);
	}

	private static String makeKey(File harvestParentDirectory,
			HistoryFile historyFile) {
		return harvestParentDirectory.getPath() + " ~ " + historyFile.getPath();
	}

	private final File harvestParentDirectory;

	private final HistoryFile historyFile;

	/**
	 * Keep it private to enforce the factory pattern.
	 */
	private HarvestWrapperImpl(File harvestParentDirectory,
			HistoryFile historyFile) {
		this.harvestParentDirectory = harvestParentDirectory;
		this.historyFile = historyFile;
	}

	@Override
	public File getHarvestParentDirectory() {
		return this.harvestParentDirectory;
	}

	/**
	 * Find the specified harvest directory, and create a wrapper around it.
	 * 
	 * The directory name can be specified either as a full path (absolute or
	 * relative to the current directory), or as relative to the harvest parent
	 * directory.
	 */
	public HarvestDirectoryWrapper getDirectoryWrapper(String directoryName)
			throws FileNotFoundException {

		File harvestDirectory = new File(directoryName);
		if (harvestDirectory.exists() && harvestDirectory.isDirectory()) {
			return new HarvestDirectoryWrapper(harvestDirectory, historyFile);
		}

		harvestDirectory = new File(harvestParentDirectory, directoryName);
		if (harvestDirectory.exists() && harvestDirectory.isDirectory()) {
			return new HarvestDirectoryWrapper(harvestDirectory, historyFile);
		}

		throw new FileNotFoundException("Cannot find harvest directory '"
				+ directoryName + "'");
	}

	/**
	 * Look at the indexedHarvests file in the current search index directory,
	 * and figure out which harvest should be indexed next.
	 * 
	 * @return null, if all harvests have been indexed.
	 */
	public HarvestDirectoryWrapper getOldestUnindexedDirectory() {
		List<HarvestDirectoryWrapper> wrappers = getUnindexedDirectories();
		if (wrappers.isEmpty()) {
			return null;
		} else {
			return wrappers.get(0);
		}
	}

	/**
	 * Get a list of all the wrappers of unindexed harvest directories, sorted
	 * by age, oldest first.
	 */
	public List<HarvestDirectoryWrapper> getUnindexedDirectories() {
		// Get a list of harvest directories (subdirectories that fit the naming
		// scheme): e.g. 2007-08-04T09-00-02Z
		File[] harvestArray = harvestParentDirectory
				.listFiles(new FileFilter() {
					public boolean accept(File file) {
						return (file.isDirectory() && (file.getName()
								.matches("[-0-9]{10}T[-0-9]{8}Z")));
					}
				});

		// Sort the list by name, which is the same as sorting it by age.
		List<File> harvestList = new ArrayList<File>(Arrays
				.asList(harvestArray));
		Collections.sort(harvestList, new Comparator<File>() {
			public int compare(File first, File second) {
				return first.getName().compareTo(second.getName());
			}
		});

		// Create a list of wrappers around any directories that have not been
		// indexed.
		List<HarvestDirectoryWrapper> result = new ArrayList<HarvestDirectoryWrapper>();
		for (Iterator<File> it = harvestList.iterator(); it.hasNext();) {
			File harvest = it.next();
			if (!historyFile.isEventPresent(harvest.getAbsolutePath())) {
				result.add(new HarvestDirectoryWrapper(harvest, historyFile));
			}
		}
		return result;
	}
}
