/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.oai.harvest;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.nsdl.searchtools.common.DateUtil;

/**
 * Analyze the log file from an OAI harvest, and present a summary of the
 * information from that file.
 * 
 * This object should be immutable, or the cache could be a problem.
 * 
 * @author jblake
 */
public class OaiHarvestLogSummary {
	public static final String CHAR_ENCODING = "UTF-8";

	public static final String DEFAULT_FROM_DATE = "1980-01-01T00:00:01Z";

	/** Keep any summaries we create, keyed by harvest directory. */
	private static final Map<File, OaiHarvestLogSummary> cache = new HashMap<File, OaiHarvestLogSummary>();

	private static final Pattern PATTERN_BASE_URL = Pattern
			.compile("OAIGet: Doing POST to (.*) args: ");
	private static final Pattern PATTERN_FROM_DATE = Pattern
			.compile("from=([^&]*)");
	private static final Pattern PATTERN_UNTIL_DATE = Pattern
			.compile("until=([^&]*)");
	private static final Pattern PATTERN_METADATA_PREFIX = Pattern
			.compile("metadataPrefix=([^&]*)");
	private static final Pattern PATTERN_WARNING = Pattern.compile(
			"Error|Warning", Pattern.CASE_INSENSITIVE);
	private static final Pattern PATTERN_NO_RECORDS_MATCH = Pattern
			.compile("[ noRecordsMatch -> \"no records match your selection criteria\" ]");
	private static final Pattern PATTERN_RUNNING_TOTAL = Pattern
			.compile("oaiharvest.pl: Got (\\d+) records \\(running total: (\\d+)\\)");
	private static final Pattern PATTERN_REQUEST_DATE = Pattern
			.compile("	Date:	(.*)");

	/**
	 * Given a path to a harvest directory, return a summary of the log. Cache
	 * the summary so we won't analyze the same log again.
	 * 
	 * @throws IOException
	 * @throws OaiLogFormatException
	 */
	public static OaiHarvestLogSummary getSummary(String harvestDirectoryPath)
			throws IOException, OaiLogFormatException {
		if (harvestDirectoryPath == null) {
			throw new NullPointerException(
					"harvestDirectoryPath may not be null.");
		}

		File harvestDirectory = new File(harvestDirectoryPath);

		// if we don't already have one in the cache, create one and cache it.
		if (!cache.containsKey(harvestDirectory)) {
			cache.put(harvestDirectory, new OaiHarvestLogSummary(
					new FileReader(new File(harvestDirectory,
							OAIHarvestConfig.HARVEST_LOG_FNAME))));
		}

		// return it from the cache.
		return cache.get(harvestDirectory);
	}

	private String baseUrl;
	private String metadataPrefix;
	private String fromDate;
	private String untilDate;
	private long numResponses;
	private long numRecsHarvested;
	private int numRecsPerResponse;
	private List<String> warnings = new ArrayList<String>();
	private String dateOfFirstRequest;

	private SimpleDateFormat requestDateParser = new SimpleDateFormat(
			"EEE, d MMM yyyy HH:mm:ss Z");

	/**
	 * Go through the log file line by line, gleaning information. Then, we may
	 * need to adjust totals and such, depending on what we find.
	 * 
	 * @throws OaiLogFormatException
	 */
	OaiHarvestLogSummary(Reader reader) throws IOException,
			OaiLogFormatException {
		BufferedReader bufferedReader = null;
		try {
			bufferedReader = new BufferedReader(reader);
			String line = "";
			while (null != (line = bufferedReader.readLine())) {
				lookForRequestArguments(line);
				lookForWarnings(line);
				lookForTotalRecords(line);
				lookForFirstRequestDate(line);
			}
			reconcileResults();
		} finally {
			if (bufferedReader != null) {
				bufferedReader.close();
			}
		}
	}

	/**
	 * Does this line describe the request? If so, see how many arguments we can
	 * get from it.
	 */
	private void lookForRequestArguments(String line) {
		Matcher matchUrl = PATTERN_BASE_URL.matcher(line);
		if (matchUrl.find()) {
			this.baseUrl = matchUrl.group(1);

			// if this included the base url, it might include other arguments
			// also.

			Matcher matchFrom = PATTERN_FROM_DATE.matcher(line);
			if (matchFrom.find()) {
				this.fromDate = decodeString(matchFrom.group(1));
			}

			Matcher matchMetadataPrefix = PATTERN_METADATA_PREFIX.matcher(line);
			if (matchMetadataPrefix.find()) {
				this.metadataPrefix = matchMetadataPrefix.group(1);
			}

			Matcher matchUntil = PATTERN_UNTIL_DATE.matcher(line);
			if (matchUntil.find()) {
				this.untilDate = decodeString(matchUntil.group(1));
			}

		}
	}

	/**
	 * Grab any error lines or warning lines, but ignore the "no match" error.
	 */
	private void lookForWarnings(String line) {
		Matcher matchWarning = PATTERN_WARNING.matcher(line);
		Matcher matchNoMatch = PATTERN_NO_RECORDS_MATCH.matcher(line);
		if ((matchWarning.find()) && (!matchNoMatch.find())) {
			this.warnings.add(line.trim());
		}
	}

	/**
	 * Each time we see the running totals line, we know the new total, and we
	 * increment the number of responses. The number of responses per batch will
	 * very likely change on the last response, so take the value from the first
	 * response and stick with it.
	 */
	private void lookForTotalRecords(String line) {
		Matcher matchTotal = PATTERN_RUNNING_TOTAL.matcher(line);
		if (matchTotal.find()) {
			this.numResponses++;
			this.numRecsHarvested = Long.parseLong(matchTotal.group(2));
			if (this.numRecsPerResponse == 0) {
				this.numRecsPerResponse = Integer.parseInt(matchTotal.group(1));
			}
		}
	}

	/**
	 * If we haven't found a request date yet, look for one. We will need it if
	 * we don't find an until date.
	 */
	private void lookForFirstRequestDate(String line)
			throws OaiLogFormatException {
		if (this.dateOfFirstRequest == null) {
			Matcher matchRequestDate = PATTERN_REQUEST_DATE.matcher(line);
			if (matchRequestDate.find()) {
				String dateString = matchRequestDate.group(1);
				try {
					Date requestDate = requestDateParser.parse(dateString);
					this.dateOfFirstRequest = DateUtil.getInstance()
							.formatDate(requestDate,
									DateUtil.ISO8601_DATE_TIME_Z);
				} catch (ParseException e) {
					throw new OaiLogFormatException("Invalid request date: "
							+ dateString, e);
				}
			}
		}
	}

	/**
	 * Now that we've searched the file, make any necessary adjustments.
	 * <ul>
	 * <li>If there were no request arguments, then we have a problem, Houston.</li>
	 * <li>If we have no from date, use the default.</li>
	 * <li>If we have no until date, see whether we picked one a request date.
	 * If no request date either, throw an Exception.</li>
	 * </ul>
	 * 
	 * @throws OaiLogFormatException
	 */
	private void reconcileResults() throws OaiLogFormatException {
		if (this.baseUrl == null) {
			throw new OaiLogFormatException(
					"OAI harvest log did not contain a request line.");
		}

		if (this.fromDate == null) {
			this.fromDate = DEFAULT_FROM_DATE;
		}

		if (this.untilDate == null) {
			if (this.dateOfFirstRequest == null) {
				throw new OaiLogFormatException(
						"OAI harvest log did not contain an 'until' date.");
			} else {
				this.untilDate = this.dateOfFirstRequest;
			}
		}
	}

	public String getBaseURLStr() {
		return this.baseUrl;
	}

	public String getMetadataPrefix() {
		return this.metadataPrefix;
	}

	public String getFromDate() {
		return this.fromDate;
	}

	public String getUntilDate() {
		return this.untilDate;
	}

	public long getNumListRecordsResponses() {
		return this.numResponses;
	}

	public long getNumRecsHarvested() {
		return this.numRecsHarvested;
	}

	public int getNumRecsPerListRecordsResponse() {
		return this.numRecsPerResponse;
	}

	public String[] getWarnings() {
		return this.warnings.toArray(new String[warnings.size()]);
	}

	/**
	 * FromDate and ToDate need to be decoded. If there is a problem, just use
	 * the raw value.
	 */
	private String decodeString(String string) {
		try {
			return URLDecoder.decode(string, CHAR_ENCODING);
		} catch (UnsupportedEncodingException e) {
			return string;
		}
	}

}
