/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.oai.process;

import java.io.File;
import java.io.IOException;
import java.io.Reader;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;
import org.nsdl.search.lucene.wrapper.IndexWrapper;
import org.nsdl.search.oai.harvest.HarvestDirectoryWrapper;
import org.nsdl.search.processoai.OaiHarvestRecordsParser;
import org.nsdl.search.processoai.OaiRecordIndexer;
import org.nsdl.search.processoai.XmlException;
import org.nsdl.search.utilities.config.PropertyFileConfig;
import org.nsdl.searchtools.common.DateUtil;
import org.xml.sax.InputSource;

/**
 * Process an OAI harvest directory, applying the OAI to the Lucene index.
 * 
 * @author jblake
 */
public class OaiHarvestProcessor {
	private static final Logger LOG = Logger
			.getLogger(OaiHarvestProcessor.class);
	private static final Logger NOTIFY_LOG = Logger
			.getLogger("NOTIFY.OaiHarvestProcessor");

	private final OaiHarvestProcessorParameters parms;
	private final OaiHarvestStatistics stats;
	private final DateUtil dateUtil = DateUtil.getInstance();

	OaiHarvestProcessor(OaiHarvestProcessorParameters parms,
			OaiHarvestStatistics stats) {
		this.parms = parms;
		this.stats = stats;
	}

	/**
	 * Parse all of the selected harvest directories.
	 */
	void parseAllHarvests() throws IOException {
		List<HarvestDirectoryWrapper> harvestDirectories = parms
				.getHarvestDirectories();
		if (harvestDirectories.isEmpty()) {
			LOG.info("Found no eligible directories.");
		} else {
			for (HarvestDirectoryWrapper wrapper : harvestDirectories) {
				parseHarvestToIndex(wrapper);
			}
		}
	}

	/**
	 * Parse all of the files in the harvest directory.
	 */
	void parseHarvestToIndex(HarvestDirectoryWrapper wrapper)
			throws IOException {
		/*
		 * Report the status before we start.
		 */
		Date startTime = dateUtil.now();
		int numberOfRecords = wrapper.getNumberOfHarvestedRecords();
		LOG.info("Begin parsing " + numberOfRecords + " from "
				+ wrapper.getPath());

		/*
		 * Go through the files in the directory, indexing each one. If an XML
		 * error is found in one file, report it, but continue processing the
		 * remaining files.
		 */
		OaiRecordIndexer indexer = OaiRecordIndexer.getInstance();
		indexer.setListener(stats);
		OaiHarvestRecordsParser parser = new OaiHarvestRecordsParser(indexer);
		for (File file : wrapper.getFiles()) {
			parseHarvestFile(wrapper, parser, file);
		}

		// Finished with the directory. Write it down.
		wrapper.setIndexed();

		/*
		 * Report the status when we finish.
		 */
		NOTIFY_LOG.info("Parsed OAI metadata records: "
				+ stats.getTotalOaiRecords() + ", (regular: "
				+ stats.getTotalOaiStandardRecords() + ", deletes: "
				+ stats.getTotalOaiDeleteRecords() + "), "
				+ stats.getTotalNewUrls() + " new URLs");
		LOG.info("End parsing " + wrapper.getPath() + " - elapsed time "
				+ dateUtil.formatInterval(startTime, dateUtil.now()));
	}

	/**
	 * Parse a single file from the OAI harvest.
	 */
	void parseHarvestFile(HarvestDirectoryWrapper wrapper,
			OaiHarvestRecordsParser parser, File harvestFile)
			throws IOException {
		/*
		 * Report the status before we start.
		 */
		Date startTime = dateUtil.now();
		stats.setCheckpoint();
		LOG.info("Parsing '" + harvestFile.getPath() + "'");

		Reader harvestReader = wrapper.openHarvestFile(harvestFile);
		try {
			// Parse the file and record that it has been indexed.
			parser.parse(new InputSource(harvestReader));

			/*
			 * Report the status when we finish.
			 */
			LOG.debug("Ending  '" + harvestFile.getPath() + "', "
					+ stats.getIncrementalOaiRecords() + " OAI records ("
					+ stats.getIncrementalOaiStandardRecords() + " standard, "
					+ stats.getIncrementalOaiDeleteRecords() + " deletes), "
					+ stats.getIncrementalNewUrls()
					+ " new URLs, elapsed time "
					+ dateUtil.formatInterval(startTime, dateUtil.now()));
		} catch (XmlException e) {
			LOG
					.error("Problem parsing file '" + harvestFile.getPath()
							+ "'", e);
		} finally {
			try {
				harvestReader.close();
			} catch (IOException e) {
				LOG.error("failed to close Reader on OAI harvest file.", e);
			}
		}
	}

	/**
	 * Parse the command line arguments and process the harvest directory.
	 */
	public static void main(String[] args) throws IOException {
		/*
		 * Report the parameters before we start.
		 */
		OaiHarvestProcessorParameters parms = new OaiHarvestProcessorParameters(
				args);
		LOG.info("Processing OAI Harvest with these options: "
				+ parms.getArgString());
		LOG.info("    index directory is: " + parms.getLuceneIndexDirectory());
		LOG.info("    harvest directories are: "
				+ parms.getHarvestDirectoryNames());

		/*
		 * Initialize the Config from the config file, and apply parameters.
		 */
		PropertyFileConfig config = new PropertyFileConfig();
		config.setLuceneIndexDirectory(parms.getLuceneIndexDirectory()
				.getPath());
		config.setHarvestParentDirectory(parms.getHarvestParentDirectory()
				.getPath());
		LOG.info(config);

		/*
		 * Do the work.
		 */
		OaiHarvestStatistics stats = new OaiHarvestStatistics();
		IndexWrapper.getInstance().setListener(stats);

		OaiHarvestProcessor ohp = new OaiHarvestProcessor(parms, stats);
		ohp.parseAllHarvests();

		/*
		 * Make sure that the IndexWrapper cache has been flushed, and summarize
		 * the index activity.
		 */
		IndexWrapper.getInstance().synchronize();
		NOTIFY_LOG.info("Lucene index activity: "
				+ stats.getTotalIndexDeletes() + " deletes, "
				+ stats.getTotalIndexWrites() + " writes.");

	}

}
