/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.oai.process;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.nsdl.search.oai.harvest.HarvestDirectoryWrapper;
import org.nsdl.search.oai.harvest.HarvestWrapper;
import org.nsdl.searchtools.common.history.HistoryFile;
import org.nsdl.searchtools.common.parameters.AbstractParameterParser;

/**
 * Parse the command line parameters for the OaiHarvestProcessor, and make them
 * easily available.
 * 
 * How to determine what directory(s) to process:
 * <ul>
 * <li>if -harvestDir is specified</li>
 * <ul>
 * <li>if the directory has not been indexed yet, we have our answer</li>
 * <li>if it has already been indexed, throw an exception unless -force is
 * specified.</li>
 * </ul>
 * <li>if -harvestDir is not specified, find which harvests have not yet been
 * indexed</li>
 * <ul>
 * <li>if there are none, throw an exception</li>
 * <li>if -all was specified, index all of the harvests.</li>
 * <li>otherwise, index just the oldest un-indexed harvest.</li>
 * </ul>
 * </ul>
 * 
 * Note: -all may not be used with -harvestDir. -force may not be used without
 * -harvestDir.
 * 
 * @author jblake
 */
public class OaiHarvestProcessorParameters extends AbstractParameterParser {
	private static final String USAGE_MESSAGE = "Usage: OaiHarvestProcessor "
			+ "{-harvestParentDir <harvest_parent_dir_path>}"
			+ "{-luceneIndexDir <lucene_dir_path>}"
			+ "[-harvestDir <harvest_dir_name>] [-all] [-force]";

	/**
	 * This file, stored in the Lucene index directory, records what OAI
	 * harvests have been applied to that index.
	 */
	public static final String HARVEST_HISTORY_FILE = "harvestHistory";

	private static final String HARVEST_DIR = "-harvestDir";
	private static final String HARVEST_PARENT_DIR = "-harvestParentDir";
	private static final String LUCENE_INDEX_DIR = "-luceneIndexDir";
	private static final String FORCE = "-force";
	private static final String ALL = "-all";

	private static final String[] ALL_PARAMETERS = new String[] {
			HARVEST_PARENT_DIR, HARVEST_DIR, LUCENE_INDEX_DIR, FORCE, ALL };

	private final HarvestWrapper harvestWrapper;
	private final File luceneIndexDirectory;
	private final String argString;
	private final boolean force;
	private final boolean all;
	private final List<HarvestDirectoryWrapper> harvestDirectories;

	/**
	 * Store the command, and the state of the "force" option. Get the harvest
	 * directory to be indexed.
	 */
	public OaiHarvestProcessorParameters(String[] args) {
		this.argString = assembleArgString(args);

		buildParameterMap(args);
		rejectUnrecognizedParms(ALL_PARAMETERS);
		checkIllegalCombinations();

		File harvestParentDirectory = new File(
				getRequiredParameterValue(HARVEST_PARENT_DIR));
		confirmHarvestParentDirectory(harvestParentDirectory);

		this.luceneIndexDirectory = new File(
				getRequiredParameterValue(LUCENE_INDEX_DIR));
		confirmLuceneIndex(luceneIndexDirectory);

		HistoryFile historyFile = HistoryFile.getInstance(new File(
				luceneIndexDirectory, HARVEST_HISTORY_FILE));

		this.harvestWrapper = HarvestWrapper.getInstance(
				harvestParentDirectory, historyFile);

		this.force = checkEmptyParameter(FORCE);
		this.all = checkEmptyParameter(ALL);

		this.harvestDirectories = figureHarvestDirectories(getOptionalParameterValue(HARVEST_DIR));
	}

	/**
	 * Put all the arguments into a printable string, with each arg in quotes to
	 * remove ambiguity.
	 */
	private String assembleArgString(String[] args) {
		StringBuffer buffer = new StringBuffer('[');
		for (int i = 0; i < args.length; i++) {
			if (i == 0) {
				buffer.append('"');
			} else {
				buffer.append(", \"");
			}
			buffer.append(args[i]).append('"');
		}
		buffer.append(']');
		return buffer.toString();
	}

	/**
	 * If they specified incompatible options, try explaining it gently.
	 */
	private void checkIllegalCombinations() {
		if (getParameterMap().containsKey(HARVEST_DIR)) {
			if (getParameterMap().containsKey(ALL)) {
				throw problem(ALL + " cannot be used with a specific "
						+ HARVEST_DIR);
			}
		} else {
			if (getParameterMap().containsKey(FORCE)) {
				throw problem(FORCE + " cannot be used without a specific "
						+ HARVEST_DIR);
			}
		}
	}

	/**
	 * Check to see that the directory exists.
	 */
	private void confirmHarvestParentDirectory(File harvestParentDirectory) {
		if (!harvestParentDirectory.exists()) {
			throw problem("harvest parent directory '"
					+ harvestParentDirectory.getPath() + "' does not exist.");
		}
		if (!harvestParentDirectory.isDirectory()) {
			throw problem("harvest parent directory '"
					+ harvestParentDirectory.getPath()
					+ "' is not a directory.");
		}
	}

	/**
	 * Check to see that the directory exists, and that it contains a "segments"
	 * file.
	 */
	private void confirmLuceneIndex(File luceneIndexDirectory) {
		if (!luceneIndexDirectory.exists()) {
			throw problem("lucene index directory '"
					+ luceneIndexDirectory.getPath() + "' does not exist.");
		}
		if (!luceneIndexDirectory.isDirectory()) {
			throw problem("lucene index directory '"
					+ luceneIndexDirectory.getPath() + "' is not a directory.");
		}
		File segmentsFile = new File(luceneIndexDirectory, "segments");
		if (!segmentsFile.exists()) {
			throw problem("lucene index directory '"
					+ luceneIndexDirectory.getPath()
					+ "' does not contain a 'segments' file.");
		}
	}

	/**
	 * Get the harvest directory. If "-harvestDirectory" is specified, find it
	 * and check that it has not already been indexed (or that "-force" was
	 * requested). If not specified,ask the {@link HarvestWrapper} to find the
	 * oldest harvest directory that has not already been indexed, or all that
	 * have not been indexed.
	 */
	private List<HarvestDirectoryWrapper> figureHarvestDirectories(
			String dirName) {

		if (dirName == null) {
			// -harvestDir not specified
			if (this.all) {
				// -all
				return harvestWrapper.getUnindexedDirectories();
			} else {
				// not -all
				HarvestDirectoryWrapper dir = harvestWrapper
						.getOldestUnindexedDirectory();
				if (dir == null) {
					return Collections.emptyList();
				} else {
					return Collections.singletonList(dir);
				}
			}
		} else {
			// -harvestDir specified
			try {
				HarvestDirectoryWrapper dir = harvestWrapper
						.getDirectoryWrapper(dirName);
				if (!dir.isIndexed() || this.force) {
					// ok to index
					return Collections.singletonList(dir);
				} else {
					// can't index again
					throw problem("Harvest directory '" + dir.getPath()
							+ "' has already been indexed. Use -" + FORCE
							+ " option to re-index.");
				}
			} catch (FileNotFoundException e) {
				throw problem("Can't find harvest directory '" + dirName + "'.");
			}
		}
	}

	public String getArgString() {
		return argString;
	}

	public List<HarvestDirectoryWrapper> getHarvestDirectories() {
		return harvestDirectories;
	}

	public String getHarvestDirectoryNames() {
		StringBuilder result = new StringBuilder();
		for (Iterator<HarvestDirectoryWrapper> it = harvestDirectories
				.iterator(); it.hasNext();) {
			HarvestDirectoryWrapper wrapper = it.next();
			result.append(wrapper.getPath());
			if (it.hasNext()) {
				result.append(", ");
			}
		}
		return result.toString();
	}

	public File getLuceneIndexDirectory() {
		return luceneIndexDirectory;
	}

	public File getHarvestParentDirectory() {
		return this.harvestWrapper.getHarvestParentDirectory();
	}

	/** Tell them how it should have been done. */
	@Override
	protected void usage() {
		System.out.println(USAGE_MESSAGE);
	}

}
