/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.oai.process;

import org.apache.log4j.Logger;
import org.nsdl.search.lucene.wrapper.IndexWrapperListener;
import org.nsdl.search.processoai.OaiHarvestRecord;

/**
 * Record the statistics of a OAI Harvest Parsing session, and report when
 * requested.
 * 
 * A convenience feature here is to "checkpoint" the stats at the beginning of
 * an interval, and then request "incremental" stats since that checkpoint.
 * 
 * @author jblake
 */
public class OaiHarvestStatistics
		implements
			OaiIndexingListener,
			IndexWrapperListener {
	private static final Logger LOG = Logger
			.getLogger(OaiHarvestStatistics.class);

	private int totalNewUrls;
	private int totalOaiDeleteRecords;
	private int totalOaiStandardRecords;
	private int checkpointNewUrls;
	private int checkpointOaiDeleteRecords;
	private int checkpointOaiStandardRecords;
	private int totalIndexWrites;
	private int totalIndexDeletes;

	/**
	 * When the OaiRecordIndexer reports a new record, add to the tallys.
	 */
	public void tallyOaiRecord(OaiHarvestRecord oaiContents) {
		if (oaiContents.isDeletedRecord()) {
			totalOaiDeleteRecords++;
		} else {
			totalOaiStandardRecords++;
		}
	}

	/**
	 * When the Lucene IndexWrapper reports an event, record the tallys.
	 */
	public void recordEvent(EventType type, int pendingDeletes,
			int pendingWrites) {
		LOG.debug("Writing to Lucene index:  reason= " + type + ", "
				+ pendingDeletes + " deletes, " + pendingWrites + " writes.");
		totalIndexDeletes += pendingDeletes;
		totalIndexWrites += pendingWrites;
	}

	public int getTotalNewUrls() {
		return totalNewUrls;
	}

	public int getTotalOaiDeleteRecords() {
		return totalOaiDeleteRecords;
	}

	public int getTotalOaiStandardRecords() {
		return totalOaiStandardRecords;
	}

	public int getTotalOaiRecords() {
		return totalOaiDeleteRecords + totalOaiStandardRecords;
	}

	public void setCheckpoint() {
		checkpointNewUrls = totalNewUrls;
		checkpointOaiDeleteRecords = totalOaiDeleteRecords;
		checkpointOaiStandardRecords = totalOaiStandardRecords;
	}

	public int getIncrementalNewUrls() {
		return totalNewUrls - checkpointNewUrls;
	}

	public int getIncrementalOaiRecords() {
		return getIncrementalOaiStandardRecords()
				+ getIncrementalOaiDeleteRecords();
	}

	public int getIncrementalOaiStandardRecords() {
		return totalOaiStandardRecords - checkpointOaiStandardRecords;
	}

	public int getIncrementalOaiDeleteRecords() {
		return totalOaiDeleteRecords - checkpointOaiDeleteRecords;
	}

	public int getTotalIndexDeletes() {
		return totalIndexDeletes;
	}

	public int getTotalIndexWrites() {
		return totalIndexWrites;
	}

}
