/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.oai.process;

import static org.nsdl.search.lucene.SearchIndexFieldSpecs.METADATA_SOURCE;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.search.lucene.searchdoc.SearchDocumentMetadata;
import org.nsdl.search.lucene.wrapper.IndexWrapper;
import org.nsdl.search.lucene.wrapper.IndexWrapperQuery;
import org.nsdl.search.lucene.wrapper.SearchResult;
import org.nsdl.search.lucene.wrapper.SearchResults;
import org.nsdl.search.processoai.OaiHarvestRecord;
import org.nsdl.search.processoai.OaiRecordIndexer;

/**
 * TODO: remove the old interface and adapter method.
 * 
 * Takes the contents of an OAI record from the parser and applies it to the
 * lucene index.
 * 
 * @author jblake
 */
public class OaiRecordIndexerImpl extends OaiRecordIndexer {
	private static final Logger LOG = Logger
			.getLogger(OaiRecordIndexerImpl.class);

	public static OaiRecordIndexerImpl getInstance() {
		return new OaiRecordIndexerImpl();
	}

	private final IndexWrapper index = IndexWrapper.getInstance();

	/**
	 * Records the parsing of OAI records and addition of new URLs. This can't
	 * be set to null, but it might not do anything.
	 */
	private OaiIndexingListener listener = OaiIndexingListener.NULL_LISTENER;

	/**
	 * Keep it private to enforce the factory pattern.
	 */
	private OaiRecordIndexerImpl() {
		// nothing to instantiate.
	}

	public void setListener(OaiIndexingListener listener) {
		this.listener = (listener == null) ? OaiIndexingListener.NULL_LISTENER
				: listener;
	}

	/**
	 * Apply the contents of the OAI record to the lucene index. Remove it from
	 * any lucene document that used to reference it, and add it to any lucene
	 * document that it now applies to (creating documents as needed).
	 */
	public void indexRecord(OaiHarvestRecord oaiContents) throws IOException {
		LOG.debug(oaiContents);

		// DEBUG: dump resourceURLsNorm2Raw if it looks interesting
		if (LOG.isDebugEnabled()) {
			Map<String, List<String>> urlMap = oaiContents.getUrlMap();
			// if the normalized URL is different from the raw one, dump it.
			for (String normalized : urlMap.keySet()) {
				for (String raw : urlMap.get(normalized)) {
					if (!normalized.equals(raw)) {
						LOG.debug("normalized: '" + normalized + "', raw: '"
								+ raw + "'");
					}
				}
			}
			// if the metadata record applies to 2 or more URLs, dump it.
			if (urlMap.size() > 1) {
				LOG.debug("multiple URLs: " + urlMap.keySet());
			}
		}

		listener.tallyOaiRecord(oaiContents);

		/**
		 * Update any documents that used to use this OAI. It may no longer
		 * reference the same documents that it did before.
		 */
		String oaiId = oaiContents.getOaiRecId();
		SearchResults oldDocResults = getDocumentsThatUseThisOai(oaiId);
		for (SearchResult oldDocResult : oldDocResults) {
			SearchDocument oldDoc = oldDocResult.getDocument();
			updateDocumentWithOaiContents(oldDoc, oaiContents);
			writeOrDeleteDocument(oldDoc);
		}

		/**
		 * Update any documents that now use this OAI. These may overlap with
		 * the previous set, but that's OK.
		 */
		for (String normResourceURL : oaiContents.getUrlMap().keySet()) {
			SearchDocument doc = getOrCreateDocument(normResourceURL);
			updateDocumentWithOaiContents(doc, oaiContents);
			writeOrDeleteDocument(doc);
		}
	}

	/**
	 * Retrieve all documents that have used this OAI as a metadata source.
	 */
	private SearchResults getDocumentsThatUseThisOai(String oaiId)
			throws IOException {
		IndexWrapperQuery query = index.createPrefixQuery(METADATA_SOURCE
				.getName(), oaiId + SearchDocument.SEPARATOR);
		return index.search(query, 0, 1000);
	}

	/**
	 * If the index contains a document with this identifier, get it. Otherwise,
	 * create one from scratch.
	 */
	private SearchDocument getOrCreateDocument(String identifier)
			throws IOException {
		SearchResults results = index.getDocument(identifier);
		if (results.getReturnCount() == 0) {
			return new SearchDocument(identifier);
		} else {
			return results.getResult(0).getDocument();
		}
	}

	/**
	 * If the document still has metadata attached to it, update it. Otherwise,
	 * remove it from the index.
	 */
	private void writeOrDeleteDocument(SearchDocument doc) throws IOException {
		LOG.debug("doc has " + doc.getMetadataSources().size()
				+ " metadata sources: " + doc.getMetadataSourceIds() + "; url="
				+ doc.getIdentifier());
		if (doc.hasMetadata()) {
			index.writeDocument(doc);
		} else {
			index.deleteDocument(doc);
		}
	}

	/**
	 * Apply this OAI to this document, as much as we can. Remove any previous
	 * version of this OAI record, and if the current version of the OAI applies
	 * to this document, add it in.
	 */
	private void updateDocumentWithOaiContents(SearchDocument doc,
			OaiHarvestRecord oaiContents) {
		doc.removeMetadataSource(oaiContents.getOaiRecId());

		String rawUrl = oaiContents.getRawUrlMapping(doc.getIdentifier());
		if (rawUrl != null) {
			doc.addMetadataSource(new SearchDocumentMetadata(rawUrl,
					oaiContents));
		}
	}

}
