/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.processoai;

import java.util.HashMap;
import java.util.Map;

import org.nsdl.search.config.ConfigException;

/**
 * Describes an item of metadata in the OAI feed. This includes the qualified
 * name and a qualified encoding type (if applicable)
 * 
 * @author jblake
 */
public class MetadataElementSpec {
	/**
	 * All existing instances, keyed by "name/encoding".
	 */
	private static final Map<String, MetadataElementSpec> specs = new HashMap<String, MetadataElementSpec>();

	/**
	 * Ask for a spec that matches these qualities. Get it from the map if it's
	 * already there, or create one and store it in the map.
	 */
	public static MetadataElementSpec getInstance(String name,
			String namespace, String encoding, String encodingNamespace) {
		return resolveInMap(new MetadataElementSpec(name, namespace, encoding,
				encodingNamespace));
	}

	/**
	 * Check the map of recorded instances to see whether a matching spec is
	 * already mapped. If so, return a reference to that spec. If a spec is
	 * mapped with the same name but different attributes, throw a
	 * ConfigException. If no such spec is found, map this one and return a
	 * reference to it.
	 * 
	 * @param newSpec
	 *            the spec that we are testing for inclusion in the map of
	 *            instances.
	 * @return a reference to the canonical spec by this name.
	 */
	private static MetadataElementSpec resolveInMap(MetadataElementSpec newSpec) {
		MetadataElementSpec result;

		String key = buildKey(newSpec.getName(), newSpec.getEncoding());
		MetadataElementSpec existingSpec = specs.get(key);
		if (existingSpec == null) {
			specs.put(key, newSpec);
			result = newSpec;
		} else {
			if (existingSpec.equals(newSpec)) {
				result = existingSpec;
			} else {
				throw new ConfigException(
						"Attempt to create similar (but not duplicate) "
								+ "MetadataElementSpec: new=" + newSpec
								+ ", existing=" + existingSpec);
			}
		}

		return result;
	}

	/**
	 * Get the specification that corresponds to the given name, if there is
	 * one.
	 * 
	 * @return either the desired spec, or null.
	 */
	public static MetadataElementSpec getSpec(String name, String encoding) {
		return specs.get(buildKey(name, encoding));
	}

	/**
	 * How the specs are keyed in the map. Note that this ensures valid keys
	 * even if name and/or encoding are null.
	 */
	private static String buildKey(String name, String encoding) {
		return String.valueOf(name) + "/" + String.valueOf(encoding);
	}

	// ----------------------------- INSTANCE --------------------------------

	private final String name;
	private final String namespace;
	private final String encoding;
	private final String encodingNamespace;

	private final int hash;
	private final String string;

	private MetadataElementSpec(final String name, final String namespace,
			final String encoding, final String encodingNamespace) {
		if (name == null) {
			throw new IllegalArgumentException("name may not be null.");
		}
		this.name = name;
		this.namespace = namespace;
		this.encoding = encoding;
		this.encodingNamespace = encodingNamespace;
		this.hash = figureHashValue();
		this.string = figureStringRepresentation();
	}

	/**
	 * @return
	 */
	private int figureHashValue() {
		return name.hashCode()
				^ ((namespace == null) ? 0 : namespace.hashCode())
				^ ((encoding == null) ? 0 : encoding.hashCode())
				^ ((encodingNamespace == null) ? 0 : encodingNamespace
						.hashCode());
	}

	/**
	 * @return
	 */
	private String figureStringRepresentation() {
		return "MetadataElementSpec[name='"
				+ name
				+ "', namespace="
				+ ((namespace == null) ? "null" : ("'" + namespace + "'"))
				+ ", encoding="
				+ ((encoding == null) ? "null" : ("'" + encoding + "'"))
				+ ", encodingNamespace="
				+ ((encodingNamespace == null) ? "null" : ("'"
						+ encodingNamespace + "'")) + "]";
	}

	public String getEncoding() {
		return encoding;
	}

	public String getEncodingNamespace() {
		return encodingNamespace;
	}

	public String getName() {
		return name;
	}

	public String getNamespace() {
		return namespace;
	}

	/**
	 * Thest that all fields are equivalent in order to determine equality.
	 */
	@Override
	public boolean equals(Object o) {
		if (o == null) {
			return false;
		}
		if (o == this) {
			return true;
		}
		if (!MetadataElementSpec.class.equals(o.getClass())) {
			return false;
		}

		MetadataElementSpec that = (MetadataElementSpec) o;
		return equiv(this.getName(), that.getName())
				&& equiv(this.getNamespace(), that.getNamespace())
				&& equiv(this.getEncoding(), that.getEncoding())
				&& equiv(this.getEncodingNamespace(), that
						.getEncodingNamespace());
	}

	/**
	 * Check that both Objects are null, or that they are equal.
	 */
	private boolean equiv(Object first, Object second) {
		return (first == null) ? (second == null) : first.equals(second);
	}

	@Override
	public int hashCode() {
		return hash;
	}

	@Override
	public String toString() {
		return string;
	}

}
