/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.processoai;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.nsdl.searchtools.common.DateUtil;

/**
 * Hold the data that comes from a parsed OAI record.
 * 
 * @author jblake
 */
public class OaiHarvestRecord {
	private static final String DATE_FORMAT = DateUtil.ISO8601_DATE_TIME_Z;
	private final DateUtil dateUtil = DateUtil.getInstance();

	private Date oaiRecDate;

	private String oaiRecId;

	private boolean deletedRecord;

	/**
	 * The URLs from the identifier fields in the OAI, keyed by their normalized
	 * values. Note that two different URLs might normalize to the same result.
	 */
	private final Map<String, List<String>> urlMap;

	/**
	 * The field values from the OAI record. Each {@link MetadataElementSpec} key
	 * may have one or more values associated with it.
	 */
	private final Map<MetadataElementSpec, List<String>> fieldValuesMap;

	private OaiCollectionInfo collectionInfo;

	public OaiHarvestRecord() {
		this.urlMap = new HashMap<String, List<String>>();
		this.fieldValuesMap = new HashMap<MetadataElementSpec, List<String>>();
	}

	public boolean isDeletedRecord() {
		return deletedRecord;
	}

	public void setDeletedRecord(boolean deletedRecord) {
		this.deletedRecord = deletedRecord;
	}

	public String getOaiRecId() {
		return oaiRecId;
	}

	public void setOaiRecId(String oaiRecId) {
		this.oaiRecId = oaiRecId;
	}

	public void setOaiRecDatestamp(String oaiRecDatestamp)
			throws ParseException {
		this.oaiRecDate = dateUtil.parseDate(oaiRecDatestamp, DATE_FORMAT);
	}

	public String getOaiRecDatestamp() {
		return dateUtil.formatDate(this.oaiRecDate, DATE_FORMAT);
	}

	public Date getOaiRecDate() {
		return this.oaiRecDate;
	}

	/**
	 * Get the Collection Info, an immutable object.
	 */
	public OaiCollectionInfo getCollectionInfo() {
		return collectionInfo;
	}

	/**
	 * Store the collection info obtained during parsing.
	 */
	public void setCollectionInfo(OaiCollectionInfo collectionInfo) {
		this.collectionInfo = collectionInfo;
	}

	/**
	 * Associate a value from the OAI with the MetadataElement that contained
	 * it.
	 */
	public void addFieldValue(MetadataElementSpec mdEl, String value) {
		if (!fieldValuesMap.containsKey(mdEl)) {
			fieldValuesMap.put(mdEl, new ArrayList<String>());
		}
		fieldValuesMap.get(mdEl).add(value);
	}

	/**
	 * Get an unmodifiable copy of the field values map. Used only for unit
	 * tests.
	 * 
	 * @deprecated
	 */
	public Map<MetadataElementSpec, List<String>> getFieldValuesMap() {
		return Collections.unmodifiableMap(fieldValuesMap);
	}

	/**
	 * Get an unmodifiable copy of the types of field values.
	 */
	public Set<MetadataElementSpec> getFieldKeys() {
		return Collections.unmodifiableSet(fieldValuesMap.keySet());
	}

	/**
	 * Get all values that have been assigned to this MetadataElementSpec. If none,
	 * returns an empty list.
	 */
	public List<String> getFieldValues(MetadataElementSpec mdEl) {
		if (fieldValuesMap.containsKey(mdEl)) {
			return Collections.unmodifiableList(fieldValuesMap.get(mdEl));
		} else {
			return Collections.emptyList();
		}
	}

	/** Convenience method. Are there any fields in the field map? */
	public boolean hasMetadataFields() {
		return !fieldValuesMap.isEmpty();
	}

	/**
	 * Add a document identifier to the map. Each identifier is a normalized
	 * URL, and is mapped to the raw URL from which it was produced.
	 * 
	 * Neither argument may be null, and both will be trimmed of white space
	 * before being added to the map.
	 */
	public void addUrlMapping(String normalized, String raw) {
		if (normalized == null) {
			throw new IllegalArgumentException(
					"normalized URL may not be null. Raw is '" + raw + "'");
		}

		if (raw == null) {
			throw new IllegalArgumentException(
					"raw URL may not be null. Normalized is '" + normalized
							+ "'");
		}

		String normal = normalized.trim();

		if (!urlMap.containsKey(normal)) {
			urlMap.put(normal, new ArrayList<String>());
		}
		urlMap.get(normal).add(raw.trim());
	}

	/**
	 * Get an unmodifiable copy of the URL map. This lists the documents that
	 * will be affected by this OAI.
	 * 
	 * Each OAI record should always affect least one document, so if there are
	 * no URLs in the map, use the OAI record ID as the document identifier.
	 */
	public Map<String, List<String>> getUrlMap() {
		if (!urlMap.isEmpty()) {
			return Collections.unmodifiableMap(urlMap);
		} else if (hasMetadataFields()) {
			Map<String, List<String>> result = new HashMap<String, List<String>>();
			result.put(getOaiRecId(), Collections.singletonList(getOaiRecId()));
			return result;
		} else {
			return Collections.emptyMap();
		}
	}

	/**
	 * Look in the URL map and get the raw URL that corresponds to this
	 * normalized URL. If there is none, return null.
	 * 
	 * This returns the first such raw URL, skirting the possibility that there
	 * might be more than one.
	 */
	public String getRawUrlMapping(String normalizedUrl) {
		List<String> rawUrls = urlMap.get(normalizedUrl);
		if ((rawUrls != null) && (!rawUrls.isEmpty())) {
			return rawUrls.get(0);
		} else {
			return null;
		}
	}

	@Override
	public String toString() {
		return "OaiHarvestRecord[oaiId=" + oaiRecId + ", date=" + oaiRecDate
				+ ", isDeleted=" + deletedRecord + ", urlMap=" + urlMap
				+ ", fieldValues=" + fieldValuesMap + ", collectionInfo="
				+ collectionInfo + "]";
	}

}
