/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.processoai;

import static org.nsdl.search.util.XmlNamespaceUtil.NS_DC_TERMS;
import static org.nsdl.search.util.XmlNamespaceUtil.NS_DUBLIN_CORE;
import static org.nsdl.search.util.XmlNamespaceUtil.NS_NSDL_ABOUT;
import static org.nsdl.search.util.XmlNamespaceUtil.NS_NSDL_DC;
import static org.nsdl.search.util.XmlNamespaceUtil.NS_NSDL_SEARCH;
import static org.nsdl.search.util.XmlNamespaceUtil.NS_OAI;
import static org.nsdl.search.util.XmlNamespaceUtil.NS_W3C_XML_SCHEMA_INSTANCE;

import java.io.IOException;
import java.text.ParseException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.nsdl.search.util.XmlNamespaceUtil;
import org.nsdl.searchtools.common.net.UrlUtil;
import org.nsdl.searchtools.common.xml.ElementLister;
import org.nsdl.searchtools.common.xml.NodeLister;
import org.nsdl.searchtools.common.xml.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * 
 * @author jblake
 */
public class OaiHarvestRecordsParser implements OaiRecordsParser {
	private static final Logger LOG = Logger
			.getLogger(OaiHarvestRecordsParser.class);

	private final OaiRecordIndexer indexer;
	private final DocumentBuilder documentBuilder;

	/**
	 * @param indexer
	 * @param config
	 */
	public OaiHarvestRecordsParser(OaiRecordIndexer indexer) {
		this.indexer = indexer;

		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			factory.setNamespaceAware(true);
			documentBuilder = factory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			throw new FatalXmlException("Can't create the DocumentBuilder", e);
		}
	}

	/**
	 * <p>
	 * Parse the input into a series of records. Inform the parse controller of
	 * each record we find. The input contains the response to an OAI
	 * ListRecords query.
	 * </p>
	 * <p>
	 * If the file is invalid, throw an exception. If a single record within the
	 * file is invalid, just log the problem and continue.
	 * </p>
	 */
	public void parse(InputSource inputSource) throws XmlException {
		try {
			Document doc = documentBuilder.parse(inputSource);

			Element topElement = doc.getDocumentElement();
			if ((!"OAI-PMH".equals(topElement.getNodeName()))
					|| (!NS_OAI.equals(topElement.getNamespaceURI()))) {
				throw new XmlException("File is not an OAI-PMH response file.");
			}

			// If there were no matching records, we'll see this sort of line:
			// <error code="noRecordsMatch"> ... </error>
			// In that case, we're done here.
			Element errorElement = XmlUtil.getChildByNameNS(topElement, NS_OAI,
					"error");
			String errorCode = XmlUtil.getAttributeValue(errorElement, "code");
			if ("noRecordsMatch".equals(errorCode)) {
				return;
			}

			Element recordsElement = XmlUtil.getChildByNameNS(topElement,
					NS_OAI, "ListRecords");
			if (recordsElement == null) {
				throw new XmlException("File contains no ListRecords tag.");
			}

			for (Element recordElement : new ElementLister(recordsElement
					.getElementsByTagNameNS(NS_OAI, "record"))) {
				try {
					OaiHarvestRecord record = parseRecord(recordElement);
					this.indexer.indexRecord(record);
				} catch (XmlException e) {
					LOG.error("XML Error in OAI record: ", e);
				} catch (ParseException e) {
					LOG.error("Date parsing exception in OAI record: ", e);
				}
			}
		} catch (IOException e) {
			throw new XmlException(e);
		} catch (SAXException e) {
			throw new XmlException(e);
		}
	}

	/**
	 * Put the record together from the fields in the OAI response.
	 */
	private OaiHarvestRecord parseRecord(Element recordElement)
			throws XmlException, ParseException {
		Element headerElement = XmlUtil.getChildByNameNS(recordElement, NS_OAI,
				"header");
		if (headerElement == null) {
			throw new XmlException(
					"Bad format in oai record: no oai:header tag.");
		}

		String recordId = getChildElementValue(headerElement, NS_OAI,
				"identifier");
		if (recordId == null) {
			throw new XmlException(
					"Bad format in oai record: no oai:identifier tag.");
		}

		try {
			OaiHarvestRecord recordData = new OaiHarvestRecord();

			recordData.setOaiRecId(recordId);
			recordData.setOaiRecDatestamp(getChildElementValue(headerElement,
					NS_OAI, "datestamp"));

			if ("deleted".equals(headerElement.getAttribute("status"))) {
				recordData.setDeletedRecord(true);
			} else {
				parsePublicMetadata(recordData, recordElement);
				recordData
						.setCollectionInfo(parseCollectionInfo(recordElement));
			}

			return recordData;
		} catch (XmlException e) {
			throw new XmlException("Bad format in oai record for " + recordId,
					e);
		}
	}

	/**
	 * Parse the public metadata. We will only catalog the fieds that are in
	 * nsdl_dc, and are not in a "native_..." group.
	 */
	private void parsePublicMetadata(OaiHarvestRecord recordData,
			Element recordElement) throws XmlException {
		Element metadataElement = XmlUtil.getChildByNameNS(recordElement,
				NS_OAI, "metadata");
		if (metadataElement == null) {
			throw new XmlException("Record contains no oai:metadata tag.");
		}

		Element searchElement = XmlUtil.getChildByNameNS(metadataElement,
				NS_NSDL_SEARCH, "nsdl_search");
		if (searchElement == null) {
			throw new XmlException("Record contains no search:nsdl_search tag.");
		}

		Element publicElement = XmlUtil.getChildByNameNS(searchElement,
				NS_NSDL_SEARCH, "public");
		if (publicElement == null) {
			throw new XmlException("Record contains no search:public tag.");
		}

		for (Element groupElement : new ElementLister(publicElement
				.getElementsByTagNameNS(NS_NSDL_DC, "nsdl_dc"))) {
			if (!groupElement.getPrefix().startsWith("native_")) {
				parsePublicMetadataGroup(recordData, groupElement);
			}
		}
	}

	/**
	 * Parse a group of public metadata fields. We're only interested in fields
	 * with a "dc" or "dct" namespace.
	 */
	private void parsePublicMetadataGroup(OaiHarvestRecord recordData,
			Node publicNode) {
		for (Node childNode : new NodeLister(publicNode.getChildNodes())) {
			if (childNode instanceof Element) {
				Element element = (Element) childNode;
				String namespaceURI = element.getNamespaceURI();
				if ((NS_DUBLIN_CORE.equals(namespaceURI))
						|| (NS_DC_TERMS.equals(namespaceURI))) {
					storeElementInHarvestRecord(element, recordData);
				}
			}
		}
	}

	/**
	 * Almost all metadata elements are stored simply in the harvest record, but
	 * identifiers with URI encoding are stored specially. Empty elements are
	 * ignored.
	 */
	private void storeElementInHarvestRecord(Element element,
			OaiHarvestRecord recordData) {
		String content = element.getTextContent();
		if ((content != null) && (content.length() > 0)) {
			if ("identifier".equals(element.getLocalName())
					&& NS_DUBLIN_CORE.equals(element.getNamespaceURI())
					&& "dct:URI".equals(element.getAttributeNS(
							NS_W3C_XML_SCHEMA_INSTANCE, "type"))) {
				String normalized = UrlUtil.getInstance().normalizeOrNot(
						content);
				recordData.addUrlMapping(normalized, content);
				LOG.debug("Stored URL, raw='" + content + "', normalized='"
						+ normalized + "'");
			} else {
				recordData
						.addFieldValue(buildMetadataElement(element), content);
			}
		}
	}

	/**
	 * Parse the fields for the OaiCollectionInfo.
	 * 
	 * @throws XmlException
	 *             if the proper fields aren't found.
	 */
	private OaiCollectionInfo parseCollectionInfo(Element recordElement)
			throws XmlException {
		Element aboutElement = XmlUtil.getChildByNameNS(recordElement, NS_OAI,
				"about");
		if (aboutElement == null) {
			throw new XmlException("Record contains no about tag.");
		}

		Element nsdlAboutElement = XmlUtil.getChildByNameNS(aboutElement,
				NS_NSDL_ABOUT, "nsdl_about");
		if (nsdlAboutElement == null) {
			throw new XmlException("Record contains no about:nsdl_about tag.");
		}

		String oaiId = null;
		for (Element linkElement : new ElementLister(nsdlAboutElement
				.getElementsByTagNameNS(NS_NSDL_ABOUT, "link"))) {
			String linkType = linkElement.getAttribute("linkType");
			if ("primaryCollection".equals(linkType)) {
				oaiId = XmlUtil.getElementValue(linkElement);
				break;
			}
		}

		Element brandElement = XmlUtil.getChildByNameNS(nsdlAboutElement,
				NS_NSDL_ABOUT, "brand");
		if (brandElement == null) {
			throw new XmlException("Record contains no about:brand tag.");
		}

		String imageUrl = getChildElementValue(brandElement, NS_NSDL_ABOUT,
				"iconURL");
		String imageHeight = getChildElementValue(brandElement, NS_NSDL_ABOUT,
				"height");
		String imageWidth = getChildElementValue(brandElement, NS_NSDL_ABOUT,
				"width");
		String imageAltText = getChildElementValue(brandElement, NS_NSDL_ABOUT,
				"title");

		return new OaiCollectionInfo(oaiId, imageUrl, imageWidth, imageHeight,
				imageAltText);
	}

	/**
	 * Look for a child element and get its value, or return null.
	 */
	private String getChildElementValue(Element parentElement,
			String childNamespace, String childName) {
		Element childElement = XmlUtil.getChildByNameNS(parentElement,
				childNamespace, childName);
		if (childElement == null) {
			return null;
		} else {
			return XmlUtil.getElementValue(childElement);
		}
	}

	/**
	 * Examine the node for the pieces we need to build a MetadataElement. The
	 * "xsi:type" attribute, which represents the encoding scheme, might include
	 * a namespace, so test for that.
	 */
	private MetadataElementSpec buildMetadataElement(Node node) {
		String localName = ((Element) node).getLocalName();
		String namespaceURI = ((Element) node).getNamespaceURI();
		String encodingScheme = null;
		String encodingSchemeUri = null;

		String type = ((Element) node).getAttributeNS(
				NS_W3C_XML_SCHEMA_INSTANCE, "type");
		if ((type != null) && (type.length() > 0)) {
			int colonHere = type.indexOf(':');
			if (colonHere == -1) {
				encodingScheme = type;
			} else {
				encodingScheme = type.substring(colonHere + 1);
				encodingSchemeUri = XmlNamespaceUtil.getNamespaceContext()
						.getNamespaceURI(type.substring(0, colonHere));
			}
		}

		return MetadataElementSpec.getInstance(localName, namespaceURI,
				encodingScheme, encodingSchemeUri);
	}
}
