/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.responses;

import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COMPOUND_AGENT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COMPOUND_AUDIENCE;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COMPOUND_DESCRIPTION;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COMPOUND_GENRE;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COMPOUND_SUBJECT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COMPOUND_TITLE;

import java.math.BigInteger;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;
import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.nsdl.search.config.MetadataMapping;
import org.nsdl.search.generated.rest.BrandType;
import org.nsdl.search.generated.rest.CollContextType;
import org.nsdl.search.generated.rest.FieldsType;
import org.nsdl.search.generated.rest.OAIIDWithURLType;
import org.nsdl.search.generated.rest.ObjectFactory;
import org.nsdl.search.generated.rest.SimpleLiteral;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.search.lucene.searchdoc.SearchDocumentMetadata;
import org.nsdl.search.lucene.wrapper.SearchResult;
import org.nsdl.search.processoai.MetadataElementSpec;
import org.nsdl.search.processoai.OaiCollectionInfo;
import org.nsdl.searchtools.common.ListsMap;

/**
 * Methods that are used by both {@link StandardQueryResponse} and
 * {@link GetDocumentResponse}.
 * 
 * @author jblake
 */
abstract class AbstractDocumentResponse extends SearchServiceResponse {
	private static final Logger LOG = Logger
			.getLogger(AbstractDocumentResponse.class);

	public AbstractDocumentResponse(HttpServletRequest request) {
		super(request);
	}

	/**
	 * Gather all of the field data from a {@link SearchDocument} for QUERY or
	 * GET_DOCUMENT.
	 */
	protected FieldsType processSearchDocumentFields(ObjectFactory factory,
			SearchResult result) {
		FieldsType fieldsHolder = factory.createFieldsType();

		SearchDocument doc = result.getDocument();
		for (String mdOaiId : doc.getMetadataSourceIds()) {
			SearchDocumentMetadata metadata = doc.getMetadata(mdOaiId);
			fieldsHolder.getMetadataSource().add(
					processMetadataSource(factory, mdOaiId, metadata));
			fieldsHolder.getCollContext().add(
					processCollectionContext(factory, metadata));
		}

		processFields(result, factory, fieldsHolder);
		return fieldsHolder;
	}

	/**
	 * Gather the header info from a metadata source.
	 */
	private OAIIDWithURLType processMetadataSource(ObjectFactory factory,
			String mdOaiId, SearchDocumentMetadata metadata) {
		OAIIDWithURLType mdSource = factory.createOAIIDWithURLType();
		mdSource.setUrl(metadata.getRawUrl());
		mdSource.setValue(mdOaiId);
		return mdSource;
	}

	/**
	 * Gather the collection context information for a metadata source.
	 */
	private CollContextType processCollectionContext(ObjectFactory factory,
			SearchDocumentMetadata metadata) {
		OaiCollectionInfo collInfo = metadata.getCollInfo();

		BrandType.Image image = factory.createBrandTypeImage();
		image.setHeight(new BigInteger(collInfo.getImageHeight()));
		image.setWidth(new BigInteger(collInfo.getImageWidth()));
		image.setValue(collInfo.getImageUrl());

		BrandType brand = factory.createBrandType();
		brand.setAltText(collInfo.getImageAltText());
		brand.setImage(image);

		CollContextType collContext = factory.createCollContextType();
		collContext.setOaiId(collInfo.getCollectionOaiId());
		collContext.setBrand(brand);
		return collContext;
	}

	/**
	 * Gather all of the fields from all of the metadata sources, and from the
	 * best passage string.
	 */
	private void processFields(SearchResult result, ObjectFactory factory,
			FieldsType fieldsHolder) {
		ListsMap<LuceneFieldSpec, String> allFields = new ListsMap<LuceneFieldSpec, String>();
		ListsMap<LuceneFieldSpec, String> compoundFields = new ListsMap<LuceneFieldSpec, String>();

		/*
		 * Collect all of the fields from all of the metadata into one table,
		 * generating any compound fields at the same time.
		 */
		SearchDocument doc = result.getDocument();
		for (String mdOaiId : doc.getMetadataSourceIds()) {
			SearchDocumentMetadata metadata = doc.getMetadata(mdOaiId);

			for (LuceneFieldSpec spec : metadata.getFields()) {
				List<String> values = metadata.getFieldValues(spec);

				if (spec.getName().startsWith("coll.")) {
					// If the field was inherited from the collection, we won't
					// serve it out. (It still may contribute to a compound
					// field).
				} else {
					allFields.put(spec, values);
				}

				LuceneFieldSpec compoundSpec = spec.getCompoundField();
				if (compoundSpec != null) {
					compoundFields.put(compoundSpec, values);
				}
			}
		}

		/*
		 * Gather all of the basic fields, sorted by field name.
		 */
		Set<LuceneFieldSpec> sortedFieldKeys = new TreeSet<LuceneFieldSpec>(
				allFields.keySet());
		for (LuceneFieldSpec field : sortedFieldKeys) {
			for (String value : allFields.get(field)) {
				MetadataElementSpec metadata = MetadataMapping.getInstance()
						.getMetadataSpec(field);
				if (metadata == null) {
					LOG.warn("Can't find a metadata mapping for Lucene field '"
							+ field.getName() + "'");
					continue;
				}
				SimpleLiteral literal = createLiteralWithProperEncoding(
						metadata, factory);
				literal.setValue(value);
				JAXBElement<SimpleLiteral> element = new JAXBElement<SimpleLiteral>(
						new QName(metadata.getNamespace(), metadata.getName()),
						SimpleLiteral.class, literal);
				fieldsHolder.getAny().add(element);
			}
		}

		/*
		 * Each of the compound fields must be marshalled separately, and only
		 * if it isn't empty.
		 */
		if (compoundFields.containsKey(COMPOUND_AGENT)) {
			fieldsHolder.setCompoundAgent(concatenateFieldValues(compoundFields
					.get(COMPOUND_AGENT)));
		}
		if (compoundFields.containsKey(COMPOUND_AUDIENCE)) {
			fieldsHolder
					.setCompoundAudience(concatenateFieldValues(compoundFields
							.get(COMPOUND_AUDIENCE)));
		}
		if (compoundFields.containsKey(COMPOUND_DESCRIPTION)) {
			fieldsHolder
					.setCompoundDescription(concatenateFieldValues(compoundFields
							.get(COMPOUND_DESCRIPTION)));
		}
		if (compoundFields.containsKey(COMPOUND_GENRE)) {
			fieldsHolder.setCompoundGenre(concatenateFieldValues(compoundFields
					.get(COMPOUND_GENRE)));
		}
		if (compoundFields.containsKey(COMPOUND_SUBJECT)) {
			fieldsHolder
					.setCompoundSubject(concatenateFieldValues(compoundFields
							.get(COMPOUND_SUBJECT)));
		}
		if (compoundFields.containsKey(COMPOUND_TITLE)) {
			fieldsHolder.setCompoundTitle(concatenateFieldValues(compoundFields
					.get(COMPOUND_TITLE)));
		}

		// Add the "best passage", if we have one.
		if (doc.hasContent()) {
			fieldsHolder.setBestPassage(result.getBestPassage());
		}
	}

	/**
	 * We need a simple literal that we can store a value in. Ah, but what kind?
	 */
	private SimpleLiteral createLiteralWithProperEncoding(
			MetadataElementSpec metadata, ObjectFactory factory) {
		String encoding = metadata.getEncoding();
		if (encoding == null) {
			return factory.createSimpleLiteral();
		}
		if ("LCSH".equals(encoding)) {
			return factory.createLCSH();
		}
		if ("MESH".equals(encoding)) {
			return factory.createMESH();
		}
		if ("DDC".equals(encoding)) {
			return factory.createDDC();
		}
		if ("LCC".equals(encoding)) {
			return factory.createLCC();
		}
		if ("UDC".equals(encoding)) {
			return factory.createUDC();
		}
		if ("Period".equals(encoding)) {
			return factory.createPeriod();
		}
		if ("W3CDTF".equals(encoding)) {
			return factory.createW3CDTF();
		}
		if ("DCMIType".equals(encoding)) {
			return factory.createDCMIType();
		}
		if ("IMT".equals(encoding)) {
			return factory.createIMT();
		}
		if ("URI".equals(encoding)) {
			return factory.createURI();
		}
		if ("ISO639-2".equals(encoding)) {
			return factory.createISO6392();
		}
		if ("RFC3066".equals(encoding)) {
			return factory.createRFC3066();
		}
		if ("Point".equals(encoding)) {
			return factory.createPoint();
		}
		if ("ISO3166".equals(encoding)) {
			return factory.createISO3166();
		}
		if ("Box".equals(encoding)) {
			return factory.createBox();
		}
		if ("TGN".equals(encoding)) {
			return factory.createTGN();
		}
		if ("GEM".equals(encoding)) {
			return factory.createGEM();
		}
		if ("NSDLEdLevel".equals(encoding)) {
			return factory.createNSDLEdLevel();
		}
		if ("NSDLType".equals(encoding)) {
			return factory.createNSDLType();
		}
		if ("NSDLAccess".equals(encoding)) {
			return factory.createNSDLAccess();
		}
		if ("NSDLAudience".equals(encoding)) {
			return factory.createNSDLAudience();
		}
		if ("ISBN".equals(encoding)) {
			return factory.createISBN();
		}
		LOG
				.warn("Unknown encoding '" + encoding + "' in metadata: "
						+ metadata);
		return factory.createSimpleLiteral();
	}

	/**
	 * Some fields appear only once, or once per metadata source. Join multiple
	 * values into one.
	 */
	private String concatenateFieldValues(List<String> values) {
		StringBuffer buffer = new StringBuffer();
		for (String value : values) {
			if (buffer.length() > 0) {
				buffer.append(SearchDocument.SEPARATOR);
			}
			buffer.append(value);
		}
		return buffer.toString();
	}

}
