/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.responses;

import java.util.Collection;

import javax.servlet.http.HttpServletRequest;

import org.nsdl.search.generated.rest.IndexFieldsType;
import org.nsdl.search.generated.rest.ObjectFactory;
import org.nsdl.search.generated.rest.IndexFieldsType.NonSearchableFields;
import org.nsdl.search.generated.rest.IndexFieldsType.ResultFieldsPopulatedInIndex;
import org.nsdl.search.generated.rest.IndexFieldsType.SearchableFields;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;
import org.nsdl.search.lucene.wrapper.IndexSummaryInfo;

/**
 * A response object for the <code>/indexFieldInfo</code> request.
 * 
 * @author jblake
 */
class FieldInfoResponse extends SearchServiceResponse {
	private static final String CC_PREFIX = "coll.";

	public FieldInfoResponse(HttpServletRequest req, IndexSummaryInfo indexInfo) {
		super(req);
		getDocRoot().setIndexFieldInfo(
				processIndexFields(getFactory(), indexInfo));

	}

	/**
	 * Process the field info for FIELD_INFO.
	 */
	private IndexFieldsType processIndexFields(ObjectFactory factory,
			IndexSummaryInfo indexInfo) {
		IndexFieldsType ift = factory.createIndexFieldsType();
		ift.setDefaultSearchField(SearchIndexFieldSpecs.ALL_FIELDS
				.getStemmedSpec().getName());
		ift.setStemmedFieldNameString(LuceneFieldSpec.STEMMING_SUFFIX);
		ift.setCollContextFieldNamePrefix(CC_PREFIX);
		ift.setSearchableFields(processSearchableFields(factory, indexInfo));
		ift.setNonSearchableFields(processNonSearchableFields(factory,
				indexInfo));
		ift.setResultFieldsPopulatedInIndex(processResultFields(factory,
				indexInfo));
		return ift;
	}

	/**
	 * Get the list of searchable fields for FIELD_INFO.
	 */
	private SearchableFields processSearchableFields(ObjectFactory factory,
			IndexSummaryInfo indexInfo) {
		SearchableFields sf = factory.createIndexFieldsTypeSearchableFields();
		SearchableFields.CollContext cc = factory
				.createIndexFieldsTypeSearchableFieldsCollContext();
		sf.setCollContext(cc);

		for (String name : indexInfo.getSearchableFieldNames()) {
			if (name.startsWith(CC_PREFIX)) {
				cc.getSearchableField().add(name);
			} else {
				sf.getSearchableField().add(name);
			}
		}

		return sf;
	}

	/**
	 * Get the list of non-searchable fields for FIELD_INFO.
	 */
	private NonSearchableFields processNonSearchableFields(
			ObjectFactory factory, IndexSummaryInfo indexInfo) {
		NonSearchableFields nsf = factory
				.createIndexFieldsTypeNonSearchableFields();
		NonSearchableFields.CollContext cc = factory
				.createIndexFieldsTypeNonSearchableFieldsCollContext();
		nsf.setCollContext(cc);

		Collection<String> searchableFields = indexInfo
				.getSearchableFieldNames();
		for (String name : indexInfo.getFieldNames()) {
			if (!searchableFields.contains(name)) {
				if (name.startsWith(CC_PREFIX)) {
					cc.getNonSearchableField().add(name);
				} else {
					nsf.getNonSearchableField().add(name);
				}
			}
		}

		return nsf;
	}

	/**
	 * Get the list of REST result fields for FIELD_INFO.
	 */
	private ResultFieldsPopulatedInIndex processResultFields(
			ObjectFactory factory, IndexSummaryInfo indexInfo) {

		ResultFieldsPopulatedInIndex rfpi = factory
				.createIndexFieldsTypeResultFieldsPopulatedInIndex();
		ResultFieldsPopulatedInIndex.CollContext cc = factory
				.createIndexFieldsTypeResultFieldsPopulatedInIndexCollContext();
		rfpi.setCollContext(cc);

		for (String name : indexInfo.getStoredFieldNames()) {
			if (name.startsWith(CC_PREFIX)) {
				cc.getResultField().add(name);
			} else {
				rfpi.getResultField().add(name);
			}
		}

		return rfpi;
	}

}
