/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.responses;

import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.PropertyException;

/**
 * A {@link ThreadLocal}-based implmenentation of {@link ResponseMarshaller}.
 * 
 * @author jblake
 */
public class ResponseMarshallerImpl extends ResponseMarshaller {
	/**
	 * Keep one instance per type for each thread.
	 */
	private static final ThreadLocal<Map<ResponseMarshallerType, ResponseMarshallerImpl>> instances = new ThreadLocal<Map<ResponseMarshallerType, ResponseMarshallerImpl>>() {
		@Override
		protected Map<ResponseMarshallerType, ResponseMarshallerImpl> initialValue() {
			return new HashMap<ResponseMarshallerType, ResponseMarshallerImpl>();
		}
	};

	/**
	 * Get the instance for this type and this thread.
	 */
	public static ResponseMarshallerImpl getInstance(ResponseMarshallerType type) {
		Map<ResponseMarshallerType, ResponseMarshallerImpl> map = instances
				.get();
		if (!map.containsKey(type)) {
			map.put(type, new ResponseMarshallerImpl(type));
		}
		return map.get(type);
	}

	private Marshaller marshaller;

	/**
	 * Create a {@link JAXBContext} for this context path, and initialize it
	 * properly. Package access to allow calling by unit tests.
	 */
	ResponseMarshallerImpl(ResponseMarshallerType type) {
		try {
			marshaller = JAXBContext.newInstance(type.getContextPath())
					.createMarshaller();
			type.initializeMarshaller(marshaller);
		} catch (PropertyException e) {
			throw new IllegalStateException(e);
		} catch (JAXBException e) {
			throw new IllegalStateException(e);
		}
	}

	/**
	 * Convert a top-level JAXB object to XML and write it out.
	 */
	public void marshal(JAXBElement<?> responseObject, OutputStream output)
			throws JAXBException {
		marshaller.marshal(responseObject, output);
	}

}
