/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.responses;

import java.io.IOException;
import java.text.ParseException;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.nsdl.search.config.MetadataMapping;
import org.nsdl.search.generated.rss2.TGuid;
import org.nsdl.search.generated.rss2.TImage;
import org.nsdl.search.generated.rss2.TRssChannel;
import org.nsdl.search.generated.rss2.TRssItem;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;
import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.search.lucene.searchdoc.SearchDocumentMetadata;
import org.nsdl.search.lucene.wrapper.IndexWrapper;
import org.nsdl.search.lucene.wrapper.SearchResult;
import org.nsdl.search.lucene.wrapper.SearchResults;
import org.nsdl.search.processoai.MetadataElementSpec;
import org.nsdl.search.util.XmlNamespaceUtil;
import org.nsdl.searchtools.common.DateUtil;
import org.nsdl.searchtools.common.http.RequestHelper;

/**
 * 
 * @author jblake
 */
public class RssQueryResponse extends RssResponse {
	private static final Logger LOG = Logger.getLogger(RssQueryResponse.class);

	private static final String EDITOR_EMAIL = "search@nsdl.org (NSDL Search Maintainer)";
	private static final String WEBMASTER_EMAIL = "search@nsdl.org (NSDL Search Maintainer)";
	private static final String GENERATOR = "National Science Digital Library (NSDL) Search Service version 2.1";
	private static final String DOCS_URL = "http://blogs.law.harvard.edu/tech/rss";
	private static final String IMAGE_URL = "http://crs.nsdl.org/brands/nsdl.nsdl.gif";

	private static final QName QNAME_DATE = new QName(
			XmlNamespaceUtil.NS_DUBLIN_CORE, "date", "dc");

	/**
	 * Create the fully-populated RSS response object, suitable for marshalling.
	 */
	public RssQueryResponse(HttpServletRequest req, SearchResults searchResults) {
		super(req);
		getDocRoot().setChannel(processSearchResults(req, searchResults));
	}

	/**
	 * Populate the RSS response with the search results.
	 */
	private TRssChannel processSearchResults(HttpServletRequest req,
			SearchResults searchResults) {
		TRssChannel channel = getFactory().createTRssChannel();
		processChannelTags(channel, req, searchResults);
		for (SearchResult result : searchResults) {
			channel.getItem().add(processItem(result));
		}

		return channel;
	}

	/**
	 * Populate all the tags that go into the channel header.
	 */
	private void processChannelTags(TRssChannel channel,
			HttpServletRequest req, SearchResults searchResults) {
		String title = figureChannelTitle(req);
		String description = figureChannelDescription(searchResults);
		String link = figureChannelLink(req);
		Date pubDate = DateUtil.getInstance().now();
		Date buildDate = figureLastBuildDate();

		List<Object> tld = channel.getTitleOrLinkOrDescription();
		tld.add(getFactory().createTRssChannelTitle(title));
		tld.add(getFactory().createTRssChannelDescription(description));
		tld.add(getFactory().createTRssChannelLink(link));
		tld.add(getFactory().createTRssChannelLanguage("en-us"));
		tld.add(getFactory().createTRssChannelManagingEditor(EDITOR_EMAIL));
		tld.add(getFactory().createTRssChannelWebMaster(WEBMASTER_EMAIL));
		tld.add(getFactory().createTRssChannelPubDate(pubDate));
		tld.add(getFactory().createTRssChannelLastBuildDate(buildDate));
		tld.add(getFactory().createTRssChannelGenerator(GENERATOR));
		tld.add(getFactory().createTRssChannelDocs(DOCS_URL));
		tld.add(getFactory().createTRssChannelImage(
				processChannelImage(title, link)));

		// channel.getAny().add(figureAtomLink(req));
	}

	/**
	 * The title tag ends with request parameters, minus the "output" parameter.
	 */
	private String figureChannelTitle(HttpServletRequest req) {
		return "NSDL Search for: " + removeOutputParm(req.getQueryString());
	}

	/**
	 * The description gives the search summary figures.
	 */
	private String figureChannelDescription(SearchResults searchResults) {
		int skip = searchResults.getSkipCount();
		int returned = searchResults.getReturnCount();
		int total = searchResults.getTotalCount();
		if (returned == 0) {
			return "Results from NSDL Search Service: 0 matching resources";
		} else {
			return String.format("Results from NSDL Search Service: "
					+ "%d - %d of %d matching resources", skip + 1, skip
					+ returned, total);
		}
	}

	/**
	 * The link tag consists of the current request, minus the "output"
	 * parameter.
	 */
	private String figureChannelLink(HttpServletRequest req) {
		return removeOutputParm(RequestHelper.assembleRequestUrl(req));
	}

	/**
	 * Remove the "output" parameter from a query string, or from a URL that
	 * contains a query string.
	 */
	private String removeOutputParm(String raw) {
		// Could be a second or subsequent parameter
		String noOutput = raw.replaceFirst("&output=[^&]*", "");
		if (noOutput.equals(raw)) {
			// Could be the first parameter after the base URL
			noOutput = raw.replaceFirst("\\?output=[^&]*&", "?");
		}
		if (noOutput.equals(raw)) {
			// Could be the first (but not the only) parameter in a raw query
			// string.
			noOutput = raw.replaceFirst("^output=[^&]*&", "");
		}
		return noOutput;
	}

	/**
	 * Find out when the index was last modified. On error, use the current
	 * date.
	 */
	private Date figureLastBuildDate() {
		try {
			return IndexWrapper.getInstance().getIndexInfo().getLastModified();
		} catch (IOException e) {
			return DateUtil.getInstance().now();
		}
	}

	/**
	 * Create and populate an image tag for the channel.
	 */
	private TImage processChannelImage(String title, String link) {
		TImage image = getFactory().createTImage();
		image.setUrl(IMAGE_URL);
		image.setTitle(title);
		image.setLink(link);
		return image;
	}

	/**
	 * Process a {@link SearchResult} to create and populate an item tag.
	 */
	private TRssItem processItem(SearchResult result) {
		DateUtil dateUtil = DateUtil.getInstance();
		SearchDocument doc = result.getDocument();
		LOG.debug("processing search result: " + doc);

		TRssItem item = getFactory().createTRssItem();

		List<Object> tld = item.getTitleOrDescriptionOrLink();
		tld.add(getFactory().createTRssItemLink(doc.getIdentifier()));
		tld.add(getFactory().createTRssItemTitle(
				getFirstValue(doc, SearchIndexFieldSpecs.DC_TITLE)));
		tld.add(getFactory().createTRssItemDescription(
				getFirstValue(doc, SearchIndexFieldSpecs.DC_DESCRIPTION)));
		tld.add(getFactory()
				.createTRssItemPubDate(doc.getDocumentLastUpdated()));
		tld.add(getFactory().createTRssItemGuid(createGuid(doc)));

		for (SearchDocumentMetadata metadata : doc.getMetadataSources()) {
			for (LuceneFieldSpec field : metadata.getFields()) {
				for (String value : metadata.getFieldValues(field)) {
					addFieldValue(tld, field, value, dateUtil);
				}
			}
		}

		return item;
	}

	private TGuid createGuid(SearchDocument doc) {
		TGuid tguid = getFactory().createTGuid();
		tguid.setValue(doc.getIdentifier());
		return tguid;
	}

	/**
	 * Go through thhe {@link SearchDocument} for the first field that matches
	 * the supplied spec.
	 */
	private String getFirstValue(SearchDocument doc, LuceneFieldSpec spec) {
		for (SearchDocumentMetadata metadata : doc.getMetadataSources()) {
			List<String> values = metadata.getFieldValues(spec);
			if (!values.isEmpty()) {
				return values.get(0);
			}
		}
		return null;
	}

	/**
	 * If this is a valid field for the RSS feed, with a valid value, wrap it in
	 * a {@link JAXBElement} and add it to the list.
	 */
	private void addFieldValue(List<Object> fields, LuceneFieldSpec spec,
			String value, DateUtil dateUtil) {
		if (spec.equals(SearchIndexFieldSpecs.DC_DATE_ISO8601)) {
			Date isoDate = parseIsoDate(dateUtil, value);
			if (isoDate != null) {
				fields.add(buildDateElement(isoDate));
			}
		} else if (spec.equals(SearchIndexFieldSpecs.DC_DATE_W3CDTF)) {
			Date w3cDate = parseW3cDate(dateUtil, value);
			if (w3cDate != null) {
				fields.add(buildDateElement(w3cDate));
			}
		} else if (isAcceptableField(spec)) {
			JAXBElement<String> fieldElement = buildFieldElement(spec, value);
			if (fieldElement != null) {
				fields.add(fieldElement);
			}
		}

	}
	private JAXBElement<Date> buildDateElement(Date date) {
		return new JAXBElement<Date>(QNAME_DATE, Date.class, date);
	}

	/**
	 * Try to parse an ISO8601 date.
	 * 
	 * @return the parsed date, or null if parsing failed.
	 */
	private Date parseIsoDate(DateUtil dateUtil, String value) {
		try {
			return dateUtil.parseDate(value, DateUtil.ISO8601_DATE_TIME_Z);
		} catch (ParseException e) {
			LOG.warn("Date was said to be ISO8601, " + "but failed to parse: '"
					+ value + "'");
			return null;
		}
	}

	/**
	 * Try to parse an W3CDTF date.
	 * 
	 * @return the parsed date, or null if parsing failed.
	 */
	private Date parseW3cDate(DateUtil dateUtil, String value) {
		String cleanValue = cleanW3cdtfTimeZone(value);
		String format;
		switch (cleanValue.length()) {
			case 4 :
				format = "yyyy";
				break;
			case 7 :
				format = "yyyy-MM";
				break;
			case 10 :
				format = "yyyy-MM-dd";
				break;
			case 21 : // long time zone (+0500)
				format = "yyyy-MM-dd'T'HH:mmz";
				break;
			case 24 : // long time zone (+0500)
				format = "yyyy-MM-dd'T'HH:mm:ssz";
				break;
			default :
				format = "yyyy-MM-dd'T'HH:mm:ss.SSSz";
				break;
		}

		try {
			return dateUtil.parseDate(cleanValue, format);
		} catch (ParseException e) {
			LOG.warn("Date was said to be W3CDTF, " + "but failed to parse: '"
					+ value + "' with format '" + format + "'");
			return null;
		}
	}

	/**
	 * W3CDTF dates can use time zones of "Z" or the form "+00:00", neither of
	 * which is recognized by SimpleDateFormat. So if we find either of those,
	 * translate to the form "+0000", so we can parse correctly.
	 */
	private String cleanW3cdtfTimeZone(String value) {
		String cleanValue = value;
		if (value.charAt(value.length() - 1) == 'Z') {
			cleanValue = value.substring(0, value.length() - 1) + "+0000";
			LOG.debug("Cleaned time zone: '" + value + "' to '" + cleanValue
					+ "'");
		} else if (value.charAt(value.length() - 3) == ':') {
			cleanValue = value.substring(0, value.length() - 3)
					+ value.substring(value.length() - 2);
			LOG.debug("Cleaned time zone: '" + value + "' to '" + cleanValue
					+ "'");
		}
		return cleanValue;
	}

	/**
	 * Is this field the sort that we would copy into RSS? Does it have a
	 * metadata mapping? Does it map to a DC or DCT field? Don't do dates here,
	 * we've already handled the only dates that were acceptable.
	 */
	private boolean isAcceptableField(LuceneFieldSpec spec) {
		MetadataElementSpec metadata = MetadataMapping.getInstance()
				.getMetadataSpec(spec);
		if (metadata == null) {
			return false;
		}
		String name = metadata.getName();
		String namespace = metadata.getNamespace();

		if ("date".equals(name)) {
			return false;
		}
		if (XmlNamespaceUtil.NS_DUBLIN_CORE.equals(namespace)) {
			return true;
		}
		if (XmlNamespaceUtil.NS_DC_TERMS.equals(namespace)) {
			return true;
		}
		return false;
	}

	/**
	 * Convert a non-date field into the appropriate {@link JAXBElement}.
	 */
	private JAXBElement<String> buildFieldElement(LuceneFieldSpec spec,
			String value) {
		MetadataElementSpec metadata = MetadataMapping.getInstance()
				.getMetadataSpec(spec);
		if (metadata == null) {
			return null;
		}

		String name = metadata.getName();
		String namespace = metadata.getNamespace();

		QName qname;
		if (XmlNamespaceUtil.NS_DC_TERMS.equals(namespace)) {
			qname = new QName(namespace, name, "dct");
		} else {
			qname = new QName(namespace, name, "dc");
		}

		return new JAXBElement<String>(qname, String.class, value);
	}
}
