/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.responses;

import java.io.OutputStream;

import javax.servlet.http.HttpServletRequest;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;

import org.nsdl.search.generated.rest.HeaderType;
import org.nsdl.search.generated.rest.NSDLSearchServiceType;
import org.nsdl.search.generated.rest.ObjectFactory;
import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.search.lucene.wrapper.IndexSummaryInfo;
import org.nsdl.search.lucene.wrapper.SearchResults;
import org.nsdl.searchtools.common.DateUtil;
import org.nsdl.searchtools.common.http.RequestHelper;

/**
 * <p>
 * Factory methods to produce an objectUtility methods that produce a
 * {@link JAXBElement} that contains an {@link NSDLSearchServiceType}, suitable
 * for marshalling into a response document.
 * </p>
 * <p>
 * The building of the {#link responseObject} is handled by an assortment of
 * private methods. These are based directly on the methods created by JAXB when
 * it generated the target classes.
 * </p>
 * 
 * @author jblake
 */
public class SearchServiceResponse {
	/**
	 * Set this as the value of the "schemaVersion" attribute in every response.
	 */
	private static final String SEARCH_REST_SCHEMA_VERSION = "2.01.000";

	// ----------------------------------------------------------------------
	// Factory methods for the various response types.
	// ----------------------------------------------------------------------

	public static SearchServiceResponse produceStandardSearchResponse(
			SearchResults searchResults, HttpServletRequest req) {
		return new StandardQueryResponse(req, searchResults);
	}

	public static SearchServiceResponse produceGetDocumentResponse(
			SearchResults searchResults, HttpServletRequest req) {
		return new GetDocumentResponse(req, searchResults);
	}

	public static SearchServiceResponse produceResourceTextResponse(
			SearchResults searchResults, HttpServletRequest req) {
		return new ResourceTextResponse(req, searchResults);
	}

	public static SearchServiceResponse produceFieldInfo(
			IndexSummaryInfo indexInfo, HttpServletRequest req) {
		return new FieldInfoResponse(req, indexInfo);
	}

	public static SearchServiceResponse produceIndexInfo(IndexSummaryInfo info,
			HttpServletRequest req) {
		return new IndexInfoResponse(info, req);
	}

	// ----------------------------------------------------------------------

	private final ObjectFactory factory;
	private final NSDLSearchServiceType docRoot;
	private final JAXBElement<NSDLSearchServiceType> responseObject;

	/**
	 * Protected. Use one of the factory methods to create an instance of a
	 * subclass.
	 */
	protected SearchServiceResponse(HttpServletRequest request) {
		factory = new ObjectFactory();
		docRoot = prepareDocumentRoot(
				RequestHelper.assembleRequestUrl(request), factory);
		this.responseObject = factory.createNSDLSearchService(docRoot);
	}

	/**
	 * Populate the root of the document with URL and time. The body of the
	 * response depends on the type, and is not done here.
	 */
	private NSDLSearchServiceType prepareDocumentRoot(String requestUrl,
			ObjectFactory factory) {
		NSDLSearchServiceType docRoot = factory.createNSDLSearchServiceType();
		docRoot.setRequest(requestUrl);

		DateUtil dateUtil = DateUtil.getInstance();
		docRoot.setResponseTime(dateUtil.now());

		docRoot.setSchemaVersion(SEARCH_REST_SCHEMA_VERSION);
		return docRoot;
	}

	/**
	 * Gather the header info from a {@link SearchDocument} for QUERY or
	 * GET_DOCUMENT or RESOURCE_TEXT.
	 */
	protected HeaderType processDocumentHeader(ObjectFactory factory,
			SearchDocument doc) {
		HeaderType headerHolder = factory.createHeaderType();
		headerHolder.setResourceIdentifier(doc.getIdentifier());
		headerHolder.setLastIndexed(doc.getDocumentLastUpdated());
		headerHolder.setMetadataLastModified(doc.getMetadataLastModified());
		if (doc.hasContent()) {
			headerHolder.setContentLastModified(doc.getContent()
					.getModifiedDate());
			headerHolder.setContentLastFetched(doc.getContent().getFetchDate());
		}
		return headerHolder;
	}

	/**
	 * Serialize this object to the output stream.
	 */
	public void marshall(OutputStream output) throws JAXBException {
		ResponseMarshaller.getInstance(ResponseMarshallerType.REST).marshal(
				this.responseObject, output);
	}

	protected ObjectFactory getFactory() {
		return factory;
	}

	protected NSDLSearchServiceType getDocRoot() {
		return docRoot;
	}

}
