/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.servlets;

import java.io.IOException;
import java.util.Arrays;
import java.util.Enumeration;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.nsdl.search.generated.rest.ErrorCodeType;
import org.nsdl.search.rest3.SearchServletException;

/**
 * Base class for the Search3 Servlets. Holds useful methods for fetching
 * parameters.
 * 
 * @author jblake
 */
public abstract class AbstractSearchServlet extends HttpServlet {
	private static final Logger LOG = Logger
			.getLogger(AbstractSearchServlet.class);
	private static final Logger QUERY_LOG = Logger.getLogger("QUERY_LOG");

	/**
	 * POST or GET, it's all the same to me.
	 */
	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		doGet(req, resp);
	}

	/**
	 * Make sure that the response is the correct type and encoding.
	 */
	@Override
	protected void service(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		resp.setContentType("text/xml;charset=UTF-8");

		if (QUERY_LOG.isInfoEnabled()) {
			QUERY_LOG.info(dumpRequestParameters(req));
		}

		super.service(req, resp);
	}

	/**
	 * Get the value of a string parameter from the servlet request. The
	 * parameter must be present.
	 */
	protected String getRequiredStringParameter(HttpServletRequest req,
			String key) throws SearchServletException {
		String value = req.getParameter(key);
		if (value == null) {
			throw new SearchServletException(req, ErrorCodeType.BAD_ARGUMENT,
					"The '" + key + "' parameter is required.");
		} else {
			return value.trim();
		}
	}

	/**
	 * Get the value of a string parameter from the servlet request. If the
	 * parameter is present, trim it. If not present, return null.
	 */
	protected String getOptionalStringParameter(HttpServletRequest req,
			String key, String defaultValue) {
		String value = req.getParameter(key);
		if (value == null) {
			return defaultValue;
		} else {
			return value.trim();
		}
	}

	/**
	 * Get the value of an integer parameter from the servlet request. The
	 * parameter must be present, must be a valid integer, and must have value
	 * between the lower and upper bounds (inclusive).
	 */
	protected int getRequiredIntegerParameter(HttpServletRequest req,
			String key, int lowerBound, int upperBound)
			throws SearchServletException {
		String value = getRequiredStringParameter(req, key);
		return parseIntegerParameter(value, lowerBound, upperBound, req, key);
	}

	protected int getOptionalIntegerParameter(HttpServletRequest req,
			String key, int lowerBound, int upperBound, int defaultValue)
			throws SearchServletException {
		String value = getOptionalStringParameter(req, key, String
				.valueOf(defaultValue));
		if ((value == null) || (value.length() == 0)) {
			return defaultValue;
		}

		return parseIntegerParameter(value, lowerBound, upperBound, req, key);
	}

	/**
	 * Parse the parameter string to an integer, with appropriate exceptions if
	 * it is not valid, or is out of range.
	 */
	private int parseIntegerParameter(String value, int lowerBound,
			int upperBound, HttpServletRequest req, String key)
			throws SearchServletException {
		int intValue;

		try {
			intValue = Integer.parseInt(value);
		} catch (NumberFormatException e) {
			throw new SearchServletException(req, ErrorCodeType.BAD_ARGUMENT,
					"The '" + key + "' parameter must be an integer.");
		}

		if (intValue < lowerBound) {
			throw new SearchServletException(req, ErrorCodeType.BAD_ARGUMENT,
					"The '" + key + "' parameter must be greater than or "
							+ "equal to " + lowerBound);
		}

		if (intValue > upperBound) {
			throw new SearchServletException(req, ErrorCodeType.BAD_ARGUMENT,
					"The '" + key + "' parameter must be less than or "
							+ "equal to " + upperBound);
		}

		return intValue;
	}

	/**
	 * Convert the request into a string for logging.
	 */
	@SuppressWarnings("unchecked")
	protected String dumpRequestParameters(HttpServletRequest req) {
		StringBuilder result = new StringBuilder("request=").append(req
				.getRequestURL());
		result.append(" parms=[");
		for (Enumeration<String> names = req.getParameterNames(); names
				.hasMoreElements();) {
			String name = names.nextElement();
			result.append(name + "=");
			result.append(Arrays.toString(req.getParameterValues(name)));
			if (names.hasMoreElements()) {
				result.append(", ");
			}
		}
		result.append("]");
		return result.toString();
	}

}
