/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.servlets;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import org.nsdl.search.content.ContentCacheConfig;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;
import org.nsdl.search.lucene.wrapper.IndexWrapper;
import org.nsdl.search.lucene.wrapper.IndexWrapperConfig;
import org.nsdl.search.lucene.wrapper.LoggingIndexWrapperListener;
import org.nsdl.search.util.Config;
import org.nsdl.search.util.SearchServiceConfig;

/**
 * <p>
 * Performs setup tasks for the web-app. Must be loaded before any other
 * servlets are executed. This is accomplished with a "load-on-startup"
 * parameter in the web.xml file.
 * </p>
 * <p>
 * <ul>
 * <li>Creates a configuration object that gets its values from the web.xml
 * init parameters. Hands this object to the {@link Config} class as the
 * solution to all configuration questions (since the object implements all
 * required config interfaces).</li>
 * <li>Sets a {@link LoggingIndexWrapperListener} on to the
 * {@link IndexWrapper}, so {@link IndexWrapper#synchronize() synchronize()}
 * events can be written to the log, if enabled in <code>log4j.properties</code>.</li>
 * </ul>
 * </p>
 * <p>
 * This servlet should not be called by any HTTP requests. Loading it into the
 * container is enough.
 * </p>
 * 
 * @author jblake
 */
public class ConfigServlet extends HttpServlet {

	@Override
	public void init() throws ServletException {
		ConfigObject configObject = new ConfigObject(getServletContext());

		Config.getInstance().setIndexWrapperConfig(configObject);
		Config.getInstance().setSearchServiceConfig(configObject);
		Config.getInstance().setContentCacheConfig(configObject);
		
		IndexWrapper.getInstance().setListener(new LoggingIndexWrapperListener());
	}

	/**
	 * The configuration object.
	 */
	private static class ConfigObject implements IndexWrapperConfig,
			SearchServiceConfig, ContentCacheConfig {
		private final ServletContext servletContext;

		ConfigObject(ServletContext servletContext) {
			this.servletContext = servletContext;
		}

		// --------------------------------------------------------------------
		// Implement IndexWrapperConfig
		// --------------------------------------------------------------------

		public String getLuceneIndexDirectory() {
			return getRequiredParameter("luceneIndexDirectory");
		}

		public String getDocIdFieldName() {
			return SearchIndexFieldSpecs.RESOURCE_IDENTIFIER.getName();
		}

		public long getInactivityThreshold() {
			return 5000;
		}

		public int getMaximumCachedWrites() {
			return 100;
		}

		// --------------------------------------------------------------------
		// Implement SearchServiceConfig
		// --------------------------------------------------------------------

		public String getServiceDescription() {
			return getRequiredParameter("serviceDescription");
		}

		public String getServiceName() {
			return getRequiredParameter("serviceName");
		}

		// --------------------------------------------------------------------
		// Implement SearchServiceConfig
		// --------------------------------------------------------------------

		public String getContentCacheBaseUrl() {
			return getRequiredParameter("contentCacheBaseUrl");
		}

		// --------------------------------------------------------------------
		// Helper methods
		// --------------------------------------------------------------------

		/**
		 * If the web app was not configured with this init parameter, barf.
		 */
		private String getRequiredParameter(String key) {
			String value = servletContext.getInitParameter(key);
			if (value == null) {
				throw new IllegalStateException("Servlet context does not "
						+ "contain an init parameter for '" + key + "'");
			}
			return value;
		}

	}
}
