/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.servlets;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import org.apache.log4j.Logger;
import org.nsdl.search.generated.rest.ErrorCodeType;
import org.nsdl.search.generated.rest.ErrorResponseType;
import org.nsdl.search.generated.rest.NSDLSearchServiceType;
import org.nsdl.search.generated.rest.ObjectFactory;
import org.nsdl.search.rest3.SearchServletException;
import org.nsdl.searchtools.common.DateUtil;

/**
 * If one of the other servlets throws an exception, create a schema-valid error
 * response.
 * 
 * If called in any way, whether by an exception or not, create a schema-valid
 * error response.
 * 
 * @author jblake
 */
public class ErrorServlet extends HttpServlet {
	private static final Logger LOG = Logger.getLogger(ErrorServlet.class);

	/**
	 * Marshall the exception into an NSDLSearchService XML document.
	 */
	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		DateUtil dateUtil = DateUtil.getInstance();

		resp.setContentType("text/xml;charset=UTF-8");
		PrintWriter out = resp.getWriter();

		SearchServletException sse = getException(req);

		try {
			ObjectFactory factory = new ObjectFactory();

			ErrorResponseType ert = factory.createErrorResponseType();
			ert.setCode(sse.getCode());
			ert.setValue(sse.getMessage());

			NSDLSearchServiceType docRoot = factory
					.createNSDLSearchServiceType();
			docRoot.setRequest(sse.getRequestUrl());
			docRoot.setResponseTime(dateUtil.now());
			docRoot.getError().add(ert);

			JAXBContext jaxbContext = JAXBContext
					.newInstance("org.nsdl.search.generated.rest");
			Marshaller marshaller = jaxbContext.createMarshaller();
			marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT,
					Boolean.TRUE);
			marshaller.marshal(factory.createNSDLSearchService(docRoot), out);
		} catch (JAXBException e) {
			LOG.error("problem in the ErrorServlet", e);
		} finally {
			out.close();
		}
	}

	/**
	 * We need a {@link SearchServletException} in order to create a
	 * schema-valid response. So if we don't have one, make one up.
	 */
	private SearchServletException getException(HttpServletRequest req) {
		Object error = req.getAttribute("javax.servlet.error.exception");

		if (error instanceof SearchServletException) {
			return (SearchServletException) error;
		}

		if (error instanceof Throwable) {
			Throwable throwable = (Throwable) error;
			return new SearchServletException(req,
					ErrorCodeType.SYSTEM_PROBLEM, throwable.getMessage(),
					throwable);
		}

		return new SearchServletException(req, ErrorCodeType.SYSTEM_PROBLEM,
				"Unknown problem: " + String.valueOf(error));
	}

}
