/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.servlets;

import java.io.IOException;
import java.util.Arrays;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.BooleanQuery;
import org.nsdl.search.generated.rest.ErrorCodeType;
import org.nsdl.search.lucene.wrapper.IndexWrapper;
import org.nsdl.search.lucene.wrapper.IndexWrapperQuery;
import org.nsdl.search.lucene.wrapper.SearchResults;
import org.nsdl.search.rest3.SearchServletException;
import org.nsdl.search.rest3.responses.RssResponse;
import org.nsdl.search.rest3.responses.SearchServiceResponse;

/**
 * <p>
 * A servlet to handle the "/search" request.
 * </p>
 * <p>
 * Check for valid parameters, put the query to the index wrapper, and marshall
 * the results.
 * </p>
 * 
 * @author jblake
 */
public class QueryServlet extends AbstractSearchServlet {
	private static final Logger LOG = Logger.getLogger(QueryServlet.class);

	private static final int MAXIMUM_RESULTS_COUNT = 1000;
	private static final int MAXIMUM_SKIP_COUNT = Integer.MAX_VALUE;
	private static final int DEFAULT_RESULTS_COUNT = 10;
	private static final int DEFAULT_SKIP_COUNT = 0;

	/** What sort of formatting do you want on the search results? */
	private enum OutputType {
		STANDARD, RSS
	}

	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		if (LOG.isDebugEnabled()) {
			LOG.debug(req.getMethod() + " " + formatRequestParameters(req));
		}

		String queryString = getRequiredStringParameter(req, "q");
		int skipCount = getOptionalIntegerParameter(req, "s", 0,
				MAXIMUM_SKIP_COUNT, DEFAULT_SKIP_COUNT);
		int maxReturnCount = getOptionalIntegerParameter(req, "n", 0,
				MAXIMUM_RESULTS_COUNT, DEFAULT_RESULTS_COUNT);

		OutputType outputType = validateOutputType(req);

		try {
			IndexWrapper index = IndexWrapper.getInstance();
			IndexWrapperQuery query = index.parseQuery(queryString);
			SearchResults results = index.search(query, skipCount,
					maxReturnCount);

			switch (outputType) {
			case RSS:
				RssResponse rssr = RssResponse.produceRssSearchResponse(
						results, req);
				rssr.marshall(resp.getOutputStream());
				break;
			default: // STANDARD
				SearchServiceResponse ssr = SearchServiceResponse
						.produceStandardSearchResponse(results, req);
				ssr.marshall(resp.getOutputStream());
				break;
			}
		} catch (ParseException e) {
			throw new SearchServletException(req, ErrorCodeType.BAD_QUERY,
					"The query string cannot be parsed: '" + queryString + "'",
					e);
		} catch (BooleanQuery.TooManyClauses e) {
			throw new SearchServletException(req, ErrorCodeType.BAD_QUERY,
					"The query is too broad: '" + queryString + "' -- "
							+ "Wildcard queries, range queries, and fuzzy "
							+ "queries are expanded by Lucene to a boolean "
							+ "expression containing all matching terms "
							+ "present in the index. Change the query so it "
							+ "matches fewer terms.", e);
		} catch (JAXBException e) {
			throw new SearchServletException(req, ErrorCodeType.SYSTEM_PROBLEM,
					"Problem while marshalling search results.", e);
		} finally {
			// don't close the writer here - the error page will need to write
			// to the response if there is an error.
		}
	}

	/**
	 * If the user doesn't specify the 'output' parameter, they get standard
	 * output. Otherwise, they should specify 'rss'.
	 */
	private OutputType validateOutputType(HttpServletRequest req)
			throws SearchServletException {
		String outputString = getOptionalStringParameter(req, "output", "");
		if ("".equals(outputString)) {
			return OutputType.STANDARD;
		} else if ("rss".equalsIgnoreCase(outputString)) {
			return OutputType.RSS;
		} else {
			throw new SearchServletException(req, ErrorCodeType.BAD_ARGUMENT,
					"'output' parameter must be either 'rss' or empty.");
		}
	}

	/**
	 * Format the request parameters into a string for writing to the log.
	 */
	private String formatRequestParameters(HttpServletRequest req) {
		StringBuilder parms = new StringBuilder();
		for (Object name : req.getParameterMap().keySet()) {
			if (parms.length() > 0) {
				parms.append(", ");
			}
			parms.append(name).append('=').append(
					Arrays.deepToString((Object[]) req.getParameterMap().get(
							name)));
		}
		String string = parms.toString();
		return string;
	}

}
