/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.util;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.XMLConstants;
import javax.xml.bind.Marshaller;
import javax.xml.bind.PropertyException;
import javax.xml.namespace.NamespaceContext;

import org.apache.log4j.Logger;

import com.sun.xml.bind.marshaller.NamespacePrefixMapper;

/**
 * Express the preferred namespace prefixes, in whatever way that we need to use
 * them.
 * 
 * Provide a subclass instance of {@link NamespacePrefixMapper} for marshalling
 * REST responses.
 * 
 * Provide an implementation of
 * {@link NamespaceContext) for parsing OAI results. 
 * 
 * @author jblake
 */
public class XmlNamespaceUtil {
	private static final Logger LOG = Logger.getLogger(XmlNamespaceUtil.class);

	// dublin core namespaces
	public static final String NS_DUBLIN_CORE = "http://purl.org/dc/elements/1.1/";
	public static final String NS_DC_TERMS = "http://purl.org/dc/terms/";

	// oai namespaces
	public static final String NS_OAI = "http://www.openarchives.org/OAI/2.0/";
	public static final String NS_OAI_DC = "http://www.openarchives.org/OAI/2.0/oai_dc/";

	// nsdl namespaces
	public static final String NS_NSDL_DC = "http://ns.nsdl.org/nsdl_dc_v1.02/";
	public static final String NS_NSDL_DC_VERSION_1 = "http://ns.nsdl.org/nsdl_dc_v1.01";
	public static final String NS_NSDL_ABOUT = "http://ns.nsdl.org/nsdl_about_v1.00";
	public static final String NS_NSDL_SEARCH = "http://ns.nsdl.org/nsdl_search_v1.02/";
	public static final String NS_NSDL_SEARCH_REST = "http://ns.nsdl.org/search/rest_v2.01/";

	// ieee namespaces
	public static final String NS_IEEE = "http://www.ieee.org/xsd/LOMv1p0";

	// w3c namespaces
	public static final String NS_W3C_XML_SCHEMA_INSTANCE = "http://www.w3.org/2001/XMLSchema-instance";

	// syndication namespaces
	public static final String NS_ATOM = "http://www.w3.org/2005/Atom";

	// schema locations
	public static final String SL_NSDL_SEARCH_REST = "http://ns.nsdl.org/schemas/search/rest_v2.01.xsd";

	/**
	 * Map the preferred prefixes to their Namespaces.
	 */
	private static final Map<String, String> bindings = new HashMap<String, String>();
	static {
		// This is the default binding.
		bindings.put(XMLConstants.DEFAULT_NS_PREFIX, NS_NSDL_SEARCH_REST);

		// These are the prefixes we use in the search index fields and in the
		// search results.
		bindings.put("about", NS_NSDL_ABOUT);
		bindings.put("atom", NS_ATOM);
		bindings.put("dc", NS_DUBLIN_CORE);
		bindings.put("dct", NS_DC_TERMS);
		bindings.put("ieee", NS_IEEE);
		bindings.put("nsdl_dc", NS_NSDL_DC);
		bindings.put("nsdl_dc_v1.01", NS_NSDL_DC_VERSION_1);
		bindings.put("oai", NS_OAI);
		bindings.put("oai_dc", NS_OAI_DC);
		bindings.put("search", NS_NSDL_SEARCH);
		bindings.put("xsi", NS_W3C_XML_SCHEMA_INSTANCE);

		// These are required by NamespaceContext.
		bindings.put(XMLConstants.XML_NS_PREFIX, XMLConstants.XML_NS_URI);
		bindings.put(XMLConstants.XMLNS_ATTRIBUTE,
				XMLConstants.XMLNS_ATTRIBUTE_NS_URI);
	}

	/**
	 * Reverse map: Namespaces to preferred prefixes.
	 */
	private static final Map<String, String> prefixes = new HashMap<String, String>();
	static {
		for (String prefix : bindings.keySet()) {
			prefixes.put(bindings.get(prefix), prefix);
		}
		prefixes.put(NS_NSDL_SEARCH_REST, "");
	}

	/**
	 * A single instance of the prefix mapper.
	 */
	private static final SearchXmlPrefixMapper prefixMapper = new SearchXmlPrefixMapper();

	/**
	 * A single instance of the namespace context.
	 */
	private static final SearchXmlNamespaceContext namespaceContext = new SearchXmlNamespaceContext();

	/**
	 * Apply the preferred prefix mapping to a JAXB Marshaller.
	 */
	public static void setPrefixMapping(Marshaller marshaller) {
		try {
			marshaller.setProperty("com.sun.xml.bind.namespacePrefixMapper",
					prefixMapper);
		} catch (PropertyException e) {
			LOG.warn("Failed to apply the XML namespace prefix mapper.", e);
		}
	}

	/**
	 * Provide the preferred prefix mapping as a {@link NamespaceContext}.
	 */
	public static NamespaceContext getNamespaceContext() {
		return namespaceContext;
	}

	/**
	 * A {@link NamespacePrefixMapper}, suitable for JAXB to use when
	 * marshalling an XML document.
	 * 
	 * @author jblake
	 */
	private static class SearchXmlPrefixMapper extends NamespacePrefixMapper {
		@Override
		public String getPreferredPrefix(String namespaceUri,
				String suggestion, boolean requirePrefix) {
			LOG.debug("Prefix for '" + namespaceUri + "' is '"
					+ prefixes.get(namespaceUri) + "'");
			return prefixes.get(namespaceUri);
		}

	}

	/**
	 * Relate XML namespace prefixes to namespace URIs, and vice versa.
	 */
	private static class SearchXmlNamespaceContext implements NamespaceContext {
		/**
		 * Implement the contract from {@link NamespaceContext}.
		 * 
		 * @see NamespaceContext#getNamespaceURI(String)
		 */
		public String getNamespaceURI(String prefix) {
			if (prefix == null) {
				throw new IllegalArgumentException("prefix may not be null.");
			} else if (bindings.containsKey(prefix)) {
				return bindings.get(prefix);
			} else {
				return XMLConstants.NULL_NS_URI;
			}
		}

		/**
		 * Implement the contract from {@link NamespaceContext}.
		 * 
		 * @see NamespaceContext#getPrefix(String)
		 */
		public String getPrefix(String namespaceUri) {
			if (namespaceUri == null) {
				throw new IllegalArgumentException(
						"namespaceUri may not be null.");
			} else {
				return prefixes.get(namespaceUri);
			}
		}

		/**
		 * Implement the contract from {@link NamespaceContext}.
		 * 
		 * @see NamespaceContext#getPrefixes(String)
		 */
		public Iterator<String> getPrefixes(String namespaceUri) {
			if (namespaceUri == null) {
				throw new IllegalArgumentException(
						"namespaceUri may not be null.");
			} else if (prefixes.containsKey(namespaceUri)) {
				return Collections.singletonList(prefixes.get(namespaceUri))
						.iterator();
			} else {
				List<String> empty = Collections.emptyList();
				return empty.iterator();
			}
		}

	}
}
