/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.config;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.nsdl.search.content.ContentCacheConfig;
import org.nsdl.search.lucene.wrapper.IndexWrapperConfig;
import org.nsdl.search.oai.harvest.HarvestWrapperConfig;
import org.nsdl.search.util.Config;
import org.nsdl.search.util.SearchServiceConfig;

/**
 * A Mutable implementation of the Config interfaces, with initial values being
 * read from a properties file.
 * 
 * @author jblake
 */
public class PropertyFileConfig implements ContentCacheConfig,
		HarvestWrapperConfig, IndexWrapperConfig, SearchServiceConfig {
	/** The name of the properties file that must be found in the class path. */
	private static final String PROPERTY_FILE_NAME = "SearchConfig.properties";

	private static final String PROP_CC_BASE_URL = "ContentCacheConfig.contentCacheBaseUrl";
	private static final String PROP_HW_PARENT_DIR = "HarvestWrapperConfig.harvestParentDirectory";
	private static final String PROP_IW_ID_FIELDNAME = "IndexWrapperConfig.docIdFieldName";
	private static final String PROP_IW_INACTIVITY_THRESHOLD = "IndexWrapperConfig.inactivityThreshold";
	private static final String PROP_IW_INDEX_DIR = "IndexWrapperConfig.luceneIndexDirectory";
	private static final String PROP_IW_MAX_CACHED_WRITES = "IndexWrapperConfig.maximumCachedWrites";
	private static final String PROP_SS_SERVICE_DESCRIPTION = "SearchServiceConfig.serviceDescription";
	private static final String PROP_SS_SERVICE_NAME = "SearchServiceConfig.serviceName";

	private String contentCacheBaseUrl;
	private String harvestParentDirectory;
	private String docIdFieldName;
	private long inactivityThreshold;
	private String luceneIndexDirectory;
	private int maximumCachedWrites;
	private String serviceDescription;
	private String serviceName;

	public PropertyFileConfig() throws IOException {
		// Read the properties file.
		InputStream input = ClassLoader
				.getSystemResourceAsStream(PROPERTY_FILE_NAME);
		Properties properties = new Properties();
		properties.load(input);

		// Load the properties as initial values (may be overridden later)
		setContentCacheBaseUrl(properties.getProperty(PROP_CC_BASE_URL));
		setHarvestParentDirectory(properties.getProperty(PROP_HW_PARENT_DIR));
		setDocIdFieldName(properties.getProperty(PROP_IW_ID_FIELDNAME));
		setInactivityThreshold(Long.parseLong(properties
				.getProperty(PROP_IW_INACTIVITY_THRESHOLD)));
		setLuceneIndexDirectory(properties.getProperty(PROP_IW_INDEX_DIR));
		setMaximumCachedWrites(Integer.parseInt(properties
				.getProperty(PROP_IW_MAX_CACHED_WRITES)));
		setServiceDescription(properties
				.getProperty(PROP_SS_SERVICE_DESCRIPTION));
		setServiceName(properties.getProperty(PROP_SS_SERVICE_NAME));

		// Set this as the current config object.
		Config.getInstance().setContentCacheConfig(this);
		Config.getInstance().setHarvestWrapperConfig(this);
		Config.getInstance().setIndexWrapperConfig(this);
		Config.getInstance().setSearchServiceConfig(this);
	}

	public String getContentCacheBaseUrl() {
		return contentCacheBaseUrl;
	}

	public void setContentCacheBaseUrl(String contentCacheBaseUrl) {
		this.contentCacheBaseUrl = contentCacheBaseUrl;
	}

	public String getHarvestParentDirectory() {
		return harvestParentDirectory;
	}

	public void setHarvestParentDirectory(String harvestParentDirectory) {
		this.harvestParentDirectory = harvestParentDirectory;
	}

	public String getDocIdFieldName() {
		return docIdFieldName;
	}

	public void setDocIdFieldName(String docIdFieldName) {
		this.docIdFieldName = docIdFieldName;
	}

	public long getInactivityThreshold() {
		return inactivityThreshold;
	}

	public void setInactivityThreshold(long inactivityThreshold) {
		this.inactivityThreshold = inactivityThreshold;
	}

	public String getLuceneIndexDirectory() {
		return luceneIndexDirectory;
	}

	public void setLuceneIndexDirectory(String luceneIndexDirectory) {
		this.luceneIndexDirectory = luceneIndexDirectory;
	}

	public int getMaximumCachedWrites() {
		return maximumCachedWrites;
	}

	public void setMaximumCachedWrites(int maximumCachedWrites) {
		this.maximumCachedWrites = maximumCachedWrites;
	}

	public String getServiceDescription() {
		return serviceDescription;
	}

	public void setServiceDescription(String serviceDescription) {
		this.serviceDescription = serviceDescription;
	}

	public String getServiceName() {
		return serviceName;
	}

	public void setServiceName(String serviceName) {
		this.serviceName = serviceName;
	}

	@Override
	public String toString() {
		return "PropertyFileConfig[" + "\n   contentCacheBaseUrl="
				+ contentCacheBaseUrl + "\n   harvestParentDirectory="
				+ harvestParentDirectory + "\n   docIdFieldName="
				+ docIdFieldName + "\n   inactivityThreshold="
				+ inactivityThreshold + "\n   luceneIndexDirectory="
				+ luceneIndexDirectory + "\n   maximumCachedWrites="
				+ maximumCachedWrites + "\n   serviceDescription="
				+ serviceDescription + "\n   serviceName=" + serviceName
				+ "\n]";
	}

}
