/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * A {@link Histogram} that partitions based on how many days have passed since
 * a given {@link Date}.
 * 
 * @author jblake
 */
public class DaysOldHistogram implements Histogram<Date> {
	private static final long MILLISECONDS_IN_A_DAY = 24 * 60 * 60 * 1000;

	private final int maxDaysPassed = 30;
	private final int minDaysPassed = 0;

	private final DaysOldBucket[] days;
	private final DaysOldBucket oldBucket;
	private final DaysOldBucket newBucket;

	private final long endOfToday;

	public DaysOldHistogram() {
		days = new DaysOldBucket[maxDaysPassed - minDaysPassed + 1];
		for (int i = 0; i < days.length; i++) {
			days[i] = new DaysOldBucket(String.valueOf(maxDaysPassed - i));
		}
		oldBucket = new DaysOldBucket("OLD");
		newBucket = new DaysOldBucket("NEW");

		endOfToday = figureEndOfToday();
	}

	public void countThis(Date date) {
		int daysOld = (int) ((this.endOfToday - date.getTime()) / MILLISECONDS_IN_A_DAY);
		if (daysOld > maxDaysPassed) {
			oldBucket.incrementCount();
		} else if (daysOld < minDaysPassed) {
			newBucket.incrementCount();
		} else {
			days[maxDaysPassed - daysOld].incrementCount();
		}
	}

	public Collection<Bucket> getBuckets() {
		List<Bucket> result = new ArrayList<Bucket>();
		result.add(oldBucket);
		result.addAll(Arrays.asList(days));
		result.add(newBucket);
		return result;
	}

	private long figureEndOfToday() {
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 0);
		cal.add(Calendar.DATE, 1);
		return cal.getTimeInMillis();
	}

	/**
	 * A simple counter with legend and count.
	 */
	private class DaysOldBucket implements Bucket {
		private final String age;
		private int count;
		public DaysOldBucket(String age) {
			this.age = age;
		}

		public void incrementCount() {
			count++;
		}

		public int getCount() {
			return count;
		}

		public String getLegend() {
			return age;
		}

	}
}
