/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.data;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;
import java.util.TreeMap;

/**
 * A {@link Histogram} that keeps a count for each different value provided to
 * it.
 * 
 * @param <T>
 *            The type of value being counted.
 * @author jblake
 */
public class DiscreteValuesHistogram<T extends Comparable<T>>
		implements
			Histogram<T> {
	private final Map<T, DiscreteValuesBucket<T>> counts = new TreeMap<T, DiscreteValuesBucket<T>>();

	/**
	 * If we haven't seen this value before, create a bucket for it. Increment
	 * the count.
	 */
	public void countThis(T value) {
		if (!counts.containsKey(value)) {
			counts.put(value, new DiscreteValuesBucket<T>(value));
		}
		counts.get(value).incrementCount();
	}

	public Collection<Bucket> getBuckets() {
		return Arrays.asList(this.counts.values().toArray(new Bucket[0]));
	}

	/** Hold the value and the count. */
	private static class DiscreteValuesBucket<T> implements Bucket {
		private final T value;
		private int count;
		public DiscreteValuesBucket(T value) {
			this.value = value;
		}

		public void incrementCount() {
			count++;
		}

		public int getCount() {
			return count;
		}

		public String getLegend() {
			return String.valueOf(value);
		}

	}

}
