/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

/**
 * 
 * @author jblake
 */
public final class FieldHelper {
	private static final Pattern FRONT_TRIMMER = Pattern
			.compile("^\\d+ ~\\^ (.*)$");
	private static final Pattern TAIL_TRIMMER = Pattern
			.compile("(.*) ~\\^ \\d+$");

	/**
	 * Given a Lucene document and a fieldname, get the fields by that name and
	 * parse their values.
	 * 
	 * @param doc
	 *            the Lucene document.
	 * @param fieldName
	 *            the name of the fields we are seeking.
	 * @return a collection containing all of the values thus found.
	 */
	public static Collection<String> parseFieldValues(Document doc,
			String fieldName) {
		return parseFieldValues(doc.getFields(fieldName));
	}

	/**
	 * Given an array of Lucene fields, remove any NSDL-specific delimiters and
	 * return all of the values that were stored there.
	 * 
	 * @param fields
	 *            an array of fields from a Lucene document (might be null).
	 * @return a collection containing all of the values thus found.
	 */
	public static Collection<String> parseFieldValues(Field[] fields) {
		List<String> values = new ArrayList<String>();

		if ((fields != null) && (fields.length > 0)) {
			for (Field field : fields) {
				values.addAll(parseFieldValue(field.stringValue()));
			}
		}

		return values;
	}

	/**
	 * Break down the value to it's effective value:
	 * <ul>
	 * <li>Strip off "n ~^ " if it appears at the beginning of the value (where
	 * n is an integer)</li>
	 * <li>Strip off " ~^ n" if it appears at the end of the value (where n is
	 * an integer) </li>
	 * <li>break at " ~^ " into distinct values.</li>
	 * </ul>
	 * 
	 * @param value
	 * @return
	 */
	private static Collection<String> parseFieldValue(String value) {
		Matcher frontMatcher = FRONT_TRIMMER.matcher(value);
		if (frontMatcher.matches()) {
			value = frontMatcher.group(1);
		}
		Matcher tailMatcher = TAIL_TRIMMER.matcher(value);
		if (tailMatcher.matches()) {
			value = tailMatcher.group(1);
		}

		String[] values = value.split(" ~\\^ ");
		return Arrays.asList(values);
	}

	private FieldHelper() {
		// no need to instantiate this.
	}
}
