/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * A {@link Histogram} where each bucket represents a range of values. On
 * construction, a set of lower bounds for the buckets is supplied. Any value
 * that is lower than the lowest bucket will be ignored (with a warning to the
 * log).
 * 
 * @param <T>
 *            The type of object that defines the bucket boundaries.
 * @author jblake
 */
public class FixedRangeHistogram<T extends Comparable<T>> implements Histogram<T> {
	private static final Logger LOG = Logger.getLogger(FixedRangeBucket.class);

	private final List<FixedRangeBucket<T>> buckets;

	public FixedRangeHistogram(T[] lowerBounds) {
		Arrays.sort(lowerBounds);
		this.buckets = new ArrayList<FixedRangeBucket<T>>(lowerBounds.length);
		for (int i = 0; i < lowerBounds.length; i++) {
			T upperBound = (i < lowerBounds.length - 1)
					? lowerBounds[i + 1]
					: null;
			this.buckets.set(i, new FixedRangeBucket<T>(lowerBounds[i],
					upperBound));
		}
	}

	public void countThis(T value) {
		for (int i = this.buckets.size() - 1; i >= 0; i--) {
			FixedRangeBucket<T> bucket = buckets.get(i);
			if (value.compareTo(bucket.getLowerBound()) >= 0) {
				bucket.incrementCount();
				return;
			}
		}
		LOG.warn("Value '" + value
				+ "' was smaller than the lowest bucket bound.");
	}

	public Collection<Bucket> getBuckets() {
		return new ArrayList<Bucket>(buckets);
	}

	/** Hold the info for each bucket. */
	private static class FixedRangeBucket<T> implements Bucket {
		private int count;
		private final T lowerBound;
		private final T upperBound;

		public FixedRangeBucket(T lowerBound, T upperBound) {
			this.lowerBound = lowerBound;
			this.upperBound = upperBound;
		}
		public T getLowerBound() {
			return lowerBound;
		}

		public void incrementCount() {
			count++;
		}

		public int getCount() {
			return count;
		}

		public String getLegend() {
			if (upperBound == null) {
				return "> " + lowerBound;
			} else {
				return lowerBound + " < " + upperBound;
			}
		}

	}
}
