/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.indexcleaner;

import java.io.File;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexReader;

/**
 * Delete any records that have no metadata sources.
 * 
 * @author jblake
 */
public class IndexCleaner {
	private static final Logger LOG = Logger.getLogger(IndexCleaner.class);

	public static final String FIELD_NAME = "metadataSource";

	private final File indexDirectory;

	public IndexCleaner(IndexCleanerParameters parms) {
		this.indexDirectory = parms.getIndexDirectory();
	}

	public void clean(IndexCleanerStats stats) throws IOException {
		IndexReader reader = IndexReader.open(indexDirectory);
		stats.setNumberOfRecords(reader.numDocs());

		try {
			for (int i = 0; i < reader.maxDoc(); i++) {
				if (reader.isDeleted(i)) {
					// Pretend it's not there.
				} else {
					Document doc = reader.document(i);
					Field[] fields = doc.getFields(FIELD_NAME);
					if ((fields == null) || (fields.length == 0)) {
						stats.incrementDeletedRecords();
						reader.deleteDocument(i);
					}
					stats.incrementCheckedRecords();
				}
			}
		} finally {
			try {
				reader.close();
			} catch (IOException e) {
				LOG.error(e);
			}
		}
	}

	private static void report(IndexCleanerStats stats) {
		System.out.println("Checked " + stats.getCheckedRecords() + " of "
				+ stats.getNumberOfRecords() + " records, marking "
				+ stats.getDeletedRecords() + " as deleted.");
		if (stats.isEndedOnException()) {
			System.out.println("Interrupted by IOException.");
		}
	}

	public static void main(String[] args) {
		IndexCleanerParameters parms = new IndexCleanerParameters(args);
		LOG.info(parms);

		IndexCleanerStats stats = new IndexCleanerStats();

		IndexCleaner cleaner = new IndexCleaner(parms);
		try {
			cleaner.clean(stats);
		} catch (IOException e) {
			LOG.error(e);
			stats.setEndedOnException();
		}

		report(stats);
	}
}
