/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.lucenescan;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.Map;
import java.util.Properties;
import java.util.regex.Pattern;

import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.lf5.LogLevel;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.nsdl.search.utilities.data.FieldHelper;
import org.nsdl.search.utilities.lucenescan.LuceneScanArguments.DocumentFilter;

/**
 * A generic utility for scanning through the Lucene index.
 * 
 * The first argument on the command line determines what concrete subclass is
 * used to scan the index. The concrete subclasses are responsible for creating
 * an array of scanner object, and for formatting the header and footer in the
 * output.
 * 
 * @author jblake
 */
public abstract class LuceneScan {
	protected static final String OPTION_FIELD_NAME = "-fieldName";
	protected static final String OPTION_FIELD_NAMES = "-fieldNames";
	protected static final String OPTION_PATTERN = "-pattern";
	protected static final String OPTION_OUTPUT_FILE = "-outputFile";

	protected LuceneScan() {
	}

	/** Subclass must supply the array of scanners. */
	protected abstract DocumentScanner[] getScanners();

	/** Write something at the beginning of the run. */
	protected abstract void printHeader(LuceneScanArguments arguments,
			LuceneScanStats stats, PrintWriter writer);

	/** Write something at the end of the run. */
	protected abstract void printFooter(LuceneScanStats stats,
			PrintWriter writer);

	/**
	 * Pass through the index, reading each document as we go. If the document
	 * is not marked as deleted, pass it to each of the scanner objects so they
	 * can take whatever action they choose.
	 * 
	 * When wondering whether we have read enough documents, remember that an
	 * unoptimized index contains deleted docs and they shouldn't count.
	 * 
	 * Note: if a filter is applied, maxDocs refers to the number of unfiltered
	 * documents, so may produce no hits at all.
	 */
	private void scanIndex(LuceneScanArguments arguments,
			LuceneScanStats stats, PrintWriter writer) throws IOException {
		File indexDirectory = arguments.getIndexDirectory();
		IndexReader reader = IndexReader.open(indexDirectory);
		stats.setNumberOfDocuments(reader.numDocs());

		stats.limitNumberOfDocumentsToScan(arguments.getMaxDocs());

		for (int i = 0; i < reader.maxDoc(); i++) {
			if (reader.isDeleted(i)) {
				stats.incrementNumberOfDeletedDocuments();
			} else {
				stats.incrementNumberOfActiveDocuments();
				Document doc = reader.document(i);
				if (filterDocument(doc, arguments)) {
					for (DocumentScanner scanner : getScanners()) {
						scanner.add(doc, writer);
					}
				}
			}
			if (stats.isScanningComplete()) {
				break;
			}
		}
	}

	/**
	 * Restrict what documents we look at, according to the provided regular
	 * expression filter.
	 * 
	 * If there is no filter, all documents pass. Otherwise, a document must
	 * have at least one value in the selected field that matches the regular
	 * expression.
	 */
	private boolean filterDocument(Document doc, LuceneScanArguments arguments) {
		DocumentFilter filter = arguments.getFilter();
		if (filter == null) {
			return true;
		}

		String fieldName = filter.getFieldName();
		Pattern pattern = filter.getPattern();
		Collection<String> fieldValues = FieldHelper.parseFieldValues(doc,
				fieldName);

		for (String fieldValue : fieldValues) {
			if (pattern.matcher(fieldValue).find()) {
				return true;
			}
		}
		// Found no matches.
		return false;
	}

	/**
	 * Ask each scanner object to write whatever summary statistics it has
	 * accumulated.
	 */
	private void printResults(PrintWriter writer) {
		for (DocumentScanner scanner : getScanners()) {
			scanner.print(writer);
		}
	}

	private static void initializeLog4J(LogLevel logLevel) {
		Properties props = new Properties();
		props.put("log4j.rootLogger", logLevel.toString() + ", myAppender");
		props.put("log4j.appender.myAppender",
				"org.apache.log4j.ConsoleAppender");
		props.put("log4j.appender.myAppender.layout",
				"org.apache.log4j.SimpleLayout");
		PropertyConfigurator.configure(props);
	}

	/**
	 * Parse the command line arguments. Create a scan instance, and use it to
	 * scan the lucene index. Print the results.
	 */
	public static void main(String[] args) throws IOException {
		try {
			PrintWriter writer = new PrintWriter(System.out, true);
			LuceneScanArguments arguments = new LuceneScanArguments(writer,
					args);

			initializeLog4J(arguments.getLogLevel());

			LuceneScanStats stats = new LuceneScanStats();
			LuceneScan scan = arguments.getScanInstance();
			scan.printHeader(arguments, stats, writer);
			scan.scanIndex(arguments, stats, writer);
			scan.printResults(writer);
			scan.printFooter(stats, writer);

		} catch (LuceneScanUsageException e) {
			System.err.println(e.getMessage());
			System.err.println(e.getUsage());
		}

	}

	/**
	 * Get an array of field names from the options -fieldName or -fieldNames,
	 * where either of these fields will contain a list of field names delimited
	 * by commas (but not blanks).
	 * 
	 * @param options
	 * @return the field names that were found.
	 * @throws LuceneScanUsageException
	 */
	protected String[] getFieldNamesFromOptions(Map<String, String> options)
			throws LuceneScanUsageException {
		String fieldName = options.get(OPTION_FIELD_NAME);
		String fieldNames = options.get(OPTION_FIELD_NAMES);
		options.remove(OPTION_FIELD_NAME);
		options.remove(OPTION_FIELD_NAMES);

		if ((fieldName == null) && (fieldNames == null)) {
			throw new LuceneScanUsageException("Need one of these options: '"
					+ OPTION_FIELD_NAME + "', '" + OPTION_FIELD_NAMES + "'",
					"BOGUS USAGE MESSAGE");
			// KLUGE how to do the real usage message for sub-options?
		}
		if (LuceneScanArguments.NO_VALUE.equals(fieldName)) {
			throw new LuceneScanUsageException("'" + OPTION_FIELD_NAME
					+ "' option requires a value.", "");
			// KLUGE ditto.
		}
		if (LuceneScanArguments.NO_VALUE.equals(fieldNames)) {
			throw new LuceneScanUsageException("'" + OPTION_FIELD_NAMES
					+ "' option requires a value.", "");
			// KLUGE ditto.
		}

		String allNames = ((fieldName == null) ? "" : fieldName)
				+ ((fieldNames == null) ? "" : fieldNames);
		return allNames.split(",");
	}

	/**
	 * Get a regular expression pattern to be matched. If the pattern is
	 * surrounded by double-quotes, remove them before returning it. The pattern
	 * is optional, so if there is none, return <code>null.</code>
	 */
	protected String getPatternFromOptions(Map<String, String> options) {
		String pattern = options.get(OPTION_PATTERN);
		options.remove(OPTION_PATTERN);

		if (pattern != null) {
			if ((pattern.startsWith("\"")) && (pattern.endsWith("\""))) {
				pattern = pattern.substring(1, pattern.length() - 1);
			}
		}
		return pattern;
	}

	protected File getOutputFileFromOptions(Map<String, String> options)
			throws LuceneScanUsageException {
		String value = options.get(OPTION_OUTPUT_FILE);
		if (value == null) {
			throw new LuceneScanUsageException("'" + OPTION_OUTPUT_FILE
					+ "' is required for URLDumper", "BOGUS USAGE MESSAGE");
			// KLUGE how to do the real usage message for sub-options?
		}
		if (value.equals(LuceneScanArguments.NO_VALUE)) {
			throw new LuceneScanUsageException("'" + OPTION_OUTPUT_FILE
					+ "' option requires a value.", "");
			// KLUGE ditto.
		}

		File outputFile = new File(value);
		File parentDir = outputFile.getParentFile();
		if (!parentDir.isDirectory()) {
			throw new LuceneScanUsageException("Parent directory for '"
					+ outputFile.getPath() + "' doesn't exist.", "");
			// KLUGE ditto.
		}
		if (!parentDir.canWrite()) {
			throw new LuceneScanUsageException("Parent directory for '"
					+ outputFile.getPath() + "' isn't writable.", "");
			// KLUGE ditto.
		}
		options.remove(OPTION_OUTPUT_FILE);
		return outputFile;
	}

}
