/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.lucenescan;

import java.io.File;
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import org.apache.log4j.lf5.LogLevel;
import org.apache.log4j.lf5.LogLevelFormatException;
import org.nsdl.search.utilities.lucenescan.countbycollection.CountByCollection;
import org.nsdl.search.utilities.lucenescan.datescanner.AllDatesScan;
import org.nsdl.search.utilities.lucenescan.describeinfo.DescribeInfo;
import org.nsdl.search.utilities.lucenescan.detailedFieldLengths.DetailedFieldLengths;
import org.nsdl.search.utilities.lucenescan.fieldcounter.FieldCounter;
import org.nsdl.search.utilities.lucenescan.fieldfrequency.FieldFrequency;
import org.nsdl.search.utilities.lucenescan.fieldvaluesdetails.FieldValuesDetails;
import org.nsdl.search.utilities.lucenescan.fieldvalueshistogram.FieldValuesHistogram;
import org.nsdl.search.utilities.lucenescan.missingfields.MissingFieldLister;
import org.nsdl.search.utilities.lucenescan.normalizerdump.NormalizedUrlDumper;
import org.nsdl.search.utilities.lucenescan.protocolcounter.ProtocolCounter;
import org.nsdl.search.utilities.lucenescan.recorddumper.RecordDumper;
import org.nsdl.search.utilities.lucenescan.recordsbyvalue.RecordsByValue;
import org.nsdl.search.utilities.lucenescan.specificfieldvalues.SpecificFieldValues;
import org.nsdl.search.utilities.lucenescan.urldumper.URLDumper;

/**
 * Encapsulate the command line arguments for the LuceneScan main routine. This
 * class will parse only those arguments that we recognize here, and pass the
 * others to the instance of the concrete class.
 * 
 * @author jblake
 */
public class LuceneScanArguments {
	private static final String OPTION_MAX_DOCS = "-maxDocs";

	private static final String OPTION_LOG_LEVEL = "-logLevel";

	private static final String OPTION_FILTER = "-filter";

	private static final long DEFAULT_MAX_DOCS = 5000000;

	private static final LogLevel DEFAULT_LOG_LEVEL = LogLevel.INFO;

	private static final int NUMBER_OF_FIXED_ARGS = 2;

	/** Stored in the options map to indicate that the option had no value. */
	public static final String NO_VALUE = "";

	/**
	 * Map each command that we recognize to the Class of the instance that will
	 * execute that command.
	 */
	private static final Map<String, Class<? extends LuceneScan>> commandsMap;

	/**
	 * The names of the options that we recognize.
	 */
	private static final Set<String> optionsSet;

	/** The string that will be passed into a UsageException, if we throw one. */
	private static final String usageMessage;

	static {
		HashMap<String, Class<? extends LuceneScan>> commands = new LinkedHashMap<String, Class<? extends LuceneScan>>();
		commands.put("countFields", FieldCounter.class);
		commands.put("countProtocols", ProtocolCounter.class);
		commands.put("dumpRecord", RecordDumper.class);
		commands.put("dumpURLs", URLDumper.class);
		commands.put("specificFieldValues", SpecificFieldValues.class);
		commands.put("fieldValuesHistogram", FieldValuesHistogram.class);
		commands.put("getDescribeInfo", DescribeInfo.class);
		commands.put("detailedFieldLengths", DetailedFieldLengths.class);
		commands.put("scanDates", AllDatesScan.class);
		commands.put("fieldValuesDetails", FieldValuesDetails.class);
		commands.put("recordsWithFieldValues", RecordsByValue.class);
		commands.put("dumpNormalizedUrls", NormalizedUrlDumper.class);
		commands.put("listMissingField", MissingFieldLister.class);
		commands.put("fieldFrequency", FieldFrequency.class);
		commands.put("countByCollection", CountByCollection.class);
		commandsMap = Collections.unmodifiableMap(commands);

		HashSet<String> options = new HashSet<String>();
		options.add(OPTION_MAX_DOCS);
		options.add(OPTION_LOG_LEVEL);
		options.add(OPTION_FILTER);
		optionsSet = Collections.unmodifiableSet(options);

		usageMessage = "Usage: LuceneScan [command] [indexDirectory] [option value]* [suboption value]*, "
				+ "where command is one of "
				+ commands.keySet()
				+ ", and option is one of " + optionsSet;
	}

	public static String getUsageMessage() {
		return usageMessage;
	}

	/**
	 * First argument, determines what class is used to create the scan
	 * instance.
	 */
	private final String command;

	/** Second argument, the directory that the Lucene index is found in. */
	private final File indexDirectory;

	/** Maximum number of docs to scan on this run. */
	private final long maxDocs;

	/** Field name and regular express used to filter the documents. */
	private final DocumentFilter filter;

	/** Log4j logging level for this run. */
	private final LogLevel logLevel;

	/**
	 * Any unrecognized command-line arguments - to be passed to the scan
	 * instance.
	 */
	private final Map<String, String> optionsForScanInstance;

	/** Yep. The scan instance. */
	private final LuceneScan scanInstance;

	/**
	 * Extract any general information from the arguments.
	 */
	public LuceneScanArguments(PrintWriter writer, String[] args)
			throws LuceneScanUsageException {
		this.command = parseCommandName(args);
		this.indexDirectory = parseIndexDirectory(args);

		Map<String, String> fullArgumentsMap = parseRemainingArguments(args);
		Map<String, String> remainingArgumentsMap = new HashMap<String, String>(
				fullArgumentsMap);
		this.maxDocs = checkMaxDocs(remainingArgumentsMap);
		this.logLevel = checkLogLevel(remainingArgumentsMap);
		this.filter = checkFilter(remainingArgumentsMap);
		this.optionsForScanInstance = remainingArgumentsMap;

		this.scanInstance = createScanInstance();

		if (this.logLevel.encompasses(LogLevel.DEBUG)) {
			writer.println("args: " + args);
			writer.println("options: " + fullArgumentsMap);
		}
	}

	/**
	 * Make sure that there is a command, and that we recognize it.
	 */
	private String parseCommandName(String[] args)
			throws LuceneScanUsageException {
		if (args.length < 1) {
			throw new LuceneScanUsageException("No command supplied",
					usageMessage);
		}

		String command = args[0];
		if (!commandsMap.containsKey(command)) {
			throw new LuceneScanUsageException("Unrecognized command: '"
					+ command + "'", usageMessage);
		}
		return command;
	}

	/**
	 * Make sure that there is a directory name on the command line, and that it
	 * represents a readable directory.
	 */
	private File parseIndexDirectory(String[] args)
			throws LuceneScanUsageException {
		if (args.length < 2) {
			throw new LuceneScanUsageException("No index directory supplied",
					usageMessage);
		}

		String directoryPath = args[1];
		File indexDirectory = new File(directoryPath);

		if (!indexDirectory.exists()) {
			throw new IllegalArgumentException("Index directory '"
					+ directoryPath + "' does not exist.");
		}

		if (!indexDirectory.isDirectory()) {
			throw new IllegalArgumentException("Index directory '"
					+ directoryPath + "' is not a directory.");
		}

		if (!indexDirectory.canRead()) {
			throw new IllegalArgumentException("Index directory '"
					+ directoryPath + "' is not readable.");
		}
		return indexDirectory;
	}

	/**
	 * Create the scan instance, based on the command we received.
	 * 
	 * @throws LuceneScanUsageException
	 *             if the instance constructor threw one.
	 * @throws IllegalStateException
	 *             if any reflection-oriented problem occurs.
	 */
	private LuceneScan createScanInstance() throws LuceneScanUsageException {
		Class<? extends LuceneScan> scanClass = commandsMap.get(this.command);
		try {
			Constructor<? extends LuceneScan> constructor = scanClass
					.getConstructor(new Class[] { Map.class });
			return constructor
					.newInstance(new Object[] { this.optionsForScanInstance });
		} catch (SecurityException e) {
			throw new IllegalStateException(e);
		} catch (IllegalArgumentException e) {
			throw new IllegalStateException(e);
		} catch (NoSuchMethodException e) {
			throw new IllegalStateException(e);
		} catch (InstantiationException e) {
			throw new IllegalStateException(e);
		} catch (IllegalAccessException e) {
			throw new IllegalStateException(e);
		} catch (InvocationTargetException e) {
			if (e.getCause() instanceof LuceneScanUsageException) {
				throw (LuceneScanUsageException) e.getCause();
			} else {
				throw new IllegalStateException(e);
			}
		}
	}

	/*
	 * Go through the args (skipping the fixed args). If an arg is not a valid
	 * option, complain. Otherwise, check to see if its followed by a
	 * (non-option) value. Store the option with it's value, or with an empty
	 * string if no value was found.
	 */
	private Map<String, String> parseRemainingArguments(String[] args)
			throws LuceneScanUsageException {
		Map<String, String> map = new HashMap<String, String>();

		for (int i = NUMBER_OF_FIXED_ARGS; i < args.length; i++) {
			String option = args[i];
			if (!isAValidOption(option)) {
				throw new LuceneScanUsageException("unrecognized option: '"
						+ option + "'", usageMessage);
			}
			if ((i + 1 < args.length) && (!isAValidOption(args[i + 1]))) {
				// found a value for option
				map.put(option, args[i + 1]);
				i++;
			} else {
				// no value for option
				map.put(option, NO_VALUE);
			}
		}
		return map;
	}

	private long checkMaxDocs(Map<String, String> optionsMap)
			throws LuceneScanUsageException {
		long maxDocs = DEFAULT_MAX_DOCS;
		String value = optionsMap.get(OPTION_MAX_DOCS);
		if (value == null) {
			// already set to default
		} else if (value.equals(NO_VALUE)) {
			throw new LuceneScanUsageException("'" + OPTION_MAX_DOCS
					+ "' option requires a value.", usageMessage);
		} else {
			try {
				maxDocs = Long.valueOf(value);
			} catch (NumberFormatException e) {
				throw new LuceneScanUsageException("'" + value
						+ "' is not a valid value for '" + OPTION_MAX_DOCS
						+ "'.", usageMessage);
			}
		}
		optionsMap.remove(OPTION_MAX_DOCS);
		return maxDocs;
	}

	private LogLevel checkLogLevel(Map<String, String> optionsMap)
			throws LuceneScanUsageException {
		LogLevel logLevel = DEFAULT_LOG_LEVEL;
		String value = optionsMap.get(OPTION_LOG_LEVEL);
		if (value == null) {
			// already set to default
		} else if (value.equals(NO_VALUE)) {
			throw new LuceneScanUsageException("'" + OPTION_LOG_LEVEL
					+ "' option requires a value.", usageMessage);
		} else {
			try {
				logLevel = LogLevel.valueOf(value);
			} catch (LogLevelFormatException e) {
				throw new LuceneScanUsageException("'" + value
						+ "' is not a valid value for '" + OPTION_MAX_DOCS
						+ "'.", usageMessage);
			}
		}
		optionsMap.remove(OPTION_LOG_LEVEL);
		return logLevel;
	}

	private DocumentFilter checkFilter(Map<String, String> optionsMap)
			throws LuceneScanUsageException {
		DocumentFilter filter = null;
		String value = optionsMap.get(OPTION_FILTER);
		if (value != null) {
			int colonHere = value.indexOf(':');
			if (colonHere == -1) {
				throw new LuceneScanUsageException("'" + OPTION_FILTER
						+ "' option contains no field name: '" + value + "'",
						usageMessage);
			}
			String fieldName = value.substring(0, colonHere);
			Pattern pattern = Pattern.compile(value.substring(colonHere + 1));
			filter = new DocumentFilter(fieldName, pattern);
		}
		optionsMap.remove(OPTION_FILTER);
		return filter;
	}

	/**
	 * Valid options begin with hyphens.
	 */
	private boolean isAValidOption(String option) {
		return (option.startsWith("-"));
	}

	public String getCommand() {
		return command;
	}

	public File getIndexDirectory() {
		return indexDirectory;
	}

	public LuceneScan getScanInstance() {
		return scanInstance;
	}

	public LogLevel getLogLevel() {
		return logLevel;
	}

	public long getMaxDocs() {
		return maxDocs;
	}

	public DocumentFilter getFilter() {
		return filter;
	}

	/**
	 * A field name, and a regular expression to match against the field
	 * contents.
	 */
	public static class DocumentFilter {
		private final String fieldName;
		private final Pattern pattern;

		public DocumentFilter(String fieldName, Pattern pattern) {
			this.fieldName = fieldName;
			this.pattern = pattern;
		}

		public String getFieldName() {
			return fieldName;
		}

		public Pattern getPattern() {
			return pattern;
		}

	}
}
