/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.lucenescan.countbycollection;

import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.nsdl.search.utilities.lucenescan.DocumentScanner;
import org.nsdl.search.utilities.lucenescan.LuceneScan;
import org.nsdl.search.utilities.lucenescan.LuceneScanArguments;
import org.nsdl.search.utilities.lucenescan.LuceneScanStats;
import org.nsdl.search.utilities.lucenescan.LuceneScanUsageException;

/**
 * Count how many records in each collection have at least one of the selected
 * fields. We are counting records, not values, so if a record has more than one
 * value of the same field name, it counts as one.
 * 
 * You must specify field names that you are interested in, or ALL if you want
 * them all.
 * 
 * You can specify collections by OAI, or we assume you want them all.
 * 
 * @author jblake
 */
public class CountByCollection extends LuceneScan {
	public static final String OPTION_COLLECTIONS = "-collections";
	public static final String MATCH_ALL_FIELDS = "ALL";

	private final CountByCollectionScanner scanner;

	public CountByCollection(Map<String, String> options)
			throws LuceneScanUsageException {
		List<String> fields = Arrays.asList(getFieldNamesFromOptions(options));
		if (fields.contains(MATCH_ALL_FIELDS)) {
			fields = Collections.emptyList();
		}

		List<String> collections = Arrays
				.asList(getCollectionIdsFromOptions(options));
		this.scanner = new CountByCollectionScanner(fields, collections);
	}

	@Override
	protected DocumentScanner[] getScanners() {
		return new DocumentScanner[] { scanner };
	}

	/**
	 * Explain what we're about to print out.
	 */
	@Override
	protected void printHeader(LuceneScanArguments arguments,
			LuceneScanStats stats, PrintWriter writer) {
		writer.println("How many records contain a particular field type?");
		writer.println("   Remember: the counts are records, not fields.");
		writer.println("   So even if a record has 3 subject fields, "
				+ "it counts as 1.");
		writer.println();
	}

	/**
	 * Nothing to see here.
	 */
	@Override
	protected void printFooter(LuceneScanStats stats, PrintWriter writer) {
	}

	/**
	 * Get the collection OAI IDs from the options map. If the option wasn't
	 * supplied, that's OK - we will match all collections.
	 */
	private String[] getCollectionIdsFromOptions(Map<String, String> options)
			throws LuceneScanUsageException {
		String collectionString = options.get(OPTION_COLLECTIONS);
		options.remove(OPTION_COLLECTIONS);
		if (collectionString == null) {
			return new String[0];
		}
		if (LuceneScanArguments.NO_VALUE.equals(collectionString)) {
			throw new LuceneScanUsageException("'" + OPTION_COLLECTIONS
					+ "' option requires a value.", "");
			// KLUGE how to do the real usage message for sub-options?
		}
		return collectionString.split(",");
	}
}
