/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.lucenescan.countbycollection;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import org.apache.log4j.Logger;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.nsdl.search.utilities.data.DiscreteValuesHistogram;
import org.nsdl.search.utilities.data.FieldHelper;
import org.nsdl.search.utilities.data.Histogram;
import org.nsdl.search.utilities.lucenescan.DocumentScanner;

/**
 * 
 * @author jblake
 */
public class CountByCollectionScanner extends DocumentScanner {
	private static final Logger LOG = Logger
			.getLogger(CountByCollectionScanner.class);

	private static final String RECORD_COUNT = "_RECORD_COUNT_";
	private static final String FIELD_COLLECTION_ID = "collImageAltText";
	private static final Comparator<String> CASE_INDEPENDANT = new IdComparator();

	private final Set<String> fields;
	private final Set<String> collections;
	private final Map<String, DiscreteValuesHistogram<String>> allCounts;

	public CountByCollectionScanner(List<String> fields,
			List<String> collections) {
		this.fields = Collections.unmodifiableSet(new HashSet<String>(fields));
		this.collections = Collections.unmodifiableSet(new HashSet<String>(
				collections));
		this.allCounts = new TreeMap<String, DiscreteValuesHistogram<String>>(
				CASE_INDEPENDANT);
	}

	/**
	 * Parse each record for collection ID and field names, and record in the
	 * data for that collection (if we care).
	 */
	@Override
	public void add(Document doc, PrintWriter writer) {
		RecordInfo info = new RecordInfo(doc);
		for (String collectionId : info.getCollectionIds()) {
			if (isInterestingCollection(collectionId)) {
				DiscreteValuesHistogram<String> collectionCounts = getCollectionCounts(collectionId);
				for (String fieldName : info.getFieldNames()) {
					if (isInterestingField(fieldName)) {
						collectionCounts.countThis(fieldName);
					}
				}
				collectionCounts.countThis(RECORD_COUNT);
			}
		}
	}

	/**
	 * A collection is interesting if (1) we asked for it, or (2) we didn't
	 * specify.
	 */
	private boolean isInterestingCollection(String collectionId) {
		return this.collections.isEmpty()
				|| this.collections.contains(collectionId);
	}

	/**
	 * A field is interesting if (1) we asked for it, or (2) we asked for all.
	 */
	private boolean isInterestingField(String fieldName) {
		return this.fields.isEmpty() || this.fields.contains(fieldName);
	}

	/**
	 * Get the counts for this collection. If we haven't seen this collection
	 * before, create an empty counts block.
	 */
	private DiscreteValuesHistogram<String> getCollectionCounts(String id) {
		if (!allCounts.containsKey(id)) {
			allCounts.put(id, new DiscreteValuesHistogram<String>());
		}
		return allCounts.get(id);
	}

	/** Print the counts for each collection. */
	@Override
	public void print(PrintWriter writer) {
		for (String collectionId : allCounts.keySet()) {
			writer.println("Collection: " + collectionId);

			DiscreteValuesHistogram<String> counts = allCounts
					.get(collectionId);
			for (Histogram.Bucket bucket : counts.getBuckets()) {
				writer.format("%1$,12d -- %2$s\n", bucket.getCount(), bucket
						.getLegend());
			}
			writer.println();
		}
	}

	/**
	 * Look through the Lucene document and get the collection ID and the names
	 * of the fields.
	 */
	private static class RecordInfo {
		private final Set<String> collectionIds;
		private final Set<String> fieldNames;

		public RecordInfo(Document doc) {
			Collection<String> values = FieldHelper.parseFieldValues(doc,
					FIELD_COLLECTION_ID);
			collectionIds = Collections.unmodifiableSet(new HashSet<String>(
					values));
			if (collectionIds.isEmpty()) {
				LOG.warn("record has no '" + FIELD_COLLECTION_ID + "' field.");
			}

			Set<String> names = new HashSet<String>();
			for (Enumeration<?> fields = doc.fields(); fields.hasMoreElements();) {
				Field field = (Field) fields.nextElement();
				names.add(field.name());
			}
			this.fieldNames = Collections.unmodifiableSet(names);
		}

		public Set<String> getCollectionIds() {
			return collectionIds;
		}

		public Set<String> getFieldNames() {
			return fieldNames;
		}

	}

	/** Sort the collections by ID, ignoring case. */
	private static class IdComparator implements Comparator<String> {
		public int compare(String s1, String s2) {
			return s1.compareToIgnoreCase(s2);
		}
	}

}
