/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.lucenescan.datescanner;

import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;
import org.apache.lucene.document.Document;
import org.nsdl.search.utilities.data.DaysOldHistogram;
import org.nsdl.search.utilities.data.Histogram.Bucket;
import org.nsdl.search.utilities.lucenescan.DocumentScanner;

/**
 * Note the values in a given date field, and show how they are distributed over
 * the last 30 days.
 * 
 * @author jblake
 */
public class ThirtyDayScanner extends DocumentScanner {
	private static final Logger LOG = Logger.getLogger(ThirtyDayScanner.class);

	private final String fieldName;
	private final DaysOldHistogram histogram;
	private final DateFormat dateFormat = new SimpleDateFormat(
			"yyyy-MM-dd'T'HH:mm:ss'Z'");

	private int tallyMissing;

	public ThirtyDayScanner(String fieldName) {
		this.fieldName = fieldName;
		this.histogram = new DaysOldHistogram();
	}

	public void add(Document doc, PrintWriter writer) {
		String[] values = doc.getValues(this.fieldName);
		if (values == null) {
			tallyMissing++;
			return;
		}

		String dateString = values[0];
		try {
			Date date = dateFormat.parse(dateString);
			histogram.countThis(date);
		} catch (ParseException e) {
			LOG.warn("Invalid date string '" + dateString
					+ "' in record for ''");
		}
	}

	public void print(PrintWriter writer) {
		int numColumns = 5;

		// number of rows is enough to include one display for each day, plus
		// two for "old" and "none", rounded up.
		List<Bucket> buckets = new ArrayList<Bucket>(histogram.getBuckets());
		int numRows = (int) Math.ceil(((float) buckets.size()) / numColumns);

		writer.println("DateFieldHistogram for '" + this.fieldName + "'");
		for (int col = 0; col < numColumns; col++) {
			writer.print("Age        Count       ");
		}
		writer.println();

		for (int row = 0; row < numRows; row++) {
			for (int col = 0; col < numColumns; col++) {
				int index = (col * numRows) + row;
				if (index < buckets.size()) {
					Bucket bucket = buckets.get(index);
					writer.format("%1$4s -- %2$,12d   ", bucket.getLegend(),
							bucket.getCount());
				} else if (index == buckets.size()) {
					writer.format("NONE -- %1$,12d   ", this.tallyMissing);
				}
			}
			writer.println();
		}

		writer.println();
	}

}
