/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.lucenescan.describeinfo;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.apache.lucene.document.Document;
import org.nsdl.search.utilities.data.DiscreteValuesHistogram;
import org.nsdl.search.utilities.data.FieldHelper;
import org.nsdl.search.utilities.data.Histogram;
import org.nsdl.search.utilities.lucenescan.DocumentScanner;

/**
 * How many resource have no description field, and what collections are
 * providing these resources?
 * 
 * @author jblake
 */
public class MissingDescriptionScanner extends DocumentScanner {
	private final DiscreteValuesHistogram<String> sources;
	private final DiscreteValuesHistogram<Integer> numberOfSources;
	private long records;
	private long recordsWithNoDescription;

	public MissingDescriptionScanner() {
		this.sources = new DiscreteValuesHistogram<String>();
		this.numberOfSources = new DiscreteValuesHistogram<Integer>();
	}

	/**
	 * If the document has no description field, find out it's collection(s) and
	 * add to the histogram.
	 */
	public void add(Document doc, PrintWriter writer) {
		records++;

		String[] descriptions = doc.getValues("dc:description");
		if ((descriptions == null) || (descriptions.length == 0)) {
			recordsWithNoDescription++;

			Collection<String> collections = FieldHelper.parseFieldValues(doc,
					"collImageAltText");
			Set<String> distinctCollections = new HashSet<String>(collections);
			for (String collection : distinctCollections) {
				sources.countThis(collection);
			}
			numberOfSources.countThis(collections.size());
		}

	}

	public void print(PrintWriter writer) {
		writer.println("Histogram of documents with no dc:description fields, "
				+ "by collection.");
		writer.println(recordsWithNoDescription + " of " + records
				+ " records had no dc:description field.");
		writer
				.println(" # resources -- contributing collection (alt image text).");
		for (Histogram.Bucket bucket : sources.getBuckets()) {
			writer.format("%1$,12d -- %2$s\n", bucket.getCount(), bucket
					.getLegend());
		}
		writer.println();

		writer.println("For documents with no description, "
				+ "how many collections are they from?");
		for (Histogram.Bucket bucket : numberOfSources.getBuckets()) {
			writer.format("%1$,12d -- %2$s\n", bucket.getCount(), bucket
					.getLegend());
		}
		writer.println();
	}

}
