/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.lucenescan.detailedFieldLengths;

import java.io.PrintWriter;
import java.util.Collection;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.nsdl.search.utilities.data.DiscreteValuesHistogram;
import org.nsdl.search.utilities.data.FieldHelper;
import org.nsdl.search.utilities.data.Histogram;
import org.nsdl.search.utilities.lucenescan.DocumentScanner;

/**
 * 
 * @author jblake
 */
public class DFLScanner extends DocumentScanner {

	/** What field are we looking at? */
	private final String fieldName;

	/** How many documents have we scanned? */
	private int numberOfDocs;

	/** How many had no fields that we wanted? */
	private int noFieldDocs;

	/** What field lengths have we found, and how many? */
	private final DiscreteValuesHistogram<Integer> fieldLengths;

	/** What length do we get if we add all matching fields for a document? */
	private final DiscreteValuesHistogram<Integer> aggregateFieldLengths;

	public DFLScanner(String fieldName) {
		this.fieldName = fieldName;
		this.fieldLengths = new DiscreteValuesHistogram<Integer>();
		this.aggregateFieldLengths = new DiscreteValuesHistogram<Integer>();
	}

	public void add(Document doc, PrintWriter writer) {
		numberOfDocs++;

		Field[] fields = doc.getFields(fieldName);
		if ((fields == null) || (fields.length == 0)) {
			noFieldDocs++;
		}

		Collection<String> values = FieldHelper.parseFieldValues(fields);

		int aggregateLength = 0;
		for (String value : values) {
			fieldLengths.countThis(value.length());
			aggregateLength += value.length();
		}
		aggregateFieldLengths.countThis(aggregateLength);
	}

	/**
	 * Print both histograms - field lengths, and aggregate field lengths.
	 */
	public void print(PrintWriter writer) {
		writer
				.println("Of " + this.numberOfDocs + " documents scanned, "
						+ this.noFieldDocs + " had no '" + this.fieldName
						+ "' fields.");
		writer.println("Histogram of " + this.fieldName
				+ " individual field lengths.");
		writer.println("       count --- length");
		for (Histogram.Bucket bucket : fieldLengths.getBuckets()) {
			writer.format("%1$,12d --- %2$s\n", bucket.getCount(), bucket
					.getLegend());
		}
		writer.println();

		writer.println("Histogram of " + this.fieldName
				+ " field lengths, aggregated by document.");
		writer.println("       count --- length");
		for (Histogram.Bucket bucket : aggregateFieldLengths.getBuckets()) {
			writer.format("%1$,12d --- %2$s\n", bucket.getCount(), bucket
					.getLegend());
		}
		writer.println();
	}
}
