/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.lucenescan.fieldvaluesdetails;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.nsdl.search.utilities.lucenescan.DocumentScanner;

/**
 * For any record that contains the chosen fields, print out the field value
 * along with the record's URL and "documentUpdated" date. If a record contains
 * more than one value for the field, print one line for each value.
 * 
 * @author jblake
 */
public class FVDScanner extends DocumentScanner {
	private final String fieldName;
	private final int maxValues;
	private final List<DocumentInfo> infos = new ArrayList<DocumentInfo>();

	public FVDScanner(String fieldName, int maxValues) {
		this.fieldName = fieldName;
		this.maxValues = maxValues;
	}

	/**
	 * If the document has any fields by the chosen name, store the document
	 * info for printing later.
	 */
	public void add(Document doc, PrintWriter writer) {
		if (infos.size() >= this.maxValues) {
			return;
		}

		Field[] fields = doc.getFields(fieldName);
		if ((fields == null) || (fields.length == 0)) {
			return;
		}

		Field urlField = doc.getField("resourceIdentifier");
		String url = (urlField == null) ? "missing URL" : urlField
				.stringValue();

		Field updateField = doc.getField("documentLastUpdated");
		String updateDate = (updateField == null)
				? "missing updateDate "
				: updateField.stringValue();

		for (Field field : fields) {
			infos.add(new DocumentInfo(url, updateDate, field.stringValue()));
		}
	}

	/**
	 * Print all of the document infos that we have accumulated.
	 */
	public void print(PrintWriter writer) {
		writer.println("values details for '" + this.fieldName + "'");

		for (DocumentInfo info : this.infos) {
			writer.println(info);
		}
	}

	/** A simple data class. */
	private static class DocumentInfo {
		private final String url;
		private final String updateDate;
		private final String fieldValue;

		public DocumentInfo(String url, String updateDate, String fieldValue) {
			this.url = url;
			this.updateDate = updateDate;
			this.fieldValue = fieldValue;
		}

		@Override
		public String toString() {
			return "date=" + this.updateDate + ", value=" + this.fieldValue
					+ ", url=" + this.url;
		}
	}
}
