/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.lucenescan.fieldvalueshistogram;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.nsdl.search.utilities.data.DiscreteValuesHistogram;
import org.nsdl.search.utilities.data.FieldHelper;
import org.nsdl.search.utilities.data.Histogram.Bucket;
import org.nsdl.search.utilities.lucenescan.DocumentScanner;

/**
 * 
 * @author jblake
 */
public class FieldValuesScanner extends DocumentScanner {
	private static final int MAX_VALUES = 5000;
	private static final String NO_VALUE = "___NO_VALUE___";

	/** What field are we looking at? */
	private final String fieldName;

	/** How many documents have this value in the field? */
	private final Map<String, Integer> valueCounts = new TreeMap<String, Integer>();

	/** How many documents have this many fields of this name? */
	private final DiscreteValuesHistogram<Integer> fieldsPerDocumentCounts;

	/** How many documents have this many field values of this name? */
	private final DiscreteValuesHistogram<Integer> valuesPerDocumentCounts;

	/** How many documents have this many unique field values of this name? */
	private final DiscreteValuesHistogram<Integer> uniqueValuesPerDocumentsCounts;

	/** Have we already printed a "too many values" message? */
	private boolean maxValuesExceeded;

	/** How many documents have we scanned? */
	private int numberOfDocs;

	/** A temporary holder to help us remove duplicate fields. */
	private Set<String> uniqueValuesInDocument = new HashSet<String>();

	public FieldValuesScanner(String fieldName) {
		this.fieldName = fieldName;
		this.fieldsPerDocumentCounts = new DiscreteValuesHistogram<Integer>();
		this.valuesPerDocumentCounts = new DiscreteValuesHistogram<Integer>();
		this.uniqueValuesPerDocumentsCounts = new DiscreteValuesHistogram<Integer>();
	}

	/**
	 * Ask the document for all fields that match this field name, and increment
	 * the count for each value found. If the document has no such fields,
	 * increment the count of the artificial value: NO_VALUE.
	 */
	public void add(Document doc, PrintWriter writer) {
		numberOfDocs++;

		Field[] fields = doc.getFields(fieldName);
		Collection<String> values = FieldHelper.parseFieldValues(fields);

		fieldsPerDocumentCounts.countThis((fields == null) ? 0 : fields.length);
		valuesPerDocumentCounts.countThis(values.size());
		addToUniqueValueCounts(values);

		if (values.isEmpty()) {
			addValueToCounts(NO_VALUE, writer);
		} else {
			for (String value : values) {
				addValueToCounts(value, writer);
			}
		}
	}

	private void addValueToCounts(String value, PrintWriter writer) {
		if (valueCounts.containsKey(value)) {
			valueCounts.put(value, valueCounts.get(value) + 1);
		} else if (valueCounts.size() < MAX_VALUES) {
			valueCounts.put(value, 1);
		} else {
			// Only report it once.
			if (!maxValuesExceeded) {
				writer.println("Exceeded maximum number of values ("
						+ MAX_VALUES + ") for '" + fieldName + "'");
				maxValuesExceeded = true;
			}
		}
	}

	/**
	 * Determine how many of these field values are unique, and add that count
	 * to the histogram.
	 */
	private void addToUniqueValueCounts(Collection<String> values) {
		uniqueValuesInDocument.clear();
		uniqueValuesInDocument.addAll(values);
		uniqueValuesPerDocumentsCounts.countThis(uniqueValuesInDocument.size());
	}

	/**
	 * Print the list of values we found and how many times.
	 */
	public void print(PrintWriter writer) {
		writer.println();
		writer.println("Histogram of values for '" + fieldName + "': found "
				+ totalNumberOfValues() + " values in " + numberOfDocs
				+ " documents.");
		writer.println("---------------------------------------------------");
		for (String value : valueCounts.keySet()) {
			int count = valueCounts.get(value);
			writer.format("%1$,12d %2$s\n", count, value);
		}
		writer.println();

		for (Bucket bucket : fieldsPerDocumentCounts.getBuckets()) {
			writer.format("%1$,12d documents have %2$s fields named '%3$s'\n",
					bucket.getCount(), bucket.getLegend(), fieldName);
		}
		writer.println();

		for (Bucket bucket : valuesPerDocumentCounts.getBuckets()) {
			writer.format("%1$,12d documents have %2$s values for '%3$s'\n",
					bucket.getCount(), bucket.getLegend(), fieldName);
		}
		writer.println();

		for (Bucket bucket : uniqueValuesPerDocumentsCounts.getBuckets()) {
			writer.format(
					"%1$,12d documents have %2$s unique values for '%3$s'\n",
					bucket.getCount(), bucket.getLegend(), fieldName);
		}
	}

	private int totalNumberOfValues() {
		int sum = 0;
		for (int count : valueCounts.values()) {
			sum += count;
		}
		return sum;
	}

}
