/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.lucenescan.specificfieldvalues;

import java.io.PrintWriter;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.nsdl.search.utilities.data.DiscreteValuesHistogram;
import org.nsdl.search.utilities.data.FieldHelper;
import org.nsdl.search.utilities.data.Histogram;
import org.nsdl.search.utilities.lucenescan.DocumentScanner;

/**
 * 
 * @author jblake
 */
public class SpecificValuesScanner extends DocumentScanner {
	private final DiscreteValuesHistogram<String> valueCounts;
	private final DiscreteValuesHistogram<String> sourceCollections;
	private final String fieldName;
	private final Set<String> specificValues;
	private int numberOfDocs;

	public SpecificValuesScanner(String fieldName, String[] specificValue) {
		this.fieldName = fieldName;
		this.specificValues = new HashSet<String>(Arrays.asList(specificValue));
		this.valueCounts = new DiscreteValuesHistogram<String>();
		this.sourceCollections = new DiscreteValuesHistogram<String>();
	}

	/**
	 * Check the values of the selected field to see if they match the specific
	 * values we are looking for. Tally any matching fields. Also, if any
	 * matches are found, tally the collections that contributed to this
	 * document.
	 */
	public void add(Document doc, PrintWriter writer) {
		numberOfDocs++;

		boolean found = false;
		Field[] fields = doc.getFields(fieldName);
		for (String value : FieldHelper.parseFieldValues(fields)) {
			if (specificValues.contains(value)) {
				valueCounts.countThis(value);
				found = true;
			}
		}
		if (found) {
			Field[] collections = doc.getFields("collImageAltText");
			for (String source : FieldHelper.parseFieldValues(collections)) {
				sourceCollections.countThis(source);
			}
		}
	}

	public void print(PrintWriter writer) {
		writer.println("Scanned the '" + fieldName
				+ "' field for these values:\n" + specificValues);
		writer.println("       count --- value");
		for (Histogram.Bucket bucket : valueCounts.getBuckets()) {
			writer.format("%1$,12d --- %2$s\n", bucket.getCount(), bucket
					.getLegend());
		}
		writer.println();

		writer.println("Documents containing these values were members of "
				+ "the following collections.");
		writer.println("       count --- source collection");
		for (Histogram.Bucket bucket : sourceCollections.getBuckets()) {
			writer.format("%1$,12d --- %2$s\n", bucket.getCount(), bucket
					.getLegend());
		}
		writer.println();
	}

}
