/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.lucenescan.urldumper;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;

import org.apache.log4j.Logger;
import org.nsdl.search.utilities.lucenescan.DocumentScanner;
import org.nsdl.search.utilities.lucenescan.LuceneScan;
import org.nsdl.search.utilities.lucenescan.LuceneScanArguments;
import org.nsdl.search.utilities.lucenescan.LuceneScanStats;
import org.nsdl.search.utilities.lucenescan.LuceneScanUsageException;

/**
 * This simple LuceneScan writes the URL of each Lucene document to the file
 * specified in the "-outputFile" parameter.
 * 
 * If you specify a sample rate, it will only write the URLs of randomly
 * selected documents - where the odds are 1 out of the rate.
 * 
 * @author jblake
 */
public class URLDumper extends LuceneScan {
	private static final Logger log4j = Logger.getLogger(URLDumper.class);

	private static final String OPTION_SAMPLE_RATE = "-sampleRate";

	private final PrintWriter outputWriter;
	private final int sampleRate;

	private DocumentScanner[] scanners;

	public URLDumper(Map<String, String> options)
			throws LuceneScanUsageException, IOException {
		File outputFile = getOutputFileFromOptions(options);
		this.outputWriter = new PrintWriter(new FileWriter(outputFile));
		this.sampleRate = getSampleRate(options);
	}

	/**
	 * Get the sample rate as a positive integer, or return 0 if no rate is
	 * specified.
	 */
	private int getSampleRate(Map<String, String> options)
			throws LuceneScanUsageException {
		int sampleRate = 0;

		String value = options.get(OPTION_SAMPLE_RATE);
		if (value == null) {
			return 0;
		}
		if (value.equals(LuceneScanArguments.NO_VALUE)) {
			throw new LuceneScanUsageException("'" + OPTION_SAMPLE_RATE
					+ "' option requires a value.", "");
			// KLUGE how to do the real usage message for sub-options?
		}

		try {
			sampleRate = Integer.parseInt(value);
		} catch (NumberFormatException e) {
			throw new LuceneScanUsageException("'" + OPTION_SAMPLE_RATE
					+ "' option must be a valid integer: '" + value + "'", "");
		}

		if (sampleRate < 2) {
			throw new LuceneScanUsageException("'" + OPTION_SAMPLE_RATE
					+ "' option must be an integer greater than 1: '"
					+ sampleRate + "'", "");
		}

		options.remove(OPTION_SAMPLE_RATE);
		return sampleRate;
	}

	/**
	 * The only scanner is one that dumps URLs to the output writer.
	 */
	@Override
	protected DocumentScanner[] getScanners() {
		if (this.scanners == null) {
			this.scanners = new DocumentScanner[] { new URLDumpScanner(
					this.outputWriter, this.sampleRate) };
		}
		return this.scanners;
	}

	/**
	 * Don't forget to close that output writer.
	 */
	@Override
	protected void printHeader(LuceneScanArguments arguments,
			LuceneScanStats stats, PrintWriter writer) {
	}

	@Override
	protected void printFooter(LuceneScanStats stats, PrintWriter writer) {
		outputWriter.close();

		log4j.debug("Scanned " + stats.getNumberOfDocumentsToScan()
				+ " documents of " + stats.getNumberOfDocuments() + " - "
				+ stats.getNumberOfActiveDocuments() + " active, "
				+ stats.getNumberOfDeletedDocuments() + " deleted");
	}

}
