/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.optimizer;

import java.io.IOException;
import java.util.Date;

import org.apache.log4j.Logger;
import org.apache.lucene.index.IndexWriter;
import org.nsdl.search.lucene.nsdl.NsdlAnalyzer;
import org.nsdl.search.util.Config;
import org.nsdl.search.utilities.config.PropertyFileConfig;
import org.nsdl.searchtools.common.DateUtil;

/**
 * Open an {@link IndexWriter} on the Lucene search index, and call optimize.
 * This will throw an exception if there are any problems, including if the
 * index does not exist, or if the index is locked.
 * 
 * @author jblake
 */
public class LuceneOptimizer {
	private static final Logger LOG = Logger.getLogger(LuceneOptimizer.class);

	/**
	 * Open IndexWriter for lucene index at the passed directory name.
	 */
	private static IndexWriter openIndexWriter(String absIndexDirName)
			throws IOException {
		return new IndexWriter(absIndexDirName, NsdlAnalyzer.getInstance(),
				false);
	}

	/**
	 * Optimize the luceneIndex.
	 */
	private void optimizeLuceneIndex() throws IOException {
		String luceneIndexDirectory = Config.getInstance()
				.getIndexWrapperConfig().getLuceneIndexDirectory();

		IndexWriter luceneIndexWriter = openIndexWriter(luceneIndexDirectory);
		luceneIndexWriter.optimize();
		luceneIndexWriter.close();
	}

	public static void main(String[] args) throws IOException {
		// Read the parms.
		LuceneOptimizerParameters parms = new LuceneOptimizerParameters(args);
		LOG.info(parms);

		// Set up the configuration.
		PropertyFileConfig config = new PropertyFileConfig();
		config.setLuceneIndexDirectory(parms.getIndexDirectory().getPath());
		LOG.info(config);

		// Do it.
		Date startTime = new Date();
		LuceneOptimizer optimizer = new LuceneOptimizer();
		optimizer.optimizeLuceneIndex();
		LOG.info("Lucene index optimization took: "
				+ DateUtil.getInstance().formatInterval(startTime, new Date()));
	}
}
