/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.summary;

import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.apache.log4j.Logger;
import org.apache.lucene.document.Document;

/**
 * @author jblake
 * 
 */
public class DateFieldHistogram implements SummaryInfo {
	private static final Logger log4j = Logger
			.getLogger(DateFieldHistogram.class);

	public static final int DEFAULT_NUMBER_OF_DAYS = 30;

	private static final long MILLISECONDS_IN_A_DAY = 24 * 60 * 60 * 1000;

	private final DateFormat dateFormat = new SimpleDateFormat(
			"yyyy-MM-dd'T'HH:mm:ss'Z'");

	private final String fieldName;

	private final int[] tallysByDay;

	private int tallyOldDocs;
	private int tallyMissing;

	private final long endOfToday;

	public DateFieldHistogram(String fieldName) {
		this.fieldName = fieldName;
		this.tallysByDay = new int[DEFAULT_NUMBER_OF_DAYS];
		this.endOfToday = figureEndOfToday();
	}

	public void add(Document doc) {
		String[] values = doc.getValues(this.fieldName);
		if (values == null) {
			// if the field isn't present, just skip it.
			log4j.debug("No value for '" + this.fieldName + "'");
			tallyMissing++;
			return;
		}

		String dateString = values[0];
		try {
			int daysOld = figureDaysOld(dateString);
			if (daysOld < tallysByDay.length) {
				tallysByDay[daysOld]++;
			} else {
				tallyOldDocs++;
			}
			log4j.trace("Date '" + dateString + "' is " + daysOld
					+ " days old.");
		} catch (ParseException e) {
			// if the field won't parse, just skip it.
			log4j.warn("Couldn't parse date: '" + dateString + "' in field '"
					+ this.fieldName + "'");
			tallyMissing++;
		}
	}

	private int figureDaysOld(String dateString) throws ParseException {
		Date date = dateFormat.parse(dateString);
		int daysOld = (int) ((this.endOfToday - date.getTime()) / MILLISECONDS_IN_A_DAY);
		if (daysOld < 0) {
			log4j.warn("Invalid age for '" + this.fieldName + "': date='"
					+ dateString + "', daysOld=" + daysOld);
			daysOld = Integer.MAX_VALUE;
		}
		return daysOld;
	}

	private long figureEndOfToday() {
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 0);
		cal.add(Calendar.DATE, 1);
		return cal.getTimeInMillis();
	}

	public void print(PrintWriter writer) {
		int numColumns = 5;
		// number of rows is enough to include one display for each day, plus
		// two for "old" and "none", rounded up.
		int numRows = (int) Math.ceil(((float) tallysByDay.length + 2)
				/ numColumns);

		writer.println("DateFieldHistogram for '" + this.fieldName + "'");
		for (int col = 0; col < numColumns; col++) {
			writer.print("Age        Count       ");
		}
		writer.println();

		for (int row = 0; row < numRows; row++) {
			for (int col = 0; col < numColumns; col++) {
				int index = (col * numRows) + row;
				if (index < tallysByDay.length) {
					writer.format("%1$4d -- %2$,12d   ", index,
							tallysByDay[index]);
				} else if (index == tallysByDay.length) {
					writer.format("OLD  -- %1$,12d   ", tallyOldDocs);
				} else if (index == tallysByDay.length + 1) {
					writer.format("NONE -- %1$,12d   ", this.tallyMissing);
				}
			}
			writer.println();
		}

		writer.println();
	}
}
