/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.summary;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;

/**
 * Scan through the Lucene index and generate statistics about when documents
 * were created and updated.
 * 
 * Take one pass through the index, reading each document in numerical order.
 * From each document, determine how long ago the document was created and last
 * modified, also when the content was last fetched and modified and when the
 * metadata was last modified. Summarize this information in data structures,
 * and display the results.
 * 
 * @author jblake
 */
public class SimpleLuceneSummarizer {
	private static final Logger log4j = Logger
			.getLogger(SimpleLuceneSummarizer.class);

	private static final int MAX_NUMBER_OF_DOCUMENTS_TO_SCAN = 5000000;
	// private static final int MAX_NUMBER_OF_DOCUMENTS_TO_SCAN = 500;

	private final File indexDirectory;

	private int numberOfDocuments;

	private int numberOfDocumentsToScan;

	private int numberOfActiveDocuments;

	private int numberOfDeletedDocuments;

	private final List<SummaryInfo> summaryInfo = Collections
			.unmodifiableList(Arrays.asList(new SummaryInfo[] {
					new DateFieldHistogram("documentCreated"),
					new DateFieldHistogram("documentLastUpdated"),
					new DateFieldHistogram("metadataLastModified"),
					new DateFieldHistogram("contentLastFetched"),
					new DateFieldHistogram("contentLastModified") }));

	public SimpleLuceneSummarizer(String directoryPath) {
		if (directoryPath == null) {
			throw new NullPointerException("directoryPath must not be null.");
		}

		this.indexDirectory = new File(directoryPath);

		if (!this.indexDirectory.exists()) {
			throw new IllegalArgumentException("Index directory '"
					+ directoryPath + "' does not exist.");
		}

		if (!this.indexDirectory.isDirectory()) {
			throw new IllegalArgumentException("Index directory '"
					+ directoryPath + "' is not a directory.");
		}

		if (!this.indexDirectory.canRead()) {
			throw new IllegalArgumentException("Index directory '"
					+ directoryPath + "' is not readable.");
		}
	}

	private void scanIndex() throws IOException {
		IndexReader reader = IndexReader.open(indexDirectory);

		this.numberOfDocuments = reader.maxDoc();
		this.numberOfDocumentsToScan = Math.min(this.numberOfDocuments,
				MAX_NUMBER_OF_DOCUMENTS_TO_SCAN);

		for (int i = 0; i < this.numberOfDocumentsToScan; i++) {
			if (reader.isDeleted(i)) {
				this.numberOfDeletedDocuments++;
			} else {
				Document doc = reader.document(i);
				this.numberOfActiveDocuments++;
				for (SummaryInfo info : this.summaryInfo) {
					info.add(doc);
				}
			}
		}
	}

	private void printResults(PrintWriter writer) {
		writer.println(new SimpleDateFormat().format(new Date()));
		writer.println("Scanned Lucene index at "
				+ this.indexDirectory.getAbsolutePath());
		writer.println("Scanned " + this.numberOfDocumentsToScan
				+ " documents of " + this.numberOfDocuments + " -- "
				+ this.numberOfActiveDocuments + " active, "
				+ this.numberOfDeletedDocuments + " deleted");
		writer.println();

		for (SummaryInfo info : this.summaryInfo) {
			log4j.debug("Printing results for " + info);
			info.print(writer);
			writer.println();
		}
	}

	private static void initializeLog4J(String logLevel) {
		Properties props = new Properties();
		props.put("log4j.rootLogger", logLevel + ", myAppender");
		props.put("log4j.appender.myAppender",
				"org.apache.log4j.ConsoleAppender");
		props.put("log4j.appender.myAppender.layout",
				"org.apache.log4j.SimpleLayout");
		PropertyConfigurator.configure(props);
	}

	private static String getIndexDirectoryName(String[] args) {
		if (args.length != 1) {
			throw new IllegalArgumentException(
					"usage: SimpleLuceneSummarizer indexDirectory");
		}
		return args[0];
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) throws IOException {
		initializeLog4J("INFO");
		SimpleLuceneSummarizer sls = new SimpleLuceneSummarizer(
				getIndexDirectoryName(args));
		sls.scanIndex();
		sls.printResults(new PrintWriter(System.out, true));
	}

}
