/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.updatecontent;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Map;
import java.util.TreeMap;

import org.apache.log4j.Logger;
import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.search.lucene.wrapper.IndexWrapper;
import org.nsdl.search.lucene.wrapper.SearchResults;
import org.nsdl.search.utilities.config.PropertyFileConfig;
import org.nsdl.searchtools.common.DateUtil;
import org.nsdl.searchtools.common.changes.ChangeDirectoryWrapper;
import org.nsdl.searchtools.common.changes.ChangeFile;
import org.nsdl.searchtools.common.changes.ChangesWrapper;
import org.nsdl.searchtools.common.changes.ChangeFile.Line;

/**
 * <p>
 * Reads one or more directories of content change files, and updates any
 * records in the search index that have been affected by these changes.
 * </p>
 * <p>
 * Each directory of change files is produced by a run of the
 * ContentChangeLogger application. The files are named "changes.1",
 * "changes.2", etc., and each line consists of an ISO8601 timestamp and a URL.
 * The timestamp indicates the time and date at which new content was retrieved
 * for the page represented by the URL.
 * </p>
 * <p>
 * This application reads the change files. For each URL, it will retrieve a
 * {@link SearchDocument} from the index, and check to see whether the
 * <code>lastModifiedDate</code> in the document is older than the timestamp
 * of the new content. If so, the new content is fetched and the record is
 * updated in the index. Otherwise, the update is ignored since it is "old
 * news."
 * </p>
 * <p>
 * When running, you must specify the parent directory that holds the change
 * file directories. You may then specify a particular change directory or not.
 * If not specified, the application runs against the oldest change directory
 * which has not yet been applied to the index, according to the history file.
 * If "-all" is specified, the application will read all directories that have
 * not been applied to the index, from oldest to newest. After each directory is
 * applied, the history file is updated to reflect this fact.
 * </p>
 * <p>
 * Command line arguments are:
 * <ul>
 * <li><em>changesParentDirectory</em> - contains a collection of change
 * directories, from which we will scan the oldest that has not yet been scanned
 * (unless changesDirectory is specified).</li>
 * <li><em>changesDirectory</em> - scan this change directory (within the
 * parent directory), even if it has already been scanned.</li>
 * <li><em>all</em> - requests scans of all non-scanned change directories
 * (incompatible with changesDirectory).</li>
 * <li><em>historyFile</em> - the full path to the file which records scans
 * (must already exist)</li>
 * </ul>
 * </p>
 * 
 * @author jblake
 */
public class UpdateContent {
	private static final Logger LOG = Logger.getLogger(UpdateContent.class);
	private static final Logger NOTIFY_LOG = Logger
			.getLogger("NOTIFY.UpdateContent");

	/**
	 * Apply a directory of change files to the index.
	 */
	private UpdateContentStats applyDirectory(ChangeDirectoryWrapper changes)
			throws IOException {
		UpdateContentStats stats = new UpdateContentStats();
		for (File changeFile : changes.getChangeFiles()) {
			applyChangeFile(changeFile, stats);
		}
		changes.setScanned();
		return stats;
	}

	/**
	 * Apply a single change file to the index.
	 */
	private void applyChangeFile(File changeFile, UpdateContentStats stats)
			throws IOException {
		LOG.info("processing change file: " + changeFile.getPath());

		IndexWrapper index = IndexWrapper.getInstance();
		for (Line line : ChangeFile.getInstance(changeFile).getLines()) {
			stats.incrementUrlCount();
			SearchResults results = index.getDocument(line.getUrl());
			if (results.getReturnCount() == 0) {
				stats.incrementUnknownCount();
			} else {
				SearchDocument doc = results.getResult(0).getDocument();
				if (doc.isChanged()) {
					index.writeDocument(doc);
				} else {
					stats.incrementOutdatedCount();
				}
			}
		}
		stats.incrementFileCount();
	}

	/**
	 * Find the next change directory to be scanned. Return null if there is no
	 * such directory.
	 */
	private static ChangeDirectoryWrapper findChangeDirectory(
			UpdateContentParameters parms, int howManyScannedAlready) {
		// If we set out to scan only one, and we've done it, run away.
		if ((howManyScannedAlready > 0)
				&& (!parms.scanMultipleChangeDirectories())) {
			return null;
		}

		ChangesWrapper parent = ChangesWrapper.getInstance(parms
				.getChangesParentDirectory(), parms.getHistoryFile());

		String changeDirectory = parms.getChangesDirectoryName();
		if (changeDirectory == null) {
			return parent.getOldestUnindexedDirectory();
		} else {
			try {
				return parent.getDirectoryWrapper(changeDirectory);
			} catch (FileNotFoundException e) {
				throw new UpdateContentException(e);
			}
		}
	}

	/**
	 * Report what we did.
	 */
	private static void report(UpdateContentParameters parms,
			Map<String, UpdateContentStats> scanned) {
		DateUtil dateUtil = DateUtil.getInstance();
		System.out.println("UpdateContent complete: "
				+ dateUtil.formatLocalDate(dateUtil.now(),
						"yyyy-MM-dd HH:mm:ss"));
		System.out.println();

		System.out.println("parameters:");
		Map<String, String> parameterMap = parms.getParameterMap();
		for (String key : parameterMap.keySet()) {
			String value = parameterMap.get(key);
			if (value == null) {
				System.out.println("    " + key);
			} else {
				System.out.println("    " + key + " = " + value);
			}
		}
		System.out.println();

		System.out.println("Scanned " + scanned.size() + " directories.");
		for (String dirName : scanned.keySet()) {
			UpdateContentStats stats = scanned.get(dirName);
			System.out.println("   " + dirName + ": " + stats.getFileCount()
					+ " files, with " + stats.getUrlCount() + " urls, ("
					+ stats.getActiveCount() + " active, "
					+ stats.getUnknownCount() + " unknown, "
					+ stats.getOutdatedCount() + " outdated)");
		}
		System.out.println();
	}

	public static void main(String[] args) throws IOException {
		// Read the parms.
		Map<String, UpdateContentStats> scanned = new TreeMap<String, UpdateContentStats>();
		UpdateContentParameters parms = new UpdateContentParameters(args);
		LOG.info(parms);

		// Set up the configuration.
		PropertyFileConfig config = new PropertyFileConfig();
		config.setLuceneIndexDirectory(parms.getIndexDirectory().getPath());
		config.setContentCacheBaseUrl(parms.getContentCacheBaseUrl());
		LOG.info(config);

		try {
			UpdateContent updater = new UpdateContent();

			// Scan change directories until we are are done, or until there are
			// no more to scan.
			while (true) {
				ChangeDirectoryWrapper changes = findChangeDirectory(parms,
						scanned.size());
				if (changes == null) {
					break;
				} else {
					UpdateContentStats stats = updater.applyDirectory(changes);
					scanned.put(changes.getPath(), stats);
					NOTIFY_LOG.info("Scanned " + changes.getPath() + ", found "
							+ stats.getUrlCount() + " URLs.");
				}
			}
		} finally {
			report(parms, scanned);
		}
	}

}
