/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.utilities.updatecontent;

import java.io.File;

import org.nsdl.searchtools.common.parameters.AbstractParameterParser;

/**
 * Encapsulate the command-line parameters for the {@link UpdateContent}
 * application. See {@link #usage()} for the command-line syntax.
 * 
 * @author jblake
 */
public class UpdateContentParameters extends AbstractParameterParser {
	public static final String PARM_INDEX = "-indexDirectory";
	public static final String PARM_HISTORY = "-historyFile";
	public static final String PARM_PARENT = "-changesParentDirectory";
	public static final String PARM_CHANGES = "-changesDirectory";
	public static final String PARM_CC_BASE_URL = "-contentCacheBaseUrl";
	public static final String PARM_ALL = "-all";
	private static final String[] ALL_PARMS = new String[] { PARM_INDEX,
			PARM_HISTORY, PARM_PARENT, PARM_CHANGES, PARM_CC_BASE_URL, PARM_ALL };

	private final boolean all;
	private final File indexDirectory;
	private final File historyFile;
	private final File changesParentDirectory;
	private final String changesDirectoryName;
	private final String contentCacheBaseUrl;

	/**
	 * Validate the arguments and store the values.
	 */
	public UpdateContentParameters(String[] args) {
		buildParameterMap(args);

		rejectUnrecognizedParms(ALL_PARMS);

		this.indexDirectory = findIndexDirectory();

		String historyFilename = getRequiredParameterValue(PARM_HISTORY);
		this.historyFile = new File(historyFilename);

		String parentFilename = getRequiredParameterValue(PARM_PARENT);
		this.changesParentDirectory = new File(parentFilename);

		this.contentCacheBaseUrl = getRequiredParameterValue(PARM_CC_BASE_URL);

		this.all = checkEmptyParameter(PARM_ALL);

		this.changesDirectoryName = parameterMap.get(PARM_CHANGES);
		if (this.all && (this.changesDirectoryName != null)) {
			throw problem(PARM_ALL + " cannot be used with " + PARM_CHANGES);
		}
	}

	private File findIndexDirectory() {
		File indexDirectory = new File(getRequiredParameterValue(PARM_INDEX));
		if (!indexDirectory.exists()) {
			throw problem("Index directory '" + indexDirectory.getPath()
					+ "' does not exist.");
		}
		if (!indexDirectory.isDirectory()) {
			throw problem("Index directory '" + indexDirectory.getPath()
					+ "' is not a directory.");
		}
		return indexDirectory;
	}

	/**
	 * Explain how it should have been done.
	 */
	@Override
	protected void usage() {
		System.out.println("usage: UpdateContent -indexDirectory {fullPath} "
				+ "-historyFile {fullPath} " + "-contentCacheBaseUrl {url} "
				+ "-changesParentDirectory {fullPath} "
				+ "[-all | -changesDirectory {name}] ");
	}

	public boolean scanMultipleChangeDirectories() {
		return this.all;
	}

	public File getHistoryFile() {
		return this.historyFile;
	}

	public File getChangesParentDirectory() {
		return this.changesParentDirectory;
	}

	public String getChangesDirectoryName() {
		return this.changesDirectoryName;
	}

	public File getIndexDirectory() {
		return this.indexDirectory;
	}

	public String getContentCacheBaseUrl() {
		return this.contentCacheBaseUrl;
	}

	public String toString() {
		return "UpdateContentParameters[indexDirectory="
				+ indexDirectory.getPath() + ", all=" + all + ", historyFile="
				+ historyFile.getPath() + ", changesParentDirectory="
				+ changesParentDirectory.getPath() + ", changesDirectoryName="
				+ changesDirectoryName + ", contentCacheBaseUrl="
				+ contentCacheBaseUrl + "]";
	}

}
