/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.util.xml;

import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.Date;

import org.nsdl.searchtools.common.DateUtil;

/**
 * Parsing and printing methods for the JAXB-generated classes to use when the
 * defaults aren't suitable.
 * 
 * @author jblake
 */
public class JaxbXmlFormats {
	/**
	 * A thread-safe formatter for the "score" field in a search result.
	 */
	private static final ThreadLocal<DecimalFormat> scoreFormatters = new ThreadLocal<DecimalFormat>() {
		@Override
		protected DecimalFormat initialValue() {
			DecimalFormat format = new DecimalFormat("0.0");
			format.setMaximumFractionDigits(6);
			return format;
		}
	};

	/**
	 * Marshall the "score" value in a search result.
	 */
	public static String printScore(Float score) {
		return scoreFormatters.get().format(score.floatValue());
	}

	/**
	 * Unmarshall the "score" value in a search result.
	 */
	public static Float parseScore(String score) {
		return Float.valueOf(score);
	}

	/**
	 * Marshall a Date object.
	 */
	public static String printDate(Date date) {
		return DateUtil.getInstance().formatDate(date,
				DateUtil.ISO8601_DATE_TIME_Z);
	}

	/**
	 * Unmarshall a Date object.
	 */
	public static Date parseDate(String dateString) {
		try {
			return DateUtil.getInstance().parseDate(dateString,
					DateUtil.ISO8601_DATE_TIME_Z);
		} catch (ParseException e) {
			throw new IllegalArgumentException("Can't parse this date: "
					+ dateString, e);
		}
	}

	private static final char TAB = 0x9;
	private static final char LINE_FEED = 0xA;
	private static final char RETURN = 0xD;
	private static final char SPACE = 0x20;
	private static final char D7FF = 0xD7FF;
	private static final char E000 = 0xE000;
	private static final char FFFD = 0xFFFD;

	/**
	 * <p>
	 * Filter bad characters from a string when Marshalling.
	 * </p>
	 * <p>
	 * Remove any characters from the string that are not allowed in XML.
	 * Exclude special characters except for white-space. Exclude the Unicode
	 * "surrogate blocks" from 0xD800 to 0xDFFF, and the characters 0xFFFE and
	 * 0xFFFF.
	 * </p>
	 * <p>
	 * Appendix C of the XML specification states: Legal characters are tab,
	 * carriage return, line feed and the legal graphic characters of Unicode
	 * and ISO/IEC 10646...
	 * </p>
	 * <p>
	 * XML Specification Guide (Graham, Quin) interprets this as:
	 * 
	 * <pre>
	 *  #x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] | [#x10000 -#x10FFF]
	 * </pre>
	 * 
	 * But Java only uses 16 bits for a character, so this last clause will be
	 * ignored.
	 * </p>
	 */
	public static String printString(String raw) {
		if (raw == null) {
			return null;
		}
		StringBuilder buffer = new StringBuilder(raw.length());
		for (int i = 0; i < raw.length(); i++) {
			char thisChar = raw.charAt(i);
			if (((thisChar >= SPACE) && (thisChar <= D7FF))
					|| (thisChar == TAB) || (thisChar == LINE_FEED)
					|| (thisChar == RETURN)
					|| ((thisChar >= E000) && (thisChar <= FFFD))) {
				buffer.append(thisChar);
			}
		}
		return buffer.toString();
	}

	/**
	 * No need to create instances - all methods are static.
	 */
	private JaxbXmlFormats() {
		// Nothing to instantiate.
	}
}
