/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package mock.search.lucene;

import java.io.IOException;

import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.BooleanQuery;
import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.search.lucene.wrapper.IndexSummaryInfo;
import org.nsdl.search.lucene.wrapper.IndexWrapper;
import org.nsdl.search.lucene.wrapper.IndexWrapperListener;
import org.nsdl.search.lucene.wrapper.IndexWrapperQuery;
import org.nsdl.search.lucene.wrapper.SearchResults;

/**
 * An implementation of IndexWrapper for use in unit tests.
 * 
 * @author jblake
 */
public class MockIndexWrapper extends IndexWrapper {
	private static MockIndexWrapper instance;

	public static MockIndexWrapper getInstance() {
		synchronized (MockIndexWrapper.class) {
			if (instance == null) {
				instance = new MockIndexWrapper();
			}
		}
		return instance;
	}

	public static void reset() {
		instance = null;
	}

	// ----------------------------------------------------------------------
	// Mocking infrastructure
	// ----------------------------------------------------------------------

	private boolean throwsIOExceptionOnSearch;
	private boolean throwsBooleanQueryExceptionOnParse;
	private boolean throwsParseExceptionOnParse;

	/** These will be the results of any query. */
	private SearchResults results;

	public void setThrowsIOExceptionOnSearch(boolean throwsIOExceptionOnSearch) {
		this.throwsIOExceptionOnSearch = throwsIOExceptionOnSearch;
	}

	public void setThrowsBooleanQueryExceptionOnParse(
			boolean throwsBooleanQueryExceptionOnSearch) {
		this.throwsBooleanQueryExceptionOnParse = throwsBooleanQueryExceptionOnSearch;
	}

	public void setThrowsParseExceptionOnParse(
			boolean throwsParseExceptionOnSearch) {
		this.throwsParseExceptionOnParse = throwsParseExceptionOnSearch;
	}

	public void setResults(SearchResults results) {
		this.results = results;
	}

	// ----------------------------------------------------------------------
	// Mocked methods
	// ----------------------------------------------------------------------

	@Override
	public IndexWrapperQuery parseQuery(String queryString)
			throws ParseException, IOException {
		if (throwsBooleanQueryExceptionOnParse) {
			throw new BooleanQuery.TooManyClauses();
		}

		if (throwsParseExceptionOnParse) {
			throw new ParseException("Can't parse that query!");
		}

		return new IndexWrapperQuery() {
		};
	}

	@Override
	public SearchResults search(IndexWrapperQuery query, int skipCount,
			int maxReturnCount) throws IOException {
		if (throwsIOExceptionOnSearch) {
			throw new IOException("This was an IOException");
		}

		return (results == null) ? MockSearchResults.EMPTY_RESULT : results;
	}

	// ----------------------------------------------------------------------
	// Un-implemented methods
	// ----------------------------------------------------------------------

	@Override
	public void createIndex() throws IOException {
		throw new RuntimeException(
				"MockIndexWrapper.createIndex() not implemented.");
	}

	@Override
	public IndexWrapperQuery createPrefixQuery(String fieldName,
			String valuePrefix) throws IOException {
		throw new RuntimeException(
				"MockIndexWrapper.createPrefixQuery() not implemented.");
	}

	@Override
	public SearchResults getDocument(String docId) throws IOException {
		throw new RuntimeException(
				"MockIndexWrapper.getDocument() not implemented.");
	}

	@Override
	public String getBestPassage(IndexWrapperQuery query, String content) {
		throw new RuntimeException(
				"MockIndexWrapper.getBestPassage() not implemented.");
	}

	@Override
	public void deleteDocument(SearchDocument doc) throws IOException {
		throw new RuntimeException(
				"MockIndexWrapper.deleteDocument() not implemented.");
	}

	@Override
	public void writeDocument(SearchDocument doc) throws IOException {
		throw new RuntimeException(
				"MockIndexWrapper.writeDocument() not implemented.");
	}

	@Override
	public void synchronize() throws IOException {
		throw new RuntimeException(
				"MockIndexWrapper.synchronize() not implemented.");
	}

	@Override
	public void setListener(IndexWrapperListener listener) {
		throw new RuntimeException(
				"MockIndexWrapper.setListener() not implemented.");
	}

	@Override
	public IndexSummaryInfo getIndexInfo() throws IOException {
		throw new RuntimeException(
				"MockIndexWrapper.getIndexInfo() not implemented.");
	}

}
