/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search;

import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_SUBJECT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_TITLE;

import java.lang.reflect.Field;
import java.text.ParseException;
import java.util.Date;

import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.search.lucene.searchdoc.SearchDocumentMetadata;
import org.nsdl.search.processoai.OaiCollectionInfo;
import org.nsdl.searchtools.common.DateUtil;

/**
 * 
 * @author jblake
 */
public class IndexContents {
	private static final String DOC1_ID = "http://doc1/";
	private static final String DOC41_ID = "http://fourway.doc41/";
	private static final String DOC42_ID = "http://fourway.doc42/";
	private static final String DOC43_ID = "http://fourway.doc43/";
	private static final String DOC44_ID = "http://fourway.doc44/";
	private static final String TWOFER_ID = "http://my.test.site/twofer";

	/**
	 * Doc1 - a simple document with one metadata source.
	 */
	public static SearchDocument getInitialDoc1() {
		OaiCollectionInfo collInfo = new OaiCollectionInfo("oai:coll1a",
				"http://image.doc1a", "120", "25", "Image for doc1, metadataA");

		SearchDocumentMetadata metadata = new SearchDocumentMetadata(
				"oai:doc1a", DOC1_ID);
		metadata.setCollInfo(collInfo);

		SearchDocument document = new SearchDocument(DOC1_ID);
		document.addMetadataSource(metadata);
		setDocumentLastUpdated(document, new Date(1000));
		setDocumentUnchanged(document);
		return document;
	}

	private IndexContents() {
		// nothing to instantiate.
	}

	/**
	 * Doc1 after we add a second metadata source.
	 */
	public static SearchDocument getExpectedDoc1A() {
		SearchDocument document = getInitialDoc1();
		document.addMetadataSource(getMetadata1B());
		setDocumentLastUpdated(document, DateUtil.getInstance().now());
		return document;
	}

	/**
	 * Doc1 with first metadata deleted and second one added.
	 */
	public static SearchDocument getExpectedDoc1B() {
		SearchDocument document = new SearchDocument(DOC1_ID);
		document.addMetadataSource(getMetadata1B());
		setDocumentLastUpdated(document, DateUtil.getInstance().now());
		return document;
	}

	private static SearchDocumentMetadata getMetadata1B() {
		OaiCollectionInfo collInfo = new OaiCollectionInfo("oai:coll1b",
				"http://image.doc1b", "120", "25", "Image for doc1, metadataB");
		SearchDocumentMetadata metadata = new SearchDocumentMetadata(
				"oai:doc1b", DOC1_ID);
		metadata.setCollInfo(collInfo);
		metadata.setOaiDateStamp(createDate("2006-11-01T15:00:18Z"));
		return metadata;
	}

	/**
	 * Create a new document with two metadata sources.
	 */
	public static SearchDocument getNewDocWithTwoMetadatas() {
		OaiCollectionInfo collInfo = new OaiCollectionInfo(
				"oai:twofer.collection", "http://image.twofer.collection",
				"99", "17", "Image for twofer collection");

		SearchDocumentMetadata metadata1 = new SearchDocumentMetadata(
				"oai:twofer.first", TWOFER_ID);
		metadata1.setCollInfo(collInfo);
		metadata1.setOaiDateStamp(createDate("2005-01-01T20:00:11Z"));

		SearchDocumentMetadata metadata2 = new SearchDocumentMetadata(
				"oai:twofer.second", TWOFER_ID);
		metadata2.setCollInfo(collInfo);
		metadata2.setOaiDateStamp(createDate("2003-01-01T20:00:11Z"));

		SearchDocument document = new SearchDocument(TWOFER_ID);
		document.addMetadataSource(metadata1);
		document.addMetadataSource(metadata2);
		setDocumentLastUpdated(document, DateUtil.getInstance().now());
		return document;
	}

	// ------------------------------------------------------------------------
	// Four-way tests
	//
	// Doc41 -- MD4A
	// Doc42 -- MD4A, MD4B
	// Doc43 -- MD4A
	// Doc44 -- MD4A, MD4B
	//
	// Setup, Apply new MD4A, which now feeds only Doc41 and Doc42
	// Setup, Apply new Delete for MD4A
	// ------------------------------------------------------------------------

	/**
	 * Set up for the Four-way test.
	 */
	public static SearchDocument[] getFourWaySetup() {
		SearchDocument doc41 = new SearchDocument(DOC41_ID);
		doc41.addMetadataSource(getOldFourWayMetadata4A(DOC41_ID));

		SearchDocument doc42 = new SearchDocument(DOC42_ID);
		doc42.addMetadataSource(getOldFourWayMetadata4A(DOC42_ID));
		doc42.addMetadataSource(getFourWayMetadata4B(DOC42_ID));

		SearchDocument doc43 = new SearchDocument(DOC43_ID);
		doc43.addMetadataSource(getOldFourWayMetadata4A(DOC43_ID));

		SearchDocument doc44 = new SearchDocument(DOC44_ID);
		doc44.addMetadataSource(getOldFourWayMetadata4A(DOC44_ID));
		doc44.addMetadataSource(getFourWayMetadata4B(DOC44_ID));

		return new SearchDocument[]{doc41, doc42, doc43, doc44};
	}

	/**
	 * The result of replacing MD4A in the Four-way test.
	 */
	public static SearchDocument[] getFourWayReplaceResult() {
		SearchDocument doc41 = new SearchDocument(DOC41_ID);
		doc41.addMetadataSource(getNewFourWayMetadata4A(DOC41_ID));
		setDocumentLastUpdated(doc41, DateUtil.getInstance().now());

		SearchDocument doc42 = new SearchDocument(DOC42_ID);
		doc42.addMetadataSource(getNewFourWayMetadata4A(DOC42_ID));
		doc42.addMetadataSource(getFourWayMetadata4B(DOC42_ID));
		setDocumentLastUpdated(doc42, DateUtil.getInstance().now());

		SearchDocument doc44 = new SearchDocument(DOC44_ID);
		doc44.addMetadataSource(getFourWayMetadata4B(DOC44_ID));
		setDocumentLastUpdated(doc44, DateUtil.getInstance().now());

		return new SearchDocument[]{doc41, doc42, doc44};
	}

	/**
	 * The result of deleting MD4A in the Four-way test.
	 */
	public static SearchDocument[] getFourWayDeleteResult() {
		SearchDocument doc42 = new SearchDocument(DOC42_ID);
		doc42.addMetadataSource(getFourWayMetadata4B(DOC42_ID));
		setDocumentLastUpdated(doc42, DateUtil.getInstance().now());

		SearchDocument doc44 = new SearchDocument(DOC44_ID);
		doc44.addMetadataSource(getFourWayMetadata4B(DOC44_ID));
		setDocumentLastUpdated(doc44, DateUtil.getInstance().now());

		return new SearchDocument[]{doc42, doc44};
	}

	/** Metadatas in the 4-way test apply to more than one URL. */
	public static SearchDocumentMetadata getOldFourWayMetadata4A(String url) {
		SearchDocumentMetadata metadata = new SearchDocumentMetadata(
				"oai:md4A", url);
		metadata.setCollInfo(getFourWayCollectionInfo());
		metadata.addFieldValue(DC_TITLE, "OldMD4A");
		return metadata;
	}

	public static SearchDocumentMetadata getNewFourWayMetadata4A(String url) {
		SearchDocumentMetadata metadata = new SearchDocumentMetadata(
				"oai:md4A", url);
		metadata.setOaiDateStamp(createDate("2007-09-01T15:00:55Z"));
		metadata.setCollInfo(getFourWayCollectionInfo());
		metadata.addFieldValue(DC_SUBJECT, "NewMD4A");
		return metadata;
	}

	public static SearchDocumentMetadata getFourWayMetadata4B(String url) {
		SearchDocumentMetadata metadata = new SearchDocumentMetadata(
				"oai:md4B", url);
		metadata.setCollInfo(getFourWayCollectionInfo());
		return metadata;
	}

	/** Both metadatas in the 4-way test are from the same collection. */
	public static OaiCollectionInfo getFourWayCollectionInfo() {
		return new OaiCollectionInfo("oai:coll4Way", "http://image.coll4Way",
				"120", "25", "Image for the 4Way collection");
	}

	// ------------------------------------------------------------------------
	// Utilities.
	// ------------------------------------------------------------------------

	private static Date createDate(String dateString) {
		try {
			return DateUtil.getInstance().parseDate(dateString,
					DateUtil.ISO8601_DATE_TIME_Z);
		} catch (ParseException e) {
			throw new IllegalArgumentException("Can't parse this date: '"
					+ dateString + "'");
		}
	}

	private static void setDocumentLastUpdated(SearchDocument doc, Date date) {
		try {
			Field dluField = SearchDocument.class
					.getDeclaredField("documentLastUpdated");
			dluField.setAccessible(true);
			dluField.set(doc, date);
		} catch (SecurityException e) {
			throw new RuntimeException("Failed to set 'documentLastUpdated'", e);
		} catch (NoSuchFieldException e) {
			throw new RuntimeException("Failed to set 'documentLastUpdated'", e);
		} catch (IllegalArgumentException e) {
			throw new RuntimeException("Failed to set 'documentLastUpdated'", e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException("Failed to set 'documentLastUpdated'", e);
		}
	}

	private static void setDocumentUnchanged(SearchDocument doc) {
		try {
			Field chField = SearchDocument.class.getDeclaredField("changed");
			chField.setAccessible(true);
			chField.set(doc, Boolean.FALSE);
		} catch (SecurityException e) {
			throw new RuntimeException("Failed to set 'documentLastUpdated'", e);
		} catch (NoSuchFieldException e) {
			throw new RuntimeException("Failed to set 'documentLastUpdated'", e);
		} catch (IllegalArgumentException e) {
			throw new RuntimeException("Failed to set 'documentLastUpdated'", e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException("Failed to set 'documentLastUpdated'", e);
		}
	}

}
