/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.fail;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DOCUMENT_CREATED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DOCUMENT_LAST_UPDATED;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.Reader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import junit.framework.AssertionFailedError;

import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.WriterAppender;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexReader;
import org.junit.After;
import org.junit.Before;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;
import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.searchtools.common.ListsMap;

/**
 * Parent class for unit tests on the search classes.
 * 
 * @author jblake
 */
public abstract class SearchUnitTest {
	private static final Set<LuceneFieldSpec> freeContentFields = new HashSet<LuceneFieldSpec>(
			Arrays.asList(new LuceneFieldSpec[] { DOCUMENT_CREATED,
					DOCUMENT_LAST_UPDATED }));

	protected ByteArrayOutputStream mockSystemOut;

	protected PrintStream saveSystemOut;

	protected ByteArrayOutputStream mockSystemError;

	protected PrintStream saveSystemError;

	/**
	 * Override this in the test class if you want to mock something that uses
	 * the {@link org.nsdl.searchtools.common.InstanceFactory InstanceFactory}
	 * pattern.
	 */
	protected Class<?>[][] getImplementationClasses() {
		return new Class[0][2];
	}

	/**
	 * Override this in the test class if you want to see the output at the
	 * console.
	 */
	protected boolean isOutputDumped() {
		return false;
	}

	@Before
	public void setupImplementationClasses() {
		Class<?>[][] mappings = getImplementationClasses();
		for (Class<?>[] mapping : mappings) {
			String key = mapping[0].getName() + ".implementation";
			String value = mapping[1].getName();
			System.setProperty(key, value);
		}
	}

	@After
	public void teardownImplementationClasses() {
		Class<?>[][] mappings = getImplementationClasses();
		for (Class<?>[] mapping : mappings) {
			String key = mapping[0].getName() + ".implementation";
			System.clearProperty(key);
		}
	}

	/**
	 * Trap any system output so we can inspect it, and so it doesn't show up on
	 * the console.
	 */
	@Before
	public void createMockSystemOut() {
		saveSystemOut = System.out;
		mockSystemOut = new ByteArrayOutputStream();
		System.setOut(new PrintStream(mockSystemOut));
	}

	@After
	public void restoreSystemOut() {
		System.setOut(saveSystemOut);
		if (isOutputDumped()) {
			System.out.println(mockSystemOut);
		}
	}

	private StringWriter logWriter;

	/**
	 * Initialize the Log4J system with an appender we can read.
	 */
	@Before
	public void initializeLog4J() {
		Logger rootLogger = Logger.getRootLogger();
		rootLogger.removeAllAppenders();
		logWriter = new StringWriter();
		rootLogger.addAppender(new WriterAppender(new PatternLayout(),
				logWriter));
	}

	protected String getLogOutput() {
		return logWriter.toString();
	}

	/**
	 * Trap any error messages so we can inspect them, and so they don't show up
	 * on the console.
	 */
	@Before
	public void createMockSystemErr() {
		saveSystemError = System.err;
		mockSystemError = new ByteArrayOutputStream();
		System.setErr(new PrintStream(mockSystemError));
	}

	@After
	public void restoreSystemErr() {
		System.setErr(saveSystemError);
		if (isOutputDumped()) {
			System.err.println(mockSystemError);
		}
	}

	/**
	 * Read a config file into a String, so it can be manipulated.
	 */
	protected static String slurpEntireFile(String filename) throws IOException {
		FileReader reader = null;
		try {
			reader = new FileReader(filename);
			StringBuffer result = new StringBuffer();
			char[] buffer = new char[4096];
			while (true) {
				int howMany = reader.read(buffer);
				if (howMany == -1) {
					break;
				} else {
					result.append(buffer, 0, howMany);
				}
			}
			return result.toString();
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					fail("failed to close reader." + e);
				}
			}
		}
	}

	/**
	 * Read a config file into a String, so it can be manipulated.
	 */
	protected static String slurpEntireResource(String resourceName)
			throws IOException {
		InputStream stream = null;
		try {
			stream = ClassLoader.getSystemResourceAsStream(resourceName);
			Reader reader = new InputStreamReader(stream);

			StringBuffer result = new StringBuffer();
			char[] buffer = new char[4096];
			while (true) {
				int howMany = reader.read(buffer);
				if (howMany == -1) {
					break;
				} else {
					result.append(buffer, 0, howMany);
				}
			}
			return result.toString();
		} finally {
			if (stream != null) {
				try {
					stream.close();
				} catch (IOException e) {
					fail("failed to close resource." + e);
				}
			}
		}
	}

	protected void assertEqualLuceneDocuments(Document expected, Document actual) {
		List<String> errors = new ArrayList<String>();

		ListsMap<LuceneFieldSpec, String> expectedFields = buildFieldMap(
				expected, errors);
		ListsMap<LuceneFieldSpec, String> actualFields = buildFieldMap(actual,
				errors);

		removeAnyMatchingFields(expectedFields, actualFields, errors);

		complainAboutMismatchedFields(expectedFields, actualFields, errors);

		if (!errors.isEmpty()) {
			for (String error : errors) {
				System.out.println(error);
			}
			fail(String.valueOf(errors));
		}
	}

	private ListsMap<LuceneFieldSpec, String> buildFieldMap(Document doc,
			List<String> errors) {
		ListsMap<LuceneFieldSpec, String> map = new ListsMap<LuceneFieldSpec, String>();
		for (Enumeration<Field> fields = getFieldsFromDocument(doc); fields
				.hasMoreElements();) {
			Field field = fields.nextElement();
			LuceneFieldSpec spec = SearchIndexFieldSpecs.getSpec(field.name());
			if (spec == null) {
				errors.add("No spec found for '" + field.name() + "'");
			} else {
				confirmProperSpec(spec, field, errors);
				map.addValue(spec, field.stringValue());
			}
		}
		return map;
	}

	/**
	 * We found a spec that matches the field name, but what about other field
	 * attributes?
	 */
	private void confirmProperSpec(LuceneFieldSpec spec, Field field,
			List<String> errors) {
		String name = spec.getName();
		try {
			assertEquals("isStored: " + name,
					(spec.getStore() == Field.Store.YES), field.isStored());
			assertEquals("isIndexed: " + name,
					(spec.getIndex() == Field.Index.TOKENIZED), field
							.isIndexed());
		} catch (AssertionFailedError e) {
			errors.add(e.getMessage() + ", value=" + field.stringValue());
		}
	}

	/**
	 * @param expectedFields
	 * @param actualFields
	 * @param errors
	 */
	private void removeAnyMatchingFields(
			ListsMap<LuceneFieldSpec, String> expectedFields,
			ListsMap<LuceneFieldSpec, String> actualFields, List<String> errors) {
		Set<LuceneFieldSpec> matches = new HashSet<LuceneFieldSpec>();
		for (LuceneFieldSpec field : expectedFields.keySet()) {
			if (!actualFields.containsKey(field)) {
				continue;
			}

			List<String> expectedValues = normalizeFieldValues(expectedFields
					.get(field));
			List<String> actualValues = normalizeFieldValues(actualFields
					.get(field));
			if (freeContentFields.contains(field)) {
				if (compareFreeContentFields(field, expectedValues,
						actualValues)) {
					matches.add(field);
				}
			} else if (field.isDerived()) {
				if (compareTokenSets(expectedValues, actualValues, errors)) {
					matches.add(field);
				}
			} else {
				if (compareValues(expectedValues, actualValues)) {
					matches.add(field);
				}
			}
		}

		for (LuceneFieldSpec match : matches) {
			expectedFields.remove(match);
			actualFields.remove(match);
		}
	}

	/**
	 * These fields don't even have to match - just be there.
	 */
	@SuppressWarnings("unused")
	private boolean compareFreeContentFields(LuceneFieldSpec field,
			List<String> expectedValues, List<String> actualValues) {
		return true;
	}

	/**
	 * For non-compound fields, each value should match each other.
	 */
	private boolean compareValues(List<String> expected, List<String> actual) {
		Collections.sort(expected);
		Collections.sort(actual);
		return expected.equals(actual);
	}

	/**
	 * For compound fields, the match is not as strict. Split them into tokens,
	 * and the sets of tokens should match.
	 */
	private boolean compareTokenSets(List<String> expected,
			List<String> actual, List<String> errors) {
		Map<String, Integer> expectedMap = buildTokenMap(expected);
		Map<String, Integer> actualMap = buildTokenMap(actual);
		boolean match = expectedMap.equals(actualMap);

		if (!match) {
			for (Iterator<String> tokens = expectedMap.keySet().iterator(); tokens
					.hasNext();) {
				String token = tokens.next();
				if (actualMap.containsKey(token)
						&& actualMap.get(token).equals(expectedMap.get(token))) {
					tokens.remove();
					actualMap.remove(token);
				}
			}
			errors.add("EXPECTED: " + expectedMap);
			errors.add("ACTUAL:   " + actualMap);
		}
		return match;
	}

	/**
	 * Take a list of values, break into tokens, and create a map showing how
	 * many times each token appears in the values. Ignore any single-character
	 * tokens, assuming they are metadata indexes.
	 */
	private Map<String, Integer> buildTokenMap(List<String> values) {
		Map<String, Integer> map = new TreeMap<String, Integer>();
		for (String value : values) {
			for (String token : value.split(" ")) {
				if (token.length() > 1) {
					if (!map.containsKey(token)) {
						map.put(token, 0);
					}
					map.put(token, map.get(token) + 1);
				}
			}
		}
		return map;
	}

	/**
	 * If any fields remain in these maps, they must be errors, so complain
	 * about them.
	 */
	private void complainAboutMismatchedFields(
			ListsMap<LuceneFieldSpec, String> expectedFields,
			ListsMap<LuceneFieldSpec, String> actualFields, List<String> errors) {

		for (LuceneFieldSpec field : expectedFields.keySet()) {
			errors.add("missing field: " + field.getName() + " ==> "
					+ normalizeFieldValues(expectedFields.get(field)));
		}
		for (LuceneFieldSpec field : actualFields.keySet()) {
			errors.add("extra field: " + field.getName() + " ==> "
					+ normalizeFieldValues(actualFields.get(field)));
		}
	}

	/**
	 * Return a list of the individual values for a set of fields, disregarding
	 * the way the values were combined into fields, and ignoring the metadata
	 * indexes.
	 */
	private List<String> normalizeFieldValues(List<String> rawValues) {
		List<String> result = new ArrayList<String>();
		for (String rawValue : rawValues) {
			String[] values = rawValue.split(SearchDocument.SEPARATOR_PATTERN);
			for (String value : values) {
				// if a value is a single character we'll call it a metadata
				// index and ignore it.
				if (value.length() == 1) {
					continue;
				}
				result.add(value);
			}
		}
		return result;
	}

	/**
	 * Get all of the documents in the search index, mapped by their IDs. Use a
	 * {@link  ListsMap} to allow for the chance that there might be two records
	 * with the same ID.
	 */
	protected ListsMap<String, SearchDocument> getContentsOfLuceneIndex(
			String indexDirectoryPath) throws IOException {
		List<SearchDocument> actualDocs = new ArrayList<SearchDocument>();

		IndexReader reader = IndexReader.open(indexDirectoryPath);
		for (int n = 0; n < reader.maxDoc(); n++) {
			if (!reader.isDeleted(n)) {
				actualDocs.add(new SearchDocument(reader.document(n)));
			}
		}
		reader.close();

		return buildDocumentMap(actualDocs);
	}

	/**
	 * Put a collection of {@link SearchDocument}s into a {@link ListsMap},
	 * keyed by IDs.
	 */
	protected ListsMap<String, SearchDocument> buildDocumentMap(
			Collection<SearchDocument> docs) {
		ListsMap<String, SearchDocument> map = new ListsMap<String, SearchDocument>();
		for (SearchDocument doc : docs) {
			map.addValue(doc.getIdentifier(), doc);
		}
		return map;
	}

	/**
	 * Erase all the files in this directory. Don't go recursive.
	 */
	protected void cleanDirectory(File directory) {
		File[] files = directory.listFiles();
		for (File file : files) {
			file.delete();
		}
	}

	/**
	 * Erase all the files in this directory, and all subdirectories.
	 */
	protected void deepCleanDirectory(File directory) {
		if (directory == null) {
			return;
		}

		if (!directory.exists()) {
			return;
		}

		if (directory.isDirectory()) {
			File[] files = directory.listFiles();
			for (File file : files) {
				if (file.isDirectory()) {
					deepCleanDirectory(file);
				}
				file.delete();
			}
		}
	}

	/**
	 * Isolate the unchecked cast.
	 */
	@SuppressWarnings("unchecked")
	protected Enumeration<Field> getFieldsFromDocument(Document doc) {
		return doc.fields();
	}

}
