/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.collContext;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.fail;
import static org.nsdl.search.collContext.CollectionContextFieldSpecs.CC_DCT_ALTERNATIVE;
import static org.nsdl.search.collContext.CollectionContextFieldSpecs.CC_DCT_EDLEVEL_NSDL_EDLEVEL;
import static org.nsdl.search.collContext.CollectionContextFieldSpecs.CC_DCT_MEDIATOR;
import static org.nsdl.search.collContext.CollectionContextFieldSpecs.CC_DC_DESCRIPTION;
import static org.nsdl.search.collContext.CollectionContextFieldSpecs.CC_DC_RIGHTS;
import static org.nsdl.search.collContext.CollectionContextFieldSpecs.CC_DC_TITLE;

import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.junit.Before;
import org.junit.Test;
import org.nsdl.search.SearchUnitTest;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.searchtools.common.ListsMap;
import org.nsdl.searchtools.common.xml.XmlUtil;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Test the CollectionFieldsCache.
 * 
 * @author jblake
 */
public class TestCollRecCache extends SearchUnitTest {
	private MyCollectionFieldsCache cache;

	@Before
	public void createTheCache() {
		cache = new MyCollectionFieldsCache();
	}

	/**
	 * Can't find a collection with a null OAI ID -- won't even try.
	 */
	@Test
	public void tryNullId() {
		ListsMap<LuceneFieldSpec, String> actualFields = cache
				.getCollContextFieldInfos(null);
		assertEquals("expecting no results.", 0, actualFields.size());
		assertEquals("expecting no fetches.", 0, cache.getFetches());
	}

	/**
	 * Treat a badly formed OAI ID like any other - can't find it!
	 */
	@Test
	public void tryInvalidOaiId() {
		ListsMap<LuceneFieldSpec, String> actualFields = cache
				.getCollContextFieldInfos("bogusRetrievalOaiId");
		assertEquals("expecting no results.", 0, actualFields.size());
		assertEquals("expecting one fetch.", 1, cache.getFetches());

		// won't try to fetch it again.
		actualFields = cache.getCollContextFieldInfos("bogusRetrievalOaiId");
		assertEquals("expecting no results.", 0, actualFields.size());
		assertEquals("expecting one fetch.", 1, cache.getFetches());
	}

	/**
	 * Don't fetch the collection of collections, nor return any fields.
	 */
	@Test
	public void tryCollectionOfCollections() {
		String oaiId = CollectionContextConstants.COLL_OF_COLL_OAI_ID;
		ListsMap<LuceneFieldSpec, String> actualFields = cache
				.getCollContextFieldInfos(oaiId);
		assertEquals("expecting no results.", 0, actualFields.size());
		assertEquals("expecting no fetches.", 0, cache.getFetches());
	}

	/**
	 * If we don't find a collection, return an empty set of fields, but cache
	 * the result so we don't try to fetch it again.
	 */
	@Test
	public void getNonexistentCollection() {
		String oaiId = "oai:nsdl.org:crs:0000";
		ListsMap<LuceneFieldSpec, String> actualFields = cache
				.getCollContextFieldInfos(oaiId);
		assertEquals("expecting no results.", 0, actualFields.size());
		assertEquals("expecting one fetch.", 1, cache.getFetches());

		// won't try to fetch it again.
		actualFields = cache.getCollContextFieldInfos(oaiId);
		assertEquals("expecting no results.", 0, actualFields.size());
		assertEquals("expecting one fetch.", 1, cache.getFetches());
	}

	/**
	 * If we ask repeatedly for a collection, only fetch it once.
	 */
	@Test
	public void getArxivCollection() throws IOException {
		String oaiId = "oai:nsdl.org:crs:5101";
		String filename = "testfiles/collectionOaiRecords/arxiv.oai.xml";
		cache.loadCollectionOai(oaiId, slurpEntireFile(filename));

		ListsMap<LuceneFieldSpec, String> expectedFields = new ListsMap<LuceneFieldSpec, String>();
		expectedFields.addValue(CC_DCT_EDLEVEL_NSDL_EDLEVEL, "Graduate");
		expectedFields.addValue(CC_DCT_EDLEVEL_NSDL_EDLEVEL,
				"Undergraduate (Upper Division)");
		expectedFields.addValue(CC_DC_DESCRIPTION, "ArXiv.org, "
				+ "the major e-Print archive in the areas of "
				+ "physics, computer science, mathematics and "
				+ "related disciplines, makes available "
				+ "online research articles in relevant "
				+ "disciplines prior to print publication. "
				+ "Articles are submitted by their authors, "
				+ "and are then available freely, in various formats.");
		expectedFields.addValue(CC_DC_TITLE, "arXiv.org e-Print archive");

		ListsMap<LuceneFieldSpec, String> actualFields = cache
				.getCollContextFieldInfos(oaiId);
		assertCorrectFields(expectedFields, actualFields);
		assertEquals("expecting one fetch.", 1, cache.getFetches());

		// make sure that it is cached and not fetched again.
		actualFields = cache.getCollContextFieldInfos(oaiId);
		assertCorrectFields(expectedFields, actualFields);
		assertEquals("expecting one fetch.", 1, cache.getFetches());
	}

	@Test
	public void getAssortedRecords() throws IOException {
		String oaiId1 = "oai:nsdl.org:crs:498742";
		String filename1 = "testfiles/collectionOaiRecords/citeSeer.oai.xml";
		cache.loadCollectionOai(oaiId1, slurpEntireFile(filename1));

		ListsMap<LuceneFieldSpec, String> expectedFields1 = new ListsMap<LuceneFieldSpec, String>();
		expectedFields1.addValue(CC_DCT_EDLEVEL_NSDL_EDLEVEL, "Postsecondary");
		expectedFields1.addValue(CC_DC_DESCRIPTION, "CiteSeer is a "
				+ "scientific literature digital library that aims to "
				+ "improve the dissemination and feedback of "
				+ "scientific literature, and to provide improvements "
				+ "in functionality, usability, availability, cost, "
				+ "comprehensiveness, efficiency, and timeliness. "
				+ "Rather than creating just another digital library, "
				+ "CiteSeer provides algorithms, techniques, and "
				+ "software that can be used in other digital libraries. ");
		expectedFields1.addValue(CC_DC_TITLE,
				"CiteSeer.IST: Scientific Literature Digital Library");
		expectedFields1.addValue(CC_DCT_ALTERNATIVE,
				"Computer and Information Science Papers "
						+ "CiteSeer Publications ResearchIndex");

		String oaiId2 = "oai:nsdl.org:crs:4794";
		String filename2 = "testfiles/collectionOaiRecords/mathForum.oai.xml";
		cache.loadCollectionOai(oaiId2, slurpEntireFile(filename2));

		ListsMap<LuceneFieldSpec, String> expectedFields2 = new ListsMap<LuceneFieldSpec, String>();
		expectedFields2.addValue(CC_DCT_EDLEVEL_NSDL_EDLEVEL,
				"Grades Pre-K to 12");
		expectedFields2.addValue(CC_DCT_EDLEVEL_NSDL_EDLEVEL, "Postsecondary");
		expectedFields2.addValue(CC_DCT_MEDIATOR, "Teachers");
		expectedFields2.addValue(CC_DC_DESCRIPTION, "The Math Forum is the "
				+ "comprehensive resource for math education on the "
				+ "Internet. Some features include a K-12 math expert "
				+ "service, an extensive database of math sites, "
				+ "online resources for teaching and learning math, "
				+ "plus much more.");
		expectedFields2.addValue(CC_DC_TITLE, "Math Forum");

		String oaiId3 = "oai:nsdl.org:crs:296577";
		String filename3 = "testfiles/collectionOaiRecords/mathWorld.oai.xml";
		cache.loadCollectionOai(oaiId3, slurpEntireFile(filename3));

		ListsMap<LuceneFieldSpec, String> expectedFields3 = new ListsMap<LuceneFieldSpec, String>();
		expectedFields3.addValue(CC_DCT_EDLEVEL_NSDL_EDLEVEL, "Middle School");
		expectedFields3.addValue(CC_DCT_EDLEVEL_NSDL_EDLEVEL, "High School");
		expectedFields3.addValue(CC_DCT_EDLEVEL_NSDL_EDLEVEL, "Postsecondary");
		expectedFields3.addValue(CC_DC_DESCRIPTION, "Eric Weisstein's "
				+ "World of Mathematics (MathWorld) is the Web's most "
				+ "complete mathematical resource, assembled over "
				+ "more than a decade by internet encyclopedist Eric "
				+ "W. Weisstein with assistance from the mathematics "
				+ "and internet communities. MathWorld is a "
				+ "comprehensive and interactive mathematics "
				+ "encyclopedia intended for students, educators, "
				+ "math enthusiasts, and researchers.  It contains "
				+ "more than 11,000 detailed entries covering "
				+ "virtually all area of mathematics.  MathWorld is a "
				+ "free service for the mathematical community "
				+ "provided by Wolfram Research, makers of Mathematica.");
		expectedFields3.addValue(CC_DC_RIGHTS, "Copyright 1999-2003 "
				+ "Wolfram Research, Inc. See "
				+ "http://mathworld.wolfram.com/terms.html for a full "
				+ "terms of use statement.");
		expectedFields3.addValue(CC_DC_TITLE,
				"Eric Weisstein's World of Mathematics");
		expectedFields3.addValue(CC_DCT_ALTERNATIVE, "World of Mathematics");

		ListsMap<LuceneFieldSpec, String> actualFields1 = cache
				.getCollContextFieldInfos(oaiId1);
		assertCorrectFields(expectedFields1, actualFields1);

		ListsMap<LuceneFieldSpec, String> actualFields2 = cache
				.getCollContextFieldInfos(oaiId2);
		assertCorrectFields(expectedFields2, actualFields2);

		ListsMap<LuceneFieldSpec, String> actualFields3 = cache
				.getCollContextFieldInfos(oaiId3);
		assertCorrectFields(expectedFields3, actualFields3);

		assertEquals("expecting one fetch for each.", 3, cache.getFetches());
	}

	/**
	 * Did we get what we expected?
	 */
	private void assertCorrectFields(
			ListsMap<LuceneFieldSpec, String> expectedFields,
			ListsMap<LuceneFieldSpec, String> actualFields) {

		String message = "";

		Set<LuceneFieldSpec> commonKeys = new HashSet<LuceneFieldSpec>(
				actualFields.keySet());
		commonKeys.retainAll(expectedFields.keySet());
		for (LuceneFieldSpec spec : commonKeys) {
			List<String> expectedValues = expectedFields.get(spec);
			List<String> actualValues = actualFields.get(spec);
			if (!expectedValues.equals(actualValues)) {
				message += "For field '" + spec.getName() + "', expected "
						+ expectedValues + ", but was " + actualValues + "\n";
			}
		}

		Set<LuceneFieldSpec> missingKeys = new HashSet<LuceneFieldSpec>(
				expectedFields.keySet());
		missingKeys.removeAll(commonKeys);
		for (LuceneFieldSpec spec : missingKeys) {
			message += "Missing field '" + spec.getName() + "' -- "
					+ expectedFields.get(spec) + "\n";
		}

		Set<LuceneFieldSpec> extraKeys = new HashSet<LuceneFieldSpec>(
				actualFields.keySet());
		extraKeys.removeAll(commonKeys);
		for (LuceneFieldSpec spec : extraKeys) {
			message += "Extra field '" + spec.getName() + "' -- "
					+ actualFields.get(spec) + "\n";
		}

		if (message.length() > 0) {
			fail(message);
		}
	}

	/**
	 * A subclass of OnDemandCache that can be force-fed a set of OAI records.
	 */
	private static class MyCollectionFieldsCache extends CollectionFieldsCache {
		private Map<String, String> loadedOaiRecords = new HashMap<String, String>();

		private int fetches;

		public MyCollectionFieldsCache() {
			super();
		}

		public void loadCollectionOai(String oaiId, String oaiRecord) {
			loadedOaiRecords.put(oaiId, oaiRecord);
		}

		public Object getFetches() {
			return fetches;
		}

		@Override
		protected Document getCollectionOaiRecordAsDocument(String oaiId) {
			fetches++;
			if (loadedOaiRecords.containsKey(oaiId)) {
				try {
					String oaiRecord = loadedOaiRecords.get(oaiId);
					DocumentBuilderFactory factory = DocumentBuilderFactory
							.newInstance();
					factory.setAttribute(XmlUtil.NAMESPACES_FEATURE_ID, true);
					DocumentBuilder builder = factory.newDocumentBuilder();
					InputSource source = new InputSource(new StringReader(
							oaiRecord));
					return builder.parse(source);
				} catch (ParserConfigurationException e) {
					throw new IllegalStateException(e);
				} catch (SAXException e) {
					throw new IllegalStateException(e);
				} catch (IOException e) {
					throw new IllegalStateException(e);
				}
			} else {
				return null;
			}
		}
	}
}
