/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.collContext;

import static junit.framework.Assert.fail;
import static org.nsdl.search.util.XmlNamespaceUtil.NS_NSDL_DC;
import static org.nsdl.search.util.XmlNamespaceUtil.NS_OAI;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.junit.Before;
import org.junit.Test;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.processoai.XmlException;
import org.nsdl.searchtools.common.ListsMap;
import org.nsdl.searchtools.common.xml.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * Test the {@link CollectionOaiParser}. We can imagine other tests besides
 * these. For example:
 * <ul>
 * <li>only native_nsdl_dc</li>
 * <li>match against something whose "type" namespace is not recognized. The
 * namespace uri would appear in the document header, but would not be known to
 * {@link org.nsdl.search.util.XmlNamespaceUtil XmlNamespaceUtil}</li>
 * <li>unrecognized NSDL_DC element: fail to match, based on name, nameNS,
 * type, or typeNS</li>
 * </ul>
 * 
 * @author jblake
 */

public class TestCollectionOaiParser {
	private static final String OAI_ID = "oai:nsdl.org:crs:498742";

	private Document collectionOai;
	private Document deletedCollection;
	private Element topElement;
	private Element getRecordElement;
	private Element recordElement;
	private Element metadataElement;
	private Element nsdlDcElement;
	private ListsMap<LuceneFieldSpec, String> result;
	private ListsMap<LuceneFieldSpec, String> expectedFields;
	private ListsMap<LuceneFieldSpec, String> emptyResult;

	@Before
	public void initializeCollectionOai() throws ParserConfigurationException,
			SAXException, IOException {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);
		DocumentBuilder builder = factory.newDocumentBuilder();
		collectionOai = builder.parse(new File(
				"testfiles/collectionOaiRecords/citeSeer.oai.xml"));
		topElement = collectionOai.getDocumentElement();
		getRecordElement = XmlUtil.getChildByNameNS(topElement, NS_OAI,
				"GetRecord");
		recordElement = XmlUtil.getChildByNameNS(getRecordElement, NS_OAI,
				"record");
		metadataElement = XmlUtil.getChildByNameNS(recordElement, NS_OAI,
				"metadata");
		nsdlDcElement = XmlUtil.getChildByNameNS(metadataElement, NS_NSDL_DC,
				"nsdl_dc");
	}

	@Before
	public void initializeDeletedCollection()
			throws ParserConfigurationException, SAXException, IOException {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);
		DocumentBuilder builder = factory.newDocumentBuilder();
		deletedCollection = builder.parse(new File(
				"testfiles/collectionOaiRecords/deletedCollection.oai.xml"));
	}

	@Before
	public void initializeExpectedFields() {
		expectedFields = new ListsMap<LuceneFieldSpec, String>();
		expectedFields.addValue(CollectionContextFieldSpecs.CC_DC_DESCRIPTION,
				"CiteSeer is a scientific literature digital library that aims "
						+ "to improve the dissemination and feedback of "
						+ "scientific literature, and to provide improvements "
						+ "in functionality, usability, availability, cost, "
						+ "comprehensiveness, efficiency, and timeliness. "
						+ "Rather than creating just another digital library, "
						+ "CiteSeer provides algorithms, techniques, and "
						+ "software that can be used in other digital "
						+ "libraries. ");
		expectedFields.addValue(CollectionContextFieldSpecs.CC_DC_TITLE,
				"CiteSeer.IST: Scientific Literature Digital Library");
		expectedFields.addValue(CollectionContextFieldSpecs.CC_DCT_ALTERNATIVE,
				"Computer and Information Science Papers CiteSeer Publications "
						+ "ResearchIndex");
		expectedFields.addValue(
				CollectionContextFieldSpecs.CC_DCT_EDLEVEL_NSDL_EDLEVEL,
				"Postsecondary");
	}

	@Before
	public void initializeEmptyResult() {
		emptyResult = new ListsMap<LuceneFieldSpec, String>();
	}

	@Test(expected = NullPointerException.class)
	public void nullArgument() throws XmlException {
		new CollectionOaiParser().parse(OAI_ID, (Document) null);
	}

	/**
	 * @param expectedFields2
	 * @param result2
	 */
	private void assertExpectedFields(
			ListsMap<LuceneFieldSpec, String> expected,
			ListsMap<LuceneFieldSpec, String> actual) {

		Set<LuceneFieldSpec> expectedKeys = expected.keySet();
		Set<LuceneFieldSpec> actualKeys = actual.keySet();
		if (!expectedKeys.equals(actualKeys)) {
			Set<LuceneFieldSpec> missingKeys = new HashSet<LuceneFieldSpec>();
			missingKeys.addAll(expectedKeys);
			missingKeys.removeAll(actualKeys);
			Set<LuceneFieldSpec> unexpectedKeys = new HashSet<LuceneFieldSpec>();
			unexpectedKeys.addAll(actualKeys);
			unexpectedKeys.removeAll(expectedKeys);
			fail("Field specs don't match: missing fields: " + missingKeys
					+ ", unexpected fields: " + unexpectedKeys);
		}

		for (LuceneFieldSpec field : actualKeys) {
			List<String> expectedValues = expected.get(field);
			List<String> actualValues = actual.get(field);
			if (!expectedValues.equals(actualValues)) {
				Set<String> missingValues = new HashSet<String>();
				missingValues.addAll(expectedValues);
				missingValues.removeAll(actualValues);
				Set<String> unexpectedValues = new HashSet<String>();
				unexpectedValues.addAll(actualValues);
				unexpectedValues.removeAll(expectedValues);
				fail("Field values don't match for " + field
						+ ": missing values: " + missingValues
						+ ", unexpected values: " + unexpectedValues);
			}
		}
	}

	@Test(expected = XmlException.class)
	public void noTopElement() throws XmlException {
		collectionOai.removeChild(topElement);
		new CollectionOaiParser().parse(OAI_ID, collectionOai);
	}

	@Test(expected = XmlException.class)
	public void topIsNotOaiPmh() throws XmlException {
		collectionOai.removeChild(topElement);
		collectionOai.appendChild(collectionOai.createElement("bogus"));
		new CollectionOaiParser().parse(OAI_ID, collectionOai);
	}

	@Test(expected = XmlException.class)
	public void noGetRecordElement() throws XmlException {
		topElement.removeChild(getRecordElement);
		new CollectionOaiParser().parse(OAI_ID, collectionOai);
	}

	@Test(expected = XmlException.class)
	public void noRecordElement() throws XmlException {
		getRecordElement.removeChild(recordElement);
		new CollectionOaiParser().parse(OAI_ID, collectionOai);
	}

	@Test(expected = XmlException.class)
	public void noMetadataElement() throws XmlException {
		recordElement.removeChild(metadataElement);
		new CollectionOaiParser().parse(OAI_ID, collectionOai);
	}

	@Test
	public void noNsdlDc() throws XmlException {
		metadataElement.removeChild(nsdlDcElement);
		result = new CollectionOaiParser().parse(OAI_ID, collectionOai);
		assertExpectedFields(emptyResult, result);
	}

	@Test
	public void simpleSuccess() throws XmlException {
		result = new CollectionOaiParser().parse(OAI_ID, collectionOai);
		assertExpectedFields(expectedFields, result);
	}

	@Test
	public void successWithDeletedRecord() throws XmlException {
		result = new CollectionOaiParser().parse(OAI_ID, deletedCollection);
		assertExpectedFields(emptyResult, result);
	}

}
