/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.content;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.fail;

import java.io.StringReader;
import java.text.ParseException;
import java.util.Date;

import javax.xml.bind.JAXBException;

import org.junit.Before;
import org.junit.Test;
import org.nsdl.searchtools.common.DateUtil;

/**
 * 
 * @author jblake
 */
public class TestContentCacheResponseParser {
	private static final String BAD_ARGUMENT = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"
			+ "<NSDLContentCache schemaVersion=\"1.00.001\" "
			+ "        xmlns=\"http://ns.nsdl.org/search/contentCache_v1.00/\">\n"
			+ "    <responseTime>2008-02-18T17:03:46Z</responseTime>\n"
			+ "    <request>http://localhost:8280/contentCache/getPage</request>\n"
			+ "    <resultCode code=\"BAD_ARGUMENT\">getPage requires a 'url' parameter.</resultCode>\n"
			+ "</NSDLContentCache>";

	private static final String SYSTEM_PROBLEM = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"
			+ "<NSDLContentCache schemaVersion=\"1.00.001\" "
			+ "        xmlns=\"http://ns.nsdl.org/search/contentCache_v1.00/\">\n"
			+ "    <responseTime>2008-02-18T17:03:46Z</responseTime>\n"
			+ "    <request>http://localhost:8280/contentCache/getPage</request>\n"
			+ "    <resultCode code=\"SYSTEM_PROBLEM\">Unknown problem: unit testing</resultCode>\n"
			+ "</NSDLContentCache>";

	private static final String NOT_IN_CACHE = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"
			+ "<NSDLContentCache schemaVersion=\"1.00.001\" "
			+ "        xmlns=\"http://ns.nsdl.org/search/contentCache_v1.00/\">\n"
			+ "    <responseTime>2008-02-18T18:40:47Z</responseTime>\n"
			+ "    <request>http://localhost:8280/contentCache/getPage?url=http://bogus.url/</request>\n"
			+ "    <resultCode code=\"SUCCESS\"></resultCode>\n"
			+ "    <GetPage>\n"
			+ "        <rawUrl>http://bogus.url/</rawUrl>\n"
			+ "        <normalizedUrl>http://bogus.url/</normalizedUrl>\n"
			+ "\n"
			+ "        <bestFetch status=\"NOT_IN_CACHE\" date=\"1970-01-01T00:00:00Z\"/>\n"
			+ "        <latestFetch status=\"NOT_IN_CACHE\" date=\"1970-01-01T00:00:00Z\"/>\n"
			+ "        <contentType></contentType>\n"
			+ "        <modifyDate>1970-01-01T00:00:00Z</modifyDate>\n"
			+ "        <title></title>\n"
			+ "        <parsedText></parsedText>\n"
			+ "    </GetPage>\n"
			+ "</NSDLContentCache>\n";

	private static final String NOT_FETCHED = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"
			+ "<NSDLContentCache schemaVersion=\"1.00.001\" "
			+ "        xmlns=\"http://ns.nsdl.org/search/contentCache_v1.00/\">\n"
			+ "    <responseTime>2008-02-18T20:19:10Z</responseTime>\n"
			+ "    <request>http://search2.nsdl.org:8080/contentCache/getPage?url=http://expertvoices.nsdl.org</request>\n"
			+ "    <resultCode code=\"SUCCESS\"></resultCode>\n"
			+ "    <GetPage>\n"
			+ "        <rawUrl>http://expertvoices.nsdl.org</rawUrl>\n"
			+ "        <normalizedUrl>http://expertvoices.nsdl.org/</normalizedUrl>\n"
			+ "\n"
			+ "        <bestFetch status=\"NOT_FETCHED\" date=\"1970-01-01T00:00:00Z\"/>\n"
			+ "        <latestFetch status=\"NOT_FETCHED\" date=\"1970-01-01T00:00:00Z\"/>\n"
			+ "        <contentType></contentType>\n"
			+ "        <modifyDate>1970-01-01T00:00:00Z</modifyDate>\n"
			+ "        <title></title>\n"
			+ "        <parsedText></parsedText>\n"
			+ "    </GetPage>\n"
			+ "</NSDLContentCache>\n";

	private static final String FETCH_FAILED = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"
			+ "<NSDLContentCache schemaVersion=\"1.00.001\" "
			+ "        xmlns=\"http://ns.nsdl.org/search/contentCache_v1.00/\">\n"
			+ "    <responseTime>2008-02-18T19:58:49Z</responseTime>\n"
			+ "    <request>http://search2.nsdl.org:8080/contentCache/getPage?url=http://dx.doi.org/10.1007/s00013-006-1788-5</request>\n"
			+ "    <resultCode code=\"SUCCESS\"></resultCode>\n"
			+ "    <GetPage>\n"
			+ "        <rawUrl>http://dx.doi.org/10.1007/s00013-006-1788-5</rawUrl>\n"
			+ "        <normalizedUrl>http://dx.doi.org/10.1007/s00013-006-1788-5</normalizedUrl>\n"
			+ "\n"
			+ "        <bestFetch status=\"FETCH_FAILED_NO_SERVER\" date=\"1970-01-01T00:00:00Z\"/>\n"
			+ "        <latestFetch status=\"FETCH_FAILED_NO_SERVER\" date=\"2008-01-07T22:22:12Z\"/>\n"
			+ "        <contentType></contentType>\n"
			+ "        <modifyDate>1970-01-01T00:00:00Z</modifyDate>\n"
			+ "        <title></title>\n"
			+ "        <parsedText></parsedText>\n"
			+ "    </GetPage>\n"
			+ "</NSDLContentCache>\n";

	private static final String FETCHED = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"
			+ "<NSDLContentCache schemaVersion=\"1.00.001\" "
			+ "        xmlns=\"http://ns.nsdl.org/search/contentCache_v1.00/\">\n"
			+ "    <responseTime>2008-02-18T19:12:35Z</responseTime>\n"
			+ "    <request>http://localhost:8280/contentCache/getPage?url=http://nsdl.org</request>\n"
			+ "    <resultCode code=\"SUCCESS\"></resultCode>\n"
			+ "    <GetPage>\n"
			+ "        <rawUrl>http://nsdl.org</rawUrl>\n"
			+ "        <normalizedUrl>http://nsdl.org/</normalizedUrl>\n"
			+ "\n"
			+ "        <bestFetch status=\"FETCHED\" date=\"2008-02-06T02:36:56Z\"/>\n"
			+ "        <latestFetch status=\"FETCHED\" date=\"2008-02-06T02:36:56Z\"/>\n"
			+ "        <contentType>text/html</contentType>\n"
			+ "        <modifyDate>2008-02-06T02:36:56Z</modifyDate>\n"
			+ "        <title>NSDL.org - The National Science Digital Library</title>\n"
			+ "        <parsedText>Good stuff about the National Science Digital Library</parsedText>\n"
			+ "    </GetPage>\n" + "\n" + "</NSDLContentCache>\n";

	private static final String STALE = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"
			+ "<NSDLContentCache schemaVersion=\"1.00.001\" "
			+ "        xmlns=\"http://ns.nsdl.org/search/contentCache_v1.00/\">\n"
			+ "    <responseTime>2008-02-18T19:12:35Z</responseTime>\n"
			+ "    <request>http://localhost:8280/contentCache/getPage?url=http://nsdl.org</request>\n"
			+ "    <resultCode code=\"SUCCESS\"></resultCode>\n"
			+ "    <GetPage>\n"
			+ "        <rawUrl>http://nsdl.org</rawUrl>\n"
			+ "        <normalizedUrl>http://nsdl.org/</normalizedUrl>\n"
			+ "\n"
			+ "        <bestFetch status=\"FETCHED\" date=\"2008-02-04T02:35:56Z\"/>\n"
			+ "        <latestFetch status=\"FETCHED\" date=\"2008-02-06T02:36:56Z\"/>\n"
			+ "        <contentType>text/html</contentType>\n"
			+ "        <modifyDate>2008-02-04T02:35:56Z</modifyDate>\n"
			+ "        <title>NSDL.org - The National Science Digital Library</title>\n"
			+ "        <parsedText>Good stuff about the National Science Digital Library</parsedText>\n"
			+ "    </GetPage>\n" + "\n" + "</NSDLContentCache>\n";

	private ContentCacheResponseParser parser;

	@Before
	public void initializeParser() {
		parser = ContentCacheResponseParser.getInstance();
	}

	@Test
	public void badArgument() throws JAXBException {
		PageContent content = parser.parse(new StringReader(BAD_ARGUMENT));
		checkExpectations(content, null, null, null, 0, null, null, false);
	}

	@Test
	public void systemProblem() throws JAXBException {
		PageContent content = parser.parse(new StringReader(SYSTEM_PROBLEM));
		checkExpectations(content, null, null, null, 0, null, null, false);
	}

	@Test
	public void notInCache() throws JAXBException {
		PageContent content = parser.parse(new StringReader(NOT_IN_CACHE));
		checkExpectations(content, null, null, null, 0, null, null, false);
	}

	@Test
	public void notFetched() throws JAXBException {
		PageContent content = parser.parse(new StringReader(NOT_FETCHED));
		checkExpectations(content, null, null, null, 0, null, null, false);
	}

	@Test
	public void fetchFailed() throws JAXBException {
		PageContent content = parser.parse(new StringReader(FETCH_FAILED));
		checkExpectations(content, null, null, null, 0, null, null, false);
	}

	@Test
	public void success() throws JAXBException {
		PageContent content = parser.parse(new StringReader(FETCHED));
		checkExpectations(content, "text/html",
				parseDate("2008-02-06T02:36:56Z"),
				parseDate("2008-02-06T02:36:56Z"), 0,
				"Good stuff about the National Science Digital Library",
				"NSDL.org - The National Science Digital Library", true);
	}

	@Test
	public void successWithLivenessIssues() throws JAXBException {
		PageContent content = parser.parse(new StringReader(STALE));
		checkExpectations(content, "text/html",
				parseDate("2008-02-04T02:35:56Z"),
				parseDate("2008-02-04T02:35:56Z"), 2,
				"Good stuff about the National Science Digital Library",
				"NSDL.org - The National Science Digital Library", true);
	}

	private void checkExpectations(PageContent content, String contentType,
			Date fetchDate, Date modifiedDate, int livenessLapseDays,
			String text, String title, boolean valid) {

		assertEquals("valid", valid, content.isValid());
		if (!valid) {
			return;
		}

		assertEquals("contentType", contentType, content.getContentType());
		assertEquals("fetchDate", fetchDate, content.getFetchDate());
		assertEquals("modifiedDate", modifiedDate, content.getModifiedDate());
		assertEquals("livenessLapse", livenessLapseDays, content
				.getLivenessLapseDays());
		assertEquals("text", text, content.getText());
		assertEquals("title", title, content.getTitle());
	}

	private Date parseDate(String dateString) {
		Date date = null;
		try {
			date = DateUtil.getInstance().parseDate(dateString,
					DateUtil.ISO8601_DATE_TIME_Z);
		} catch (ParseException e) {
			e.printStackTrace();
			fail("Failed to parse date: " + dateString);
		}
		return date;
	}

}
