/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertFalse;
import static junit.framework.Assert.assertTrue;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COMPOUND_AGENT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.COMPOUND_DESCRIPTION;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_DESCRIPTION;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.lucene.document.Field;
import org.apache.lucene.document.Field.Index;
import org.apache.lucene.document.Field.Store;
import org.apache.lucene.document.Field.TermVector;
import org.junit.Before;
import org.junit.Test;

/**
 * Unit tests for {@link LuceneFieldSpec}.
 * 
 * @author jblake
 */
public class TestLuceneFieldSpec {
	private static final LuceneFieldSpec COMPOUND_SPEC = COMPOUND_AGENT;
	private static final String COMPOUND_SPEC_NAME = COMPOUND_SPEC.getName();

	private static final LuceneFieldSpec OTHER_COMPOUND_SPEC = COMPOUND_DESCRIPTION;

	private static final LuceneFieldSpec NON_COMPOUND_SPEC = DC_DESCRIPTION;

	private String specName;
	private Store specStore;
	private Index specIndex;
	private boolean specCaseFold;
	private float specBoost;
	private LuceneFieldSpec specCompoundField;
	private boolean specStemmed;
	private boolean specTokenize;

	@Before
	public void setBaseAttributes() {
		specName = "name";
		specStore = Store.YES;
		specIndex = Index.TOKENIZED;
		specCaseFold = false;
		specBoost = 1.2F;
		specCompoundField = COMPOUND_AGENT;
		specStemmed = true;
		specTokenize = true;
	}

	// ------------------------------------------------------------------------
	// Test Data constructor
	// ------------------------------------------------------------------------

	@Test
	public void dataSimpleSuccess() {
		LuceneFieldSpec spec = new LuceneFieldSpec("myName",
				Field.Store.COMPRESS, Field.Index.NO_NORMS, false, 5.5F,
				COMPOUND_SPEC, false, true, true, true);
		assertCorrectSpec(spec, "myName", Field.Store.COMPRESS,
				Field.Index.NO_NORMS, false, Field.TermVector.NO, 5.5F,
				COMPOUND_SPEC_NAME, false, true, true, true);
	}

	@Test(expected = IllegalArgumentException.class)
	public void dataNoName() {
		new LuceneFieldSpec(null, Field.Store.COMPRESS, Field.Index.NO_NORMS,
				false, 5.5F, null, false, true, false, true);
	}

	@Test(expected = IllegalArgumentException.class)
	public void dataNoStore() {
		new LuceneFieldSpec("myName", null, Field.Index.NO_NORMS, false, 5.5F,
				null, false, true, false, true);
	}

	@Test(expected = IllegalArgumentException.class)
	public void dataNoIndex() {
		new LuceneFieldSpec("myName", Field.Store.COMPRESS, null, false, 5.5F,
				null, false, true, false, true);
	}

	@Test(expected = IllegalArgumentException.class)
	public void dataNonDerivedCompoundField() {
		new LuceneFieldSpec("myName", Field.Store.COMPRESS, null, false, 5.5F,
				NON_COMPOUND_SPEC, false, true, false, true);
	}

	// ------------------------------------------------------------------------
	// Test Functions
	// ------------------------------------------------------------------------

	/**
	 * Test that a spec is equal to its copy and equal to itself, and not equal
	 * to any spec that differs from it. Also, equal implies same hashCode.
	 */
	@Test
	public void testEqualsAndHashCodeMethods() {
		List<LuceneFieldSpec> specs = createAssortedSpecs();
		for (LuceneFieldSpec spec1 : specs) {
			LuceneFieldSpec copy = copySpec(spec1);
			assertEquals("copy should be equal to self", spec1, copy);
			assertEquals("copy should have same hashcode", spec1.hashCode(),
					copy.hashCode());
			for (LuceneFieldSpec spec2 : specs) {
				if (spec1 == spec2) {
					assertEquals("spec should be equal to self", spec1, spec2);
				} else {
					assertFalse("different should not be equal: spec1=" + spec1
							+ ", spec2=" + spec2, spec1.equals(spec2));
					assertFalse("different should not be equal: spec2=" + spec2
							+ ", spec1=" + spec1, spec2.equals(spec1));
				}
			}
		}
	}

	@Test
	public void testHashing() {
		List<LuceneFieldSpec> specList = createAssortedSpecs();
		Set<LuceneFieldSpec> specSet = new HashSet<LuceneFieldSpec>(specList);
		assertEquals("everything went in properly", specList.size(), specSet
				.size());
		for (LuceneFieldSpec spec : specList) {
			assertTrue("should find this one: " + spec, specSet.contains(spec));
		}
	}

	@Test
	public void stemmedSpec() {
		LuceneFieldSpec spec = createSpecFromValues();
		LuceneFieldSpec stemmed = spec.getStemmedSpec();
		assertCorrectSpec(stemmed, "name.Stems", Field.Store.NO,
				Field.Index.TOKENIZED, false, Field.TermVector.NO, 1.2F,
				COMPOUND_SPEC_NAME, false, true, false, true);
	}

	@Test(expected = UnsupportedOperationException.class)
	public void noStemmedSpec() {
		specStemmed = false;
		LuceneFieldSpec spec = createSpecFromValues();
		spec.getStemmedSpec();
	}

	// ------------------------------------------------------------------------
	// Helper methods
	// ------------------------------------------------------------------------

	/**
	 * Each of these specs differs from another in only one attribute.
	 */
	private List<LuceneFieldSpec> createAssortedSpecs() {
		List<LuceneFieldSpec> specs = new ArrayList<LuceneFieldSpec>();
		specs.add(createSpec("name", Store.YES, Index.TOKENIZED, false, 1.0F,
				COMPOUND_SPEC, true, true, false, true));
		specs.add(createSpec("name2", Store.YES, Index.TOKENIZED, false, 1.0F,
				COMPOUND_SPEC, true, true, false, true));
		specs.add(createSpec("name", Store.NO, Index.TOKENIZED, false, 1.0F,
				COMPOUND_SPEC, true, true, false, true));
		specs.add(createSpec("name", Store.YES, Index.NO, false, 1.0F,
				COMPOUND_SPEC, true, true, false, true));
		specs.add(createSpec("name", Store.YES, Index.TOKENIZED, true, 1.0F,
				COMPOUND_SPEC, true, true, false, true));
		specs.add(createSpec("name", Store.YES, Index.TOKENIZED, false, 0.6F,
				COMPOUND_SPEC, true, true, false, true));
		specs.add(createSpec("name", Store.YES, Index.TOKENIZED, false, 1.0F,
				OTHER_COMPOUND_SPEC, true, true, false, true));
		specs.add(createSpec("name", Store.YES, Index.TOKENIZED, false, 1.0F,
				COMPOUND_SPEC, false, true, false, true));
		specs.add(createSpec("name", Store.YES, Index.TOKENIZED, false, 1.0F,
				COMPOUND_SPEC, true, false, false, true));
		specs.add(createSpec("name", Store.YES, Index.TOKENIZED, false, 1.0F,
				COMPOUND_SPEC, true, true, true, true));
		specs.add(createSpec("name", Store.YES, Index.TOKENIZED, false, 1.0F,
				COMPOUND_SPEC, true, true, false, false));
		return specs;
	}

	private LuceneFieldSpec createSpec(String name, Store store, Index index,
			boolean caseFold, float boost, LuceneFieldSpec compoundField,
			boolean stemmable, boolean tokenized, boolean derived,
			boolean inDefaultField) {
		return new LuceneFieldSpec(name, store, index, caseFold, boost,
				compoundField, stemmable, tokenized, derived, inDefaultField);
	}

	private LuceneFieldSpec createSpecFromValues() {
		return new LuceneFieldSpec(specName, specStore, specIndex,
				specCaseFold, specBoost, specCompoundField, specStemmed,
				specTokenize, false, true);
	}

	private LuceneFieldSpec copySpec(LuceneFieldSpec spec) {
		return new LuceneFieldSpec(spec.getName(), spec.getStore(), spec
				.getIndex(), spec.isCaseFold(), spec.getBoost(), spec
				.getCompoundField(), spec.isStemmable(), spec.isTokenized(),
				spec.isDerived(), spec.isInDefaultField());
	}

	private void assertCorrectSpec(LuceneFieldSpec spec, String name,
			Store store, Index index, boolean caseFold, TermVector termVector,
			float boost, String compoundFieldName, boolean stemmable,
			boolean tokenized, boolean derived, boolean inDefaultField) {
		assertEquals("field name", name, spec.getName());
		assertEquals("field Store", store, spec.getStore());
		assertEquals("field Index", index, spec.getIndex());
		assertEquals("isIndexed", (index == Field.Index.TOKENIZED), spec
				.isIndexed());
		assertEquals("case fold", caseFold, spec.isCaseFold());
		assertEquals("field Term Vector", termVector, spec.getTermVector());
		assertEquals("boost", boost, spec.getBoost());
		assertEquals("compound field name", compoundFieldName, spec
				.getCompoundFieldName());
		assertEquals("stemmed", stemmable, spec.isStemmable());
		assertEquals("special parsing", !tokenized, spec.isSpecialParsing());
		assertEquals("tokenized", tokenized, spec.isTokenized());
		assertEquals("derived", derived, spec.isDerived());
		assertEquals("inDefaultField", inDefaultField, spec.isInDefaultField());
	}

}
