/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.nsdl;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.fail;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.lucene.analysis.PerFieldAnalyzerWrapper;
import org.nsdl.search.SearchUnitTest;

/**
 * 
 * @author jblake
 */
public abstract class AbstractNsdlAnalyzerTest extends SearchUnitTest {
	private static final Pattern PATTERN_FULL_STRING = Pattern
			.compile(".*\\{(.*)\\}, (.*)\\)");
	private static final Pattern PATTERN_MAPPING = Pattern
			.compile("(.*)=(.*)@[0-9a-f]+");

	/**
	 * Determine what analyzers are mapped to what fields by analyzing the
	 * output of a call to "toString()".
	 */
	protected Map<String, String> buildAnalyzerMap(
			PerFieldAnalyzerWrapper analyzer) {
		Map<String, String> analyzerMap = new HashMap<String, String>();

		String fullString = analyzer.toString();
		Matcher fullMatch = PATTERN_FULL_STRING.matcher(fullString);
		if (!fullMatch.matches()) {
			fail("Didn't match the analyzer string: '" + fullString + "'");
		}

		String defaultString = fullMatch.group(2);
		Matcher defaultMatch = PATTERN_MAPPING.matcher(defaultString);
		if (!defaultMatch.matches()) {
			fail("Can't find the default analyzer: '" + defaultString + "'");
		} else {
			analyzerMap.put(defaultMatch.group(1), defaultMatch.group(2));
		}

		String wholeMapString = fullMatch.group(1);
		String[] mapStrings = (wholeMapString.length() > 0) ? wholeMapString
				.split(", ") : new String[0];
		for (String mapString : mapStrings) {
			Matcher matcher = PATTERN_MAPPING.matcher(mapString);
			if (!matcher.matches()) {
				fail("Can't find analyzer here: '" + mapString + "'");
			} else {
				analyzerMap.put(matcher.group(1), matcher.group(2));
			}
		}
		return analyzerMap;
	}

	protected void assertEqualMaps(Map<String, String> expected,
			Map<String, String> actual) {
		Set<String> missingKeys = new HashSet<String>(expected.keySet());
		missingKeys.removeAll(actual.keySet());
		if (!missingKeys.isEmpty()) {
			fail("These keys are missing: " + missingKeys);
		}

		Set<String> extraKeys = new HashSet<String>(actual.keySet());
		extraKeys.removeAll(expected.keySet());
		if (!extraKeys.isEmpty()) {
			fail("Found extra keys: " + extraKeys);
		}

		for (String key : expected.keySet()) {
			assertEquals("values for '" + key + "' don't match", expected
					.get(key), actual.get(key));
		}
	}

}
