/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.nsdl;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

import org.apache.lucene.analysis.PerFieldAnalyzerWrapper;
import org.apache.lucene.document.Field.Index;
import org.apache.lucene.document.Field.Store;
import org.junit.Before;
import org.junit.Test;
import org.nsdl.search.lucene.LuceneFieldSpec;

/**
 * Test the functions of the NsdlAnalyzer.
 * 
 * @author jblake
 */
public class TestNsdlAnalyzer extends AbstractNsdlAnalyzerTest {
	private Map<String, String> expectedAnalyzerMap;
	private NsdlAnalyzer analyzer;

	@Before
	public void initializeEmptyAnalyzer() throws NoSuchFieldException,
			IllegalAccessException {
		analyzer = new NsdlAnalyzer();

		Field analyzerMapField = PerFieldAnalyzerWrapper.class
				.getDeclaredField("analyzerMap");
		analyzerMapField.setAccessible(true);
		Map<?, ?> analyzerMap = (Map<?, ?>) analyzerMapField.get(analyzer);
		analyzerMap.clear();
	}

	@Before
	public void initializeExpectedMapping() {
		expectedAnalyzerMap = new HashMap<String, String>();
		putExpectedMapping("default", NsdlAnalyzer.TextAnalyzer.class);
	}

	@Test
	public void tryEmptyAnalyzer() {
		assertCorrectMapping();
	}

	/** Not indexed? then not analyzed. */
	@Test
	public void nonIndexedAddsNothing() {
		analyzeThis("nonIndexedField", false, true, true, true, null);
		assertCorrectMapping();
	}

	/** Tokenized and case-folded. */
	@Test
	public void indexedTokenized() {
		analyzeThis("tokenized", true, true, true, false,
				NsdlAnalyzer.TextAnalyzer.class);
		assertCorrectMapping();
	}

	/** Tokenized and not case-folded: we're going to fold it anyway. */
	@Test
	public void indexedTokenizedNoFolding() {
		analyzeThis("tokenized", true, true, false, false,
				NsdlAnalyzer.TextAnalyzer.class);
		assertCorrectMapping();
	}

	/** Not tokenized, but case-folded: what fields use this? */
	@Test
	public void indexedNotTokenizedCasefold() {
		analyzeThis("notTokenizedFolded", true, false, true, false,
				NsdlAnalyzer.FoldedKeywordAnalyzer.class);
		assertCorrectMapping();
	}

	/** Not tokenized, not case-folded: it's a keyword! */
	@Test
	public void indexedNotTokenizedNotCasefold() {
		analyzeThis("notTokenizedNotFolded", true, false, false, false,
				NsdlAnalyzer.KeywordAnalyzer.class);
		assertCorrectMapping();
	}

	/** Stemming a field. */
	@Test
	public void stemmedSimpleName() {
		String name = "stemmedSimple";
		String stemmedName = name + ".Stems";
		analyzeThis(name, true, true, true, true);
		putExpectedMapping(name, NsdlAnalyzer.TextAnalyzer.class);
		putExpectedMapping(stemmedName, NsdlAnalyzer.StemmedTextAnalyzer.class);
		assertCorrectMapping();
	}

	private void analyzeThis(String name, boolean index, boolean tokenize,
			boolean caseFold, boolean stemmed, Class<?> clazz) {
		analyzeThis(name, index, tokenize, caseFold, stemmed);
		putExpectedMapping(name, clazz);
	}

	private void analyzeThis(String name, boolean index, boolean tokenize,
			boolean caseFold, boolean stemmed) {
		Index index2 = index ? Index.TOKENIZED : Index.NO;
		LuceneFieldSpec spec = new LuceneFieldSpec(name, Store.YES, index2,
				caseFold, 1.0F, null, stemmed, tokenize, false, false);
		analyzer.addAnalyzer(spec);
	}
	private void putExpectedMapping(String fieldname, Class<?> clazz) {
		if (clazz != null) {
			expectedAnalyzerMap.put(fieldname, clazz.getName());
		}
	}

	private void assertCorrectMapping() {
		assertEqualMaps(expectedAnalyzerMap, buildAnalyzerMap(analyzer));
	}

}
