/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.nsdl;

import static junit.framework.Assert.assertEquals;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.TreeSet;

import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.Query;
import org.junit.Before;
import org.junit.Test;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;

/**
 * 
 * @author jblake
 */
public class TestNsdlQueryParser {
	/**
	 * This string works well as a query tester, since it will show differences
	 * if it is tokenized, case-folded, trimmed, or URL-normalized.
	 */
	private static final String QUERY_TEXT = "http\\://fred.com/THIS#junk  ";

	private NsdlQueryParser qp;
	private Set<String> expectedQueries;

	@Before
	public void createQueryParser() {
		qp = new NsdlQueryParser();

	}

	@Test
	public void checkQueries() throws ParseException {
		String queryText = QUERY_TEXT;
		Set<String> queries = new TreeSet<String>();

		Query q = qp.parse(queryText);
		queries.add("default-->" + q.getClass().getSimpleName() + "-->" + q);

		for (LuceneFieldSpec spec : SearchIndexFieldSpecs.getSpecs()) {
			String name = spec.getName().replace(":", "\\:");
			q = qp.parse(name + ":" + queryText);
			queries
					.add(name + "-->" + q.getClass().getSimpleName() + "-->"
							+ q);
		}

		assertEquals("", compareSets("Queries", expectedQueries, queries));
	}

	private String compareSets(String label, Set<String> expected,
			Set<String> actual) {
		String result = "";

		Set<String> missing = new TreeSet<String>(expected);
		missing.removeAll(actual);
		if (!missing.isEmpty()) {
			result += "Missing " + label + ": " + missing + "\n";
		}

		Set<String> extra = new TreeSet<String>(actual);
		extra.removeAll(expected);
		if (!extra.isEmpty()) {
			result += "Extra " + label + ": " + extra + "\n";
		}

		return result;
	}

	@Before
	public void setExpectedQueries() {
		expectedQueries = new HashSet<String>(
				Arrays
						.asList(new String[] {
								"allFields-->PhraseQuery-->allFields:\"http fred.com junk\"",
								"allFields.Stems-->PhraseQuery-->allFields.Stems:\"http fred.com junk\"",
								"coll.dc\\:description-->PhraseQuery-->coll.dc:description:\"http fred.com junk\"",
								"coll.dc\\:description.Stems-->PhraseQuery-->coll.dc:description.Stems:\"http fred.com junk\"",
								"coll.dc\\:rights-->PhraseQuery-->coll.dc:rights:\"http fred.com junk\"",
								"coll.dc\\:rights.Stems-->PhraseQuery-->coll.dc:rights.Stems:\"http fred.com junk\"",
								"coll.dc\\:title-->PhraseQuery-->coll.dc:title:\"http fred.com junk\"",
								"coll.dc\\:title.Stems-->PhraseQuery-->coll.dc:title.Stems:\"http fred.com junk\"",
								"coll.dct\\:accessRights-->PhraseQuery-->coll.dct:accessRights:\"http fred.com junk\"",
								"coll.dct\\:accessRights.Stems-->PhraseQuery-->coll.dct:accessRights.Stems:\"http fred.com junk\"",
								"coll.dct\\:alternative-->PhraseQuery-->coll.dct:alternative:\"http fred.com junk\"",
								"coll.dct\\:alternative.Stems-->PhraseQuery-->coll.dct:alternative.Stems:\"http fred.com junk\"",
								"coll.dct\\:audience-->PhraseQuery-->coll.dct:audience:\"http fred.com junk\"",
								"coll.dct\\:audience.Stems-->PhraseQuery-->coll.dct:audience.Stems:\"http fred.com junk\"",
								"coll.dct\\:audience.nsdl_dc\\:NSDLEdLevel-->PrefixQuery-->coll.dct:audience.nsdl_dc:NSDLEdLevel:http://fred.com/this#junk ~^ *",
								"coll.dct\\:educationLevel-->PhraseQuery-->coll.dct:educationLevel:\"http fred.com junk\"",
								"coll.dct\\:educationLevel.Stems-->PhraseQuery-->coll.dct:educationLevel.Stems:\"http fred.com junk\"",
								"coll.dct\\:educationLevel.nsdl_dc\\:NSDLEdLevel-->PrefixQuery-->coll.dct:educationLevel.nsdl_dc:NSDLEdLevel:http://fred.com/this#junk ~^ *",
								"coll.dct\\:license-->PhraseQuery-->coll.dct:license:\"http fred.com junk\"",
								"coll.dct\\:license.Stems-->PhraseQuery-->coll.dct:license.Stems:\"http fred.com junk\"",
								"coll.dct\\:mediator-->PhraseQuery-->coll.dct:mediator:\"http fred.com junk\"",
								"coll.dct\\:mediator.Stems-->PhraseQuery-->coll.dct:mediator.Stems:\"http fred.com junk\"",
								"collImageAltText-->PhraseQuery-->collImageAltText:\"http fred.com junk\"",
								"collImageHeight-->PhraseQuery-->collImageHeight:\"http fred.com junk\"",
								"collImageUrl-->PhraseQuery-->collImageUrl:\"http fred.com junk\"",
								"collImageWidth-->PhraseQuery-->collImageWidth:\"http fred.com junk\"",
								"compoundAgent-->PhraseQuery-->compoundAgent:\"http fred.com junk\"",
								"compoundAgent.Stems-->PhraseQuery-->compoundAgent.Stems:\"http fred.com junk\"",
								"compoundAudience-->PhraseQuery-->compoundAudience:\"http fred.com junk\"",
								"compoundAudience.Stems-->PhraseQuery-->compoundAudience.Stems:\"http fred.com junk\"",
								"compoundDescription-->PhraseQuery-->compoundDescription:\"http fred.com junk\"",
								"compoundDescription.Stems-->PhraseQuery-->compoundDescription.Stems:\"http fred.com junk\"",
								"compoundGenre-->PhraseQuery-->compoundGenre:\"http fred.com junk\"",
								"compoundGenre.Stems-->PhraseQuery-->compoundGenre.Stems:\"http fred.com junk\"",
								"compoundSubject-->PhraseQuery-->compoundSubject:\"http fred.com junk\"",
								"compoundSubject.Stems-->PhraseQuery-->compoundSubject.Stems:\"http fred.com junk\"",
								"compoundTitle-->PhraseQuery-->compoundTitle:\"http fred.com junk\"",
								"compoundTitle.Stems-->PhraseQuery-->compoundTitle.Stems:\"http fred.com junk\"",
								"content-->PhraseQuery-->content:\"http fred.com junk\"",
								"content.Stems-->PhraseQuery-->content.Stems:\"http fred.com junk\"",
								"contentLastFetched-->TermQuery-->contentLastFetched:http://fred.com/this#junk",
								"contentLastModified-->TermQuery-->contentLastModified:http://fred.com/this#junk",
								"dc\\:contributor-->PhraseQuery-->dc:contributor:\"http fred.com junk\"",
								"dc\\:contributor.Stems-->PhraseQuery-->dc:contributor.Stems:\"http fred.com junk\"",
								"dc\\:coverage-->PhraseQuery-->dc:coverage:\"http fred.com junk\"",
								"dc\\:coverage.Stems-->PhraseQuery-->dc:coverage.Stems:\"http fred.com junk\"",
								"dc\\:coverage.dct\\:Box-->PhraseQuery-->dc:coverage.dct:Box:\"http fred.com junk\"",
								"dc\\:coverage.dct\\:Box.Stems-->PhraseQuery-->dc:coverage.dct:Box.Stems:\"http fred.com junk\"",
								"dc\\:coverage.dct\\:ISO3166-->PhraseQuery-->dc:coverage.dct:ISO3166:\"http fred.com junk\"",
								"dc\\:coverage.dct\\:ISO3166.Stems-->PhraseQuery-->dc:coverage.dct:ISO3166.Stems:\"http fred.com junk\"",
								"dc\\:coverage.dct\\:ISO8601-->PrefixQuery-->dc:coverage.dct:ISO8601:http://fred.com/this#junk ~^ *",
								"dc\\:coverage.dct\\:Period-->PhraseQuery-->dc:coverage.dct:Period:\"http fred.com junk\"",
								"dc\\:coverage.dct\\:Period.Stems-->PhraseQuery-->dc:coverage.dct:Period.Stems:\"http fred.com junk\"",
								"dc\\:coverage.dct\\:Point-->PhraseQuery-->dc:coverage.dct:Point:\"http fred.com junk\"",
								"dc\\:coverage.dct\\:Point.Stems-->PhraseQuery-->dc:coverage.dct:Point.Stems:\"http fred.com junk\"",
								"dc\\:coverage.dct\\:TGN-->PhraseQuery-->dc:coverage.dct:TGN:\"http fred.com junk\"",
								"dc\\:coverage.dct\\:TGN.Stems-->PhraseQuery-->dc:coverage.dct:TGN.Stems:\"http fred.com junk\"",
								"dc\\:coverage.dct\\:W3CDTF-->PrefixQuery-->dc:coverage.dct:W3CDTF:http://fred.com/this#junk ~^ *",
								"dc\\:creator-->PhraseQuery-->dc:creator:\"http fred.com junk\"",
								"dc\\:creator.Stems-->PhraseQuery-->dc:creator.Stems:\"http fred.com junk\"",
								"dc\\:date-->PhraseQuery-->dc:date:\"http fred.com junk\"",
								"dc\\:date.Stems-->PhraseQuery-->dc:date.Stems:\"http fred.com junk\"",
								"dc\\:date.dct\\:ISO8601-->PrefixQuery-->dc:date.dct:ISO8601:http://fred.com/this#junk ~^ *",
								"dc\\:date.dct\\:Period-->PhraseQuery-->dc:date.dct:Period:\"http fred.com junk\"",
								"dc\\:date.dct\\:Period.Stems-->PhraseQuery-->dc:date.dct:Period.Stems:\"http fred.com junk\"",
								"dc\\:date.dct\\:W3CDTF-->PrefixQuery-->dc:date.dct:W3CDTF:http://fred.com/this#junk ~^ *",
								"dc\\:description-->PhraseQuery-->dc:description:\"http fred.com junk\"",
								"dc\\:description.Stems-->PhraseQuery-->dc:description.Stems:\"http fred.com junk\"",
								"dc\\:format-->PhraseQuery-->dc:format:\"http fred.com junk\"",
								"dc\\:format.Stems-->PhraseQuery-->dc:format.Stems:\"http fred.com junk\"",
								"dc\\:format.dct\\:IMT-->PrefixQuery-->dc:format.dct:IMT:http://fred.com/this#junk ~^ *",
								"dc\\:identifier-->PhraseQuery-->dc:identifier:\"http fred.com junk\"",
								"dc\\:identifier.Stems-->PhraseQuery-->dc:identifier.Stems:\"http fred.com junk\"",
								"dc\\:identifier.dct\\:ISBN-->PrefixQuery-->dc:identifier.dct:ISBN:http://fred.com/this#junk ~^ *",
								"dc\\:identifier.dct\\:ISSN-->PrefixQuery-->dc:identifier.dct:ISSN:http://fred.com/this#junk ~^ *",
								"dc\\:identifier.dct\\:URI-->PrefixQuery-->dc:identifier.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dc\\:language-->PhraseQuery-->dc:language:\"http fred.com junk\"",
								"dc\\:language.Stems-->PhraseQuery-->dc:language.Stems:\"http fred.com junk\"",
								"dc\\:language.dct\\:ISO639-2-->PrefixQuery-->dc:language.dct:ISO639-2:http://fred.com/this#junk ~^ *",
								"dc\\:language.dct\\:RFC3066-->PrefixQuery-->dc:language.dct:RFC3066:http://fred.com/this#junk ~^ *",
								"dc\\:publisher-->PhraseQuery-->dc:publisher:\"http fred.com junk\"",
								"dc\\:publisher.Stems-->PhraseQuery-->dc:publisher.Stems:\"http fred.com junk\"",
								"dc\\:relation-->PhraseQuery-->dc:relation:\"http fred.com junk\"",
								"dc\\:relation.Stems-->PhraseQuery-->dc:relation.Stems:\"http fred.com junk\"",
								"dc\\:relation.dct\\:URI-->PrefixQuery-->dc:relation.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dc\\:rights-->PhraseQuery-->dc:rights:\"http fred.com junk\"",
								"dc\\:rights.Stems-->PhraseQuery-->dc:rights.Stems:\"http fred.com junk\"",
								"dc\\:rights.dct\\:URI-->PrefixQuery-->dc:rights.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dc\\:source-->PhraseQuery-->dc:source:\"http fred.com junk\"",
								"dc\\:source.Stems-->PhraseQuery-->dc:source.Stems:\"http fred.com junk\"",
								"dc\\:source.dct\\:URI-->PrefixQuery-->dc:source.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dc\\:subject-->PhraseQuery-->dc:subject:\"http fred.com junk\"",
								"dc\\:subject.Stems-->PhraseQuery-->dc:subject.Stems:\"http fred.com junk\"",
								"dc\\:subject.dct\\:DDC-->PrefixQuery-->dc:subject.dct:DDC:http://fred.com/this#junk ~^ *",
								"dc\\:subject.dct\\:LCC-->PrefixQuery-->dc:subject.dct:LCC:http://fred.com/this#junk ~^ *",
								"dc\\:subject.dct\\:LCSH-->PrefixQuery-->dc:subject.dct:LCSH:http://fred.com/this#junk ~^ *",
								"dc\\:subject.dct\\:MESH-->PrefixQuery-->dc:subject.dct:MESH:http://fred.com/this#junk ~^ *",
								"dc\\:subject.dct\\:UDC-->PrefixQuery-->dc:subject.dct:UDC:http://fred.com/this#junk ~^ *",
								"dc\\:subject.nsdl_dc\\:GEM-->PrefixQuery-->dc:subject.nsdl_dc:GEM:http://fred.com/this#junk ~^ *",
								"dc\\:title-->PhraseQuery-->dc:title:\"http fred.com junk\"",
								"dc\\:title.Stems-->PhraseQuery-->dc:title.Stems:\"http fred.com junk\"",
								"dc\\:type-->PhraseQuery-->dc:type:\"http fred.com junk\"",
								"dc\\:type.Stems-->PhraseQuery-->dc:type.Stems:\"http fred.com junk\"",
								"dc\\:type.dct\\:DCMIType-->PrefixQuery-->dc:type.dct:DCMIType:http://fred.com/this#junk ~^ *",
								"dc\\:type.nsdl_dc\\:NSDLType-->PrefixQuery-->dc:type.nsdl_dc:NSDLType:http://fred.com/this#junk ~^ *",
								"dct\\:abstract-->PhraseQuery-->dct:abstract:\"http fred.com junk\"",
								"dct\\:abstract.Stems-->PhraseQuery-->dct:abstract.Stems:\"http fred.com junk\"",
								"dct\\:accessRights-->PhraseQuery-->dct:accessRights:\"http fred.com junk\"",
								"dct\\:accessRights.Stems-->PhraseQuery-->dct:accessRights.Stems:\"http fred.com junk\"",
								"dct\\:accessRights.nsdl_dc\\:NSDLAccess-->PrefixQuery-->dct:accessRights.nsdl_dc:NSDLAccess:http://fred.com/this#junk ~^ *",
								"dct\\:accessRights.dct\\:URI-->PrefixQuery-->dct:accessRights.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:accessibility-->PhraseQuery-->dct:accessibility:\"http fred.com junk\"",
								"dct\\:accessibility.Stems-->PhraseQuery-->dct:accessibility.Stems:\"http fred.com junk\"",
								"dct\\:accrualMethod-->PhraseQuery-->dct:accrualMethod:\"http fred.com junk\"",
								"dct\\:accrualMethod.Stems-->PhraseQuery-->dct:accrualMethod.Stems:\"http fred.com junk\"",
								"dct\\:accrualPeriodicity-->PhraseQuery-->dct:accrualPeriodicity:\"http fred.com junk\"",
								"dct\\:accrualPeriodicity.Stems-->PhraseQuery-->dct:accrualPeriodicity.Stems:\"http fred.com junk\"",
								"dct\\:accrualPolicy-->PhraseQuery-->dct:accrualPolicy:\"http fred.com junk\"",
								"dct\\:accrualPolicy.Stems-->PhraseQuery-->dct:accrualPolicy.Stems:\"http fred.com junk\"",
								"dct\\:alternative-->PhraseQuery-->dct:alternative:\"http fred.com junk\"",
								"dct\\:alternative.Stems-->PhraseQuery-->dct:alternative.Stems:\"http fred.com junk\"",
								"dct\\:audience-->PhraseQuery-->dct:audience:\"http fred.com junk\"",
								"dct\\:audience.Stems-->PhraseQuery-->dct:audience.Stems:\"http fred.com junk\"",
								"dct\\:audience.nsdl_dc\\:NSDLAudience-->PrefixQuery-->dct:audience.nsdl_dc:NSDLAudience:http://fred.com/this#junk ~^ *",
								"dct\\:audience.nsdl_dc\\:NSDLEdLevel-->PrefixQuery-->dct:audience.nsdl_dc:NSDLEdLevel:http://fred.com/this#junk ~^ *",
								"dct\\:available-->PhraseQuery-->dct:available:\"http fred.com junk\"",
								"dct\\:available.Stems-->PhraseQuery-->dct:available.Stems:\"http fred.com junk\"",
								"dct\\:available.dct\\:ISO8601-->PrefixQuery-->dct:available.dct:ISO8601:http://fred.com/this#junk ~^ *",
								"dct\\:available.dct\\:Period-->PhraseQuery-->dct:available.dct:Period:\"http fred.com junk\"",
								"dct\\:available.dct\\:Period.Stems-->PhraseQuery-->dct:available.dct:Period.Stems:\"http fred.com junk\"",
								"dct\\:available.dct\\:W3CDTF-->PrefixQuery-->dct:available.dct:W3CDTF:http://fred.com/this#junk ~^ *",
								"dct\\:bibliographicCitation-->PhraseQuery-->dct:bibliographicCitation:\"http fred.com junk\"",
								"dct\\:bibliographicCitation.Stems-->PhraseQuery-->dct:bibliographicCitation.Stems:\"http fred.com junk\"",
								"dct\\:conformsTo-->PhraseQuery-->dct:conformsTo:\"http fred.com junk\"",
								"dct\\:conformsTo.Stems-->PhraseQuery-->dct:conformsTo.Stems:\"http fred.com junk\"",
								"dct\\:conformsTo.dct\\:URI-->PrefixQuery-->dct:conformsTo.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:created-->PhraseQuery-->dct:created:\"http fred.com junk\"",
								"dct\\:created.Stems-->PhraseQuery-->dct:created.Stems:\"http fred.com junk\"",
								"dct\\:created.dct\\:ISO8601-->PrefixQuery-->dct:created.dct:ISO8601:http://fred.com/this#junk ~^ *",
								"dct\\:created.dct\\:Period-->PhraseQuery-->dct:created.dct:Period:\"http fred.com junk\"",
								"dct\\:created.dct\\:Period.Stems-->PhraseQuery-->dct:created.dct:Period.Stems:\"http fred.com junk\"",
								"dct\\:created.dct\\:W3CDTF-->PrefixQuery-->dct:created.dct:W3CDTF:http://fred.com/this#junk ~^ *",
								"dct\\:dateAccepted-->PhraseQuery-->dct:dateAccepted:\"http fred.com junk\"",
								"dct\\:dateAccepted.Stems-->PhraseQuery-->dct:dateAccepted.Stems:\"http fred.com junk\"",
								"dct\\:dateAccepted.dct\\:ISO8601-->PrefixQuery-->dct:dateAccepted.dct:ISO8601:http://fred.com/this#junk ~^ *",
								"dct\\:dateAccepted.dct\\:Period-->PhraseQuery-->dct:dateAccepted.dct:Period:\"http fred.com junk\"",
								"dct\\:dateAccepted.dct\\:Period.Stems-->PhraseQuery-->dct:dateAccepted.dct:Period.Stems:\"http fred.com junk\"",
								"dct\\:dateAccepted.dct\\:W3CDTF-->PrefixQuery-->dct:dateAccepted.dct:W3CDTF:http://fred.com/this#junk ~^ *",
								"dct\\:dateCopyrighted-->PhraseQuery-->dct:dateCopyrighted:\"http fred.com junk\"",
								"dct\\:dateCopyrighted.Stems-->PhraseQuery-->dct:dateCopyrighted.Stems:\"http fred.com junk\"",
								"dct\\:dateCopyrighted.dct\\:ISO8601-->PrefixQuery-->dct:dateCopyrighted.dct:ISO8601:http://fred.com/this#junk ~^ *",
								"dct\\:dateCopyrighted.dct\\:Period-->PhraseQuery-->dct:dateCopyrighted.dct:Period:\"http fred.com junk\"",
								"dct\\:dateCopyrighted.dct\\:Period.Stems-->PhraseQuery-->dct:dateCopyrighted.dct:Period.Stems:\"http fred.com junk\"",
								"dct\\:dateCopyrighted.dct\\:W3CDTF-->PrefixQuery-->dct:dateCopyrighted.dct:W3CDTF:http://fred.com/this#junk ~^ *",
								"dct\\:dateSubmitted-->PhraseQuery-->dct:dateSubmitted:\"http fred.com junk\"",
								"dct\\:dateSubmitted.Stems-->PhraseQuery-->dct:dateSubmitted.Stems:\"http fred.com junk\"",
								"dct\\:dateSubmitted.dct\\:ISO8601-->PrefixQuery-->dct:dateSubmitted.dct:ISO8601:http://fred.com/this#junk ~^ *",
								"dct\\:dateSubmitted.dct\\:Period-->PhraseQuery-->dct:dateSubmitted.dct:Period:\"http fred.com junk\"",
								"dct\\:dateSubmitted.dct\\:Period.Stems-->PhraseQuery-->dct:dateSubmitted.dct:Period.Stems:\"http fred.com junk\"",
								"dct\\:dateSubmitted.dct\\:W3CDTF-->PrefixQuery-->dct:dateSubmitted.dct:W3CDTF:http://fred.com/this#junk ~^ *",
								"dct\\:educationLevel-->PhraseQuery-->dct:educationLevel:\"http fred.com junk\"",
								"dct\\:educationLevel.Stems-->PhraseQuery-->dct:educationLevel.Stems:\"http fred.com junk\"",
								"dct\\:educationLevel.nsdl_dc\\:NSDLEdLevel-->PrefixQuery-->dct:educationLevel.nsdl_dc:NSDLEdLevel:http://fred.com/this#junk ~^ *",
								"dct\\:extent-->PhraseQuery-->dct:extent:\"http fred.com junk\"",
								"dct\\:extent.Stems-->PhraseQuery-->dct:extent.Stems:\"http fred.com junk\"",
								"dct\\:hasFormat-->PhraseQuery-->dct:hasFormat:\"http fred.com junk\"",
								"dct\\:hasFormat.Stems-->PhraseQuery-->dct:hasFormat.Stems:\"http fred.com junk\"",
								"dct\\:hasFormat.dct\\:URI-->PrefixQuery-->dct:hasFormat.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:hasPart-->PhraseQuery-->dct:hasPart:\"http fred.com junk\"",
								"dct\\:hasPart.Stems-->PhraseQuery-->dct:hasPart.Stems:\"http fred.com junk\"",
								"dct\\:hasPart.dct\\:URI-->PrefixQuery-->dct:hasPart.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:hasVersion-->PhraseQuery-->dct:hasVersion:\"http fred.com junk\"",
								"dct\\:hasVersion.Stems-->PhraseQuery-->dct:hasVersion.Stems:\"http fred.com junk\"",
								"dct\\:hasVersion.dct\\:URI-->PrefixQuery-->dct:hasVersion.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:instructionalMethod-->PhraseQuery-->dct:instructionalMethod:\"http fred.com junk\"",
								"dct\\:instructionalMethod.Stems-->PhraseQuery-->dct:instructionalMethod.Stems:\"http fred.com junk\"",
								"dct\\:isFormatOf-->PhraseQuery-->dct:isFormatOf:\"http fred.com junk\"",
								"dct\\:isFormatOf.Stems-->PhraseQuery-->dct:isFormatOf.Stems:\"http fred.com junk\"",
								"dct\\:isFormatOf.dct\\:URI-->PrefixQuery-->dct:isFormatOf.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:isPartOf-->PhraseQuery-->dct:isPartOf:\"http fred.com junk\"",
								"dct\\:isPartOf.Stems-->PhraseQuery-->dct:isPartOf.Stems:\"http fred.com junk\"",
								"dct\\:isPartOf.dct\\:URI-->PrefixQuery-->dct:isPartOf.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:isReferencedBy-->PhraseQuery-->dct:isReferencedBy:\"http fred.com junk\"",
								"dct\\:isReferencedBy.Stems-->PhraseQuery-->dct:isReferencedBy.Stems:\"http fred.com junk\"",
								"dct\\:isReferencedBy.dct\\:URI-->PrefixQuery-->dct:isReferencedBy.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:isReplacedBy-->PhraseQuery-->dct:isReplacedBy:\"http fred.com junk\"",
								"dct\\:isReplacedBy.Stems-->PhraseQuery-->dct:isReplacedBy.Stems:\"http fred.com junk\"",
								"dct\\:isReplacedBy.dct\\:URI-->PrefixQuery-->dct:isReplacedBy.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:isRequiredBy-->PhraseQuery-->dct:isRequiredBy:\"http fred.com junk\"",
								"dct\\:isRequiredBy.Stems-->PhraseQuery-->dct:isRequiredBy.Stems:\"http fred.com junk\"",
								"dct\\:isRequiredBy.dct\\:URI-->PrefixQuery-->dct:isRequiredBy.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:isVersionOf-->PhraseQuery-->dct:isVersionOf:\"http fred.com junk\"",
								"dct\\:isVersionOf.Stems-->PhraseQuery-->dct:isVersionOf.Stems:\"http fred.com junk\"",
								"dct\\:isVersionOf.dct\\:URI-->PrefixQuery-->dct:isVersionOf.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:issued-->PhraseQuery-->dct:issued:\"http fred.com junk\"",
								"dct\\:issued.Stems-->PhraseQuery-->dct:issued.Stems:\"http fred.com junk\"",
								"dct\\:issued.dct\\:ISO8601-->PrefixQuery-->dct:issued.dct:ISO8601:http://fred.com/this#junk ~^ *",
								"dct\\:issued.dct\\:Period-->PhraseQuery-->dct:issued.dct:Period:\"http fred.com junk\"",
								"dct\\:issued.dct\\:Period.Stems-->PhraseQuery-->dct:issued.dct:Period.Stems:\"http fred.com junk\"",
								"dct\\:issued.dct\\:W3CDTF-->PrefixQuery-->dct:issued.dct:W3CDTF:http://fred.com/this#junk ~^ *",
								"dct\\:license-->PhraseQuery-->dct:license:\"http fred.com junk\"",
								"dct\\:license.Stems-->PhraseQuery-->dct:license.Stems:\"http fred.com junk\"",
								"dct\\:license.dct\\:URI-->PrefixQuery-->dct:license.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:mediator-->PhraseQuery-->dct:mediator:\"http fred.com junk\"",
								"dct\\:mediator.Stems-->PhraseQuery-->dct:mediator.Stems:\"http fred.com junk\"",
								"dct\\:medium-->PhraseQuery-->dct:medium:\"http fred.com junk\"",
								"dct\\:medium.Stems-->PhraseQuery-->dct:medium.Stems:\"http fred.com junk\"",
								"dct\\:medium.dct\\:IMT-->PrefixQuery-->dct:medium.dct:IMT:http://fred.com/this#junk ~^ *",
								"dct\\:modified-->PhraseQuery-->dct:modified:\"http fred.com junk\"",
								"dct\\:modified.Stems-->PhraseQuery-->dct:modified.Stems:\"http fred.com junk\"",
								"dct\\:modified.dct\\:ISO8601-->PrefixQuery-->dct:modified.dct:ISO8601:http://fred.com/this#junk ~^ *",
								"dct\\:modified.dct\\:Period-->PhraseQuery-->dct:modified.dct:Period:\"http fred.com junk\"",
								"dct\\:modified.dct\\:Period.Stems-->PhraseQuery-->dct:modified.dct:Period.Stems:\"http fred.com junk\"",
								"dct\\:modified.dct\\:W3CDTF-->PrefixQuery-->dct:modified.dct:W3CDTF:http://fred.com/this#junk ~^ *",
								"dct\\:provenance-->PhraseQuery-->dct:provenance:\"http fred.com junk\"",
								"dct\\:provenance.Stems-->PhraseQuery-->dct:provenance.Stems:\"http fred.com junk\"",
								"dct\\:references-->PhraseQuery-->dct:references:\"http fred.com junk\"",
								"dct\\:references.Stems-->PhraseQuery-->dct:references.Stems:\"http fred.com junk\"",
								"dct\\:references.dct\\:URI-->PrefixQuery-->dct:references.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:replaces-->PhraseQuery-->dct:replaces:\"http fred.com junk\"",
								"dct\\:replaces.Stems-->PhraseQuery-->dct:replaces.Stems:\"http fred.com junk\"",
								"dct\\:replaces.dct\\:URI-->PrefixQuery-->dct:replaces.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:requires-->PhraseQuery-->dct:requires:\"http fred.com junk\"",
								"dct\\:requires.Stems-->PhraseQuery-->dct:requires.Stems:\"http fred.com junk\"",
								"dct\\:requires.dct\\:URI-->PrefixQuery-->dct:requires.dct:URI:http://fred.com/THIS#junk ~^ *",
								"dct\\:rightsHolder-->PhraseQuery-->dct:rightsHolder:\"http fred.com junk\"",
								"dct\\:rightsHolder.Stems-->PhraseQuery-->dct:rightsHolder.Stems:\"http fred.com junk\"",
								"dct\\:spatial-->PhraseQuery-->dct:spatial:\"http fred.com junk\"",
								"dct\\:spatial.Stems-->PhraseQuery-->dct:spatial.Stems:\"http fred.com junk\"",
								"dct\\:spatial.dct\\:Box-->PhraseQuery-->dct:spatial.dct:Box:\"http fred.com junk\"",
								"dct\\:spatial.dct\\:Box.Stems-->PhraseQuery-->dct:spatial.dct:Box.Stems:\"http fred.com junk\"",
								"dct\\:spatial.dct\\:ISO3166-->PhraseQuery-->dct:spatial.dct:ISO3166:\"http fred.com junk\"",
								"dct\\:spatial.dct\\:ISO3166.Stems-->PhraseQuery-->dct:spatial.dct:ISO3166.Stems:\"http fred.com junk\"",
								"dct\\:spatial.dct\\:Point-->PhraseQuery-->dct:spatial.dct:Point:\"http fred.com junk\"",
								"dct\\:spatial.dct\\:Point.Stems-->PhraseQuery-->dct:spatial.dct:Point.Stems:\"http fred.com junk\"",
								"dct\\:spatial.dct\\:TGN-->PhraseQuery-->dct:spatial.dct:TGN:\"http fred.com junk\"",
								"dct\\:spatial.dct\\:TGN.Stems-->PhraseQuery-->dct:spatial.dct:TGN.Stems:\"http fred.com junk\"",
								"dct\\:tableOfContents-->PhraseQuery-->dct:tableOfContents:\"http fred.com junk\"",
								"dct\\:tableOfContents.Stems-->PhraseQuery-->dct:tableOfContents.Stems:\"http fred.com junk\"",
								"dct\\:temporal-->PhraseQuery-->dct:temporal:\"http fred.com junk\"",
								"dct\\:temporal.Stems-->PhraseQuery-->dct:temporal.Stems:\"http fred.com junk\"",
								"dct\\:temporal.dct\\:ISO8601-->PrefixQuery-->dct:temporal.dct:ISO8601:http://fred.com/this#junk ~^ *",
								"dct\\:temporal.dct\\:Period-->PhraseQuery-->dct:temporal.dct:Period:\"http fred.com junk\"",
								"dct\\:temporal.dct\\:Period.Stems-->PhraseQuery-->dct:temporal.dct:Period.Stems:\"http fred.com junk\"",
								"dct\\:temporal.dct\\:W3CDTF-->PrefixQuery-->dct:temporal.dct:W3CDTF:http://fred.com/this#junk ~^ *",
								"dct\\:valid-->PhraseQuery-->dct:valid:\"http fred.com junk\"",
								"dct\\:valid.Stems-->PhraseQuery-->dct:valid.Stems:\"http fred.com junk\"",
								"dct\\:valid.dct\\:ISO8601-->PrefixQuery-->dct:valid.dct:ISO8601:http://fred.com/this#junk ~^ *",
								"dct\\:valid.dct\\:Period-->PhraseQuery-->dct:valid.dct:Period:\"http fred.com junk\"",
								"dct\\:valid.dct\\:Period.Stems-->PhraseQuery-->dct:valid.dct:Period.Stems:\"http fred.com junk\"",
								"dct\\:valid.dct\\:W3CDTF-->PrefixQuery-->dct:valid.dct:W3CDTF:http://fred.com/this#junk ~^ *",
								"default-->PhraseQuery-->allFields.Stems:\"http fred.com junk\"",
								"documentCreated-->TermQuery-->documentCreated:http://fred.com/this#junk",
								"documentLastUpdated-->TermQuery-->documentLastUpdated:http://fred.com/this#junk",
								"ieee\\:interactivityLevel-->PhraseQuery-->ieee:interactivityLevel:\"http fred.com junk\"",
								"ieee\\:interactivityLevel.Stems-->PhraseQuery-->ieee:interactivityLevel.Stems:\"http fred.com junk\"",
								"ieee\\:interactivityType-->PhraseQuery-->ieee:interactivityType:\"http fred.com junk\"",
								"ieee\\:interactivityType.Stems-->PhraseQuery-->ieee:interactivityType.Stems:\"http fred.com junk\"",
								"ieee\\:typicalLearningTime-->PhraseQuery-->ieee:typicalLearningTime:\"http fred.com junk\"",
								"ieee\\:typicalLearningTime.Stems-->PhraseQuery-->ieee:typicalLearningTime.Stems:\"http fred.com junk\"",
								"inCollection-->PrefixQuery-->inCollection:http://fred.com/THIS#junk ~^ *",
								"metadataLastModified-->TermQuery-->metadataLastModified:http://fred.com/this#junk",
								"metadataSource-->PrefixQuery-->metadataSource:http://fred.com/THIS#junk ~^ *",
								"resourceIdentifier-->TermQuery-->resourceIdentifier:http://fred.com/THIS" }));
	}
}
