/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.searchdoc;
import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.fail;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DCT_AUDIENCE;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DCT_CREATED_W3CDTF;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DCT_DATE_ACCEPTED_W3CDTF;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DCT_DATE_SUBMITTED_W3CDTF;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DCT_EDLEVEL;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DCT_EDLEVEL_NSDL_EDLEVEL;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DCT_EXTENT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DCT_MEDIATOR;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DCT_MODIFIED_W3CDTF;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_CREATOR;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_DATE;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_DATE_W3CDTF;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_DESCRIPTION;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_FORMAT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_FORMAT_IMT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_IDENTIFIER;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_LANGUAGE_RFC3066;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_PUBLISHER;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_RIGHTS;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_SUBJECT;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_SUBJECT_GEM;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_SUBJECT_LCSH;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_TITLE;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_TYPE;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DC_TYPE_DCMITYPE;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.DOCUMENT_CREATED;
import static org.nsdl.search.lucene.SearchIndexFieldSpecs.RESOURCE_IDENTIFIER;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import mock.search.content.MockContentCacheWrapper;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.junit.Ignore;
import org.junit.Test;
import org.nsdl.search.SearchUnitTest;
import org.nsdl.search.content.ContentCacheWrapper;
import org.nsdl.search.lucene.FatalSearchIndexException;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.processoai.OaiCollectionInfo;
import org.nsdl.search.testoaiparsing.ExpectedDocuments;
import org.nsdl.searchtools.common.DateUtil;
import org.nsdl.searchtools.common.ListsMap;

/**
 * Test the SearchDocumentLoader class by loading some SearchDocument objects.
 * 
 * @author jblake
 */
public class TestSearchDocumentLoader extends SearchUnitTest {
	@Override
	protected Class<?>[][] getImplementationClasses() {
		return new Class[][]{{ContentCacheWrapper.class,
				MockContentCacheWrapper.class}};
	}

	@Override
	protected boolean isOutputDumped() {
		return false;
	}

	@Test
	public void testStringConstructor() {
		SearchDocument searchDoc = new SearchDocument("someURL");
		assertEquals("someURL", searchDoc.getIdentifier());
		assertNotNull(searchDoc.getDocumentCreated());

		Collection<SearchDocumentMetadata> metadatas = searchDoc
				.getMetadataSources();
		assertEquals(0, metadatas.size());
	}

	@Test
	public void testDocumentConstructorOneMetadata() {
		SearchDocument searchDoc = new SearchDocument(ExpectedDocuments
				.getDocument1());
		assertEquals(
				"http://dcr.mathdl.org/mathDL/3/?pa=content&sa=viewDocument&nodeId=679",
				searchDoc.getIdentifier());
		assertEqualDates("2007-04-12T21:01:21Z", searchDoc.getDocumentCreated());
		assertEqualDates("2006-11-03T02:01:55Z", searchDoc
				.getDocumentLastUpdated());
		assertEqualDates("2006-11-01T15:00:18Z", searchDoc
				.getMetadataLastModified());

		String[][] sources = new String[][]{{"oai:nsdl.org:MathDL:679",
				"http://dcr.mathdl.org/mathDL/3/?pa=content&sa=viewDocument&nodeId=679"}};
		assertProperMetadataMapping(sources, searchDoc);

		ListsMap<LuceneFieldSpec, String> metadata = new ListsMap<LuceneFieldSpec, String>();
		metadata.addValue(DC_CREATOR, "Douglas Ensley, Barbara Kaskosz");
		metadata.addValue(DC_DATE, "2006-10-27T10:44:02Z");
		metadata.addValue(DC_DESCRIPTION,
				"A forum for educators interested in developing "
						+ "classroom material using Macromedia Flash.");
		metadata.addValue(DC_SUBJECT, "Flash Forum Collection");
		metadata.addValue(DC_SUBJECT, "flash, developer, programming");
		metadata.addValue(DC_TITLE, "MathDL Flash Forum");
		metadata.addValue(DC_TYPE_DCMITYPE, "Collection");

		OaiCollectionInfo collectionInfo = new OaiCollectionInfo(
				"oai:nsdl.org:nsdl.nsdl:00171",
				"http://crs.nsdl.org/brands/MathDL.jpg", "100", "34",
				"Mathematical Sciences Digital Library");

		assertCorrectMetadata(searchDoc, "oai:nsdl.org:MathDL:679", metadata,
				collectionInfo);
	}
	@Test
	public void testDocumentConstructorFourMetadatas() {
		SearchDocument searchDoc = new SearchDocument(ExpectedDocuments
				.getDocumentNQAll());
		assertEquals("http://quest.arc.nasa.gov/", searchDoc.getIdentifier());
		assertEqualDates("2007-05-06T23:01:21Z", searchDoc.getDocumentCreated());
		assertEqualDates("2007-05-05T01:14:00Z", searchDoc
				.getDocumentLastUpdated());
		assertEqualDates("2007-04-03T01:35:50Z", searchDoc
				.getMetadataLastModified());

		String[][] sources = new String[][]{
				{"oai:nsdl.org:2200-20061002172429873T",
						"http://quest.arc.nasa.gov/"},
				{"oai:nsdl.org:2200-20061002193307574T",
						"http://quest.arc.nasa.gov/"},
				{"oai:nsdl.org:crs:4726", "http://quest.arc.nasa.gov/"},
				{"oai:nsdl.org:2200-20061003055539958T",
						"http://quest.arc.nasa.gov/"}};
		assertProperMetadataMapping(sources, searchDoc);

		ListsMap<LuceneFieldSpec, String> metadata0 = new ListsMap<LuceneFieldSpec, String>();
		metadata0.addValue(DCT_AUDIENCE, "Learner");
		metadata0.addValue(DCT_AUDIENCE, "Teacher");
		metadata0.addValue(DC_CREATOR, "Donald James");
		metadata0.addValue(DC_DESCRIPTION,
				"NASA Quest is dedicated to bringing NASA people "
						+ "and science to classrooms through the internet. "
						+ "NASA Quest is a resource for educators and kids "
						+ "interested in meeting and learning about NASA "
						+ "people and space science.");
		metadata0.addValue(DC_DESCRIPTION, "Check out this site for "
				+ "activities related to NASA's science and engineering "
				+ "projects . Topics range from solving air traffic control "
				+ "problems to designing a new planet, and there's something "
				+ "for inquisitive minds of all ages. Also, every week you "
				+ "can read a bio of a NASA worker.");
		metadata0.addValue(DC_DESCRIPTION, "The NSDL Middle School Pathway "
				+ "at ENC is an evolving site tailored to the needs of "
				+ "mathematics, science, and technology educators. On the "
				+ "site, ENC offers a middle school view into excellent "
				+ "online materials from the National Science Digital Library "
				+ "(NSDL). In this first phase of the project, teachers can "
				+ "examine three electronic publications in which content "
				+ "specialists spotlight superior digital resources and offer "
				+ "teaching tips for important topics.");
		metadata0.addValue(DC_FORMAT, "video/realvideo");
		metadata0.addValue(DC_FORMAT_IMT, "text/html");
		metadata0.addValue(DC_IDENTIFIER, "compadre.org:172");
		metadata0.addValue(DC_LANGUAGE_RFC3066, "en");
		metadata0.addValue(DC_PUBLISHER,
				"NASA Ames Educational Technology Team");
		metadata0.addValue(DC_RIGHTS, "Has a copyright or other restriction.");
		metadata0.addValue(DC_RIGHTS, "Copyright");
		metadata0.addValue(DC_TITLE, "NASA Quest");
		metadata0.addValue(DC_TITLE, "Middle School Pathway at ENC");
		metadata0.addValue(DC_TYPE, "Activity");
		metadata0.addValue(DC_TYPE, "Reference");
		metadata0.addValue(DC_TYPE, "Curriculum support");
		metadata0.addValue(DC_TYPE, "Lesson or Activity");
		metadata0.addValue(DC_TYPE_DCMITYPE, "Event");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Middle School");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Kindergarten");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 1");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 2");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 3");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 4");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 5");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 6");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 7");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 8");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 9");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 10");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 11");
		metadata0.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grade 12");
		metadata0.addValue(DCT_EDLEVEL, "Teacher");
		metadata0.addValue(DCT_MEDIATOR, "Teachers");
		metadata0.addValue(DCT_CREATED_W3CDTF, "2003-10-10T18:17:16Z");
		metadata0.addValue(DCT_CREATED_W3CDTF, "2003-07-09T18:35:17Z");
		metadata0.addValue(DCT_DATE_ACCEPTED_W3CDTF, "2003-10-10T18:17:17Z");
		metadata0.addValue(DCT_DATE_ACCEPTED_W3CDTF, "2003-12-05T14:34:21Z");
		metadata0.addValue(DCT_DATE_ACCEPTED_W3CDTF, "2005-12-28T13:57:26Z");
		metadata0.addValue(DCT_DATE_SUBMITTED_W3CDTF, "2003-10-10T18:17:16Z");
		metadata0.addValue(DCT_MODIFIED_W3CDTF, "2003-12-03T19:45:46Z");

		OaiCollectionInfo collectionInfo0 = new OaiCollectionInfo(
				"oai:nsdl.org:crs:1007936",
				"http://crs.nsdl.org/brands/1007936.jpg", "100", "35",
				"Middle School Pathway at ENC");

		assertCorrectMetadata(searchDoc,
				"oai:nsdl.org:2200-20061002172429873T", metadata0,
				collectionInfo0);

		ListsMap<LuceneFieldSpec, String> metadata1 = new ListsMap<LuceneFieldSpec, String>();
		metadata1.addValue(DCT_AUDIENCE, "Teachers");
		metadata1.addValue(DC_CREATOR,
				"National Aeronautics and Space Administration (NASA)");
		metadata1.addValue(DC_DESCRIPTION,
				"This site \"brings NASA people and science to K-12 "
						+ "classrooms via the Internet.\" The site includes "
						+ "chat opportunities, live broadcasts, and video "
						+ "series. The site provides a schedule of events "
						+ "with appropriate grade levels indicated and "
						+ "suggested classroom activities.");
		metadata1.addValue(DC_DESCRIPTION,
				"Michigan Teacher Network supports Michigan K-12 educators in "
						+ "the successful use of technology by providing best "
						+ "practice, professional development, and "
						+ "evaluated curriculum resources. Curriculum "
						+ "materials are correlated to Michigan "
						+ "Curriculum Framework content standards.");
		metadata1.addValue(DC_RIGHTS, "Metadata may be used without "
				+ "restrictions as long as the oai identifier "
				+ "remains attached to it.");
		metadata1.addValue(DC_SUBJECT, "Science - Space/Astronomy");
		metadata1.addValue(DC_SUBJECT, "Kindergarten");
		metadata1.addValue(DC_SUBJECT, "Early Elementary");
		metadata1.addValue(DC_SUBJECT, "Later Elementary");
		metadata1.addValue(DC_SUBJECT, "Middle School");
		metadata1.addValue(DC_SUBJECT, "High School");
		metadata1.addValue(DC_SUBJECT, "space sciences");
		metadata1.addValue(DC_SUBJECT, "astronomy");
		metadata1.addValue(DC_SUBJECT, "space exploration");
		metadata1.addValue(DC_SUBJECT, "solar system");
		metadata1.addValue(DC_TITLE, "NASA Quest");
		metadata1.addValue(DC_TITLE, "Michigan Teacher Network");
		metadata1.addValue(DC_TYPE, "Lesson/activity plans");
		metadata1.addValue(DC_TYPE, "Student Email interaction "
				+ "(includes Keypals, Ask an Expert services)");
		metadata1.addValue(DC_TYPE, "Video");
		metadata1.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grades Pre-K to 12");
		metadata1.addValue(DCT_MEDIATOR, "Teachers");

		OaiCollectionInfo collectionInfo1 = new OaiCollectionInfo(
				"oai:nsdl.org:crs:4708",
				"http://crs.nsdl.org/brands/merit.jpg", "100", "35",
				"Michigan Teacher Network");

		assertCorrectMetadata(searchDoc,
				"oai:nsdl.org:2200-20061002193307574T", metadata1,
				collectionInfo1);

		ListsMap<LuceneFieldSpec, String> metadata2 = new ListsMap<LuceneFieldSpec, String>();
		metadata2.addValue(DC_DATE_W3CDTF, "2001-02-06");
		metadata2.addValue(DC_DESCRIPTION, "NASA Quest is dedicated to "
				+ "bringing NASA people and science to classrooms through "
				+ "the internet via Space Team Online, Aerospace Team Online, "
				+ "Solar System Online, and Deep Space Online. NASA Quest is "
				+ "the ultimate resource for educators and kids interested in "
				+ "meeting and learning about NASA people and space science. "
				+ "These NASA Quest projects allow students to share in the "
				+ "excitement of NASA's authentic scientific and engineering "
				+ "pursuits like flying the shuttle and International Space "
				+ "Station, exploring distant planets with amazing "
				+ "spacecraft, and aeronautics/airplane research.");
		metadata2.addValue(DC_FORMAT_IMT, "text/html");
		metadata2.addValue(DC_FORMAT_IMT, "video");
		metadata2.addValue(DC_LANGUAGE_RFC3066, "en");
		metadata2.addValue(DC_PUBLISHER, "United States. National "
				+ "Aeronautics and Space Administration. Ames Research Center");
		metadata2.addValue(DC_SUBJECT, "NASA; education; quest; "
				+ "space science; science; space; education; resource; "
				+ "educator; teacher; kid; k-12; space team; aerospace; "
				+ "solar system; deep space; lesson plan; webcast; events; "
				+ "space shuttle; mars; rockets; astronauts; "
				+ "apollo; support; community; newsletters; about nasa");
		metadata2.addValue(DC_SUBJECT_GEM, "Space sciences");
		metadata2.addValue(DC_SUBJECT_LCSH,
				"Science--Study and teaching--United States");
		metadata2.addValue(DC_SUBJECT_LCSH, "Solar system");
		metadata2.addValue(DC_TITLE, "NASA Quest");
		metadata2.addValue(DC_TYPE_DCMITYPE, "Collection");
		metadata2.addValue(DC_TYPE_DCMITYPE, "InteractiveResource");
		metadata2.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Grades Pre-K to 12");
		metadata2.addValue(DCT_MEDIATOR, "Teachers");
		metadata2.addValue(DCT_EXTENT, "10636 bytes");

		OaiCollectionInfo collectionInfo2 = new OaiCollectionInfo(
				"oai:nsdl.org:nsdl:4002",
				"http://crs.nsdl.org/brands/nasaquest.jpg", "100", "30",
				"NASA Quest");

		assertCorrectMetadata(searchDoc, "oai:nsdl.org:crs:4726", metadata2,
				collectionInfo2);

		ListsMap<LuceneFieldSpec, String> metadata3 = new ListsMap<LuceneFieldSpec, String>();
		metadata3.addValue(DCT_AUDIENCE, "Teacher");
		metadata3.addValue(DCT_AUDIENCE, "Learner");
		metadata3.addValue(DC_CREATOR, "Donald James");
		metadata3.addValue(DC_DESCRIPTION, "NASA Quest is dedicated to "
				+ "bringing NASA people and science to classrooms through the "
				+ "internet. NASA Quest is a resource for educators and kids "
				+ "interested in meeting and learning about NASA people "
				+ "and space science.");
		metadata3.addValue(DC_DESCRIPTION, "Check out this site for "
				+ "activities related to NASA's science and engineering "
				+ "projects . Topics range from solving air traffic control "
				+ "problems to designing a new planet, and there's "
				+ "something for inquisitive minds of all ages. "
				+ "Also, every week you can read a bio of a NASA worker.");
		metadata3.addValue(DC_DESCRIPTION, "ComPADRE is a set of communities "
				+ "of teachers and students in physics and astronomy, and "
				+ "web-based collections of resources to support their "
				+ "needs.  The communities supported by ComPADRE are groups "
				+ "that can benefit from the sharing of materials, "
				+ "information, and experiences in a web environment.  "
				+ "Example communities include teachers of specific physics "
				+ "or astronomy courses, societies of undergraduate physics "
				+ "and astronomy students, and teachers in specific grade "
				+ "levels, such as high school or middle school teachers.  "
				+ "Resources included in the collections are chosen to "
				+ "enhance the teaching and learning experience, and include "
				+ "multimedia learning objects, lesson plans, tutorials, "
				+ "laboratories and other student activities, and discussion "
				+ "forums on the use these materials.  The different "
				+ "collections are organized under the ComPADRE umbrella, "
				+ "which provides a central database (the Physical Sciences "
				+ "Resource Center), technical support, support for "
				+ "collection editors and community leaders, and the means to "
				+ "coordinate efforts across the communities.");
		metadata3.addValue(DC_FORMAT, "video/realvideo");
		metadata3.addValue(DC_FORMAT_IMT, "text/html");
		metadata3.addValue(DC_IDENTIFIER, "compadre.org:172");
		metadata3.addValue(DC_LANGUAGE_RFC3066, "en");
		metadata3.addValue(DC_PUBLISHER,
				"NASA Ames Educational Technology Team");
		metadata3.addValue(DC_RIGHTS, "Has a copyright or other restriction.");
		metadata3.addValue(DC_RIGHTS, "Copyright");
		metadata3.addValue(DC_SUBJECT, "flight");
		metadata3.addValue(DC_SUBJECT, "space exploration");
		metadata3.addValue(DC_SUBJECT, "biographies");
		metadata3.addValue(DC_SUBJECT, "Astronomy");
		metadata3.addValue(DC_SUBJECT, "Astronomy Education");
		metadata3.addValue(DC_TITLE, "NASA Quest");
		metadata3.addValue(DC_TITLE,
				"comPADRE: Resources for physics and astronomy education");
		metadata3.addValue(DC_TYPE, "Reference");
		metadata3.addValue(DC_TYPE, "Curriculum support");
		metadata3.addValue(DC_TYPE, "Activity");
		metadata3.addValue(DC_TYPE_DCMITYPE, "Event");
		metadata3.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "High School");
		metadata3.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Middle School");
		metadata3.addValue(DCT_EDLEVEL_NSDL_EDLEVEL, "Postsecondary");
		metadata3.addValue(DCT_MEDIATOR, "Teachers");
		metadata3.addValue(DCT_CREATED_W3CDTF, "2003-10-10T18:17:16Z");
		metadata3.addValue(DCT_CREATED_W3CDTF, "2003-07-09T18:35:17Z");
		metadata3.addValue(DCT_DATE_ACCEPTED_W3CDTF, "2003-10-10T18:17:17Z");
		metadata3.addValue(DCT_DATE_ACCEPTED_W3CDTF, "2003-12-05T14:34:21Z");
		metadata3.addValue(DCT_DATE_ACCEPTED_W3CDTF, "2005-12-28T13:57:26Z");
		metadata3.addValue(DCT_DATE_SUBMITTED_W3CDTF, "2003-10-10T18:17:16Z");
		metadata3.addValue(DCT_MODIFIED_W3CDTF, "2003-12-03T19:45:46Z");
		OaiCollectionInfo collectionInfo3 = new OaiCollectionInfo(
				"oai:nsdl.org:crs:439869",
				"http://crs.nsdl.org/brands/439869.jpg", "100", "20",
				"comPADRE: Resources for physics and astronomy education");

		assertCorrectMetadata(searchDoc,
				"oai:nsdl.org:2200-20061003055539958T", metadata3,
				collectionInfo3);
	}

	@Test(expected = FatalSearchIndexException.class)
	public void testNoIdentifier() {
		Document doc = ExpectedDocuments.getDocument1();
		doc.removeField(RESOURCE_IDENTIFIER.getName());
		new SearchDocument(doc);
	}

	/**
	 * This no longer throws an exception, it merely produces a log entry.
	 * What's a good way to test for that?
	 */
	@Test(expected = FatalSearchIndexException.class)
	@Ignore
	public void testTwoCreateDates() {
		Document doc = ExpectedDocuments.getDocument1();
		Field field = new Field(DOCUMENT_CREATED.getName(),
				"2007-04-12T21:01:22Z", Field.Store.YES, Field.Index.TOKENIZED);
		doc.add(field);
		new SearchDocument(doc);
	}

	@Test
	@Ignore
	public void testUnloadWithContent() {
		// If content exists in the content cache, it should be added to
		// AllFields.

		// ContentLastFetched and ContentLastModified should be set by
		// setContent, and when loaded.

		// Content gets added to the AllFields when unloaded.
		fail("not implemented");
	}

	private void assertProperMetadataMapping(String[][] mappingPairs,
			SearchDocument searchDoc) {
		Map<String, String> expected = new HashMap<String, String>();
		for (String[] mapping : mappingPairs) {
			expected.put(mapping[0], mapping[1]);
		}

		Collection<SearchDocumentMetadata> metadatas = searchDoc
				.getMetadataSources();
		assertEquals(mappingPairs.length, metadatas.size());

		Map<String, String> actual = new HashMap<String, String>();
		for (SearchDocumentMetadata metadata : metadatas) {
			actual.put(metadata.getOaiId(), metadata.getRawUrl());
		}

		assertEquals(expected, actual);
	}

	private void assertCorrectMetadata(SearchDocument doc, String oaiId,
			ListsMap<LuceneFieldSpec, String> expectedFieldData,
			OaiCollectionInfo expectedCollInfo) {
		SearchDocumentMetadata metadata = doc.getMetadata(oaiId);

		OaiCollectionInfo actualCollInfo = metadata.getCollInfo();
		assertEqualCollectionInfo(expectedCollInfo, actualCollInfo);

		Set<LuceneFieldSpec> expectedFields = expectedFieldData.keySet();
		Set<LuceneFieldSpec> actualFields = metadata.getFields();

		Set<LuceneFieldSpec> extraFields = new HashSet<LuceneFieldSpec>(
				actualFields);
		extraFields.removeAll(expectedFields);
		Set<LuceneFieldSpec> missingFields = new HashSet<LuceneFieldSpec>(
				expectedFields);
		missingFields.removeAll(actualFields);
		if (!extraFields.isEmpty() || !missingFields.isEmpty()) {
			fail("Wrong fields - extra=" + extraFields + ", missing="
					+ missingFields);
		}

		for (LuceneFieldSpec spec : expectedFields) {
			List<String> expectedValues = expectedFieldData.get(spec);
			List<String> actualValues = metadata.getFieldValues(spec);
			assertEquals(expectedValues, actualValues);
		}
	}

	private void assertEqualCollectionInfo(OaiCollectionInfo expected,
			OaiCollectionInfo actual) {
		assertEquals("Collection OAI ID", expected.getCollectionOaiId(), actual
				.getCollectionOaiId());
		assertEquals("Collection Image URL", expected.getImageUrl(), actual
				.getImageUrl());
		assertEquals("Collection Image Height", expected.getImageHeight(),
				actual.getImageHeight());
		assertEquals("Collection Image Width", expected.getImageWidth(), actual
				.getImageWidth());
		assertEquals("Collection Image Alt Text", expected.getImageAltText(),
				actual.getImageAltText());
	}

	private void assertEqualDates(String expectedString, Date actualDate) {
		assertEquals(expectedString, DateUtil.getInstance().formatDate(
				actualDate, SearchDocument.DATE_FORMAT));
	}

}
