/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.searchdoc;

import static junit.framework.Assert.assertEquals;

import java.text.ParseException;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import mock.search.collContext.MockCollRecCache;

import org.junit.Test;
import org.nsdl.search.SearchUnitTest;
import org.nsdl.search.collContext.CollRecCache;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;
import org.nsdl.search.processoai.OaiCollectionInfo;
import org.nsdl.search.processoai.OaiHarvestRecord;
import org.nsdl.searchtools.common.DateUtil;
import org.nsdl.searchtools.common.ListsMap;

/**
 * 
 * @author jblake
 */
public class TestSearchDocumentMetadata extends SearchUnitTest {
	private static final String OAI_ID = "oaiId";
	private static final String RAW_URL = "rawUrl";
	private static final OaiCollectionInfo EMPTY_COLLECTION_INFO = new OaiCollectionInfo(
			null, null, null, null, null);

	private SearchDocumentMetadata md;

	@Override
	protected Class<?>[][] getImplementationClasses() {
		return new Class[][] { { CollRecCache.class, MockCollRecCache.class } };
	}

	@Test
	public void createEmptyRecord() {
		md = new SearchDocumentMetadata(OAI_ID, RAW_URL);

		assertExpectedMetadata(OAI_ID, new Date(0L), RAW_URL,
				EMPTY_COLLECTION_INFO, new Field[0]);
	}

	@Test
	public void createFromHarvestRecord() throws ParseException {
		String dateString = DateUtil.getInstance().formatDate(new Date(0L),
				DateUtil.ISO8601_DATE_TIME_Z);
		OaiCollectionInfo collectionInfo = new OaiCollectionInfo(
				"collectionOaiId", "imageUrl", "79", "40", "imageAltText");
		ListsMap<LuceneFieldSpec, String> fieldInfo = new ListsMap<LuceneFieldSpec, String>();
		fieldInfo.addValue(SearchIndexFieldSpecs.COLL_DC_DESCRIPTION,
				"Silly description");

		MockCollRecCache cache = MockCollRecCache.getInstance();
		cache.addCollectionInfo("collectionOaiId", fieldInfo);

		Field[] expectedFields = new Field[] { new Field(
				SearchIndexFieldSpecs.COLL_DC_DESCRIPTION,
				new String[] { "Silly description" }) };

		OaiHarvestRecord oai = new OaiHarvestRecord();
		oai.setOaiRecId(OAI_ID);
		oai.setOaiRecDatestamp(dateString);
		oai.setCollectionInfo(collectionInfo);

		md = new SearchDocumentMetadata(RAW_URL, oai);

		assertExpectedMetadata(OAI_ID, new Date(0L), RAW_URL, collectionInfo,
				expectedFields);
	}

	/**
	 * Confirm that the actual metadata is the same as we were expecting.
	 */
	private void assertExpectedMetadata(String oaiId, Date dateStamp,
			String rawUrl, OaiCollectionInfo collectionInfo, Field[] fields) {
		assertEquals("oaiID", oaiId, md.getOaiId());
		assertEquals("date stamp", dateStamp, md.getOaiDateStamp());
		assertEquals("rawURL", rawUrl, md.getRawUrl());
		assertEquals("collectionInfo", collectionInfo, md.getCollInfo());

		Set<LuceneFieldSpec> specs = new HashSet<LuceneFieldSpec>();
		for (Field field : fields) {
			specs.add(field.getSpec());
		}
		assertEquals("fieldSpecs", specs, md.getFields());

		for (Field field : fields) {
			LuceneFieldSpec spec = field.getSpec();
			assertEquals("fieldValues for " + spec.getName(), Arrays
					.asList(field.getValues()), md.getFieldValues(spec));
		}
	}

	/** Holds expected field data. */
	private static class Field {
		private final LuceneFieldSpec spec;
		private final String[] values;

		public Field(LuceneFieldSpec spec, String[] values) {
			this.spec = spec;
			this.values = values;
		}

		public LuceneFieldSpec getSpec() {
			return spec;
		}

		public String[] getValues() {
			return values;
		}

	}

}
