/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.lucene.wrapper;

import static junit.framework.Assert.assertEquals;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.List;

import mock.search.content.MockContentCacheWrapper;

import org.apache.lucene.queryParser.ParseException;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.nsdl.search.SearchUnitTest;
import org.nsdl.search.content.ContentCacheWrapper;
import org.nsdl.search.lucene.LuceneFieldSpec;
import org.nsdl.search.lucene.LuceneUtil;
import org.nsdl.search.lucene.SearchIndexFieldSpecs;
import org.nsdl.search.lucene.searchdoc.SearchDocument;
import org.nsdl.search.lucene.searchdoc.SearchDocumentMetadata;
import org.nsdl.search.util.ConfigImpl;
import org.nsdl.searchtools.common.ListsMap;

/**
 * 
 * @author jblake
 */
public class TestIndexWrapper extends SearchUnitTest {
	private static final String INDEX_DIRECTORY_PATH = "./testfiles/testIndex/";

	/** Use this Config object so the IndexWrapper works with our test index. */
	private static final MyConfig indexWrapperConfig = new MyConfig();

	@Override
	protected boolean isOutputDumped() {
		return false;
	}

	@Override
	protected Class<?>[][] getImplementationClasses() {
		return new Class[][] { { ContentCacheWrapper.class,
				MockContentCacheWrapper.class } };
	}

	/**
	 * Set the IndexWrapper to point to the test index.
	 */
	@BeforeClass
	public static void initializeIndexWrapper() {
		ConfigImpl.getInstance().setIndexWrapperConfig(indexWrapperConfig);
		IndexWrapperImpl.reset();
	}

	private IndexWrapper index;
	private MyIndexListener indexListener;

	/**
	 * Create an empty test index every time.
	 */
	@Before
	public void initializeIndexAndWrapper() throws IOException {
		cleanDirectory(new File(INDEX_DIRECTORY_PATH));
		index = IndexWrapper.getInstance();
		index.createIndex();
		index.synchronize();

		// now set a fresh listener on the index.
		indexListener = new MyIndexListener();
		index.setListener(indexListener);
	}

	/**
	 * An index should already exist. Write to it, confirm that we have written,
	 * then re-create the index. Voila, empty!
	 */
	@Test
	public void checkCreateIndex() throws IOException {
		// start with an empty index
		assertEquals("Should be just a segments file", 1, howManyFilesInIndex());

		// add a document and see it there.
		index.writeDocument(new SearchDocument("http://my.silly.document/"));
		index.synchronize();
		assertEquals("Should have one document", 1, howManyDocsInIndex());
		assertEquals("Should be a segment also and a deletable", 3,
				howManyFilesInIndex());

		// re-create the index. document is gone.
		index.createIndex();
		index.synchronize();
		assertEquals("Document should be gone", 0, howManyDocsInIndex());
		assertEquals("Should be just a segments file", 1, howManyFilesInIndex());

		// check index activity.
		assertEquals("events", 2, indexListener.getTotalEvents());
		assertEquals("deletes", 0, indexListener.getTotalDeletes());
		assertEquals("writes", 1, indexListener.getTotalWrites());
	}

	/**
	 * Test that functions work correctly after synchronize: write a doc,
	 * synchronize, show that it is there. search for it, also getDocument.
	 * 
	 * Then delete the doc, synchronize, show that it is gone.
	 */
	@Test
	public void testADocumentAfterSynchronize() throws IOException,
			ParseException {
		String url = "http://some.strange.document";
		String oaiId = "oai:metadata";
		LuceneFieldSpec spec = SearchIndexFieldSpecs.DC_SUBJECT;
		String fieldValue = "thesaurus";
		SearchResults results = null;

		SearchDocument doc = makeDocWithOneField(url, oaiId, spec, fieldValue);

		IndexWrapperQuery valueQuery = index.parseQuery(fieldValue);
		IndexWrapperQuery fieldQuery = makeFieldQuery(spec, fieldValue);

		index.writeDocument(doc);
		index.synchronize();
		assertEquals("Should have one document", 1, howManyDocsInIndex());

		results = index.search(valueQuery, 0, 10);
		assertEquals("Should find the doc", 1, results.getReturnCount());

		results = index.search(fieldQuery, 0, 10);
		assertEquals("Should find the doc", 1, results.getReturnCount());

		results = index.getDocument(url);
		assertEquals("Should find the doc", 1, results.getReturnCount());

		index.deleteDocument(doc);
		index.synchronize();
		assertEquals("Should have zero documents", 0, howManyDocsInIndex());

		results = index.search(valueQuery, 0, 10);
		assertEquals("Doc should be gone", 0, results.getReturnCount());

		results = index.getDocument(url);
		assertEquals("Doc should be gone", 0, results.getReturnCount());

		// check index activity.
		assertEquals("events", 2, indexListener.getTotalEvents());
		assertEquals("deletes", 1, indexListener.getTotalDeletes());
		assertEquals("writes", 1, indexListener.getTotalWrites());
	}

	/**
	 * Show how the index reacts to adds, deletes and changes even before
	 * synchronizing. Then synchronize.
	 * 
	 * Before synchronizing, an added doc is not found in a search, a deleted
	 * doc is not returned in a search result, and a revised doc is revised in
	 * the search result.
	 */
	@Test
	public void preAndPostSynchronize() throws IOException, ParseException {
		SearchResults results = null;

		// Set up the initial contents, and synchronize.
		String url1 = "http://doc.no.change/";
		String oaiId1 = "oai:md.no.change";
		LuceneFieldSpec spec1 = SearchIndexFieldSpecs.DC_SUBJECT;
		String fieldValue1 = "noChange";
		SearchDocument doc1 = makeDocWithOneField(url1, oaiId1, spec1,
				fieldValue1);

		String url2 = "http://doc.replace/";
		String oaiId2 = "oai:md.replace";
		LuceneFieldSpec spec2 = SearchIndexFieldSpecs.DC_SUBJECT;
		String fieldValue2 = "toBeReplaced";
		SearchDocument doc2 = makeDocWithOneField(url2, oaiId2, spec2,
				fieldValue2);

		String url3 = "http://doc.delete/";
		String oaiId3 = "oai:md.delete";
		LuceneFieldSpec spec3 = SearchIndexFieldSpecs.DC_SUBJECT;
		String fieldValue3 = "toBeDeleted";
		SearchDocument doc3 = makeDocWithOneField(url3, oaiId3, spec3,
				fieldValue3);

		index.writeDocument(doc1);
		index.writeDocument(doc2);
		index.writeDocument(doc3);
		index.synchronize();
		assertEquals("Should have three documents", 3, howManyDocsInIndex());
		results = index.getDocument(url1);
		assertEquals("Should find doc1", 1, results.getReturnCount());
		results = index.getDocument(url2);
		assertEquals("Should find doc2", 1, results.getReturnCount());
		results = index.search(makeFieldQuery(spec2, fieldValue2), 0, 10);
		assertEquals("Should find doc2", 1, results.getReturnCount());
		results = index.getDocument(url3);
		assertEquals("Should find doc3", 1, results.getReturnCount());

		// Add changes, but don't synchronize.
		String fieldValue2a = "updated";
		SearchDocument doc2a = makeDocWithOneField(url2, oaiId2, spec2,
				fieldValue2a);

		String url4 = "http://doc.new/";
		String oaiId4 = "oai:md.new";
		LuceneFieldSpec spec4 = SearchIndexFieldSpecs.DC_SUBJECT;
		String fieldValue4 = "newDocument";
		SearchDocument doc4 = makeDocWithOneField(url4, oaiId4, spec4,
				fieldValue4);

		index.writeDocument(doc2a);
		index.deleteDocument(doc3);
		index.writeDocument(doc4);

		// physical index hasn't changed.
		assertEquals("Should have three documents", 3, howManyDocsInIndex());

		results = index.getDocument(url1);
		assertEquals("Should find doc1", 1, results.getReturnCount());
		results = index.getDocument(url2);
		assertEquals("Should find doc2", 1, results.getReturnCount());
		results = index.search(makeFieldQuery(spec2, fieldValue2), 0, 10);
		assertEquals("Should find doc2 with old contents", 1, results
				.getReturnCount());
		// ... but returned value has new contents.
		assertCorrectFieldValue(results, spec2, fieldValue2a);
		results = index.search(makeFieldQuery(spec2, fieldValue2a), 0, 10);
		assertEquals("Should not find doc2 with new contents", 0, results
				.getReturnCount());
		results = index.getDocument(url3);
		assertEquals("Should not find doc3", 0, results.getReturnCount());
		results = index.getDocument(url4);
		assertEquals("Should not find doc4", 0, results.getReturnCount());

		// now synchronize and test the changes.
		index.synchronize();
		assertEquals("Should have three documents", 3, howManyDocsInIndex());

		results = index.getDocument(url1);
		assertEquals("Should still find doc1", 1, results.getReturnCount());
		results = index.getDocument(url2);
		assertEquals("Should find doc2", 1, results.getReturnCount());
		results = index.search(makeFieldQuery(spec2, fieldValue2), 0, 10);
		assertEquals("Should not find doc2 with old contents", 0, results
				.getReturnCount());
		results = index.search(makeFieldQuery(spec2, fieldValue2a), 0, 10);
		assertEquals("Should find doc2 with new contents", 1, results
				.getReturnCount());
		// ... and returned value has new contents.
		assertCorrectFieldValue(results, spec2, fieldValue2a);
		results = index.getDocument(url3);
		assertEquals("Should not find doc3", 0, results.getReturnCount());
		results = index.getDocument(url4);
		assertEquals("Should find doc4", 1, results.getReturnCount());

		// check index activity.
		assertEquals("events", 2, indexListener.getTotalEvents());
		assertEquals("deletes", 1, indexListener.getTotalDeletes());
		assertEquals("writes", 5, indexListener.getTotalWrites());
	}

	// ------------------------------------------------------------------------
	// utilities
	// ------------------------------------------------------------------------

	private int howManyFilesInIndex() {
		File[] files = new File(INDEX_DIRECTORY_PATH)
				.listFiles(new FileFilter() {
					public boolean accept(File file) {
						return !file.getName().equals("CVS")
								&& !file.getName().equals(".svn");
					}
				});
		return files.length;
	}

	private int howManyDocsInIndex() throws IOException {
		int howManyDocs = 0;
		ListsMap<String, SearchDocument> contents = getContentsOfLuceneIndex(INDEX_DIRECTORY_PATH);
		for (String key : contents.keySet()) {
			howManyDocs += contents.get(key).size();
		}
		return howManyDocs;
	}

	private IndexWrapperQuery makeFieldQuery(LuceneFieldSpec spec,
			String fieldValue) throws ParseException, IOException {
		String fieldName = LuceneUtil.getInstance().encodeLuceneChars(
				spec.getName());
		return index.parseQuery(fieldName + ":" + fieldValue);
	}

	private SearchDocument makeDocWithOneField(String url, String oaiId,
			LuceneFieldSpec spec, String fieldValue) {
		SearchDocumentMetadata metadata = new SearchDocumentMetadata(oaiId, url);
		metadata.addFieldValue(spec, fieldValue);
		SearchDocument doc = new SearchDocument(url);
		doc.addMetadataSource(metadata);
		return doc;
	}

	/**
	 * In the search results we expect one doc with one metadata source with a
	 * particular value of a given field.
	 */
	private void assertCorrectFieldValue(SearchResults results,
			LuceneFieldSpec spec, String expected) {
		assertEquals("expecting one result", 1, results.getReturnCount());
		for (SearchResult result : results) {
			SearchDocument sd = result.getDocument();
			assertEquals("expecting one metadata", 1, sd.getMetadataSourceIds()
					.size());
			for (SearchDocumentMetadata md : sd.getMetadataSources()) {
				List<String> values = md.getFieldValues(spec);
				assertEquals("expecting one value", 1, values.size());
				assertEquals("wrong value", expected, values.get(0));
				break;
			}
			break;
		}
	}

	/**
	 * Point to the test index directory, set large limits so we can control the
	 * synchronization.
	 */
	private static class MyConfig implements IndexWrapperConfig {
		private int cacheLimit = 100;

		public String getDocIdFieldName() {
			return SearchIndexFieldSpecs.RESOURCE_IDENTIFIER.getName();
		}

		public String getLuceneIndexDirectory() {
			return INDEX_DIRECTORY_PATH;
		}

		public long getInactivityThreshold() {
			return 1000;
		}

		public int getMaximumCachedWrites() {
			return cacheLimit;
		}

		public void setMaximumCachedWrites(int cacheLimit) {
			this.cacheLimit = cacheLimit;
		}
	}

	/**
	 * Keep tallys of records written and/or deleted.
	 */
	private static class MyIndexListener implements IndexWrapperListener {
		private int totalEvents;
		private int totalDeletes;
		private int totalWrites;

		public void recordEvent(EventType type, int pendingDeletes,
				int pendingWrites) {
			totalEvents++;
			totalDeletes += pendingDeletes;
			totalWrites += pendingWrites;
		}

		public int getTotalEvents() {
			return totalEvents;
		}

		public int getTotalDeletes() {
			return totalDeletes;
		}

		public int getTotalWrites() {
			return totalWrites;
		}

	}

}
