/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.oai.harvest;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.fail;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.junit.Test;
import org.nsdl.searchtools.common.io.FileUtil;

/**
 * Test the functioning of {@link OaiHarvestLogSummary}.
 * 
 * @author jblake
 */
public class TestOaiHarvestLogSummary {
	private static final String TESTFILES_PARENT_DIRECTORY = "./testfiles/oaiHarvestLogs";
	private static final String HARVEST_DIRECTORY_NO_LOG = TESTFILES_PARENT_DIRECTORY
			+ "/noLog";
	@SuppressWarnings("unused")
	private static final String HARVEST_DIRECTORY_ERROR_ABORT = TESTFILES_PARENT_DIRECTORY
			+ "/errorAbort";
	private static final String HARVEST_DIRECTORY_NO_MATCH = TESTFILES_PARENT_DIRECTORY
			+ "/noMatch";
	private static final String HARVEST_DIRECTORY_ONE_RESPONSE = TESTFILES_PARENT_DIRECTORY
			+ "/oneResponse";
	private static final String HARVEST_DIRECTORY_MANY_RESPONSES = TESTFILES_PARENT_DIRECTORY
			+ "/manyResponses";

	@Test(expected = IOException.class)
	public void noSuchDirectory() throws IOException, OaiLogFormatException {
		OaiHarvestLogSummary.getSummary("bogusPath");
	}

	@Test(expected = IOException.class)
	public void noLogFileInTheDirectory() throws IOException,
			OaiLogFormatException {
		OaiHarvestLogSummary.getSummary(HARVEST_DIRECTORY_NO_LOG);
	}

	@Test
	public void noMatch() throws IOException, OaiLogFormatException {
		OaiHarvestLogSummary summary = OaiHarvestLogSummary
				.getSummary(HARVEST_DIRECTORY_NO_MATCH);
		assertLogSummary(summary, "http://ndr.nsdl.org/search-oai",
				"nsdl_search", 0, "2007-01-08T22:21:34Z",
				"2007-01-09T18:10:59Z", 0, 0, new String[0]);
	}

	@Test
	public void oneResponse() throws IOException, OaiLogFormatException {
		OaiHarvestLogSummary summary = OaiHarvestLogSummary
				.getSummary(HARVEST_DIRECTORY_ONE_RESPONSE);
		assertLogSummary(summary, "http://ndr.nsdl.org/search-oai",
				"nsdl_search", 5, "2007-03-14T09:00:01Z",
				"2007-03-15T09:00:02Z", 1, 5, new String[0]);
	}

	@Test
	public void manyResponses() throws IOException, OaiLogFormatException {
		OaiHarvestLogSummary summary = OaiHarvestLogSummary
				.getSummary(HARVEST_DIRECTORY_MANY_RESPONSES);
		assertLogSummary(summary, "http://ndr.nsdl.org/search-oai",
				"nsdl_search", 250, "2007-01-08T22:21:34Z",
				"2007-01-10T18:49:02Z", 38, 9259, new String[0]);
	}

	/**
	 * If there is no arguments line, we should see an exception.
	 */
	@Test(expected = OaiLogFormatException.class)
	public void harvestArgumentsAreMalformed() throws IOException,
			OaiLogFormatException {
		File logFile = new File(HARVEST_DIRECTORY_ONE_RESPONSE,
				OAIHarvestConfig.HARVEST_LOG_FNAME);
		String logContents = FileUtil.getInstance().getContents(logFile);
		String modifiedLog = logContents.replace("Doing POST to",
				"BOGUS ARGS LINE");
		new OaiHarvestLogSummary(new StringReader(modifiedLog));
	}

	@Test
	public void oneResponseWithSomeWarningMessages() throws IOException,
			OaiLogFormatException {
		String warning1 = "WARNING: can you hear me now?";
		String warning2 = "WarninG: mixed case";
		String error1 = "Error: a simple error";

		File logFile = new File(HARVEST_DIRECTORY_ONE_RESPONSE,
				OAIHarvestConfig.HARVEST_LOG_FNAME);
		String logContents = FileUtil.getInstance().getContents(logFile);
		String modifiedLog = logContents + '\n' + warning1 + '\n' + warning2
				+ '\n' + error1 + '\n';

		OaiHarvestLogSummary summary = new OaiHarvestLogSummary(
				new StringReader(modifiedLog));
		assertLogSummary(summary, "http://ndr.nsdl.org/search-oai",
				"nsdl_search", 5, "2007-03-14T09:00:01Z",
				"2007-03-15T09:00:02Z", 1, 5, new String[]{warning1, warning2,
						error1});
	}

	/**
	 * If no from date is found, go with a hardcoded early date.
	 */
	@Test
	public void noFromDate() throws IOException, OaiLogFormatException {
		File logFile = new File(HARVEST_DIRECTORY_ONE_RESPONSE,
				OAIHarvestConfig.HARVEST_LOG_FNAME);
		String logContents = FileUtil.getInstance().getContents(logFile);
		String modifiedLog = logContents.replace("&from=", "&bogus=");

		OaiHarvestLogSummary summary = new OaiHarvestLogSummary(
				new StringReader(modifiedLog));
		assertLogSummary(summary, "http://ndr.nsdl.org/search-oai",
				"nsdl_search", 5, OaiHarvestLogSummary.DEFAULT_FROM_DATE,
				"2007-03-15T09:00:02Z", 1, 5, new String[0]);
	}

	/**
	 * If no until date is found, go with the date of the first request.
	 */
	@Test
	public void noUntilDate() throws IOException, OaiLogFormatException {
		File logFile = new File(HARVEST_DIRECTORY_ONE_RESPONSE,
				OAIHarvestConfig.HARVEST_LOG_FNAME);
		String logContents = FileUtil.getInstance().getContents(logFile);
		String modifiedLog = logContents.replace(" until=", " bogus=");

		OaiHarvestLogSummary summary = new OaiHarvestLogSummary(
				new StringReader(modifiedLog));
		assertLogSummary(summary, "http://ndr.nsdl.org/search-oai",
				"nsdl_search", 5, "2007-03-14T09:00:01Z",
				"2007-03-15T09:00:02Z", 1, 5, new String[0]);
	}

	/**
	 * Confirm that all of the info from the log analyzer is correct.
	 */
	private void assertLogSummary(OaiHarvestLogSummary summary,
			String expectedBaseUrl, String expectedMetadataPrefix,
			int expectedNumRecordsPerResponse, String expectedFromDate,
			String expectedUntilDate, long expectedNumResponses,
			long expectedNumRecords, String[] expectedWarnings)
			throws IOException {
		assertEquals("base URL", expectedBaseUrl, summary.getBaseURLStr());
		assertEquals("from date", expectedFromDate, summary.getFromDate());
		assertEquals("metadata prefix", expectedMetadataPrefix, summary
				.getMetadataPrefix());
		assertEquals("number of responses", expectedNumResponses, summary
				.getNumListRecordsResponses());
		assertEquals("number of records", expectedNumRecords, summary
				.getNumRecsHarvested());
		assertEquals("number of records per response",
				expectedNumRecordsPerResponse, summary
						.getNumRecsPerListRecordsResponse());
		assertEquals("until date", expectedUntilDate, summary.getUntilDate());
		assertEqualStringArrays("warnings", expectedWarnings, summary
				.getWarnings());
	}

	private void assertEqualStringArrays(String message, String[] expected,
			String[] actual) {
		List<String> expectedList = Arrays.asList(expected);
		List<String> actualList = Arrays.asList(actual);
		List<String> missing = new ArrayList<String>(expectedList);
		missing.removeAll(actualList);
		List<String> unexpected = new ArrayList<String>(actualList);
		missing.removeAll(expectedList);

		if (missing.isEmpty() && unexpected.isEmpty()) {
			return;
		}

		String fullMessage = "Arrays don't match: " + message + '\n';
		if (!missing.isEmpty()) {
			fullMessage += "Missing strings:\n";
			for (String notThere : missing) {
				fullMessage += "  '" + notThere + "'\n";
			}
		}
		if (!unexpected.isEmpty()) {
			fullMessage += "Unexpected strings:\n";
			for (String extra : unexpected) {
				fullMessage += "  '" + extra + "'\n";
			}
		}
		fail(fullMessage);
	}

}
