/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.oai.process;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.fail;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import org.junit.After;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.nsdl.search.SearchUnitTest;
import org.nsdl.search.oai.harvest.HarvestDirectoryWrapper;
import org.nsdl.searchtools.common.DateUtil;
import org.nsdl.searchtools.common.history.HistoryException;
import org.nsdl.searchtools.common.history.HistoryFile;

/**
 * 
 * @author jblake
 */
public class TestOaiHarvestProcessorParameters extends SearchUnitTest {

	// @Override
	// protected Class<?>[][] getImplementationClasses() {
	// // TODO Auto-generated method stub
	// throw new RuntimeException(
	// "TestOaiHarvestProcessorParameters.getImplementationClasses() not
	// implemented.");
	// }

	private static File tempDir;
	private static File harvestParentEmpty;
	private static File harvestParentOne;
	private static File harvestParentMultiple;
	private static File theOne;
	private static File theOldest;
	private static File theMiddle;
	private static File theYoungest;
	private static File historyFile;
	private static File luceneIndex;
	private static File luceneSegmentsFile;

	private OaiHarvestProcessorParameters parms;

	@Before
	public void createHarvestDirectories() throws IOException {
		String tempPath = System.getProperty("java.io.tmpdir");
		tempDir = new File(tempPath, "TestOaiHarvestProcessorParameters");
		tempDir.mkdir();

		harvestParentEmpty = new File(tempDir, "empty");
		harvestParentEmpty.mkdir();

		harvestParentOne = new File(tempDir, "oneHarvest");
		harvestParentOne.mkdir();
		theOne = new File(harvestParentOne, "1111-11-11T11-11-11Z");
		theOne.mkdir();

		harvestParentMultiple = new File(tempDir, "multiple");
		harvestParentMultiple.mkdir();
		theOldest = new File(harvestParentMultiple, "2000-01-01T00-00-00Z");
		theOldest.mkdir();
		theMiddle = new File(harvestParentMultiple, "2000-01-01T11-11-11Z");
		theMiddle.mkdir();
		theYoungest = new File(harvestParentMultiple, "2000-01-01T22-22-22Z");
		theYoungest.mkdir();

		luceneIndex = new File(tempDir, "lucene");
		luceneIndex.mkdir();
		luceneSegmentsFile = new File(luceneIndex, "segments");
		luceneSegmentsFile.createNewFile();
		historyFile = new File(luceneIndex,
				OaiHarvestProcessorParameters.HARVEST_HISTORY_FILE);
		historyFile.createNewFile();

		HistoryFile.getInstance(historyFile).reload();
	}

	@After
	public void removeHarvestDirectories() {
		deepCleanDirectory(tempDir);
		tempDir.delete();
	}

	@Test(expected = IllegalArgumentException.class)
	public void unexpectedParameter() {
		parms = createParms("-harvestParentDir", harvestParentEmpty.getPath(),
				"-BOGUS", "-luceneIndexDir", luceneIndex.getPath());
	}

	@Test(expected = IllegalArgumentException.class)
	public void missingParentDir() {
		parms = createParms("-luceneIndexDir", luceneIndex.getPath());
	}

	@Test(expected = IllegalArgumentException.class)
	public void parentDirDoesntExist() {
		parms = createParms("-harvestParentDir", "bogus", "-luceneIndexDir",
				luceneIndex.getPath());
	}

	@Test(expected = IllegalArgumentException.class)
	public void missingLuceneDir() {
		parms = createParms("-harvestParentDir", harvestParentEmpty.getPath());
	}

	@Test(expected = IllegalArgumentException.class)
	public void luceneDirDoesntExist() {
		parms = createParms("-harvestParentDir", harvestParentEmpty.getPath(),
				"-luceneIndexDir", "bogus");
	}

	@Test(expected = IllegalArgumentException.class)
	public void invalidLuceneDir() {
		luceneSegmentsFile.delete();
		parms = createParms("-harvestParentDir", harvestParentEmpty.getPath(),
				"-luceneIndexDir", luceneIndex.getPath());
	}

	@Test(expected = HistoryException.class)
	@Ignore
	// HistoryFile has already loaded the file, so deleting it doesn't produce
	// the error.
	public void historyFileDoesntExist() {
		historyFile.delete();
		parms = createParms("-harvestParentDir", harvestParentEmpty.getPath(),
				"-luceneIndexDir", luceneIndex.getPath());
	}

	@Test(expected = IllegalArgumentException.class)
	public void forceWithoutHarvestDir() {
		parms = createParms("-harvestParentDir", harvestParentEmpty.getPath(),
				"-luceneIndexDir", luceneIndex.getPath(), "-force");
	}

	@Test(expected = IllegalArgumentException.class)
	public void allWithHarvestDir() {
		parms = createParms("-harvestParentDir", harvestParentOne.getPath(),
				"-luceneIndexDir", luceneIndex.getPath(), "-all",
				"-harvestDir", theOne.getPath());
	}

	@Test(expected = IllegalArgumentException.class)
	public void harvestDirDoesntExist() {
		parms = createParms("-harvestParentDir", harvestParentOne.getPath(),
				"-luceneIndexDir", luceneIndex.getPath(), "-harvestDir",
				"bogus");
	}

	@Test(expected = IllegalArgumentException.class)
	public void harvestDirAlreadyIndexedNoForce() {
		loadHistoryFile(theOne);
		parms = createParms("-harvestParentDir", harvestParentOne.getPath(),
				"-luceneIndexDir", luceneIndex.getPath(), "-harvestDir", theOne
						.getPath());
	}

	@Test
	public void harvestDirAlreadyIndexedForce() {
		loadHistoryFile(theOne);

		parms = createParms("-harvestParentDir", harvestParentOne.getPath(),
				"-luceneIndexDir", luceneIndex.getPath(), "-harvestDir", theOne
						.getPath(), "-force");
		confirmParms(luceneIndex, harvestParentOne, new File[] { theOne });
	}

	@Test
	public void noHarvestsNoAll() {
		parms = createParms("-harvestParentDir", harvestParentEmpty.getPath(),
				"-luceneIndexDir", luceneIndex.getPath());
		confirmParms(luceneIndex, harvestParentEmpty, new File[] {});
	}

	@Test
	public void noHarvestsAll() {
		parms = createParms("-harvestParentDir", harvestParentEmpty.getPath(),
				"-luceneIndexDir", luceneIndex.getPath(), "-all");
		confirmParms(luceneIndex, harvestParentEmpty, new File[] {});
	}

	@Test
	public void oneHarvestAll() {
		parms = createParms("-harvestParentDir", harvestParentOne.getPath(),
				"-luceneIndexDir", luceneIndex.getPath(), "-all");
		confirmParms(luceneIndex, harvestParentOne, new File[] { theOne });
	}

	@Test
	public void oneHarvestNoAll() {
		parms = createParms("-harvestParentDir", harvestParentOne.getPath(),
				"-luceneIndexDir", luceneIndex.getPath());
		confirmParms(luceneIndex, harvestParentOne, new File[] { theOne });
	}

	@Test
	public void multiHarvestNoHistoryAll() {
		parms = createParms("-harvestParentDir", harvestParentMultiple
				.getPath(), "-luceneIndexDir", luceneIndex.getPath(), "-all");
		confirmParms(luceneIndex, harvestParentMultiple, new File[] {
				theOldest, theMiddle, theYoungest });
	}

	@Test
	public void multiHarvestNoHistoryNoAll() {
		parms = createParms("-harvestParentDir", harvestParentMultiple
				.getPath(), "-luceneIndexDir", luceneIndex.getPath());
		confirmParms(luceneIndex, harvestParentMultiple,
				new File[] { theOldest });
	}

	@Test
	public void multiHarvestOneHistoryAll() {
		loadHistoryFile(theOldest);
		parms = createParms("-harvestParentDir", harvestParentMultiple
				.getPath(), "-luceneIndexDir", luceneIndex.getPath(), "-all");
		confirmParms(luceneIndex, harvestParentMultiple, new File[] {
				theMiddle, theYoungest });
	}

	@Test
	public void multiHarvestOneHistoryNoAll() {
		loadHistoryFile(theOldest);
		parms = createParms("-harvestParentDir", harvestParentMultiple
				.getPath(), "-luceneIndexDir", luceneIndex.getPath());
		confirmParms(luceneIndex, harvestParentMultiple,
				new File[] { theMiddle });
	}

	@Test
	public void multiHarvestTwoHistoryAll() {
		loadHistoryFile(theOldest, theMiddle);
		parms = createParms("-harvestParentDir", harvestParentMultiple
				.getPath(), "-luceneIndexDir", luceneIndex.getPath(), "-all");
		confirmParms(luceneIndex, harvestParentMultiple,
				new File[] { theYoungest });
	}

	@Test
	public void multiHarvestTwoHistoryNoAll() {
		loadHistoryFile(theOldest, theMiddle);
		parms = createParms("-harvestParentDir", harvestParentMultiple
				.getPath(), "-luceneIndexDir", luceneIndex.getPath());
		confirmParms(luceneIndex, harvestParentMultiple,
				new File[] { theYoungest });
	}

	@Test
	public void multiHarvestThreeHistoryAll() {
		loadHistoryFile(theOldest, theMiddle, theYoungest);
		parms = createParms("-harvestParentDir", harvestParentMultiple
				.getPath(), "-luceneIndexDir", luceneIndex.getPath(), "-all");
		confirmParms(luceneIndex, harvestParentMultiple, new File[] {});
	}

	@Test
	public void multiHarvestThreeHistoryNoAll() {
		loadHistoryFile(theOldest, theMiddle, theYoungest);
		parms = createParms("-harvestParentDir", harvestParentMultiple
				.getPath(), "-luceneIndexDir", luceneIndex.getPath());
		confirmParms(luceneIndex, harvestParentMultiple, new File[] {});
	}

	/**
	 * Rewrite the history file to show that these directories have been
	 * indexed, and reload the instance of the {@link HistoryFile}.
	 */
	private void loadHistoryFile(File... indexedDirs) {
		try {
			FileWriter writer = new FileWriter(historyFile);
			for (File dir : indexedDirs) {
				writer.write(DateUtil.getInstance().formatDate(
						DateUtil.ISO8601_DATE_TIME_Z)
						+ " " + dir.getPath() + "\n");
			}
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
		HistoryFile.getInstance(historyFile).reload();
	}

	private OaiHarvestProcessorParameters createParms(String... args) {
		return new OaiHarvestProcessorParameters(args);
	}

	private void confirmParms(File expectedLucene, File expectedParent,
			File[] expectedHarvests) {
		assertEquals("luceneIndex", expectedLucene, parms
				.getLuceneIndexDirectory());
		assertEquals("harvestParent", expectedParent, parms
				.getHarvestParentDirectory());

		Set<String> expectedHarvestPaths = new HashSet<String>();
		for (File expectedHarvest : expectedHarvests) {
			expectedHarvestPaths.add(expectedHarvest.getPath());
		}

		Set<String> actualHarvestPaths = new HashSet<String>();
		for (HarvestDirectoryWrapper wrapper : parms.getHarvestDirectories()) {
			actualHarvestPaths.add(wrapper.getPath());
		}

		assertEquals("harvestDirectories", expectedHarvestPaths,
				actualHarvestPaths);
	}
}
