/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.processoai;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertFalse;
import static junit.framework.Assert.assertNull;
import static junit.framework.Assert.assertTrue;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.junit.Test;
import org.nsdl.search.config.ConfigException;

/**
 * Tests the function of {@link MetadataElementSpec}.
 * 
 * @author jblake
 */
public class TestMetadataElementSpec {

	// ------------------------------------------------------------------------
	// Test Data constructor
	// ------------------------------------------------------------------------

	@Test
	public void constructNoEncoding() {
		MetadataElementSpec spec = invokeConstructor("name", "namespace", null,
				null);
		assertCorrectSpec(spec, "name", "namespace", null, null);
	}

	@Test
	public void constructWithEncoding() {
		MetadataElementSpec spec = invokeConstructor("name", "namespace",
				"encode", "encodeNS");
		assertCorrectSpec(spec, "name", "namespace", "encode", "encodeNS");
	}

	@Test(expected = IllegalArgumentException.class)
	public void noName() {
		invokeConstructor(null, "namespace", null, null);
	}

	// ------------------------------------------------------------------------
	// Test Mapping
	// ------------------------------------------------------------------------

	@Test
	public void tryCreatingDuplicateSpec() {
		clearSpecMap();
		MetadataElementSpec first = MetadataElementSpec.getInstance("name1",
				"namespace", null, null);
		MetadataElementSpec second = MetadataElementSpec.getInstance("name1",
				"namespace", null, null);
		assertEquals(first, second);
		assertEquals(1, getMapSize());
	}

	@Test(expected = ConfigException.class)
	public void tryCreatingMismatchedSpec() {
		MetadataElementSpec.getInstance("name1", "namespace", null, null);
		MetadataElementSpec.getInstance("name1", null, null, null);
	}

	/**
	 * The specs can be different as long as the encoding is different also.
	 */
	@Test
	public void tryCreatingDifferentEncodingSpecs() {
		MetadataElementSpec.getInstance("name1", "namespace", null, null);
		MetadataElementSpec.getInstance("name1", null, "encoding", null);
		assertEquals(2, getMapSize());
	}

	@Test
	public void findSpecNoEncoding() {
		clearSpecMap();
		MetadataElementSpec spec = MetadataElementSpec.getInstance("name1",
				"namespace", null, null);
		assertEquals(spec, MetadataElementSpec.getSpec("name1", null));
	}

	@Test
	public void findSpecWithEncoding() {
		clearSpecMap();
		MetadataElementSpec spec = MetadataElementSpec.getInstance("name1",
				"namespace", "encoding", "encodingNamespace");
		assertEquals(spec, MetadataElementSpec.getSpec("name1", "encoding"));
	}

	@Test
	public void findNoSpec() {
		clearSpecMap();
		MetadataElementSpec.getInstance("name1", "namespace", "encoding",
				"encodingNamespace");
		assertNull(MetadataElementSpec.getSpec("otherName", "encoding"));
	}

	// ------------------------------------------------------------------------
	// Test Functions
	// ------------------------------------------------------------------------
	/**
	 * Test that a spec is equal to its copy and equal to itself, and not equal
	 * to any spec that differs from it. Also, equal implies same hashCode.
	 */
	@Test
	public void testEquals() {
		List<MetadataElementSpec> specs = createAssortedSpecs();
		for (MetadataElementSpec spec1 : specs) {
			for (MetadataElementSpec spec2 : specs) {
				if (spec1 == spec2) {
					assertEquals("spec should be equal to self", spec1, spec2);
				} else {
					assertFalse("different should not be equal: spec1=" + spec1
							+ ", spec2=" + spec2, spec1.equals(spec2));
					assertFalse("different should not be equal: spec2=" + spec2
							+ ", spec1=" + spec1, spec2.equals(spec1));
				}
			}
		}
	}

	@Test
	public void testHashing() {
		List<MetadataElementSpec> specList = createAssortedSpecs();
		Set<MetadataElementSpec> specSet = new HashSet<MetadataElementSpec>(
				specList);
		assertEquals("everything went in properly", specList.size(), specSet
				.size());
		for (MetadataElementSpec spec : specList) {
			assertTrue("should find this one: " + spec, specSet.contains(spec));
		}
	}

	// ------------------------------------------------------------------------
	// Helper methods
	// ------------------------------------------------------------------------

	/**
	 * Each of these specs differs from another in only one attribute.
	 */
	private List<MetadataElementSpec> createAssortedSpecs() {
		List<MetadataElementSpec> specs = new ArrayList<MetadataElementSpec>();
		specs.add(invokeConstructor("name", "namespace", "encoding",
				"encodingNS"));
		specs.add(invokeConstructor("otherName", "namespace", "encoding",
				"encodingNS"));
		specs.add(invokeConstructor("name", "otherSpace", "encoding",
				"encodingNS"));
		specs.add(invokeConstructor("name", "namespace", "otherEncoding",
				"encodingNS"));
		specs
				.add(invokeConstructor("name", "namespace", "encoding",
						"otherNS"));
		specs.add(invokeConstructor("name", null, "encoding", "encodingNS"));
		specs.add(invokeConstructor("name", "namespace", null, null));
		specs.add(invokeConstructor("name", "namespace", "encoding", null));
		return specs;
	}

	/**
	 * Check all fields of the spec -- did we get what we were expecting?
	 */
	private void assertCorrectSpec(MetadataElementSpec spec, String name,
			String namespace, String encoding, String encodingNamespace) {
		assertEquals("wrong name", name, spec.getName());
		assertEquals("wrong namespace", namespace, spec.getNamespace());
		assertEquals("wrong encoding", encoding, spec.getEncoding());
		assertEquals("wrong name", encodingNamespace, spec
				.getEncodingNamespace());
	}

	/**
	 * Invoke the private constructor directly, so we won't add any instances to
	 * the map.
	 */
	private MetadataElementSpec invokeConstructor(String name,
			String namespace, String encoding, String encodingNamespace) {
		try {
			Class<?>[] argTypes = new Class[]{String.class, String.class,
					String.class, String.class};
			Object[] args = new Object[]{name, namespace, encoding,
					encodingNamespace};

			Constructor<MetadataElementSpec> constructor = MetadataElementSpec.class
					.getDeclaredConstructor(argTypes);
			constructor.setAccessible(true);
			return constructor.newInstance(args);
		} catch (NoSuchMethodException e) {
			throw new IllegalStateException(e);
		} catch (IllegalAccessException e) {
			throw new IllegalStateException(e);
		} catch (InstantiationException e) {
			throw new IllegalStateException(e);
		} catch (InvocationTargetException e) {
			throw new IllegalArgumentException(e);
		}
	}

	/**
	 * Clear the map of specification instances. It's private, so use
	 * reflection.
	 */
	private void clearSpecMap() {
		try {
			java.lang.reflect.Field mapField = MetadataElementSpec.class
					.getDeclaredField("specs");
			mapField.setAccessible(true);
			Map<?, ?> map = (Map<?, ?>) mapField.get(null);
			map.clear();
		} catch (NoSuchFieldException e) {
			throw new IllegalStateException(e);
		} catch (IllegalAccessException e) {
			throw new IllegalStateException(e);
		}
	}

	/**
	 * Find out how many specs have been stored in the map. It's private so use
	 * reflection.
	 */
	private int getMapSize() {
		try {
			java.lang.reflect.Field mapField = MetadataElementSpec.class
					.getDeclaredField("specs");
			mapField.setAccessible(true);
			Map<?, ?> map = (Map<?, ?>) mapField.get(null);
			return map.size();
		} catch (NoSuchFieldException e) {
			throw new IllegalStateException(e);
		} catch (IllegalAccessException e) {
			throw new IllegalStateException(e);
		}
	}
}
