/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.processoai;

import static junit.framework.Assert.assertEquals;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_DESCRIPTION;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_IDENTIFIER;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_SUBJECT;

import java.text.ParseException;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.junit.Before;
import org.junit.Test;
import org.nsdl.search.SearchUnitTest;
import org.nsdl.searchtools.common.ListsMap;

/**
 * 
 * @author jblake
 */
public class TestOaiHarvestRecord extends SearchUnitTest {
	private static final String EXAMPLE_OAI_ID = "oai:ExampleId";

	private static final String EXAMPLE_DATESTAMP = "1999-09-12T13:44:55Z";

	private MetadataElementSpec specDescription = MD_DC_DESCRIPTION;
	private MetadataElementSpec specIdentifier = MD_DC_IDENTIFIER;
	private MetadataElementSpec specSubject = MD_DC_SUBJECT;

	private OaiHarvestRecord record;

	@Before
	public void initializeRecord() throws ParseException {
		record = new OaiHarvestRecord();
		record.setOaiRecId(EXAMPLE_OAI_ID);
		record.setOaiRecDatestamp(EXAMPLE_DATESTAMP);
	}

	/**
	 * If we have a URL mapping, then the URL map will reflect that.
	 */
	@Test
	public void checkUrlMapWithUrls() {
		record.addFieldValue(specDescription, "description");
		record.addFieldValue(specSubject, "subject");
		record.addFieldValue(specIdentifier, "http://valid.url");
		record.addUrlMapping("mailto://should.be.normalized",
				"mailto://should.be.raw");

		ListsMap<MetadataElementSpec, String> fieldMap = new ListsMap<MetadataElementSpec, String>();
		fieldMap.addValue(specDescription, "description");
		fieldMap.addValue(specSubject, "subject");
		fieldMap.addValue(specIdentifier, "http://valid.url");

		ListsMap<String, String> urlMap = new ListsMap<String, String>();
		urlMap.addValue("mailto://should.be.normalized",
				"mailto://should.be.raw");

		assertCorrectOaiHarvestRecord(EXAMPLE_OAI_ID, EXAMPLE_DATESTAMP,
				fieldMap, urlMap);
	}

	/**
	 * If we have no URL mapping, use the OAI ID.
	 */
	@Test
	public void checkUrlMapNoUrls() {
		record.addFieldValue(specDescription, "description");
		record.addFieldValue(specSubject, "subject");

		ListsMap<MetadataElementSpec, String> fieldMap = new ListsMap<MetadataElementSpec, String>();
		fieldMap.addValue(specDescription, "description");
		fieldMap.addValue(specSubject, "subject");

		ListsMap<String, String> urlMap = new ListsMap<String, String>();
		urlMap.addValue(EXAMPLE_OAI_ID, EXAMPLE_OAI_ID);

		assertCorrectOaiHarvestRecord(EXAMPLE_OAI_ID, EXAMPLE_DATESTAMP,
				fieldMap, urlMap);
	}

	/**
	 * If we have no URL mapping, and no metadata fields, return an empty map.
	 */
	@Test
	public void checkUrlMapNoUrlsNoMetadata() {
		ListsMap<MetadataElementSpec, String> fieldMap = new ListsMap<MetadataElementSpec, String>();
		ListsMap<String, String> urlMap = new ListsMap<String, String>();
		assertCorrectOaiHarvestRecord(EXAMPLE_OAI_ID, EXAMPLE_DATESTAMP,
				fieldMap, urlMap);
	}

	/**
	 * Check that the harvest record is everything we expected it to be.
	 */
	private void assertCorrectOaiHarvestRecord(String expectedOaiId,
			String expectedDatestamp,
			ListsMap<MetadataElementSpec, String> fieldMap,
			ListsMap<String, String> urlMap) {
		assertEquals("oaiId", expectedOaiId, record.getOaiRecId());
		assertEquals("datestamp", expectedDatestamp, record
				.getOaiRecDatestamp());
		assertCorrectFieldValuesMap(fieldMap);
		assertCorrectUrlMap(urlMap);
	}

	/**
	 * Check for the correct URL mapping.
	 */
	private void assertCorrectUrlMap(ListsMap<String, String> expectedUrlMap) {
		Map<String, List<String>> actualUrlMap = record.getUrlMap();
		assertEquals("normalized URLs", expectedUrlMap.keySet(), actualUrlMap
				.keySet());
		for (String normalized : expectedUrlMap.keySet()) {
			assertEquals("raw URLs for '" + normalized + "'", expectedUrlMap
					.get(normalized), actualUrlMap.get(normalized));
		}
	}

	/**
	 * Check for the correct field values.
	 */
	private void assertCorrectFieldValuesMap(
			ListsMap<MetadataElementSpec, String> expectedFieldValuesMap) {
		Set<MetadataElementSpec> expectedKeys = expectedFieldValuesMap.keySet();
		assertEquals("field specs", expectedKeys, record.getFieldKeys());
		for (MetadataElementSpec key : expectedFieldValuesMap.keySet()) {
			String name = key.getName();
			String encoding = key.getEncoding();
			String label = (encoding == null)
					? name
					: (name + "(" + encoding + ")");
			assertEquals("values for " + label,
					expectedFieldValuesMap.get(key), record.getFieldValues(key));
		}
	}

}
