/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.processoai;

import static org.nsdl.search.config.OaiMetadataElements.MD_DCT_EDLEVEL_NSDL_EDLEVEL;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_CONTRIBUTOR;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_CREATOR;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_DATE;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_DATE_W3CDTF;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_DESCRIPTION;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_FORMAT;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_FORMAT_IMT;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_IDENTIFIER;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_LANGUAGE_RFC3066;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_PUBLISHER;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_RIGHTS;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_SOURCE;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_SUBJECT;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_SUBJECT_GEM;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_TITLE;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_TYPE;
import static org.nsdl.search.config.OaiMetadataElements.MD_DC_TYPE_DCMITYPE;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.text.ParseException;

import mock.search.collContext.MockCollRecCache;
import mock.search.processoai.MockOaiListRecordsProcessor;

import org.junit.Before;
import org.junit.Test;
import org.nsdl.search.SearchUnitTest;
import org.nsdl.search.collContext.CollRecCache;
import org.xml.sax.SAXException;

/**
 * 
 * @author jblake
 */
public class TestOaiHarvestRecordsParser extends SearchUnitTest {
	private static final String TESTFILES_PARENT_DIRECTORY = "./testfiles/oaiParsingExamples";

	private MockOaiListRecordsProcessor olrp;
	private OaiRecordsParser parser;

	@Override
	protected Class<?>[][] getImplementationClasses() {
		return new Class[][]{{CollRecCache.class, MockCollRecCache.class}};
	}

	@Before
	public void emptyTheCollectionCache() {
		MockCollRecCache.getInstance().reset();
	}

	@Before
	public void initializeTheParserAndProcessor() {
		olrp = new MockOaiListRecordsProcessor();
		parser = new OaiHarvestRecordsParser(olrp);
		olrp.setParser(parser);
	}

	@Test
	public void parse4152T() throws IOException, SAXException, XmlException,
			ParseException {
		Reader reader = new FileReader(new File(TESTFILES_PARENT_DIRECTORY,
				"oai20061003062444152T.xml"));
		olrp.parseOaiResponseFromReader(reader);

		OaiCollectionInfo collectionInfo = new OaiCollectionInfo(
				"oai:nsdl.org:crs:65224",
				"http://crs.nsdl.org/brands/PlanetMath.jpg", "32", "32",
				"PlanetMath.org: Math for the people, by the people");

		OaiHarvestRecord expected = new OaiHarvestRecord();
		expected.setDeletedRecord(false);
		expected.setCollectionInfo(collectionInfo);
		expected.setOaiRecDatestamp("2007-04-18T12:03:07Z");
		expected.setOaiRecId("oai:nsdl.org:2200-20061003062444152T");
		expected.addUrlMapping(
				"http://planetmath.org/encyclopedia/Diameter.html",
				"http://planetmath.org/encyclopedia/Diameter.html");
		expected.addFieldValue(MD_DC_PUBLISHER, "PlanetMath");
		expected.addFieldValue(MD_DC_FORMAT_IMT, "text/html");
		expected.addFieldValue(MD_DC_DATE_W3CDTF, "2002-02-15");
		expected.addFieldValue(MD_DC_SUBJECT, "msc:54-00");
		expected.addFieldValue(MD_DC_SUBJECT, "mathematics");
		expected.addFieldValue(MD_DC_TYPE_DCMITYPE, "Text");
		expected.addFieldValue(MD_DC_DESCRIPTION, "Let <i>A</i> a subset of a "
				+ "pseudometric space ... . The ... diameter "
				+ "of <i>A</i> is defined to be ... whenever "
				+ "the supremum exists. If the supremum "
				+ "doesn't exist, diameter of <i>A</i> is defined "
				+ "to be infinite. Having finite diameter is not a "
				+ "topological invariant.");
		expected.addFieldValue(MD_DC_TITLE, "diameter");
		expected.addFieldValue(MD_DC_LANGUAGE_RFC3066, "en-US");
		olrp.assertExpectedResults(expected);
	}

	@Test
	public void parse8358T() throws IOException, SAXException, XmlException,
			ParseException {
		Reader reader = new FileReader(new File(TESTFILES_PARENT_DIRECTORY,
				"oai20061003155518358T.xml"));
		olrp.parseOaiResponseFromReader(reader);

		OaiCollectionInfo collectionInfo = new OaiCollectionInfo(
				"oai:nsdl.org:crs:380603",
				"http://crs.nsdl.org/brands/380603.jpg", "100", "35",
				"Earth Science Education Catalog");

		OaiHarvestRecord expected = new OaiHarvestRecord();
		expected.setDeletedRecord(false);
		expected.setCollectionInfo(collectionInfo);
		expected.setOaiRecDatestamp("2007-02-18T09:33:29Z");
		expected.setOaiRecId("oai:nsdl.org:2200-20061003155518358T");
		expected.addUrlMapping("http://acswebcontent.acs.org/education/"
				+ "chemmatters/archive/cm_sep01.pdf",
				"http://acswebcontent.acs.org/education/"
						+ "chemmatters/archive/cm_sep01.pdf");
		expected.addFieldValue(MD_DCT_EDLEVEL_NSDL_EDLEVEL, "High School");
		expected.addFieldValue(MD_DC_RIGHTS,
				"This product is free and clear for general use.");
		expected.addFieldValue(MD_DC_FORMAT_IMT, "text/html");
		expected.addFieldValue(MD_DC_SUBJECT_GEM, "Science");
		expected.addFieldValue(MD_DC_SUBJECT_GEM, "Earth science");
		expected.addFieldValue(MD_DC_SUBJECT_GEM, "Physical sciences");
		expected.addFieldValue(MD_DC_SUBJECT_GEM, "Meteorology");
		expected.addFieldValue(MD_DC_SUBJECT_GEM, "Oceanography");
		expected.addFieldValue(MD_DC_SUBJECT_GEM, "Chemistry");
		expected.addFieldValue(MD_DC_SUBJECT_GEM, "Astronomy");
		expected.addFieldValue(MD_DC_SUBJECT_GEM, "Space sciences");
		expected.addFieldValue(MD_DC_DATE_W3CDTF, "2001-09-01");
		expected.addFieldValue(MD_DC_SUBJECT, "Physical oceanography");
		expected.addFieldValue(MD_DC_SUBJECT, "Atmospheric science");
		expected.addFieldValue(MD_DC_SUBJECT, "Climatology");
		expected.addFieldValue(MD_DC_SUBJECT, "Environmental science");
		expected.addFieldValue(MD_DC_SUBJECT, "satellite missions");
		expected.addFieldValue(MD_DC_SUBJECT, "Air quality");
		expected.addFieldValue(MD_DC_SUBJECT, "Ozone");
		expected.addFieldValue(MD_DC_SUBJECT, "Climate");
		expected.addFieldValue(MD_DC_SUBJECT, "Global Warming");
		expected.addFieldValue(MD_DC_SUBJECT, "Aerosols");
		expected.addFieldValue(MD_DC_SUBJECT, "Composition and Chemistry");
		expected.addFieldValue(MD_DC_TYPE, "Artifact");
		expected.addFieldValue(MD_DC_TYPE, "Periodical");
		expected.addFieldValue(MD_DC_TYPE, "Remotely Sensed Imagery");
		expected.addFieldValue(MD_DC_TYPE_DCMITYPE, "Image");
		expected.addFieldValue(MD_DC_TYPE_DCMITYPE, "Text");
		expected
				.addFieldValue(
						MD_DC_DESCRIPTION,
						"ChemMatters is a quarterly publication of the American Chemical Society (ACS) geared to a high school audience. The magazine is designed and written to demystify everyday chemistry. This EOS-Aura special edition is the first of of four special issues focusing on atmospheric chemistry and the Aura mission. It features articles on global climate change, asthma and air pollution, good and bad ozone, and spectroscopy.");
		expected.addFieldValue(MD_DC_FORMAT, "Adobe Acrobat reader");
		expected.addFieldValue(MD_DC_TITLE,
				"ChemMatters September 2001 Special Edition: NASA's EOS Aura");
		expected.addFieldValue(MD_DC_LANGUAGE_RFC3066, "en");
		olrp.assertExpectedResults(expected);
	}

	@Test
	public void parseNoUrl() throws IOException, SAXException, XmlException,
			ParseException {
		Reader reader = new FileReader(new File(TESTFILES_PARENT_DIRECTORY,
				"noUrlIdentifier.xml"));
		olrp.parseOaiResponseFromReader(reader);

		OaiCollectionInfo collectionInfo = new OaiCollectionInfo(
				"oai:nsdl.org:crs:297934",
				"http://crs.nsdl.org/brands/FEOL.jpg", "100", "28",
				"Florida Environments Online");

		OaiHarvestRecord expected = new OaiHarvestRecord();
		expected.setDeletedRecord(false);
		expected.setCollectionInfo(collectionInfo);
		expected.setOaiRecDatestamp("2007-05-05T14:11:44Z");
		expected.setOaiRecId("oai:nsdl.org:2200-20061002160939356T");
		expected.addUrlMapping("oai:nsdl.org:2200-20061002160939356T",
				"oai:nsdl.org:2200-20061002160939356T");
		expected.addFieldValue(MD_DC_IDENTIFIER,
				"inzprocessNOT YET AVAILABLE; "
						+ "in process for electronic access.");
		expected.addFieldValue(MD_DC_CREATOR,
				"L'Isle, Guillaume de, 1675-1726.");
		expected.addFieldValue(MD_DC_DATE, "1730?]");
		expected.addFieldValue(MD_DC_CONTRIBUTOR,
				"Florida State College for Women. Class of 1945.");
		expected.addFieldValue(MD_DC_SUBJECT, "New France--Maps.");
		expected.addFieldValue(MD_DC_SUBJECT, "North America--Maps--To 1800.");
		expected.addFieldValue(MD_DC_SUBJECT,
				"European Discovery and Settlement in Florida, 1492-1821.");
		expected.addFieldValue(MD_DC_TITLE,
				"Novissima tabula regionis Lvdoviciana gallice dictae "
						+ "La Lovgsiane iam olim quidem sub Canadae et "
						+ "Floridae nomine in America Septentrionali "
						+ "[electronic resource] / notre nunc vero "
						+ "sub auspiciis sereniss: Ducis Aurelianensis "
						+ "maxime inclarescentis Coloniae Gallicae, "
						+ "delineata ex recentissimis itinerariis et "
						+ "commentariis Gallicis ]a Guil; Insulano "
						+ "Geographo et Socio Regiae Scientarum "
						+ "Academiae et recognita a Chr:Weigelio, Nor.");
		olrp.assertExpectedResults(expected);
	}

	@Test
	public void parseNonNormalizedUrl() throws IOException, SAXException,
			XmlException, ParseException {
		Reader reader = new FileReader(new File(TESTFILES_PARENT_DIRECTORY,
				"nonNormalizedUrl.xml"));
		olrp.parseOaiResponseFromReader(reader);

		OaiCollectionInfo collectionInfo = new OaiCollectionInfo(
				"oai:nsdl.org:crs:491770",
				"http://crs.nsdl.org/brands/491770.jpg", "100", "26",
				"Directory of Open Access Journals");

		OaiHarvestRecord expected = new OaiHarvestRecord();
		expected.setDeletedRecord(false);
		expected.setCollectionInfo(collectionInfo);
		expected.setOaiRecDatestamp("2007-05-04T21:08:06Z");
		expected.setOaiRecId("oai:nsdl.org:2200-20070312155813611T");
		expected.addUrlMapping(
				"http://www.economistascoruna.org/eawp/eawp.asp?"
						+ "qsa=ES&qsb=1&qsc=142&qsd=179",
				"http://www.economistascoruna.org/eawp/eawp.asp?"
						+ "qsa=ES&qsb=1&qsc=142&qsd=179#n5");
		expected.addUrlMapping(
				"http://www.doaj.org/doaj?func=openurl&genre=article&"
						+ "issn=15791475&date=2006&volume=5&issue=09&spage=",
				"http://www.doaj.org/doaj?func=openurl&genre=article&"
						+ "issn=15791475&date=2006&volume=5&issue=09&spage=");
		expected.addFieldValue(MD_DC_CREATOR, "Jos? Antonio Fern?ndez S?nchez");
		expected.addFieldValue(MD_DC_PUBLISHER,
				"Economists Association of La Coru?a, Spain");
		expected.addFieldValue(MD_DC_DATE_W3CDTF, "2006");
		expected
				.addFieldValue(MD_DC_SOURCE, "Economic Analysis Working Papers");
		expected.addFieldValue(MD_DC_DESCRIPTION,
				"En los ?ltimos a?os se ha hablado y escrito mucho "
						+ "sobre el tema de las reestructuraciones de "
						+ "plantillas y sus resultados. Adem?s, la prensa "
						+ "econ?mica nos muestra que estos procesos no han "
						+ "terminado y que se van a seguir repitiendo en el "
						+ "futuro. En este contexto, hemos decidido realizar "
						+ "un estudio acerca de los procesos de "
						+ "reestructuraci?n y sus principales implicaciones "
						+ "en la econom?a actual. En el presente trabajo se "
						+ "muestra un resumen de las conclusiones m?s "
						+ "importantes de un estudio realizado al respecto en "
						+ "el ?mbito de la provincia de Alicante, indicando "
						+ "las razones que pueden empujar a tomar la decisi?n "
						+ "de reestructurar la plantilla, los m?todos "
						+ "empleados y las nuevas tendencias en el resultado "
						+ "de estos procesos.");
		expected.addFieldValue(MD_DC_TITLE,
				"?Estan mejorando los resultados de los procesos de "
						+ "reestructuraci?n de las plantillas? Un estudio "
						+ "emp?rico en la provincia de Alicante");
		olrp.assertExpectedResults(expected);
	}

	@Test
	public void parseMultipleUrls() throws IOException, SAXException,
			XmlException, ParseException {
		Reader reader = new FileReader(new File(TESTFILES_PARENT_DIRECTORY,
				"multipleUrls.xml"));
		olrp.parseOaiResponseFromReader(reader);

		OaiCollectionInfo collectionInfo = new OaiCollectionInfo(
				"oai:nsdl.org:crs:491770",
				"http://crs.nsdl.org/brands/491770.jpg", "100", "26",
				"Directory of Open Access Journals");

		OaiHarvestRecord expected = new OaiHarvestRecord();
		expected.setDeletedRecord(false);
		expected.setCollectionInfo(collectionInfo);
		expected.setOaiRecDatestamp("2007-05-04T21:12:10Z");
		expected.setOaiRecId("oai:nsdl.org:2200-20070312163652126T");
		expected.addUrlMapping(
				"http://www.doaj.org/doaj?func=openurl&genre=article&"
						+ "issn=17464269&date=2007&volume=3&issue=1&spage=7",
				"http://www.doaj.org/doaj?func=openurl&genre=article&"
						+ "issn=17464269&date=2007&volume=3&issue=1&spage=7");
		expected.addUrlMapping("http://www.ethnobiomed.com/content/3/1/7",
				"http://www.ethnobiomed.com/content/3/1/7");
		expected.addFieldValue(MD_DC_CREATOR, "Kitula Rukia");
		expected.addFieldValue(MD_DC_PUBLISHER, "BioMed Central");
		expected.addFieldValue(MD_DC_DATE_W3CDTF, "2007");
		expected.addFieldValue(MD_DC_SOURCE,
				"Journal of Ethnobiology and Ethnomedicine");
		expected.addFieldValue(MD_DC_DESCRIPTION,
				"<p>Abstract</p> <p>The dependence of local people "
						+ "on plant medicine from natural forests has a long "
						+ "tradition in Tanzania and is becoming increasingly "
						+ "popular among rural and urban communities due to "
						+ "among others increase in living costs. The study "
						+ "on utilization of medicinal plants for meeting "
						+ "heath care needs was carried out between March "
						+ "2001 and March 2002 in New Dabaga Ulongambi Forest "
						+ "Reserve, Tanzania. The study aimed at generating "
						+ "necessary data for the Udzungwa Mountains Forest "
						+ "Management project to draft sound Joint Forest "
						+ "Management plans. Specific objectives of the study "
						+ "among others were to assess knowledge associated "
						+ "with utilization of medicinal plants for health "
						+ "care needs as well as factors associated in using "
						+ "plant medicines in the study area. A questionnaire "
						+ "survey, market survey and literature review were "
						+ "used to collect information. Tools used for data "
						+ "analysis were Statistical Packages for Social "
						+ "Science and content analysis. A total of 45 plant "
						+ "species were documented curing about 22 human "
						+ "diseases. Medicinal plants were readily available "
						+ "throughout the year and plentiful in the forest "
						+ "reserve. Roots and leaves were the plant parts "
						+ "harvested for medicinal purposes. Processing of "
						+ "plant medicines involved boiling, pounding, "
						+ "soaking in water and chewing. Distance to health "
						+ "facility, income level of the household and "
						+ "beliefs contributed to the use of plant medicines. "
						+ "The study concluded that medicinal plants play an "
						+ "important role in providing primary health care to "
						+ "the rural communities. It is recommended that in "
						+ "achieving joint forest management (JFM), villagers "
						+ "adjacent to the forest reserve should be "
						+ "sensitised on the importance of JFM through "
						+ "seminars, workshops, drama, school songs or video "
						+ "show. During the development of a joint draft "
						+ "management plan, villagers as an informal "
						+ "institution must define their priority needs of "
						+ "use of parts of the forest in collaboration with "
						+ "the Udzungwa Mountains Forest Management "
						+ "project.</p>");
		expected.addFieldValue(MD_DC_TITLE,
				"Use of medicinal plants for human health in Udzungwa "
						+ "Mountains Forests: a case study of New Dabaga "
						+ "Ulongambi Forest Reserve, Tanzania");
		olrp.assertExpectedResults(expected);
	}

	@Test
	public void parseNoMatchingRecords() throws IOException, SAXException,
			XmlException, ParseException {
		Reader reader = new FileReader(new File(TESTFILES_PARENT_DIRECTORY,
				"noMatchingRecords.xml"));
		olrp.parseOaiResponseFromReader(reader);
		olrp.confirmNoRecordsIndexed();
	}

}
