/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.servlets;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import mock.javax.servlet.http.MockHttpServletRequest;
import mock.javax.servlet.http.MockHttpServletResponse;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.junit.After;
import org.junit.Before;

/**
 * Some base methods that will be useful in unit tests of the Servlets.
 * 
 * @author jblake
 */
public abstract class ServletUnitTest {
	protected MockHttpServletRequest request;
	protected MockHttpServletResponse response;

	protected ServletUnitTest() {
		// Turn off the query logs.
		Logger.getLogger("QUERY_LOG").setLevel(Level.OFF);
	}

	/**
	 * Each test class may override the implementations. Provide a 2 x n array
	 * where each row contains a base class and an implementation class.
	 */
	protected abstract Class<?>[][] getImplementationClasses();

	@Before
	public void setupImplementationClasses() {
		Class<?>[][] mappings = getImplementationClasses();
		for (Class<?>[] mapping : mappings) {
			String key = mapping[0].getName() + ".implementation";
			String value = mapping[1].getName();
			System.setProperty(key, value);
		}
	}

	@After
	public void teardownImplementationClasses() {
		Class<?>[][] mappings = getImplementationClasses();
		for (Class<?>[] mapping : mappings) {
			String key = mapping[0].getName() + ".implementation";
			System.clearProperty(key);
		}
	}

	/**
	 * Each test class may set the Logging Level for any class with
	 * {@link #setLoggerLevel(Class, Level) setLoggerLevel()}.
	 */
	/** The appender we use for logging. */
	private static final ConsoleAppender appender = new ConsoleAppender(
			new PatternLayout(
					"%p %d{yyyy-MM-dd' 'HH:mm:ss.SSS} [%t] (%c{1}) %m%n"));

	private static final List<Class<?>> alteredLoggers = new ArrayList<Class<?>>();

	@Before
	public void initializeLogging() {
		Logger.getRootLogger().addAppender(appender);
		Logger.getRootLogger().setLevel(Level.INFO);
		alteredLoggers.clear();
	}

	@After
	public void restoreLogging() {
		for (Class<?> clazz : alteredLoggers) {
			Logger.getLogger(clazz).setLevel(null);
		}
		Logger.getRootLogger().removeAppender(appender);
	}

	/**
	 * Call this in a "@Before" method to change the logging level of a
	 * particular class.
	 */
	protected void setLoggerLevel(Class<?> clazz, Level level) {
		Logger.getLogger(clazz).setLevel(level);
		alteredLoggers.add(clazz);
	}

	/**
	 * Provide a simple request and response object for the servlet to use.
	 */
	@Before
	public void initializeReqAndResp() {
		request = new MockHttpServletRequest();
		request.setMethod("GET");
		request.setRequestInfo("myServer", 80, "servlet");

		response = new MockHttpServletResponse();
	}

	/**
	 * Exercise the servlet life-cycle methods.
	 */
	protected void runServlet(HttpServlet servlet,
			MockHttpServletRequest request, MockHttpServletResponse response)
			throws ServletException, IOException {
		servlet.init();
		servlet.service(request, response);
		servlet.destroy();
	}
}
