/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.servlets;

import static junit.framework.Assert.assertEquals;

import org.junit.Before;
import org.junit.Test;
import org.nsdl.search.rest3.SearchServletException;

/**
 * 
 * @author jblake
 */
public class TestAbstractSearchServlet extends ServletUnitTest {
	private static final String KEY = "key";
	private static final String DEFAULT_STRING = "default";
	private static final int DEFAULT_INTEGER = 5;
	private static final int LOWER_BOUND = -40;
	private static final int UPPER_BOUND = 1400;

	private MyServlet servlet;

	@Override
	protected Class<?>[][] getImplementationClasses() {
		return new Class[0][];
	}

	@Before
	public void initializeServlet() {
		servlet = new MyServlet();
	}

	// -------------------------------------------------------------------------
	// Optional string.
	// -------------------------------------------------------------------------

	@Test
	public void optionalStringNoValue() {
		assertOptionalString(null, DEFAULT_STRING);
	}

	@Test
	public void optionalStringEmpty() {
		assertOptionalString("", "");
	}

	@Test
	public void optionalStringBlank() {
		assertOptionalString(" ", "");
	}

	@Test
	public void optionalStringOK() {
		assertOptionalString("Hi", "Hi");
	}

	@Test
	public void optionalStringTrims() {
		assertOptionalString(" Hi", "Hi");
	}

	// -------------------------------------------------------------------------
	// Required string.
	// -------------------------------------------------------------------------

	@Test(expected = SearchServletException.class)
	public void requiredStringNoValue() throws SearchServletException {
		assertRequiredString(null, "bogus");
	}

	@Test
	public void requiredStringEmpty() throws SearchServletException {
		assertRequiredString("", "");
	}

	@Test
	public void requiredStringBlank() throws SearchServletException {
		assertRequiredString("   ", "");
	}

	@Test
	public void requiredStringOK() throws SearchServletException {
		assertRequiredString("boogie!", "boogie!");
	}

	@Test
	public void requiredStringTrims() throws SearchServletException {
		assertRequiredString("boogie!   ", "boogie!");
	}

	// -------------------------------------------------------------------------
	// Optional integer.
	// -------------------------------------------------------------------------

	@Test
	public void optionalIntegerNoValue() throws SearchServletException {
		assertOptionalInteger(null, DEFAULT_INTEGER);
	}

	@Test
	public void optionalIntegerEmpty() throws SearchServletException {
		assertOptionalInteger("", DEFAULT_INTEGER);
	}

	@Test
	public void optionalIntegerBlank() throws SearchServletException {
		assertOptionalInteger(" ", DEFAULT_INTEGER);
	}

	@Test
	public void optionalIntegerOK() throws SearchServletException {
		assertOptionalInteger("123", 123);
	}

	@Test
	public void optionalIntegerTrims() throws SearchServletException {
		assertOptionalInteger(" 13", 13);
	}

	@Test(expected = SearchServletException.class)
	public void optionalIntegerTooHigh() throws SearchServletException {
		assertOptionalInteger("1000000", 0);
	}

	@Test(expected = SearchServletException.class)
	public void optionalIntegerTooLow() throws SearchServletException {
		assertOptionalInteger("-1000", 0);
	}

	@Test(expected = SearchServletException.class)
	public void optionalIntegerNotValid() throws SearchServletException {
		assertOptionalInteger("12.5", 0);
	}

	// -------------------------------------------------------------------------
	// Required integer.
	// -------------------------------------------------------------------------

	@Test(expected = SearchServletException.class)
	public void requiredIntegerNoValue() throws SearchServletException {
		assertRequiredInteger(null, 0);
	}

	@Test(expected = SearchServletException.class)
	public void requiredIntegerEmpty() throws SearchServletException {
		assertRequiredInteger("", 0);
	}

	@Test(expected = SearchServletException.class)
	public void requiredIntegerBlank() throws SearchServletException {
		assertRequiredInteger(" ", 0);
	}

	@Test
	public void requiredIntegerOK() throws SearchServletException {
		assertRequiredInteger("123", 123);
	}

	@Test
	public void requiredIntegerTrims() throws SearchServletException {
		assertRequiredInteger(" 123 ", 123);
	}

	@Test(expected = SearchServletException.class)
	public void requiredIntegerTooHigh() throws SearchServletException {
		assertRequiredInteger("100000", 0);
	}

	@Test(expected = SearchServletException.class)
	public void requiredIntegerTooLow() throws SearchServletException {
		assertRequiredInteger("-5400", 0);
	}

	@Test(expected = SearchServletException.class)
	public void requiredIntegerNotValid() throws SearchServletException {
		assertRequiredInteger("12A", 0);
	}

	// -------------------------------------------------------------------------
	// Helpers.
	// -------------------------------------------------------------------------

	private void assertOptionalString(String parameterValue,
			String expectedValue) {
		request.setParameter(KEY, parameterValue);
		String actual = servlet.getOptionalStringParameter(request, KEY,
				DEFAULT_STRING);
		assertEquals("", expectedValue, actual);
	}

	private void assertRequiredString(String parameterValue,
			String expectedValue) throws SearchServletException {
		request.setParameter(KEY, parameterValue);
		String actual = servlet.getRequiredStringParameter(request, KEY);
		assertEquals("", expectedValue, actual);
	}

	private void assertOptionalInteger(String parameterValue, int expectedValue)
			throws SearchServletException {
		request.setParameter(KEY, parameterValue);
		int actual = servlet.getOptionalIntegerParameter(request, KEY,
				LOWER_BOUND, UPPER_BOUND, DEFAULT_INTEGER);
		assertEquals("", expectedValue, actual);
	}

	private void assertRequiredInteger(String parameterValue, int expectedValue)
			throws SearchServletException {
		request.setParameter(KEY, parameterValue);
		int actual = servlet.getRequiredIntegerParameter(request, KEY,
				LOWER_BOUND, UPPER_BOUND);
		assertEquals("", expectedValue, actual);
	}

	/** Need a concrete servlet class. */
	private static class MyServlet extends AbstractSearchServlet {
		// Nothing to override.
	}
}
