/*
 * -----------------------------------------------------------------------------
 *
 * License and Copyright: 
 *
 * The contents of this file are subject to the Educational Community License 
 * (the "License"); you may not use this file except in compliance with the 
 * License. You may obtain a copy of the License at 
 * http://www.opensource.org/licenses/ecl1.txt.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The entire file consists of original code.  Copyright &copy; 2002-2008 by 
 * The Rector and Visitors of Cornell University. All rights reserved.
 *
 * -----------------------------------------------------------------------------
 */

package org.nsdl.search.rest3.servlets;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.fail;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.TimeZone;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import mock.nsdl.searchtools.common.MockDateUtil;
import mock.search.content.MockContentCacheWrapper;
import mock.search.lucene.MockIndexWrapper;
import mock.search.lucene.MockSearchResults;

import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.nsdl.search.content.ContentCacheWrapper;
import org.nsdl.search.generated.rest.ErrorCodeType;
import org.nsdl.search.lucene.wrapper.IndexWrapper;
import org.nsdl.search.lucene.wrapper.SearchResult;
import org.nsdl.search.lucene.wrapper.SearchResultImpl;
import org.nsdl.search.rest3.SearchServletException;
import org.nsdl.searchtools.common.DateUtil;

import samples.SampleQueryResults;
import samples.SampleSearchDocuments;

/**
 * Test the functions of the QueryServlet.
 * 
 * @author jblake
 */
public class TestQueryServlet extends ServletUnitTest {
	private MockIndexWrapper indexWrapper;
	private MockDateUtil dateUtil;

	@Override
	protected Class<?>[][] getImplementationClasses() {
		return new Class[][] { { IndexWrapper.class, MockIndexWrapper.class },
				{ DateUtil.class, MockDateUtil.class },
				{ ContentCacheWrapper.class, MockContentCacheWrapper.class } };
	}

	/**
	 * We'll use these parameters on the request, unless a particular test
	 * requires otherwise.
	 */
	@Before
	public void setBasicParameters() {
		request.setParameter("q", "frogs");
		request.setParameter("n", "10");
		request.setParameter("s", "0");
	}

	/**
	 * Get a fresh {@link MockIndexWrapper} for each test.
	 */
	@Before
	public void clearMockIndexWrapper() {
		MockIndexWrapper.reset();
		indexWrapper = MockIndexWrapper.getInstance();
	}

	/**
	 * Get a fresh {@link MockDateUtil} for each test.
	 */
	@Before
	public void useMockDateUtil() {
		MockDateUtil.reset();
		dateUtil = MockDateUtil.getInstance();
	}

	// ----------------------------------------------------------------------
	// Check parameters for validity.
	// ----------------------------------------------------------------------

	@Test
	public void noQueryParameter() throws ServletException, IOException {
		request.removeParameter("q");
		runExpectingException(new QueryServlet(), ErrorCodeType.BAD_ARGUMENT);
	}

	@Test
	public void skipParameterNotInteger() throws ServletException, IOException {
		request.setParameter("s", "4.5");
		runExpectingException(new QueryServlet(), ErrorCodeType.BAD_ARGUMENT);
	}

	@Test
	public void skipParameterNegative() throws ServletException, IOException {
		request.setParameter("s", "-5");
		runExpectingException(new QueryServlet(), ErrorCodeType.BAD_ARGUMENT);
	}

	@Test
	public void countParameterNotInteger() throws ServletException, IOException {
		request.setParameter("n", "BOGUS");
		runExpectingException(new QueryServlet(), ErrorCodeType.BAD_ARGUMENT);
	}

	@Test
	public void countParameterNegative() throws ServletException, IOException {
		request.setParameter("n", "-123");
		runExpectingException(new QueryServlet(), ErrorCodeType.BAD_ARGUMENT);
	}

	@Test
	public void countParameterTooBig() throws ServletException, IOException {
		request.setParameter("n", "1000000");
		runExpectingException(new QueryServlet(), ErrorCodeType.BAD_ARGUMENT);
	}

	@Test
	public void invalidOutputParameter() throws ServletException, IOException {
		request.setParameter("output", "bogus");
		runExpectingException(new QueryServlet(), ErrorCodeType.BAD_ARGUMENT);
	}

	// ----------------------------------------------------------------------
	// Test reactions to Exceptions.
	// ----------------------------------------------------------------------

	@Test
	public void indexThrowsBooleanQueryException() throws ServletException,
			IOException {
		indexWrapper.setThrowsBooleanQueryExceptionOnParse(true);
		runExpectingException(new QueryServlet(), ErrorCodeType.BAD_QUERY);
	}

	@Test
	public void indexThrowsParseException() throws ServletException,
			IOException {
		indexWrapper.setThrowsParseExceptionOnParse(true);
		runExpectingException(new QueryServlet(), ErrorCodeType.BAD_QUERY);
	}

	@Test(expected = IOException.class)
	public void indexThrowsIOException() throws ServletException, IOException {
		indexWrapper.setThrowsIOExceptionOnSearch(true);
		runServlet(new QueryServlet(), request, response);
	}

	// ----------------------------------------------------------------------
	// Test the results of the query.
	// ----------------------------------------------------------------------

	@Test
	public void searchReturnsNoResult() throws ServletException, IOException {
		// 2007-06-14T13:27:35.781Z
		setDateNow(2007, 06, 14, 13, 27, 35, 781);

		runServlet(new QueryServlet(), request, response);
		assertEquals(SampleQueryResults.getEmptyResult(), response
				.getOutputResult());
	}

	@Test
	public void searchReturnsOneResult() throws ServletException, IOException {
		// 2007-06-14T13:27:36.656Z
		setDateNow(2007, 06, 14, 13, 27, 36, 656);

		List<SearchResult> mockList = new ArrayList<SearchResult>();
		mockList.add(new SearchResultImpl(0, 1.0F, SampleSearchDocuments
				.getFrogDissectionDocument(), "bestPassage"));
		indexWrapper.setResults(new MockSearchResults(50, 10, mockList));

		runServlet(new QueryServlet(), request, response);
		assertEquals(SampleQueryResults.getFrogDissectionResult(), response
				.getOutputResult());
	}

	@Test
	public void searchReturnsTwoResults() throws ServletException, IOException {
		// 2007-06-14T15:56:33.463Z
		setDateNow(2007, 06, 14, 15, 56, 33, 463);

		List<SearchResult> mockList = new ArrayList<SearchResult>();
		mockList.add(new SearchResultImpl(0, 1.0F, SampleSearchDocuments
				.getFrogDissectionDocument(), "bestPassage"));
		mockList.add(new SearchResultImpl(1, 0.5F, SampleSearchDocuments
				.getFrogExploratoriumDocument(), "bestPassage"));
		indexWrapper.setResults(new MockSearchResults(50, 10, mockList));

		runServlet(new QueryServlet(), request, response);
		assertEquals(SampleQueryResults.getTwoFrogsResult(), response
				.getOutputResult());
	}

	/**
	 * Be sure that the resourceIdentifier is not "decoded" on the way out.
	 */
	@Test
	public void searchReturnsResultWithFunkyUri() throws ServletException,
			IOException {
		// 2007-06-14T13:27:36.656Z
		setDateNow(2007, 06, 14, 13, 27, 36, 656);

		List<SearchResult> mockList = new ArrayList<SearchResult>();
		mockList.add(new SearchResultImpl(0, 1.0F, SampleSearchDocuments
				.getFunkyUriDocument(), "bestPassage"));
		indexWrapper.setResults(new MockSearchResults(50, 10, mockList));

		runServlet(new QueryServlet(), request, response);
		assertEquals(SampleQueryResults.getFunkyUriResult(), response
				.getOutputResult());
	}

	/**
	 * Be sure that the resourceIdentifier is not "encoded" on the way out.
	 */
	@Test
	public void searchReturnsResultWithFunkyUri2() throws ServletException,
			IOException {
		// 2007-06-14T13:27:36.656Z
		setDateNow(2007, 06, 14, 13, 27, 36, 656);

		List<SearchResult> mockList = new ArrayList<SearchResult>();
		mockList.add(new SearchResultImpl(0, 1.0F, SampleSearchDocuments
				.getFunkyUriDocument2(), "bestPassage"));
		indexWrapper.setResults(new MockSearchResults(50, 10, mockList));

		runServlet(new QueryServlet(), request, response);
		assertEquals(SampleQueryResults.getFunkyUriResult2(), response
				.getOutputResult());
	}

	@Ignore
	@Test
	public void testWithContents() {
		// need to test at least one doc with contents.
	}

	// ----------------------------------------------------------------------
	// Test the RSS output.
	// ----------------------------------------------------------------------

	@Test
	@Ignore
	public void rssNoResult() throws ServletException, IOException {
		// 2007-06-14T13:27:35.781Z
		setDateNow(2007, 06, 14, 13, 27, 35, 781);

		request.setParameter("output", "rss");

		runServlet(new QueryServlet(), request, response);
		assertEquals(SampleQueryResults.getEmptyResult(), response
				.getOutputResult());
	}

	@Test
	@Ignore
	public void rssOneResult() throws ServletException, IOException {
		// 2007-06-14T13:27:36.656Z
		setDateNow(2007, 06, 14, 13, 27, 36, 656);

		request.setParameter("output", "rss");

		List<SearchResult> mockList = new ArrayList<SearchResult>();
		mockList.add(new SearchResultImpl(0, 1.0F, SampleSearchDocuments
				.getFrogDissectionDocument(), "bestPassage"));
		indexWrapper.setResults(new MockSearchResults(50, 10, mockList));

		runServlet(new QueryServlet(), request, response);
		assertEquals(SampleQueryResults.getFrogDissectionResult(), response
				.getOutputResult());
	}

	@Test
	@Ignore
	public void rssTwoResults() throws ServletException, IOException {
		// 2007-06-14T15:56:33.463Z
		setDateNow(2007, 06, 14, 15, 56, 33, 463);

		request.setParameter("output", "rss");

		List<SearchResult> mockList = new ArrayList<SearchResult>();
		mockList.add(new SearchResultImpl(0, 1.0F, SampleSearchDocuments
				.getFrogDissectionDocument(), "bestPassage"));
		mockList.add(new SearchResultImpl(1, 0.5F, SampleSearchDocuments
				.getFrogExploratoriumDocument(), "bestPassage"));
		indexWrapper.setResults(new MockSearchResults(50, 10, mockList));

		runServlet(new QueryServlet(), request, response);
		assertEquals(SampleQueryResults.getTwoFrogsResult(), response
				.getOutputResult());
	}

	// ----------------------------------------------------------------------
	// Helper methods.
	// ----------------------------------------------------------------------

	/**
	 * Force the "now" date in the MockDateUtil.
	 */
	private void setDateNow(int year, int month, int day, int hour, int minute,
			int second, int milli) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTimeZone(TimeZone.getTimeZone("GMT"));
		calendar.set(year, month - 1, day, hour, minute, second);
		calendar.set(Calendar.MILLISECOND, milli);
		dateUtil.setNow(calendar.getTime());
	}

	private void runExpectingException(HttpServlet servlet, ErrorCodeType code)
			throws ServletException, IOException {
		try {
			runServlet(servlet, request, response);
			fail("Expected a SearchServletException");
		} catch (SearchServletException e) {
			assertEquals(code, e.getCode());
		}
	}

}
